// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


/** Document the purpose of this class.
 *
 * @version 1.0
 * @author Timo Stich
 */

package org.columba.coder;

import java.io.*;

public class Base64Decoder extends Decoder {

    private Reader input;
    private Writer output;
    private int pads;

    
    public Base64Decoder()
    {
        super();
        coding = new String("base64");
    }
    
    
    public void setupDecoder(Reader input, Writer output)
    {        
        this.input = input;
        this.output = output;
    }
        
    public void run()
    {
        int block = 0;

        char[] inByte = new char[4];
        char[] outByte = new char[3];

        
        pads = 0;

        try {

            while( input.read( inByte ) == 4 ) {                
                removeCRLF( inByte );
                
                block += getCode( inByte[0] );
                block = block << 6;                    
                block += getCode( inByte[1] );
                block = block << 6;                    
                block += getCode( inByte[2] );
                block = block << 6;                    
                block += getCode( inByte[3] );
                
                outByte[0] = (char) ((block & 0x0ff0000) >> 16 );
                outByte[1] = (char) ((block & 0x000ff00) >> 8 );
                outByte[2] = (char) (block & 0x00000ff);
                
                if( pads != 0 ) {
                    output.write( outByte, 0, 3 - pads );
                    output.close();            
                    return;
                }
                
                output.write( outByte );
                    
                block = 0;
            }
            output.close();
        }
        catch( IOException e ) {
            System.out.println( e );
        }
        
    }
    
    private int getCode( char code )
    {
        if( code == '+' ) return 62;
        if( code == '/' ) return 63;
        if( code <= '9' ) return (code - '0' + 52);
        if( code == '=' ) {
            pads ++;
            return 0;
        }            
        if( code <= 'Z' ) return ( code - 'A' );
        if( code <= 'z' ) return (code - 'a'+ 26);
        return -1;
    }

    private void removeCRLF( char[] inByte ) throws IOException
    {
        while( inByte[0] == '\n' ) {
            inByte[0] = inByte[1];
            inByte[1] = inByte[2];
            inByte[2] = inByte[3];
            input.read( inByte,3,1);
        }
        while( inByte[1] == '\n' ) {
            inByte[1] = inByte[2];
            inByte[2] = inByte[3];
            input.read( inByte,3,1);
        }
        while( inByte[2] == '\n' ) {
            inByte[2] = inByte[3];
            input.read( inByte,3,1);
        }
        while( inByte[3] == '\n' ) {
            input.read( inByte,3,1);
        }
    }

}
