// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


/** Document the purpose of this class.
 *
 * @version 1.0
 * @author Timo Stich
 */

package org.columba.coder;


import java.io.*;


public class Base64Encoder extends Encoder {

    private Reader input;
    private Writer output;

    public Base64Encoder()
    {
        super();
        coding = new String("base64");
    }
    
    public void setupEncoder(Reader input, Writer output)
        {
            this.input = input;
            this.output = output;
        }

    public void run()
    {
        char[] quad = new char[4];
        char[] tri = new char[3];
        int codeWord;
        long pos = 3;
        int i = 0;
        int rest;

        try {
        
            rest = input.read(tri);
            
            while( rest == 3  )
            {
                codeWord = tri[0] << 16;
                codeWord+= tri[1] << 8;
                codeWord+= tri[2];

                quad[3] = getCode( codeWord & 0x03f );
                codeWord = codeWord >> 6;
                quad[2] = getCode( codeWord & 0x03f );
                codeWord = codeWord >> 6;
                quad[1] = getCode( codeWord & 0x03f );
                codeWord = codeWord >> 6;
                quad[0] = getCode( codeWord );
                
                output.write(quad);
                
                pos+=3;
                i++;
                
                if( i == 18 ) {
                    output.write('\n');
                    i = 0;
                }
                
                rest = input.read(tri);            
            }
            
            if( rest > 0 )
            {            
                codeWord = tri[0] << 16;
                codeWord+= tri[1] << 8;
                codeWord+= tri[2];
                
                output.write( getCode( (codeWord & 0x0fc0000) >> 18 ) );
                output.write( getCode( (codeWord & 0x003f000) >> 12 ) );
                if( rest == 1 ) {
                    output.write('=');
                    output.write('=');
                }
                else {
                    output.write( getCode( (codeWord & 0x0000fc0) >> 6 ) );
                    output.write('=');
                }
            }

            output.close();
        }
        catch( IOException e ) {
            System.out.println( e );
        }
    }
    
    private char getCode( int val )
        {
            if( val < 26 ) return (char) ('A'+val);
            if( val < 52 ) return (char) ('a'+val-26);
            if( val < 62 ) return (char) ('0'+val-52);
            if( val == 62 ) return '+';

            return '/';
        }
    

}
