// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


/** Document the purpose of this class.
 *
 * @version 1.0
 * @author Timo Stich
 */

package org.columba.coder;

import java.io.*;
import java.util.*;

public class EncodedWordDecoder extends Decoder {

    private BufferedReader input;
    private Writer output;

    public EncodedWordDecoder()
    {
        super();
        coding = new String("encoded-word");
    }


    public void setupDecoder(Reader input, Writer output)
    {
        this.input = new BufferedReader( input );
        this.output = output;
    }

    public void run()
    {
        String aktLine;

        int start = 0;
        int end = 0;

        try {
            aktLine = input.readLine();

            while( aktLine != null ) {

                start = aktLine.indexOf("=?",end);

                while( start != -1) {
                    output.write( aktLine, end, start - end );
                    end = aktLine.indexOf("?=", start) + 2;

                    output.write(decode(aktLine.substring(start, end)));

                    start = aktLine.indexOf("=?",end);
                }
                aktLine = input.readLine();
            }

            output.close();

        }
        catch( IOException e ) {
            System.out.println( e );
        }

    }

    public String decode( String input )
    {
        StringBuffer output = new StringBuffer();

        int start = 0;
        int end = 0;
        int markPos;

        try {

            start = input.indexOf("=?",end);

            if( start == -1 ) return input;

            while( start != -1) {
                output.append( input.substring( end, start ) );
                markPos = input.indexOf( "?", start+2 ) +1;          // second ?
                markPos = input.indexOf( "?", markPos ) +1;        // third ?
                end = input.indexOf("?=", markPos) + 2;

                output.append(decodeEncodedWord(input.substring(start, end)));

                start = input.indexOf("=?",end);
            }

            if( end < input.length() )
                output.append( input.substring(end) );

            return output.toString();
        }
        catch( Exception e )
        {
            System.out.println( e );
            System.out.println( input );
        }

        return input;
    }


    private String decodeEncodedWord( String encodedWord )
    {
    	String charset;
        char encoding;
        String codedWord;
        StringTokenizer items;

        items = new StringTokenizer( encodedWord, "?" );

        if( items.countTokens() < 4 ) return encodedWord;

        items.nextToken();   // The beginning "="

        charset = items.nextToken().toLowerCase();
        encoding = items.nextToken().toLowerCase().charAt(0);
        codedWord = items.nextToken();

        switch( encoding ) {
        case( 'q' ) :
        {
            return  decodeQP( codedWord );
        }
        case( 'b' ) :
        {
            return decodeB64( codedWord );
        }
        }

        return encodedWord;
    }

// The Base64 Decoder Section

    int pads;

    private String decodeB64( String decodedString )
        {
            int i = 0;
            int j = 0;
            int block = 0;
            char buffer;

            pads = 0;

            byte[] outByte = new byte[3];

            StringBuffer output = new StringBuffer();

            while( i < decodedString.length() ) {
                //System.out.println(i);

                for( j=0; j<=2; j++) {
                    buffer = decodedString.charAt(i+j);
                    block += getCode( buffer );
                    block = block << 6;
                }

                buffer = decodedString.charAt(i+3);
                block += getCode( buffer );

                outByte[0] = new Integer( (block & 0x0ff0000) >> 16 ).byteValue();
                outByte[1] = new Integer( (block & 0x000ff00) >> 8 ).byteValue();
                outByte[2] = new Integer( block & 0x00000ff ).byteValue();


                if( pads != 0 )
                {
                    output.append( new String( outByte, 0, 3 - pads ));
                    return output.toString();
                }

                output.append( new String( outByte ));

                i += 4;
                block = 0;
            }

            return output.toString();
        }

    private int getCode( char code )
        {
            if( code == '+' ) return 62;
            if( code == '/' ) return 63;
            if( code <= '9' ) return (code - '0' + 52);
            if( code == '=' ) {
                pads ++;
                return 0;
            }
            if( code <= 'Z' ) return ( code - 'A' );
            if( code <= 'z' ) return (code - 'a'+ 26);
            return -1;
        }

// The Quoted Prinatble Decoder Section

    private String decodeQP( String decodedString )
        {
            String aktLine = decodedString;
            StringBuffer output = new StringBuffer();
            int i = 0;
            int crlfPos;
            char aktChar;

            while( i < aktLine.length()  ) {

                if( aktLine.charAt( i ) == '=') {
                    output.append( getChar( aktLine.substring(i+1,i+3) ) );
                    i += 3;
                }
                else {
                    aktChar = aktLine.charAt(i);
                    if( aktChar == '_' ) output.append(' ');
                    else output.append( aktChar );
                    i++;
                }
            }
            return output.toString();
        }

    private String getChar( String code )
        {
            char[] digit = new char[2];
            int number = 0;
            byte[] output = new byte[1];

            code = code.toLowerCase();

            digit[0] = code.charAt(0);
            digit[1] = code.charAt(1);

            for(int i=0; i<=1; i++) {
                if(digit[i]>='a') number += digit[i]-'a'+10;
                else number += digit[i]-'0';

                if( i!=1) number *= 16;
            }


            output[0] = new  Integer(number).byteValue();

            return new String( output );

        }



}


