// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


/* Document the purpose of this class.
 *
 * @version 1.0
 * @author Timo Stich
 */

package org.columba.coder;

import java.io.*;

public class NullEncoder extends Encoder {

    private static final int BUFFER_SIZE = 2048;
    private Reader input;
    private Writer output;

    public NullEncoder()
    {
        coding = new String("null");
    }    
    
    public void setupEncoder(Reader input, Writer output)
    {
        this.input = input;
        this.output = output;
    }

    public void run()
    {
        if( input == null ) return;
        
        char[] buffer = new char[BUFFER_SIZE];
        int len;

        try {
            
            while( true ) {
                len = input.read( buffer );
                if(len == -1) {
                    output.close();
                    return;
                }
                output.write( buffer,0, len );
                if(len < BUFFER_SIZE) {
                    output.close();
                    return;
                }
            }
        }
        catch( IOException e ) {
            System.out.println(e);
        }
    }

    public String decode( String input )
    {
        return null;
    }
    

}
