// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


/** Document the purpose of this class.
 *
 * @version 1.0
 * @author Timo Stich
 */

package org.columba.coder;


import java.io.*;

public class QuotedPrintableDecoder extends Decoder {

    private BufferedReader input;
    private Writer output;

    public QuotedPrintableDecoder()
    {
        super();
        coding = new String("quoted-printable");

    }

    
    public void setupDecoder(Reader input, Writer output)
    {
        this.input = new BufferedReader( input );
        this.output = output;
    }
    
    public void run()
    {
        String aktLine = null;
        int i = 0;
        int crlfPos;

        try {
            aktLine = input.readLine();
            
            while( aktLine != null )
            {
                while( i < aktLine.length()  ) {
                    
                    if( aktLine.charAt( i )=='=' ) {
                        if( i == aktLine.length()-1 ) {
                            aktLine = input.readLine();
                            i = 0;
                        }
                        else {
                            output.write( getCode( aktLine.substring(i+1,i+3) ) );
                            i += 3;
                        }
                    }
                    else {
                        output.write( aktLine.charAt( i ) );
                        i++;    
                    }
                }
                output.write('\n');
                aktLine = input.readLine();
                i = 0;
            }

            output.close();
        }
        catch( IOException  e )
        {
            System.out.println( e );
        }
    }

    
    private String getCode( String code )
        {
            char[] digit = new char[2];
            int number = 0;
            byte[] output = new byte[1];

            code = code.toLowerCase();
            
            digit[0] = code.charAt(0);
            digit[1] = code.charAt(1);
            
            for(int i=0; i<=1; i++) {
                if(digit[i]>='a') number += digit[i]-'a'+10;
                else number += digit[i]-'0';
                
                if( i!=1) number *= 16;                
            }

            
            output[0] = new Integer(number).byteValue();
            
            return new String( output );
            
        }
    

}





