// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.config;

import java.util.Vector;
import org.columba.message.*;
import org.columba.folder.*;

import org.w3c.dom.*;


public class AccountItem extends DefaultItem
{
    private AdapterNode name;
    private AdapterNode uid;

    private boolean pop3;

    private IdentityItem identity;
    private PopItem pop;
    private ImapItem imap;
    private SmtpItem smtp;
    private PGPItem pgp;
    private SpecialFoldersItem folder;

    private AdapterNode rootNode;

    public AccountItem( AdapterNode rootNode, Document doc )
    {
        super( doc );

        this.rootNode = rootNode;

        pop3=false;

        identity = null;
        pop = null;
        imap = null;
        identity = null;
        smtp = null;

        if ( rootNode != null ) parse();



    }

    public AdapterNode getRootNode()
    {
        return rootNode;
    }

    protected void parse()
    {
        //System.out.println("parseing accountitem");

        for ( int i=0; i<getRootNode().getChildCount(); i++ )
        {
            AdapterNode child = getRootNode().getChildAt(i);

            if ( child.getName().equals("name") )
            {
                name = child;
            }
            else  if ( child.getName().equals("uid") )
            {
                uid = child;
            }
            else  if ( child.getName().equals("identity") )
            {
                identity = new IdentityItem( child, getDocument() );
            }
            else  if ( child.getName().equals("popserver") )
            {
                pop3 = true;
                pop = new PopItem( child, getDocument() );
            }
            else  if ( child.getName().equals("imapserver") )
            {
                pop3 = false;
                imap = new ImapItem( child, getDocument() );
            }
            else  if ( child.getName().equals("smtpserver") )
            {
                smtp = new SmtpItem( child, getDocument() );
            }
            else  if ( child.getName().equals("pgp") )
            {
                pgp = new PGPItem( getDocument(), child );
            }
            else  if ( child.getName().equals("specialfolders") )
            {
                folder = new SpecialFoldersItem( getDocument(), child );
            }
        }
    }

    protected void createMissingElements()
    {
    }

    public boolean isPopAccount()
    {
        return pop3;
    }

    public void remove()
    {
        AdapterNode parent = new AdapterNode( name.domNode.getParentNode() );
        AdapterNode top = new AdapterNode( parent.domNode.getParentNode() );

	top.removeChild( parent );
    }



    public void setNameNode( AdapterNode node )
    {
        name = node;
    }

    public void setUidNode( AdapterNode node )
    {
        uid = node;
    }


    public void setPopItem( PopItem item )
    {
        pop = item;
    }

    public void setImapItem( ImapItem item )
    {
        imap = item;
    }

    public void setIdentityItem( IdentityItem item )
    {
        identity = item;
    }

    public void setSmtpItem( SmtpItem item )
    {
        smtp = item;
    }

    public SpecialFoldersItem getSpecialFoldersItem()
    {
        return folder;
    }

    public PopItem getPopItem()
    {
        return pop;
    }

    public SmtpItem getSmtpItem()
    {
        return smtp;
    }

    public PGPItem getPGPItem()
    {
        return pgp;
    }

    public ImapItem getImapItem()
    {
        return imap;
    }

    public IdentityItem getIdentityItem()
    {
        return identity;
    }


    /*
    public boolean isPopAccount()
    {
        if ( pop == null )
            return false;
        else
            return true;
    }
    */


    public void setName( String str )
    {
        setTextValue( name, str );
    }

    public String getName()
    {
        return getTextValue( name );
    }

    public void setUid( int i )
    {
        Integer h = new Integer( i );

        setTextValue( uid, h.toString() );
    }


    public int getUid()
    {
        Integer i = new Integer(  getTextValue( uid ) );

        return i.intValue();
    }



}




