// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.config;




import java.io.*;
import java.net.*;
import java.util.zip.*;

import java.util.*;


import org.columba.folder.*;
import org.columba.main.*;
import org.columba.util.*;



import org.w3c.dom.*;

public class Config
{
    private MainInterface mainInterface;

    private File file;

    public static String lineSeparator;
    public static String fileSeparator;
    public static String osName;
    public static String userName;
    public static String userHome;
    public File configDirectory;
    public static File inboxDirectory;
    public static File sentDirectory;
    public static File headerDirectory;
    public static File pop3Directory;
    public static String userDir;


      //private XmlConfig xmlConfig;
    private AccountList accountList;

    private AccountXmlConfig accountConfig;
    private FolderXmlConfig folderConfig;
    private OptionsXmlConfig optionsConfig;
    private AddressbookXmlConfig addressbookConfig;


    private MainFrameOptionsXmlConfig mainFrameOptionsConfig;
    private AddressbookOptionsXmlConfig addressbookOptionsConfig;
    private PopManageOptionsXmlConfig popManageOptionsConfig;
    private ComposerOptionsXmlConfig composerOptionsConfig;

    public Config(MainInterface mainInterface)
    {
	this.mainInterface = mainInterface;

	getSystemProperties();

        init();
    }

    public Config(MainInterface mainInterface,String cdir)
    {
	this.mainInterface = mainInterface;

        configDirectory = new File(cdir);

	getSystemProperties();

        init();
    }

    protected void init()
    {

        File accountFile = new File( configDirectory.toString() + fileSeparator + "account.xml");
        accountConfig = new AccountXmlConfig( accountFile );



        File folderFile = new File( configDirectory.toString() + fileSeparator + "tree.xml");
        folderConfig = new FolderXmlConfig( folderFile );
        File optionsFile = new File( configDirectory.toString() + fileSeparator + "options.xml");
        optionsConfig = new OptionsXmlConfig( optionsFile );
        File addressbookFile = new File( configDirectory.toString() + fileSeparator + "addressbook.xml");
        addressbookConfig = new AddressbookXmlConfig( addressbookFile );

	File mainFrameOptionsFile = new
	    File( configDirectory.toString() + fileSeparator + "mainframeoptions.xml");
        mainFrameOptionsConfig = new MainFrameOptionsXmlConfig( mainFrameOptionsFile );
	File popManageOptionsFile = new
	    File( configDirectory.toString() + fileSeparator + "popmanageoptions.xml");
        popManageOptionsConfig = new PopManageOptionsXmlConfig( popManageOptionsFile );
	File composerOptionsFile = new
	    File( configDirectory.toString() + fileSeparator + "composeroptions.xml");
        composerOptionsConfig = new ComposerOptionsXmlConfig( composerOptionsFile );
	File addressbookOptionsFile = new
	    File( configDirectory.toString() + fileSeparator + "addressbookoptions.xml");
        addressbookOptionsConfig = new AddressbookOptionsXmlConfig( addressbookOptionsFile );


        load();

        accountList = accountConfig.getAccountList();

        pop3Directory = new File( configDirectory, "pop3server" );
        if ( !pop3Directory.exists() )
        {
            pop3Directory.mkdir();
        }

    }

    public void save()
    {
        accountConfig.save();
        folderConfig.save();
        optionsConfig.save();
        addressbookConfig.save();

	mainFrameOptionsConfig.save();
	composerOptionsConfig.save();
	addressbookOptionsConfig.save();
	popManageOptionsConfig.save();
    }

    public void load()
    {
        accountConfig.load();
        folderConfig.load();
        optionsConfig.load();
        addressbookConfig.load();

	mainFrameOptionsConfig.load();
	composerOptionsConfig.load();
	addressbookOptionsConfig.load();
	popManageOptionsConfig.load();
    }

    public AccountList getAccountList()
    {
        return accountList;
    }

    public AccountXmlConfig getAccountConfig()
    {
        return accountConfig;
    }

    public FolderXmlConfig getFolderConfig()
    {
        return folderConfig;
    }

    public OptionsXmlConfig getOptionsConfig()
    {
        return optionsConfig;
    }

    public AddressbookXmlConfig getAddressbookConfig()
    {
        return addressbookConfig;
    }

    public MainFrameOptionsXmlConfig getMainFrameOptionsConfig()
    {
        return mainFrameOptionsConfig;
    }

    public PopManageOptionsXmlConfig getPopManageOptionsConfig()
    {
        return popManageOptionsConfig;
    }

    public ComposerOptionsXmlConfig getComposerOptionsConfig()
    {
        return composerOptionsConfig;
    }

    public AddressbookOptionsXmlConfig getAddressbookOptionsConfig()
    {
        return addressbookOptionsConfig;
    }



    protected void copy( String sourceDir, String outputDir )
    {
        URL url = ClassLoader.getSystemResource(
            "org/columba/config/"+sourceDir);

        File destFile = new File( outputDir+sourceDir );

        java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
        String protocol = url.getProtocol();

        if( protocol.equals("file") ) {
            try {
                FileUtilities.copy( new File( url.getPath() ), destFile  );
            }
            catch( IOException e )
            {
                System.out.println( e );
            }
        }
        if( protocol.equals("jar") ) {
            String path = url.getPath();
            String jarpath = path.substring(
                path.indexOf("file:")+5,
                path.indexOf("!"));

            String jarEntry = path.substring(
                path.indexOf("!")+2 );


            try
            {
                ZipFile zipfile = new ZipFile( jarpath );
                ZipEntry zipentry = zipfile.getEntry( jarEntry );

                FileOutputStream out = new FileOutputStream( destFile );

                if( zipentry != null )
                {
                    long size = zipentry.getSize();
                    if( size > 0 )
                    {
                        byte bytes[] = new byte[ (int) size ];
                        InputStream in = new BufferedInputStream(
                            zipfile.getInputStream( zipentry ) );
                        in.read( bytes );
                        in.close();
                        out.write( bytes );
                        out.close();
                    }
                }
            }
            catch( ZipException e )
            {
                System.out.println(e);
                return;
            }
            catch( IOException e )
            {
                System.out.println(e);
                return;
            }
            return;
        }

        return;
    }


    protected void createDefaultConfigFiles()
    {
        copy( "account.xml", configDirectory.toString()+fileSeparator);
        copy( "account.dtd", configDirectory.toString()+fileSeparator);
        copy( "addressbook.xml", configDirectory.toString()+fileSeparator);
        copy( "addressbook.dtd", configDirectory.toString()+fileSeparator);
        copy( "options.xml", configDirectory.toString()+fileSeparator);
        copy( "options.dtd", configDirectory.toString()+fileSeparator);
        copy( "tree.xml", configDirectory.toString()+fileSeparator);
        copy( "tree.dtd", configDirectory.toString()+fileSeparator);

        copy( "mainframeoptions.xml", configDirectory.toString()+fileSeparator);
        copy( "popmanageoptions.xml", configDirectory.toString()+fileSeparator);
        copy( "addressbookoptions.xml", configDirectory.toString()+fileSeparator);
        copy( "composeroptions.xml", configDirectory.toString()+fileSeparator);
    }


    protected boolean isWindowsPlatform()
    {
        if ( osName.equals("Windows 95") ) return true;
        if ( osName.equals("Windows 98") ) return true;

        return false;
    }

    protected void getSystemProperties()
    {
        lineSeparator = new String( System.getProperty("line.separator") );
        fileSeparator = new String( System.getProperty("file.separator") );
        osName = new String( System.getProperty("os.name") );

        //System.out.println("os.arch: "+System.getProperty("os.arch") );
        //System.out.println("os.version: "+System.getProperty("os.version") );

        userName = new String( System.getProperty("user.name") );
        userHome = new String( System.getProperty("user.home") );
        userDir = new String( System.getProperty("user.dir") );
        //System.out.println("User home: "+userHome);


        if ( isWindowsPlatform() == true )
        {
            // this os has no home directory
            // for example windows9x

            if ( configDirectory == null )
                configDirectory = new File( "config" );


            if ( !configDirectory.exists() )
            {
                configDirectory.mkdir();

                createDefaultConfigFiles();

            }
        }
        else
        {
            if ( configDirectory == null )
                configDirectory = new File( userHome+fileSeparator+".columba" );

            if ( !configDirectory.exists() )
            {
                configDirectory.mkdir();

                createDefaultConfigFiles();

            }
        }

    }



    public void saveConfiguration()
    {
        save();
    }



      /*
    public AccountItem getAccountItem( AdapterNode accountNode )
    {
        AdapterNode child = accountNode;

        AccountItem item = new AccountItem( getDocument() );

        item.setNameNode( child.getChild("name") );

        if ( isPopAccount( child ) == true )
        {
            PopItem pop = getPopItem( child.getChild("popserver") );
            item.setPopItem( pop );
        }
        else
        {
            ImapItem imap = getImapItem( child.getChild("imapserver") );
            item.setImapItem( imap );
        }

        IdentityItem identity = getIdentityItem( child.getChild("identity") );
        item.setIdentityItem( identity );

        SmtpItem smtp = getSmtpItem( child.getChild("smtpserver") );
        item.setSmtpItem( smtp );


        return item;
    }


    public AdapterNode getAccountNode( int i )
    {
	AdapterNode node = getXmlConfig().getSectionNode( "accountlist", 0);

        AdapterNode child = node.getChild( i );

	return child;
    }

    public AdapterNode getAccountNode( String n )
    {
        AdapterNode node = getXmlConfig().getSectionNode( "accountlist", 0);


	for ( int i=0; i<node.getChildCount(); i++ )
        {
            AdapterNode child = node.getChild( i );

            AdapterNode nameNode  = child.getChild("name");
            if ( nameNode != null )
            {
                String name = nameNode.getValue();

                if ( name.equals( n ) )
                    return child;
            }
        }
	return null;
    }


    public String getAccountName( String s )
    {
        String accountName=null;

        for ( int i=0; i<getAccountCount(); i++)
        {
            AdapterNode accountNode = getAccountNode( i );
            AdapterNode serverNode = accountNode.getChild("popserver");
            if( serverNode == null) serverNode = accountNode.getChild("imapserver");
            AdapterNode hostNode = serverNode.getChild("host");
            String hostName = hostNode.getValue();

            if( hostName.equals( s ) )
            {
                System.out.println("host found: "+ hostName +" - "+s);
                AdapterNode nameNode = accountNode.getChild("name");
                accountName = nameNode.getValue();
                break;
            }
        }

        return accountName;
    }
      */


      /*
    public boolean isPopAccount( AdapterNode parent )
    {
	if ( parent.getChild("popserver") != null ) return true;
	return false;
    }

    public boolean isImapAccount( AdapterNode parent )
    {
	if ( parent.getChild("imapserver") != null ) return true;
	return false;
    }

    public AdapterNode getIncomingServerNode( AdapterNode accountNode )
    {
	AdapterNode serverNode = null;

	for ( int i=0; i< accountNode.getChildCount(); i++)
	    {
		serverNode = accountNode.getChild( i );

		if ( serverNode.getName().equals("popserver") )
		    {
			return serverNode;
		    }
		else if ( serverNode.getName().equals("imapserver") )
		    {
			return serverNode;
		    }

	    }
	return null;
    }

    public AdapterNode getOutgoingServerNode( AdapterNode accountNode )
    {
	AdapterNode serverNode = null;

	for ( int i=0; i< accountNode.getChildCount(); i++)
	    {
		serverNode = accountNode.getChild( i );

		if ( serverNode.getName().equals("smtpserver") )
		    {
			return serverNode;
		    }
	    }
	return null;
    }

    public String getAccountName( AdapterNode accountNode )
    {
	String name = null;
	AdapterNode accountNameNode = accountNode.getChild("name");

	if ( accountNameNode != null )  name = accountNameNode.getValue();
	return name;
    }


    public AdapterNode getIdentityNode( AdapterNode accountNode )
    {
	AdapterNode identityNode = accountNode.getChild("identity");

	return identityNode;
    }

    public IdentityItem getIdentityItem( AdapterNode identityNode )
    {

	IdentityItem item = new IdentityItem( getDocument() );

	item.setNameNode( identityNode.getChild("name") );
	item.setAddressNode( identityNode.getChild("address") );
	item.setOrganisationNode( identityNode.getChild("organisation") );
	item.setReplyNode( identityNode.getChild("replyaddress") );
        item.setSignatureNode( identityNode.getChild("signature") );
        item.setAttachSignatureNode( identityNode.getChild("attachsignature") );

	return item;
    }

    public PopItem getPopItem( AdapterNode popserverNode )
    {
	PopItem item = new PopItem( getDocument() );

	item.setUserNode( popserverNode.getChild( "user" ) );
	item.setPasswordNode( popserverNode.getChild(  "password" ) );
	item.setHostNode( popserverNode.getChild( "host" ) );
	item.setPortNode( popserverNode.getChild( "port" ) );
	item.setLeaveMessageNode( popserverNode.getChild( "leave" ) );
	item.setSavePasswordNode( popserverNode.getChild( "savepassword" ) );
        item.setExcludeNode( popserverNode.getChild( "exclude" ) );
	item.setUidNode( popserverNode.getChild(  "uid" ) );

	return item;
    }


    public ImapItem getImapItem( AdapterNode imapserverNode )
    {
	ImapItem item = new ImapItem( getDocument() );

	item.setUserNode( imapserverNode.getChild( "user" ) );
	item.setPasswordNode( imapserverNode.getChild( "password" ) );
	item.setHostNode( imapserverNode.getChild( "host" ) );
	item.setPortNode( imapserverNode.getChild( "port" ) );
        item.setSavePasswordNode( imapserverNode.getChild( "savepassword" ) );


	return item;
    }
      */


      /*
        public void addEmptyAccount( boolean b )
        {
	getXmlConfig().addEmptyAccount( b );
        }

        public AdapterNode addFolderNode( AdapterNode parentNode, String name, String access,
        String messagefolder, String type, int uid )
        {
	return getXmlConfig().addFolderNode( parentNode, name, access, messagefolder, type, uid );
        }
      */

      /*
    public FolderItem getFolderItem( AdapterNode node )
    {
        if ( node != null )
        {
            if ( node.getName().equals("folder") )
            {


                FolderItem folderItem = new FolderItem( getDocument() );

                folderItem.setNameNode( node.getChild( "name" ) );

                  //System.out.println("folder found: "+ node.getChild("name").getValue() );
                folderItem.setAccessRightsNode( node.getChild( "accessrights" ) );
                folderItem.setMessageFolderNode( node.getChild( "messagefolder" ) );
                folderItem.setTypeNode( node.getChild( "type" ) );
                folderItem.setUidNode( node.getChild( "uid" ) );


                String type = node.getChild("type").getValue();

                if ( ( type.equals("imap") ) || ( type.equals("imaproot") ) )
                    folderItem.setAccountUidNode( node.getChild("accountuid") );


                AdapterNode filterNode = node.getChild( "filterlist" );
                if ( filterNode != null )
                {
                      //System.out.println("filter found");
                    AdapterNode child;
                    for ( int i=0; i< filterNode.getChildCount(); i++)
                    {
                        child = filterNode.getChild( i );
                        folderItem.addFilterNode( child );
                    }
                }
                else
                {
                    AdapterNode searchNode = node.getChild("search");
                    if ( searchNode != null )
                    {
                          //System.out.println("search node found");

                        folderItem.setSearchNode( searchNode );
                    }

                }

                return folderItem;
            }

        }

        return null;
    }
      */


      /*
    public HeaderTableItem getHeaderTableItem()
    {
	HeaderTableItem headerTableItem = new HeaderTableItem();

        AdapterNode headerTableItemNode = getXmlConfig().getSectionNode( "headertable", 0 );

        for ( int i=0; i< headerTableItemNode.getChildCount(); i++)
        {
            AdapterNode parent = headerTableItemNode.getChild(i);

            headerTableItem.addHeaderItem( parent.getChild(0),
                                           parent.getChild(3),
                                           parent.getChild(1),
                                           parent.getChild(2)
                                           );
        }

	return headerTableItem;
    }
      */


      /*
    public SmtpItem getSmtpItem( AdapterNode smtpNode )
    {
	SmtpItem smtpItem = new SmtpItem( getDocument() );

        smtpItem.setUserNode( smtpNode.getChild("user") );
        smtpItem.setHostNode( smtpNode.getChild("host") );
        smtpItem.setPortNode( smtpNode.getChild("port") );
        smtpItem.setESmtpNode( smtpNode.getChild("esmtp") );
	smtpItem.setUidNode( smtpNode.getChild("uid") );
	smtpItem.setBccYourselfNode( smtpNode.getChild("bccyourself") );
	smtpItem.setBccAnotherNode( smtpNode.getChild("bccanother") );
	return smtpItem;
    }


    public int getAccountCount()
    {
        AdapterNode node = getXmlConfig().getSectionNode( "accountlist", 0);

        int count  = node.getChildCount();

	return count;
    }
      */

      /*
    public Document getDocument()
    {
        return xmlConfig.getDocument();
    }

    public XmlConfig getXmlConfig()
    {
        return xmlConfig;
    }
      */





}







