// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.filter;

import org.columba.config.*;
import org.columba.folder.*;
import org.columba.message.*;
import org.columba.main.*;

import java.util.Vector;

import org.w3c.dom.*;


public class Filter extends DefaultItem
{
    private MainInterface mainInterface;
    private AdapterNode node;
    private Vector actionList;
    private FilterRule rule;
    private Folder folder;
    private AdapterNode actionListNode;
    private AdapterNode nameNode;
    private AdapterNode enabledNode;

    public Filter(AdapterNode node, MainInterface mainInterface)
    {
        super( mainInterface.config.getFolderConfig().getDocument() );
        this.mainInterface = mainInterface;
        this.node = node;
        actionList = new Vector();
          //System.out.println("node: "+node);

        if ( node != null ) parseNode();

    }

    public AdapterNode getRootNode()
    {
        return node;
    }

    public AdapterNode getActionListNode()
    {
        return actionListNode;
    }


    public void parseNode()
    {
        AdapterNode child;

        for ( int i=0; i<node.getChildCount(); i++)
        {
            child = node.getChild(i);

            if ( child.getName().equals("actionlist") )
            {
                actionListNode = child;

                for ( int j=0; j<child.getChildCount(); j++)
                {
                    AdapterNode subChild = child.getChild(j);
                    if ( subChild.getName().equals("action") )
                        actionList.add( new FilterAction( subChild, getDocument() ) );
                }

            }
            else if ( child.getName().equals("filterrule") )
            {
                rule = new FilterRule( child, getDocument() );
            }
            else if ( child.getName().equals("description") )
            {
                nameNode = child;
            }
            else if ( child.getName().equals("enabled") )
            {
                enabledNode = child;
            }
        }

    }

    public FilterRule getFilterRule()
    {
        return rule;
    }

    public void addEmptyAction()
    {
        AdapterNode n = mainInterface.config.getFolderConfig().addEmptyFilterAction( getActionListNode() );

        FilterAction action = new FilterAction( n, getDocument() );

        actionList.add( action );
    }

    public int getActionCount()
    {
        return actionList.size();
    }


    public FilterAction getFilterAction( int index )
    {
        return (FilterAction) actionList.get(index);
    }

    public void removeAction( int index )
    {
        actionList.remove( index );

        AdapterNode actionNode = getActionListNode().getChildAt( index );
        actionNode.remove();
    }


    public void removeLastAction()
    {
        int index = actionList.size()-1;

        removeAction( index );

    }



    public void addEmptyCriteria()
    {
        rule.addEmptyCriteria( mainInterface );
    }

    public void removeCriteria( int index )
    {
        rule.remove( index );
    }

    public void removeLastCriteria()
    {
        rule.removeLast();
    }

    public void setFolder( Folder f )
    {
        this.folder = f;
    }

    public AdapterNode getNode()
    {
        return node;
    }

    public Boolean getEnabled()
    {
        String str = (String) getTextValue( enabledNode );

        Boolean b = new Boolean( str );

        return b;
    }

    public void setEnabled( Boolean bool )
    {
        setTextValue( enabledNode, bool.toString() );
    }

    public void setName( String s )
    {
        setTextValue( nameNode, s );
    }
    public String getName()
    {
        return getTextValue( nameNode );
    }

    /*
    public boolean getFilterResult(  Rfc822Header header )
    {
        return rule.process( header );
    }
    */

    public Object[] getFilterResult( Object[] uids ) throws Exception
    {
        Vector v = rule.process( folder, uids );

        return v.toArray();
    }


    public boolean processFilter( Object[] uids ) throws Exception
    {


        if ( getEnabled().equals(Boolean.FALSE) ) return true;

        Vector v = rule.process( folder, uids  );
		if ( v.size() == 0 ) return true;
		
              //System.out.println("rule is true");
        FilterAction action;

              //System.out.println(" actionlist.size() = "+ actionList.size() );
        boolean moved = false;

            for ( int i=0; i<actionList.size(); i++)
            {
                action = (FilterAction) actionList.get(i);

                if ( action.processAction( folder, v, mainInterface ) )
                    {
                        moved = true;
                        break;
                    }


            }
            //return moved;

        return true;
    }


}
