// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.filter;


import java.util.*;
import org.columba.message.*;
import org.columba.config.*;
import org.columba.main.*;
import org.columba.folder.*;

import org.w3c.dom.*;

public class FilterRule extends DefaultItem
{
        // list of FilterCriteria
    private Vector list;

    private AdapterNode node;

           // condition: match all (AND) = 0, match any (OR) = 1
    private AdapterNode conditionNode;


    public FilterRule( AdapterNode node, Document doc)
    {
        super( doc );

        list = new Vector();
        this.node = node;

        if ( node != null )
            {
                parseNode();
            }

        else
        {
            System.out.println(" node == null ");

        }


    }

    public AdapterNode getRootNode()
    {
        return node;
    }

    public void addEmptyCriteria( MainInterface mainInterface )
    {
        AdapterNode n = mainInterface.config.getFolderConfig().addEmptyFilterCriteria( getRootNode() );
        FilterCriteria criteria = new FilterCriteria( n, getDocument() );

        list.add( criteria );
    }



    public void remove( int index )
    {
        if ( ( index>=0 ) && ( index<list.size() ) )
        {
            list.remove( index );

            int result = -1;

            for ( int i=0; i<getRootNode().getChildCount(); i++ )
            {
                AdapterNode child = (AdapterNode) getRootNode().getChildAt(i);
                String name = child.getName();

                if ( name.equals("filtercriteria") ) result++;

                if ( result == index )
                {
                    child.remove();
                    break;
                }
            }

            //AdapterNode child = getRootNode().getChildAt(index);


        }
    }

    public void removeLast()
    {
        int index = list.size()-1;

        remove( index );
    }

    public FilterCriteria get( int index )
    {
        return (FilterCriteria) list.get(index);
    }

    public int count()
    {
        return list.size();
    }




    public void parseNode()
    {
        AdapterNode child;

        //System.out.println("filter rule:  rule size = "+ node.getChildCount() );

        for ( int i=0; i<node.getChildCount(); i++)
        {
            child = node.getChild(i);
            //System.out.println("filter rule: child-name: "+ child.getName() );

            if ( child.getName().equals("filtercriteria") )
                list.add( new FilterCriteria( child, getDocument() ) );
            else if ( child.getName().equals("condition") )
            {
                //System.out.println("condition value oops "+ child.getValue() );

                conditionNode = child;
            }

        }
    }


    public String getCondition()
    {
        if ( conditionNode == null )
        {
            System.out.println("---------------------------> failure: conditionNode == null !");

            return new String("matchany");
        }
        else
            return getTextValue( conditionNode );
    }

    public void setCondition( String s )
    {
        setTextValue( conditionNode, s );
    }

    public FilterCriteria getCriteria( int index )
    {
        return (FilterCriteria) list.get(index);
    }

    public int getConditionInt()
    {
          //System.out.println("condigtion: "+ condition );

        if ( getCondition().equals("matchall") ) return 0;
        if ( getCondition().equals("matchany") ) return 1;
        return -1;
    }


    /*********************** filtering action *************************/



    public Vector process( Folder folder, Object[] uids ) throws Exception
    {
        FilterCriteria criteria;
        boolean result = true;

        Vector v = processCriteria( folder, uids );

        return v;

    }






    public Vector processCriteria( Folder folder, Object[] uids ) throws Exception
        {
            Vector v = new Vector();
            Vector list = new Vector();
            Vector result = new Vector();

            int match = getConditionInt();

            for ( int i=0; i<count(); i++ )
            {
                FilterCriteria criteria = getCriteria(i);

                String type = criteria.getType();
                String pattern = criteria.getPattern();
                String condition = criteria.getCriteria();

                //System.out.println("i="+i+ " - type="+ type );

                if ( type.equalsIgnoreCase("Custom Headerfield") )
                {
                    String headerField = criteria.getHeaderItem();

                    v = folder.processHeaderCriteria( uids, headerField, pattern, condition );

                    //printList( v );
                }
                else if ( type.equalsIgnoreCase("To or Cc") )
                {
                    Vector v1 = folder.processHeaderCriteria( uids, "To", pattern, condition );
                    Vector v2 = folder.processHeaderCriteria( uids, "Cc", pattern, condition );

                    v = mergeLists( v1, v2 );
                }
                else if ( type.equalsIgnoreCase("Date") )
                {
                    v = folder.processDateCriteria( uids, pattern, condition );
                }
                else if ( type.equalsIgnoreCase("Size") )
                {
                    v = folder.processSizeCriteria( uids, pattern, condition );
                }
                else if ( type.equalsIgnoreCase("Flags") )
                {
                    v = folder.processFlagsCriteria( uids, pattern, condition );
                }
                else if ( type.equalsIgnoreCase("Priority") )
                {
                    v = folder.processPriorityCriteria( uids, pattern, condition );
                }
                else if ( type.equalsIgnoreCase("Body") )
                {
                    v = folder.processBodyCriteria( uids, pattern, condition );
                }
                else
                {
                    v = folder.processHeaderCriteria( uids, type, pattern, condition );
                    //printList( v );
                }
                	
                list.add( v );
            }


            result = mergeFilterResult( list, match );

            // only for debugging purpose
            //printList( result );

            return result;
        }


        // only for debugging
        protected void printList( Vector v )
        {
            //System.out.println("print list:");

            for ( int i=0; i<v.size(); i++ )
            {
                System.out.println( i+"->"+ v.get(i) );
            }
        }


        protected Vector mergeLists( Vector v1, Vector v2 )
        {
            Vector v = new Vector(v1);
            v.addAll( v2 );

/*
            for ( int j=0; j<v2.size(); j++ )
            {
                Object uid = v2.get(j);

                if ( !v1.contains( uid ) ) v.add( uid );
            }
*/

            return v;
        }


         protected Vector mergeFilterResult( Vector list, int match )
        {
            Vector result = (Vector) list.get(0);

			if( list.size() == 1 ) return result;

            if ( match == 0 )
            {
                // match all

                //System.out.println("match all");
				
				/*
				
                if ( list.size() > 1 )
                {
                    Vector base = (Vector) list.get(0);

                    for ( int j=0; j<base.size(); j++ )
                    {
                        Object uid = base.get(j);

                        boolean b = true;

                        for ( int i=1; i<list.size(); i++ )
                        {
                            Vector child = (Vector) list.get(i);
                            if ( !child.contains( uid ) ) b = false;
                        }

                        if ( b == true ) result.add( uid );
                    }
                }
                else
                {
                    result = (Vector) list.get(0);
                }*/
                
                for( int i=1; i<list.size(); i++ ) {
					result.retainAll( (Vector) list.get(i) );
					if( result.size() == 0 ) break;
                }
            }
            else if ( match == 1 )
            {
            	
                // match any

                //System.out.println("match any");

				/*				
                for ( int i=0; i<list.size(); i++ )
                {
                    Vector child = (Vector) list.get(i);

                    for ( int j=0; j<child.size(); j++ )
                    {
                        Object uid = child.get(j);

                        if ( !result.contains( uid ) )
                           result.add( uid );
                    }
                }*/
                
 				int j=1;
                
                // First make one big List
                
                for( int i=1; i<list.size(); i++ ) {
                	result.addAll( (Vector) list.get(i) );
                }

                if( result.size() == 0 ) return result;
                
                // Sort it
                
                Collections.sort( result );
                
                // remove redundancies
                
                
                Object lastuid = result.get(0);
                Object actuid;
                
                int size = result.size();
                
                for( int i=1; i<size; i++ ) {
                	actuid = result.get(j);

                	if( actuid.equals( lastuid ) ) 
                		result.remove( j );
                	else 
                		j++;

                	lastuid = actuid;
                }
            }

            return result;
        }


}
