// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.filter;

import org.columba.config.*;
import org.columba.folder.*;
import org.columba.folder.virtual.*;
import org.columba.message.*;
import org.columba.gui.tree.*;
import org.columba.gui.tree.util.*;
import org.columba.main.*;

import javax.swing.*;

import java.util.Vector;
import java.util.Enumeration;


public class Search extends DefaultItem
{
    private MainInterface mainInterface;
    private AdapterNode vFolderNode;
    private VirtualFolder vFolder;
    private Filter filter;
    private AdapterNode searchNode;

    public Search(AdapterNode node, VirtualFolder vFolder, MainInterface mainInterface)
    {
        super( mainInterface.config.getFolderConfig().getDocument() );
        this.mainInterface = mainInterface;
        this.vFolderNode = node;
        this.vFolder = vFolder;


        vFolder.setSearchFilter( this );

        if ( node != null )
           parseNode();
    }

    protected void parseNode()
    {
        searchNode = vFolderNode.getChild("search");
        AdapterNode filterNode = searchNode.getChild("filter");

        filter = new Filter( filterNode, mainInterface );

    }

    public Filter getFilter()
    {
        return filter;
    }

    public int getUid()
    {
        AdapterNode uidNode = searchNode.getChild("uid");
        String uidStr = getTextValue( uidNode );
        Integer iStr = new Integer( uidStr );
        int uid = iStr.intValue();

        return uid;
    }

    public void setUid( int i )
    {
        Integer uid = new Integer( i );

        AdapterNode uidNode = searchNode.getChild("uid");

        setTextValue( uidNode, uid.toString() );

    }

    public void setInclude( String s )
    {
        AdapterNode includeNode = searchNode.getChild("include");
        setTextValue( includeNode, s );
    }

    public boolean isInclude()
    {
        AdapterNode includeNode = searchNode.getChild("include");
        String include = getTextValue( includeNode );


        if ( include.equals("true") ) return true;
           else return false;
    }

    public void applySearch() throws Exception
    {

        filter = getFilter();


        int uid = getUid();


        vFolder.removeAll();


        Folder srcFolder = mainInterface.treeViewer.getFolderTree().getFolder( uid );

        if ( srcFolder != null )
        {
            try
            {
                srcFolder.getHeaderList( null );
            }
            catch ( Exception ex )
            {
                System.out.println("Error while loading folder: "+ ex.getMessage() );
            }


            filter.setFolder( srcFolder );
            boolean result = false;

            Object[] uids = srcFolder.getUids();
            Object[] resultUids = filter.getFilterResult( uids );

            for ( int i=0; i<resultUids.length; i++ )
            {
                Message message = srcFolder.uidGet( resultUids[i] );
                try
                {
                    vFolder.add( srcFolder, message.getUID(), i );
                }
                catch ( Exception ex )
                {
                    System.out.println("Search exception: "+ ex.getMessage() );
                    ex.printStackTrace();
                }
            }

            /*
                for ( int i=0; i< srcFolder.count(); i++)
                {
                    Message message = srcFolder.get(i);
                    Rfc822Header header = message.getHeader();

                    result = false;

                    result = filter.getFilterResult( header );

                    if ( result )
                    {
                        try
                        {
                            vFolder.add( srcFolder, message.getUID(), i );
                        }
                        catch ( Exception ex )
                        {
                            System.out.println("search: "+ ex.getMessage() );
                        }

                    }
                }
                */


            }
            else
            {
                JOptionPane.showMessageDialog(
                    mainInterface.mainFrame,
                    "The folder you specified does not exist anymore, please update your configuration."
                    );
            }




            if ( isInclude() == true )
                applySearch( srcFolder, filter, vFolder);


    }


    public void applySearch( Folder parent, Filter filter, VirtualFolder destFolder) throws Exception
    {

        Folder folder=null;

        FolderItem item = null;

        for (Enumeration e = parent.children() ; e.hasMoreElements() ;)
        {
            folder = (Folder) e.nextElement();
            item = folder.getFolderItem();


            if ( ( folder != null ) && ( !(item.getType().equals("virtual") ) ) )
            {
                try
                {
                    folder.getHeaderList( null );
                }
                catch ( Exception ex )
                {
                    System.out.println("Error while loading folder: "+ ex.getMessage() );
                }


                filter.setFolder( folder );

                boolean result = false;

                Object[] uids = folder.getUids();
                Object[] resultUids = filter.getFilterResult( uids );

                for ( int i=0; i<resultUids.length; i++ )
                {
                    Message message = folder.uidGet( resultUids[i] );
                    try
                    {
                        vFolder.add( folder, message.getUID(), i );
                    }
                    catch ( Exception ex )
                    {
                        System.out.println("Search exception: "+ ex.getMessage() );
                        ex.printStackTrace();
                    }
                }

                /*
                for ( int i=0; i<folder.count(); i++)
                {
                    Message message = folder.get(i);
                    Rfc822Header header = message.getHeader();

                    result = false;


                    result = filter.getFilterResult( header );
                    if ( result )
                    {
                        try
                        {
                            vFolder.add( folder, (Integer) message.getUID(), i );
                        }
                        catch ( Exception ex )
                        {
                            System.out.println("search: "+ ex.getMessage() );
                        }

                    }
                }
                */

            }
            applySearch( folder, filter, vFolder );

        }

    }




}


