// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.folder;

import java.util.*;

public class Channel
{
    private int pufferSize;
    private Object[] puffer;
    private int pRead;
    private int pWrite;
    private int written;
    private int read;
    private Worker writer;
    private Worker reader;
    private boolean writerWaiting;
    private boolean readerWaiting;

    private int transVolume;
    
    private boolean cancel;

    private boolean mutex;

    public Channel( int pufferSize, int transVolume )
    {
	this.pufferSize = pufferSize;
	this.transVolume = transVolume;
	if( pufferSize < 1 ) pufferSize = 2;
	puffer = new Object[pufferSize];
	pRead = 0;
	pWrite = 0;
	written = 0;
	read = 0;
	cancel = false;
    }

    private synchronized void getMutex()
    {
        while( mutex ) {
            try {
                wait();
            }
            catch( InterruptedException e )
            {
            }
        }

        mutex = true;
    }

    private synchronized void releaseMutex()
    {
        mutex = false;
        notify();
    }

    private void incpWrite()
    {
	pWrite++;
    }

    private void incpRead()
    {
	pRead++;
    }

    public void cancel()
    {

		getMutex();

		if( !cancel ) {
			cancel = true;

	        System.out.println("cancel writer: "+ writer );

			if( writer != null ) {
				writer.channelCanceled = true;
			    writer.setCancel(true);
			}	

	        System.out.println("cancel reader: "+ reader );

			if( reader != null ) {
				reader.channelCanceled = true;
			    reader.setCancel(true);
			}

		}
		
		releaseMutex();
		
		if( !cancel ) {
			if( writer != null ) writer.wakeUp();
	    	if( reader != null ) reader.wakeUp();
		}

    }

    public boolean canWrite( Worker writer )
    {
	boolean output;

	getMutex();

	if( cancel )
        {
            writer.setCancel( true );
            return false;
        }

	output =  ((written - read) < pufferSize );

	System.out.println( "Write: "+pWrite+ " read: "+pRead+" output: "+output);

	if( !output )
	    writerWaiting = true;
	    
	this.writer = writer;

	releaseMutex();

	return output;
    }

    public boolean canRead( Worker reader )
    {
	boolean output;

	getMutex();

	if( cancel )
        {
            reader.setCancel( true );
            return false;
        }

	if( read >= transVolume )
	    System.out.println( "Reader wants to read more than the transVolume!!!");

	output = (written > read);

	System.out.println( "write: "+pWrite+ " Read: "+pRead+" output: "+output);

	if( !output )
	    readerWaiting = true;
	
	this.reader = reader;
	
	releaseMutex();

	return output;
    }


    public void write(Object input)
    {
	
	
	Boolean wakeup;

	getMutex();

	written++;

	pWrite %= pufferSize;

	puffer[pWrite] = input;
	incpWrite();

	if( readerWaiting ) {
	    
	    reader.wakeUp();
	    readerWaiting = false;
	}

	releaseMutex();

    }

    public Object read()
    {
	Object output;

	getMutex();

	read++;

	pRead %= pufferSize;

	output = puffer[pRead];
	incpRead();

	if( writerWaiting ) {
	    
	    writer.wakeUp();
	    writerWaiting = false;
	}

	releaseMutex();

	return output;
    }

    public int getTransVolume()
    {
	return transVolume;
    }

    public void setTransVolume( int v )
    {
        transVolume = v;
    }
}







