// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.folder;

import java.util.*;
import java.lang.reflect.*;
import org.columba.gui.statusbar.*;
import org.columba.main.*;
import org.columba.folder.imap.*;
import org.columba.folder.gui.*;
import org.columba.folder.columba.*;
import org.columba.folder.outbox.*;
import org.columba.folder.encryption.*;
import org.columba.folder.temp.*;
import org.columba.folder.virtual.*;
import org.columba.folder.composer.*;
import org.columba.config.*;


import javax.swing.*;



public class Crossbar
{
	private TaskManager taskManager;
	private MainInterface mainInterface;

	//private Folder selectedFolder;
	//private Object[] uidList;

	private Folder trashFolder;
	private Folder outboxFolder;
	private ColumbaFolder clipBoard;

	public GuiOperator guiOperator;
	public EncryptionOperator encryptionOperator;
	public ComposerOperator composerOperator;

	public Crossbar(MainInterface mainInterface)
	{
		this.mainInterface = mainInterface;
		taskManager = mainInterface.taskManager;

		trashFolder = mainInterface.treeViewer.getFolderTree().getFolder(105);
		outboxFolder = mainInterface.treeViewer.getFolderTree().getFolder(103);
		clipBoard = new ColumbaFolder("clipboard", mainInterface);
		clipBoard.setOperator(
			mainInterface.treeViewer.getFolderTree().getFolder(100).getOperator());

		//ColumbaOperator operator = new ColumbaOperator(mainInterface);
		//operator.start();
		//clipBoard.setOperator( operator );
	}

	public ColumbaFolder getClipBoard()
	{
		return clipBoard;
	}

	public synchronized void operate(Operation op)
	{

		System.out.println(op.toString());

		switch (op.type)
		{
			case (Operation.GUI) :
				{

					GuiOperation guiOp = (GuiOperation) op;

					if (op.mode != Operation.HEADERLIST)
					{
						if (guiOp.source instanceof VirtualFolder)
						{
							VirtualOperator vOperator = (VirtualOperator) guiOp.source.getOperator();
							vOperator.operate(guiOp);

							return;
						}
					}

					if (op.mode == Operation.MESSAGEBODY)
					{
						Channel channelBody = new Channel(1, 1);
						Channel channelAttachment = new Channel(1, 1);

						/* MESSAGEBODY */

						guiOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.MESSAGEBODY,
								op.priority + 2,
								guiOp.uid,
								guiOp.source,
								channelBody));

						guiOperator.addOp(
							new GuiOperation(
								Operation.MESSAGEBODY,
								op.priority,
								guiOp.source,
								guiOp.uid,
								channelBody));

						/* ATTACHMENTLIST */

						guiOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.ATTACHMENTLIST,
								op.priority + 2,
								guiOp.uid,
								guiOp.source,
								channelAttachment));

						guiOperator.addOp(
							new GuiOperation(
								Operation.ATTACHMENTLIST,
								op.priority,
								guiOp.source,
								guiOp.uid,
								channelAttachment));

						break;
					}
					if (op.mode == Operation.MESSAGESOURCE)
					{
						Channel channelBody = new Channel(1, 1);

						/* MESSAGEBODY */

						guiOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.MESSAGESOURCE,
								op.priority + 1,
								guiOp.uid,
								guiOp.source,
								channelBody));

						guiOperator.addOp(
							new GuiOperation(
								Operation.MESSAGESOURCE,
								op.priority,
								guiOp.source,
								guiOp.uid,
								channelBody));
						break;
					}

					if (op.mode == Operation.HEADERLIST)
					{
						Channel channel = new Channel(1, 1);
						//GuiOperation guiOp = (GuiOperation) op;

						/* HEADERLIST */

						guiOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.HEADERLIST,
								op.priority + 5,
								guiOp.source,
								channel));

						guiOperator.addOp(new GuiOperation(Operation.HEADERLIST, op.priority, channel));

						break;
					}

					if (op.mode == Operation.ATTACHMENT)
					{
						Channel channel = new Channel(1, 1);
						//GuiOperation guiOp = (GuiOperation) op;

						/* HEADERLIST */

						guiOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.ATTACHMENT,
								op.priority + 1,
								guiOp.uid,
								guiOp.source,
								guiOp.part,
								channel));

						guiOperator.addOp(
							new GuiOperation(Operation.ATTACHMENT, guiOp.submode, op.priority, channel));

						break;
					}

					break;
				}
			case (Operation.FOLDER) :
				{
					FolderOperation folderOp = (FolderOperation) op;

					if (folderOp.source instanceof VirtualFolder)
					{
						VirtualOperator vOperator = (VirtualOperator) folderOp.source.getOperator();
						vOperator.operate(folderOp);

						return;
					}

					if (op.mode == Operation.FOLDER_SUBLIST)
					{
						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.FOLDER_SUBLIST,
								op.priority,
								folderOp.source,
								folderOp.node,
								folderOp.dialog));
						break;
					}
					if (op.mode == Operation.FOLDER_REFRESHLIST)
					{
						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.FOLDER_REFRESHLIST,
								op.priority,
								folderOp.source,
								folderOp.dialog));
						break;
					}
					if (op.mode == Operation.FOLDER_SUBSCRIBE)
					{
						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.FOLDER_SUBSCRIBE,
								op.priority,
								folderOp.source,
								folderOp.node,
								folderOp.dialog));
						break;
					}
					if (op.mode == Operation.FOLDER_UNSUBSCRIBE)
					{
						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.FOLDER_UNSUBSCRIBE,
								op.priority,
								folderOp.source,
								folderOp.node,
								folderOp.dialog));
						break;
					}
					if (op.mode == Operation.FOLDER_APPLYFILTER)
					{
						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.FOLDER_APPLYFILTER,
								op.priority,
								folderOp.uidList,
								folderOp.source));
						break;
					}
					if (op.mode == Operation.FOLDER_LIST)
					{
						Channel channel = new Channel(1, 1);
						//FolderOperation folderOp = (FolderOperation) op;

						/* FOLDER LIST */

						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.FOLDER_LIST,
								op.priority + 1,
								folderOp.source,
								channel));

						guiOperator.addOp(
							new GuiOperation(Operation.FOLDER_LIST, op.priority, folderOp.source, channel));

						break;
					}

					if (op.mode == Operation.FOLDER_ADD)
					{
						Channel channel = new Channel(1, 1);
						//FolderOperation folderOp = (FolderOperation) op;

						/* FOLDER ADD */

						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.FOLDER_ADD,
								op.priority + 1,
								folderOp.source,
								folderOp.name,
								channel));

						guiOperator.addOp(
							new GuiOperation(
								Operation.FOLDER_ADD,
								op.priority,
								folderOp.source,
								folderOp.name,
								channel));

						break;
					}
					if (op.mode == Operation.FOLDER_RENAME)
					{
						Channel channel = new Channel(1, 1);
						//FolderOperation folderOp = (FolderOperation) op;

						/* FOLDER ADD */

						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.FOLDER_RENAME,
								op.priority + 1,
								folderOp.source,
								folderOp.name,
								channel));

						guiOperator.addOp(
							new GuiOperation(
								Operation.FOLDER_RENAME,
								op.priority,
								folderOp.source,
								folderOp.name,
								channel));

						break;
					}
					if (op.mode == Operation.FOLDER_REMOVE)
					{
						Channel channel = new Channel(1, 1);
						///folderOperation guiOp = (FolderOperation) op;

						/* FOLDER ADD */

						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.FOLDER_REMOVE,
								op.priority + 1,
								folderOp.source,
								channel));

						guiOperator.addOp(
							new GuiOperation(
								Operation.FOLDER_REMOVE,
								op.priority,
								folderOp.source,
								channel));

						break;
					}

					if (folderOp.mode == Operation.MARK_READ)
					{
						Channel channel = new Channel(1, 1);

						try
						{
							folderOp.source.workerViewMark(folderOp.uidList, FolderOp.READ);

							FolderEvent event =
								new FolderEvent(
									folderOp.source,
									folderOp.uidList,
									FolderEvent.MARK,
									Operation.MARK_READ);
							fireFolderChanged(event, folderOp.source);
						}
						catch (Exception ex)
						{
							ex.printStackTrace();
						}

						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.MARK_READ,
								op.priority + 1,
								folderOp.uidList,
								folderOp.source,
								channel));

						guiOperator.addOp(
							new GuiOperation(Operation.MARK_READ, op.priority, folderOp.source, channel));
						break;
					}
					else if (folderOp.mode == Operation.MARK_FLAGGED)
					{
						Channel channel = new Channel(1, 1);

						try
						{
							folderOp.source.workerViewMark(folderOp.uidList, FolderOp.FLAGGED);
							FolderEvent event =
								new FolderEvent(
									folderOp.source,
									folderOp.uidList,
									FolderEvent.MARK,
									Operation.MARK_FLAGGED);
							fireFolderChanged(event, folderOp.source);
						}
						catch (Exception ex)
						{
							ex.printStackTrace();
						}

						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.MARK_FLAGGED,
								op.priority + 1,
								folderOp.uidList,
								folderOp.source,
								channel));
						guiOperator.addOp(
							new GuiOperation(
								Operation.MARK_FLAGGED,
								op.priority,
								folderOp.source,
								channel));
						break;
					}
					else if (folderOp.mode == Operation.MARK_EXPUNGED)
					{
						Channel channel = new Channel(1, 1);

						try
						{
							folderOp.source.workerViewMark(folderOp.uidList, FolderOp.EXPUNGED);
							FolderEvent event =
								new FolderEvent(
									folderOp.source,
									folderOp.uidList,
									FolderEvent.MARK,
									Operation.MARK_EXPUNGED);
							fireFolderChanged(event, folderOp.source);
						}
						catch (Exception ex)
						{
							ex.printStackTrace();
						}

						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.MARK_EXPUNGED,
								op.priority + 1,
								folderOp.uidList,
								folderOp.source,
								channel));
						guiOperator.addOp(
							new GuiOperation(
								Operation.MARK_EXPUNGED,
								op.priority,
								folderOp.source,
								channel));
						break;
					}
					else if (folderOp.mode == Operation.MARK_ANSWERED)
					{
						Channel channel = new Channel(1, 1);

						try
						{
							folderOp.source.workerViewMark(folderOp.uidList, FolderOp.ANSWERED);
							FolderEvent event =
								new FolderEvent(
									folderOp.source,
									folderOp.uidList,
									FolderEvent.MARK,
									Operation.MARK_ANSWERED);
							fireFolderChanged(event, folderOp.source);
						}
						catch (Exception ex)
						{
						}

						folderOp.source.getOperator().addOp(
							new FolderOperation(
								Operation.MARK_ANSWERED,
								op.priority + 1,
								folderOp.uidList,
								folderOp.source,
								channel));
						guiOperator.addOp(
							new GuiOperation(
								Operation.MARK_ANSWERED,
								op.priority,
								folderOp.source,
								channel));
						break;
					}

					/* Delete Operation ( Move to Trash / Delete from Trash ) */

					if ((folderOp.mode == Operation.DELETE))
					{
						try
						{
							folderOp.source.workerViewRemove(folderOp.uidList);

						}
						catch (Exception e)
						{
						}

						
						if (folderOp.source != trashFolder)
						{
							folderOp.mode = Operation.MOVE;

							// if this is not an imap-folder take the local trash-folder
							if (!(folderOp.source.getOperator() instanceof ImapOperator))
								folderOp.drain = trashFolder;
							else
							{
								// imap folder, use the trash-folder from imap-config

								ImapFolder imapFolder = (ImapFolder) folderOp.source;
								ImapRootFolder rootFolder = imapFolder.getImapRootFolder();
								int accountUid = rootFolder.getAccountUid();
								AccountItem accountItem =
									mainInterface.config.getAccountList().uidGet(accountUid);
								SpecialFoldersItem foldersItem = accountItem.getSpecialFoldersItem();
								String trashUid = foldersItem.getTrash();
								int uidInt = Integer.parseInt(trashUid);
								Folder imapTrashFolder =
									mainInterface.treeViewer.getFolderTree().getFolder(uidInt);
								
								if ( folderOp.source != imapTrashFolder )
									folderOp.drain = imapTrashFolder;
								else
								{
									folderOp.mode = Operation.DELETE;
									folderOp.source.getOperator().addOp(folderOp);
									break;
								}

							}
						}
						else
						{
							folderOp.source.getOperator().addOp(folderOp);
							break;
						}
					}

					/* Intern Move and Copy Operation */

					if ((folderOp.source.getOperator() == folderOp.drain.getOperator()))
					{
						if (folderOp.mode == Operation.MOVE)
						{
							try
							{
								folderOp.source.workerViewRemove(folderOp.uidList);

							}
							catch (Exception e)
							{
							}
						}

						folderOp.source.getOperator().addOp(folderOp);
						break;
					}
					else
					{
						/* IO Operation */

						Channel channel = new Channel(1, Array.getLength(folderOp.uidList));

						FolderOperation sendOp =
							new FolderOperation(
								Operation.SEND,
								op.priority,
								folderOp.uidList,
								folderOp.source,
								channel);

						if (folderOp.mode == Operation.MOVE)
						{
							try
							{
								folderOp.source.workerViewRemove(folderOp.uidList);

							}
							catch (Exception e)
							{
							}
							sendOp.destructive = true;
						}

						folderOp.source.getOperator().addOp(sendOp);

						folderOp.drain.getOperator().addOp(
							new FolderOperation(Operation.RECEIVE, op.priority, folderOp.drain, channel));
						break;
					}
				}
				/********************* POP3 **************************/
			case Operation.POP3 :
				{
					Pop3Operation pop3Op = (Pop3Operation) op;
					Channel channel = new Channel(1, 1);

					if (op.mode == Operation.POP3_FETCHALL)
					{

						Pop3Operation fetchOp =
							new Pop3Operation(Operation.POP3_FETCHALL, pop3Op.priority, pop3Op.source,
							//pop3Op.rawString,
		//pop3Op.message,
	pop3Op.popServer, pop3Op.mailcheck);
						//channel );
						pop3Op.popServer.getOperator().addOp(fetchOp);

						break;
					}
					if (op.mode == Operation.POP3_MANAGE_SYNCHRONIZE)
					{

						Pop3Operation fetchOp =
							new Pop3Operation(Operation.POP3_MANAGE_SYNCHRONIZE, pop3Op.priority,
							//pop3Op.source,
		//pop3Op.rawString,
		//pop3Op.message,
	pop3Op.popServer);
						pop3Op.popServer.getOperator().addOp(fetchOp);

						break;
					}

					if (op.mode == Operation.POP3_MANAGE_FETCH)
					{

						Pop3Operation fetchOp =
							new Pop3Operation(
								Operation.POP3_MANAGE_FETCH,
								pop3Op.priority,
								pop3Op.uidList,
								pop3Op.source,
								pop3Op.popServer);
						pop3Op.popServer.getOperator().addOp(fetchOp);

						break;
					}

					if (op.mode == Operation.POP3_MANAGE_REMOVE)
					{

						Pop3Operation fetchOp =
							new Pop3Operation(Operation.POP3_MANAGE_REMOVE, pop3Op.priority,
							//pop3Op.source,
		//pop3Op.rawString,
		//pop3Op.message,
	pop3Op.uidList, pop3Op.popServer);
						pop3Op.popServer.getOperator().addOp(fetchOp);

						break;
					}

				}
				/******************** SMTP *************************/

			case Operation.SMTP :
				{
					OutboxOperation smtpOp = (OutboxOperation) op;

					if (op.mode == Operation.SMTP_SENDCOMPOSED)
					{
						Channel channel = new Channel(1, 1);

						FolderOperation sendop =
							new FolderOperation(
								Operation.SEND,
								op.priority,
								smtpOp.source.getUids(),
								smtpOp.source,
								channel);
						sendop.destructive = true;

						smtpOp.source.getOperator().addOp(sendop);

						outboxFolder.getOperator().addOp(
							new FolderOperation(Operation.RECEIVE, op.priority, outboxFolder, channel));

						smtpOp.mode = Operation.SMTP_SENDALL;
					}

					outboxFolder.getOperator().addOp(smtpOp);

					break;
				}
				
		/****************** encryption **********************************/
		
			case Operation.ENCRYPTION :
				{

					EncryptionOperation encryptionOp = (EncryptionOperation) op;

					if (op.mode == Operation.DECRYPT)
					{

						Channel channel = new Channel(1, 1);

						EncryptionOperation op1 =
							new EncryptionOperation(
								Operation.DECRYPT,
								op.priority,
								mainInterface.treeViewer.getSelected(),
								encryptionOp.uid);
						encryptionOperator.addOp(op1);

						break;
					}

					if (op.mode == Operation.VERIFY)
					{

						Channel channel = new Channel(1, 1);

						EncryptionOperation op1 =
							new EncryptionOperation(
								Operation.VERIFY,
								op.priority,
								encryptionOp.source,
								encryptionOp.uid);

						encryptionOperator.addOp(op1);

						break;
					}

					break;
				}
				/********************* COMPOSER **************************/
			case Operation.COMPOSER :
				{
					ComposerOperation composerOp = (ComposerOperation) op;
					if (op.mode == Operation.COMPOSER_REPLY)
					{
						Channel channel = new Channel(1, 1);

						
						/*
						Object[] uidList = new Object[1];
						uidList[0] = composerOp.uid;
						*/

						FolderOperation op2 =
							new FolderOperation(
								Operation.MESSAGEBODY,
								composerOp.priority,
								composerOp.uid,
								composerOp.source,
								channel);
						composerOp.source.getOperator().addOp(op2);
						
						ComposerOperation op1 =
							new ComposerOperation(Operation.COMPOSER_REPLY, composerOp.priority, channel);
						composerOperator.addOp(op1);

						break;
					}

					if (op.mode == Operation.COMPOSER_REPLY_ALL)
					{
						Channel channel = new Channel(1, 1);

						ComposerOperation op1 =
							new ComposerOperation(
								Operation.COMPOSER_REPLY_ALL,
								composerOp.priority,
								channel);
						composerOperator.addOp(op1);

						/*
						Object[] uidList = new Object[1];
						uidList[0] = composerOp.uid;
						*/
						
						FolderOperation op2 =
							new FolderOperation(
								Operation.MESSAGEBODY,
								composerOp.priority,
								composerOp.uid,
								composerOp.source,
								channel);
						composerOp.source.getOperator().addOp(op2);

						break;
					}

					if (op.mode == Operation.COMPOSER_REPLY_AS_ATTACHMENT)
					{
						Channel channel = new Channel(1, 1);

						ComposerOperation op1 =
							new ComposerOperation(
								Operation.COMPOSER_REPLY_AS_ATTACHMENT,
								composerOp.priority,
								channel);
						composerOperator.addOp(op1);

						
						Object[] uidList = new Object[1];
						uidList[0] = composerOp.uid;

						FolderOperation op2 =
							new FolderOperation(
								Operation.SEND,
								composerOp.priority,
								uidList,
								composerOp.source,
								channel);
						op2.destructive = false;
						composerOp.source.getOperator().addOp(op2);

						break;
					}

					if (op.mode == Operation.COMPOSER_REPLY_MAILINGLIST)
					{
						Channel channel = new Channel(1, 1);

						ComposerOperation op1 =
							new ComposerOperation(
								Operation.COMPOSER_REPLY_MAILINGLIST,
								composerOp.priority,
								channel);
						composerOperator.addOp(op1);

						/*
						Object[] uidList = new Object[1];
						uidList[0] = composerOp.uid;
						*/

						FolderOperation op2 =
							new FolderOperation(
								Operation.MESSAGEBODY,
								composerOp.priority,
								composerOp.uid,
								composerOp.source,
								channel);
						composerOp.source.getOperator().addOp(op2);
						break;
					}

					if (op.mode == Operation.COMPOSER_FORWARD)
					{
						Channel channel = new Channel(1, 1);

						ComposerOperation op1 =
							new ComposerOperation(Operation.COMPOSER_FORWARD, composerOp.priority, channel);
						composerOperator.addOp(op1);

		
						Object[] uidList = new Object[1];
						uidList[0] = composerOp.uid;

						FolderOperation op2 =
							new FolderOperation(
								Operation.SEND,
								composerOp.priority,
								uidList,
								composerOp.source,
								channel);
						op2.destructive = false;
						composerOp.source.getOperator().addOp(op2);

						break;
					}

					if (op.mode == Operation.COMPOSER_FORWARD_INLINE)
					{
						Channel channel = new Channel(1, 1);

						ComposerOperation op1 =
							new ComposerOperation(
								Operation.COMPOSER_FORWARD_INLINE,
								composerOp.priority,
								channel);
						composerOperator.addOp(op1);

						/*
						Object[] uidList = new Object[1];
						uidList[0] = composerOp.uid;
						*/

						FolderOperation op2 =
							new FolderOperation(
								Operation.MESSAGEBODY,
								composerOp.priority,
								composerOp.uid,
								composerOp.source,
								channel);
						composerOp.source.getOperator().addOp(op2);
						break;
					}

					if (op.mode == Operation.COMPOSER_OPEN)
					{
						Channel channel = new Channel(1, 1);

						ComposerOperation op1 =
							new ComposerOperation(Operation.COMPOSER_OPEN, composerOp.priority, channel);
						composerOperator.addOp(op1);

						Object[] uidList = new Object[1];
						uidList[0] = composerOp.uid;

						FolderOperation op2 =
							new FolderOperation(
								Operation.SEND,
								composerOp.priority,
								uidList,
								composerOp.source,
								channel);
						op2.destructive = false;
						composerOp.source.getOperator().addOp(op2);
						break;
					}
					
					if ( op.mode == Operation.COMPOSER_SAVEDRAFT )
					{
						ComposerOperation op1 =
							new ComposerOperation(Operation.COMPOSER_SAVEDRAFT, composerOp.priority, composerOp.frame);
						composerOperator.addOp(op1);
						
						
						break;
					}
					
					if ( op.mode == Operation.COMPOSER_SAVETEMPLATE )
					{
						ComposerOperation op1 =
							new ComposerOperation(Operation.COMPOSER_SAVETEMPLATE, composerOp.priority, composerOp.frame);
						composerOperator.addOp(op1);
						
						
						break;
					}
					
					if ( op.mode == Operation.COMPOSER_SEND )
					{
						ComposerOperation op1 =
							new ComposerOperation(Operation.COMPOSER_SEND, composerOp.priority, composerOp.frame);
						composerOperator.addOp(op1);
						
						
						break;
					}
					
					if ( op.mode == Operation.COMPOSER_SENDLATER )
					{
						ComposerOperation op1 =
							new ComposerOperation(Operation.COMPOSER_SENDLATER, composerOp.priority, composerOp.frame);
						composerOperator.addOp(op1);
						
						
						break;
					}

					break;
				}

		}

	}

	/******************************* notification ****************************/

	/*********** tree component **************/

	public void fireTreeNodeChanged(TreeNodeEvent e)
	{
		mainInterface.treeViewer.treeNodeChanged(e);
	}

	/************ table component ***********************/

	public void fireFolderChanged(FolderEvent e, Folder folder)
	{
		//mainInterface.headerTableViewer.folderChanged( e );

		if (mainInterface.treeViewer.getSelected() == null)
			return;

		if (mainInterface.treeViewer.getSelected().equals(folder))
		{
			mainInterface.headerTableViewer.folderChanged(e);			
		}

	}

}