// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.folder;

import org.columba.gui.statusbar.*;
import org.columba.main.*;
import java.util.*;

public class Operator extends Thread
{
    private Hashtable lock;
    protected Hashtable workerTable;
    private Vector queue;
    public MainInterface mainInterface;

    protected int maxOut;
    private int actOut;

    private boolean mutex;

    public Operator(MainInterface mainInterface)
    {
	this.mainInterface = mainInterface;

	lock = new Hashtable();
	workerTable = new Hashtable();
	queue = new Vector();
	maxOut = 1; // Default

	actOut = 0;
    }


    private synchronized void getMutex()
    {
        while( mutex ) {
            try {
                wait();
            }
            catch( InterruptedException e )
            {
            }
        }

        mutex = true;
    }

    private synchronized void releaseMutex()
    {
        mutex = false;
        notify();
    }


    public synchronized void addOp( Operation op )
    {
	getMutex();


	int p = queue.size()-1;

	while( p!=-1 ) {
	    if( ((Operation)queue.get(p)).priority < op.priority)
		p--;
	    else break;
	}

	queue.insertElementAt( op, p+1 );

	releaseMutex();
    }

    private synchronized Operation getNextOp()
    {
	getMutex();

        while( (queue.size() == 0)  | (actOut == maxOut) ) {
            try {
		releaseMutex();
                wait();
            }
            catch( InterruptedException e )
            {
            }
	}
	releaseMutex();

	//System.out.println(queue.firstElement());

        return (Operation) queue.remove( 0 );
    }

    public synchronized void operationFinished()
    {
	actOut--;
	notify();
    }

    private void prepareOperation( Operation op )
    {
	actOut++;
    }

    private Worker getWorker( Operation op )
    {
	Worker worker = null;

	//System.out.println( "OPMODE : " + op.mode );

        Class workerClass = (Class) workerTable.get( new Integer(op.mode) );
        if ( workerClass != null )
        {
            //System.out.println("single worker class found");
        }
        else
        {
	    workerClass = (Class) workerTable.get(new Integer(op.mode & 0x0FF00));
            if ( workerClass == null )  System.out.println("workerclass == null");
        }

	try {
	    worker = (Worker) workerClass.newInstance();
	}
	catch( Exception e ) {
            e.printStackTrace();
	}

	return worker;
    }

    public void run() // Scheduler
    {
        Operation op;
        Worker worker;

        while(true) {
            op = getNextOp();
	    prepareOperation( op );
            worker = getWorker( op );
            //System.out.println("operator: "+ op );
            //System.out.println("worker: "+worker );
	    worker.init( this, op );
            worker.register( mainInterface.taskManager );
            worker.start();
	    //System.out.println( "Operation started..." );
        }
    }
}














