// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.folder;

import org.columba.util.*;
import org.columba.gui.statusbar.*;

public abstract class Worker extends SwingWorker
{
    protected Operation op;
    protected Operator root;
    protected Channel channel;
    protected boolean kill;
    public boolean channelCanceled = false;

    public void init( Operator root, Operation op )
    {
	this.op = op;
	this.root = root;
	kill = false;
    }
    
    public abstract Object construct();

    public synchronized void sleep() throws InterruptedException
    {
	wait();
    }

    public synchronized void wakeUp()
    {
	notify();
    }

    public synchronized void kill()
    {
	kill = true;
	notify();
    }

    public void finished()
    {
	root.operationFinished();
    }

	public void setCancel( boolean val ) {
		System.out.println( "Cancel operation" );
		
		if( getCancel() == true ) return;
		
		super.setCancel( val );

		if( val & !channelCanceled ) {
			if( channel != null ) channel.cancel();
			channelCanceled = true;
		}
	}

	/*

	public boolean getCancel(){
		boolean result = super.getCancel();
		
		if( result & !channelCanceled ) {
			if( channel != null ) channel.cancel();
			channelCanceled = true;
		}
		
		return result;
	}

	*/

	protected boolean waitForWrite() {
		if( channel == null ) return false;
		if( getCancel() ) { 
			if( !channelCanceled ) {
				if( channel != null ) channel.cancel();
				channelCanceled = true;
			}
			return false;
		}
		
		while (!channel.canWrite(this)) {
			if( getCancel() ) { 
				if( !channelCanceled ) {
					if( channel != null ) channel.cancel();
					channelCanceled = true;
				}
				return false;
			}
			try {
				sleep();
			} catch (InterruptedException e) {}	
		}
		
		return true;
	}

	protected boolean waitForRead() {
		if( channel == null ) return false;
		if( getCancel() ) { 
			if( !channelCanceled ) {
				if( channel != null ) channel.cancel();
				channelCanceled = true;
			}
			return false;
		}

		while (!channel.canRead(this)) {
			if( getCancel() ) { 
				if( !channelCanceled ) {
					if( channel != null ) channel.cancel();
					channelCanceled = true;
				}
				return false;
			}
			try {
				sleep();
			} catch (InterruptedException e) {}
		}
		
		return true;
	}

    public void register( TaskManager t )
    {
	getMutex();

        this.taskManager = t;

	if ( t==null ) System.out.println("taskmanager==null");

	int priority = op.priority;

	taskManager.register( getThread(), priority );

	releaseMutex();
    }
}

