// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.folder.columba;

import org.columba.message.*;
import org.columba.config.*;
import org.columba.main.*;
import org.columba.gui.tree.util.*;
import org.columba.coder.*;
import org.columba.parser.*;
import org.columba.folder.*;

import java.io.*;
import java.util.Vector;
import java.util.*;
import java.lang.reflect.*;

import org.columba.parser.*;
import org.columba.filter.*;
import org.columba.util.*;

public class ColumbaFolder extends LocalFolder
{

	private File messageFile;
	// name of message file, only number of message needs to be appended

	protected MainInterface mainInterface;

	private FileWriter fileWriter;
	private FileReader fileReader;

	protected Message aktMessage;
	protected int aktMessageIndex;
	protected Integer aktMessageUid;

	private Vector filterList;

	protected boolean alreadyLoaded;

	protected boolean headercacheMutex=false;

	public ColumbaFolder(
		AdapterNode node,
		FolderItem item,
		MainInterface mainInterface)
	{
		super(node, item, mainInterface);

		this.mainInterface = mainInterface;

		// this file caches all parsed header info
		headerFile = new File(directoryFile, "header");

		// this file contains the complete message,
		// the message UID is going to be appended for identification
		messageFile = new File(directoryFile, "message");

		alreadyLoaded = false;

	}

	public ColumbaFolder(String name, MainInterface mainInterface)
	{
		super(name, mainInterface);

		// this file caches all parsed header info
		headerFile = new File(directoryFile, "header");

		// this file contains the complete message,
		// the message UID is going to be appended for identification
		messageFile = new File(directoryFile, "message");

		alreadyLoaded = false;
	}

	private void loadHeaderList(SwingWorker worker)
	{
		if (worker != null)
			worker.setText("Loading Headerlist ...");

		if (worker != null)
			worker.startTimer();

		if (headerFile.exists())
		{
			try
			{
				load(worker);
			}
			catch (Exception ex)
			{
				System.out.println("exception: " + ex.getMessage());
				ex.printStackTrace();

				System.out.println("recreating index file");
				recreateIndex();
			}
		}
		else
		{
			recreateIndex();

		}

		if (worker != null)
			worker.stopTimer();

		alreadyLoaded = true;
	}

	public HeaderList getHeaderList(SwingWorker worker) throws Exception
	{
		HeaderList list = new HeaderList();

		getHeadercacheMutex();
		
		if (alreadyLoaded == false)
		{
			loadHeaderList(worker);
		}

		getMutex();
		Enumeration keys = uidTable.keys();
		releaseMutex();

		while (keys.hasMoreElements())
		{
			Object uid = keys.nextElement();
			Message message = uidGet(uid);

			list.add(message.getHeader(), message.getUID());
		}

		releaseHeadercacheMutex();
		
		return list;
	}

	public Object[] getUids()
	{
		
		getHeadercacheMutex();
		
		if (!alreadyLoaded)
		{
			loadHeaderList(null);
		}
		
		releaseHeadercacheMutex();
		

		return super.getUids();
	}

	protected String loadMessage(Integer uid) throws Exception
	{
		StringBuffer strbuf;
		File file = uidgetMessageFile(uid);

		try
		{
			BufferedReader in = new BufferedReader(new FileReader(file));
			String str;
			strbuf = new StringBuffer();

			while ((str = in.readLine()) != null)
			{
				strbuf.append(str + "\n");
			}

			in.close();
		}
		catch (IOException ex)
		{
			throw new MessageFolderInconsistentException(ex.getMessage());
		}

		return strbuf.toString();
	}

	protected String loadMessage(int i) throws Exception
	{
		StringBuffer strbuf = new StringBuffer();
		String filename = new String(messageFile.toString() + i);
		File file = new File(filename);

		try
		{
			BufferedReader in = new BufferedReader(new FileReader(file));
			String str;

			while ((str = in.readLine()) != null)
			{
				strbuf.append(str + "\n");
			}

			in.close();
		}
		catch (IOException ex)
		{
			System.out.println(filename + " : " + ex);
		}

		return strbuf.toString();
	}

	/*
	    protected File getMessageFile(int index)
	    {
		File file;
	        Integer uid = (Integer) getUid( index );
	
	        int fileNumber = uid.intValue();
	
	
		file = new File( messageFile.toString()+fileNumber );
		return file;
	    }
	*/

	protected File uidgetMessageFile(Integer uid)
	{
		File file;

		file = new File(messageFile.toString() + uid.toString());
		return file;
	}

	public Message uidGetParsedMessage(Object uid) throws Exception
	{
		//System.out.println("--------------> columbafolder->uidGet");

		Integer uidInt = (Integer) uid;

		if ((uidInt != aktMessageUid))
		{
			uidsetAktMessage(uidInt);
		}

		System.out.println("---------> uidgetparsedmessage:"+aktMessage);
		
		return aktMessage;
	}

	/*
	private void setAktMessage( int index ) throws Exception
	{
	    System.out.println("----------------> setAktMessage");
	
	    Integer uid = (Integer) getUid( index );
	
	    String message = loadMessage( uid );
	
	    Folder parentFolder;
	    int parentIndex;
	
	    if( message == null ) {
	        aktMessageIndex = -1;
	        return;
	    }
	    aktMessage = new Rfc822Parser().parse( message, false, null, MimeTypeParser.SHOW_MESSAGE );
	
	    Message tableMessage = uidGet( uid );
	
	    tableMessage.setMimePartCollection( aktMessage.getMimePartCollection());
	
	    tableMessage.setHeader( aktMessage.getHeader() );
	
	    aktMessageIndex = index;
	    aktMessageUid = uid;
	
	
	}
	*/

	protected Message uidsetAktMessage(Integer uid) throws Exception
	{
	

		String message = loadMessage(uid);

		

		Folder parentFolder;
		int parentIndex;

		if (message == null)
		{
			aktMessageIndex = -1;

			return null;
		}

		if (aktMessage != null)
		{
			/* Release Memory */
			aktMessage.freeMemory();
		}

		

		aktMessage = uidGet(uid);

		System.out.println("---------> uidsetaktmessage:"+aktMessage);
		
		Message parsedMessage =
			new Rfc822Parser().parse(
				message,
				true,
				aktMessage.getHeader(),
				MimeTypeParser.SHOW_MESSAGE);

		aktMessage.setRawString(message);
		aktMessage.setMimePartCollection(parsedMessage.getMimePartCollection());
		

		aktMessage.setHeader(parsedMessage.getHeader());

		aktMessageUid = uid;
	
		System.out.println("---------> uidsetaktmessage2:"+aktMessage);
		
		return aktMessage;
	}

	/*
	public int getMimePartCount( int index ) throws Exception
	{
	
	    Integer uid = (Integer) getUid( index );
	
	    if( (uid != aktMessageUid) | (aktMessage==null) )
	    {
	        setAktMessage( index );
	    }
	
	    MimePartCollection mimeparts = aktMessage.getMimePartCollection();
	
	    int count = mimeparts.count();
	
	    if( count == 1 ) {
	        MimeHeader mimeheader = mimeparts.get( 0 ).getHeader();
	        if( mimeheader.getContentType().equals("text") &
	            mimeheader.getContentSubtype().equals("plain") ) {
	            count--;
	        }
	    }
	
	    return count;
	}
	*/

	public void uidSetMimePartCollection(Object uid, MimePartCollection mc)
		throws Exception
	{
		if (aktMessage != null)
		{
			aktMessage.setMimePartCollection(mc);
		}
		else
		{
			super.uidSetMimePartCollection(uid, mc);
		}
	}

	public MimePartCollection uidgetMimePartCollection(Object uid) throws Exception
	{
		Integer uidInt = (Integer) uid;

		if ((uidInt != aktMessageUid) || (aktMessage == null))
		{
			uidsetAktMessage(uidInt);
		}

		return aktMessage.getMimePartCollection();
	}

	public int uidgetMimePartCount(Object uid) throws Exception
	{
		Integer uidInt = (Integer) uid;

		if ((uidInt != aktMessageUid) | (aktMessage == null))
		{
			uidsetAktMessage(uidInt);
		}

		MimePartCollection mimeparts = aktMessage.getMimePartCollection();

		int count = mimeparts.count();

		if (count == 1)
		{
			MimeHeader mimeheader = mimeparts.get(0).getHeader();
			if (mimeheader.getContentType().equals("text")
				& mimeheader.getContentSubtype().equals("plain"))
			{
				count--;
			}
		}

		return count;
	}

	/*
	public MimePart getMimePart( int index, int number ) throws Exception
	{
	
	    Integer uid = (Integer) getUid( index );
	
	    if( (uid != aktMessageUid) | (aktMessage==null) )
	    {
	        setAktMessage( index );
	    }
	
	    return aktMessage.getMimePartCollection().get( number );
	}
		*/

	public MimePart uidgetMimePart(Object uid, int number, SwingWorker worker)
		throws Exception
	{
		Integer uidInt = (Integer) uid;

		if ((uidInt != aktMessageUid) || (aktMessage == null))
		{
			uidsetAktMessage(uidInt);
		}

		return aktMessage.getMimePartCollection().get(number);
	}

	/*
	public String getBodyText( int index, SwingWorker worker ) throws Exception
	{
	    MimePartCollection aktCollection;
	    MimePart aktPart;
	
	    Integer uid = (Integer) getUid( index );
	
	    if( (uid != aktMessageUid) | (aktMessage==null) ) {
	
	        setAktMessage( index );
	    }
	
	    aktCollection = aktMessage.getMimePartCollection();
	
	    for(int i=0; i < aktCollection.count(); i++) {
	        aktPart = aktCollection.get(i);
	        if( aktPart.getHeader().contentType.equals("text") ) {
	            Decoder decoder = CoderRouter.getDecoder(
	                aktPart.getHeader().contentTransferEncoding);
	
	            StringWriter decodedBody = new StringWriter();
	
	            decoder.setupDecoder( new StringReader( aktPart.getBody() ), decodedBody );
	            decoder.run();
	
	            return decodedBody.toString();
	        }
	    }
	
	return new String();
	}
	*/

	public String uidgetBodyText(Object uid, SwingWorker worker) throws Exception
	{
		Integer uidInt = (Integer) uid;

		MimePartCollection aktCollection;
		MimePart aktPart;

		if ((uidInt != aktMessageUid) || (aktMessage == null))
		{

			uidsetAktMessage(uidInt);
		}

		aktCollection = aktMessage.getMimePartCollection();

		for (int i = 0; i < aktCollection.count(); i++)
		{
			aktPart = aktCollection.get(i);
			if (aktPart.getHeader().contentType.equals("text"))
			{
				Decoder decoder =
					CoderRouter.getDecoder(aktPart.getHeader().contentTransferEncoding);

				StringWriter decodedBody = new StringWriter();

				decoder.setupDecoder(new StringReader(aktPart.getBody()), decodedBody);
				decoder.run();

				return decodedBody.toString();
			}
		}

		return new String();
	}

	public String uidToString(Object uid, SwingWorker worker) throws Exception
	{
		Integer uidInt = (Integer) uid;

		return loadMessage(uidInt);
	}

	/*
	public String toString( int index, SwingWorker worker )
	{
	    return toString( index );
	}
	*/

	/*
	public String toString( int index )
	{
	
	    StringBuffer strbuf = new StringBuffer();
	String str = new String();
	int line=0;
	
	        //System.out.println("trying to show message number: "+index);
	
	    System.out.println("columba->toString()" );
	try
	    {
	        File file = getMessageFile( index );
	        BufferedReader in = new BufferedReader( new FileReader( file ) );
	
	        while ((str = in.readLine()) != null)
	        {
	            strbuf.append( str+"\n" );
	        }
	
	        in.close();
	
	
	
	    }
	catch (IOException e)
	    {
	        System.out.println("exception: "+e.getMessage() );
	
	    }
	
	return strbuf.toString();
	}
	*/

	/*
	protected Object calculateNewUid()
	{
	  int max = -1;
	  Integer actUid;
	
	  for( int i=0; i<count(); i++)
	  {
	      actUid = (Integer) getUid( i );
	
	      if( actUid.intValue() > max ) {
	          max = actUid.intValue();
	      }
	  }
	
	  return new Integer( max+1 );
	}
	
	*/

	public Object add(Message message, String str) throws Exception
	{
		/*
		getHeadercacheMutex();
		
		if (!alreadyLoaded)
		{
			getHeaderList(null);
		}
		
		releaseHeadercacheMutex();
		*/
		
		Object uid = super.add(message, str, calculateNewUid());

		int length = 0;

		File file = new File(messageFile.getPath() + message.getUID().toString());

		saveStringInFile(file, str);

		message.freeMemory();

		return uid;
	}

	/*************************** Worker Methods ***************************/

	public Message workerRemove(Object uid) throws Exception
	{
		Integer uidInt = (Integer) uid;

		/*
		    String rawString = loadMessage( uidInt );
		
		    Message message = super.uidRemove( uid );
		
		    message.setRawString( rawString  );
		*/

		File file = new File(messageFile.toString() + uidInt);
		file.delete();

		return super.workerRemove(uid);

	}

	// we need these two methods here to be sure
	// the header list cache is already loaded
	public Object workerAdd(Message message) throws Exception
	{
		getHeadercacheMutex();
		
		if (alreadyLoaded == false)
		{
			if (headerFile.exists())
			{
				try
				{
					load(null);
				}
				catch (Exception ex)
				{
					System.out.println("exception: " + ex.getMessage());
					ex.printStackTrace();

					System.out.println("recreating index file");
					recreateIndex();

				}

				//fireTreeNodeUpdate();
				alreadyLoaded = true;
			}
		}
		
		releaseHeadercacheMutex();		
	
		//System.out.println("---------->message source: "+ message.getRawString() );
		Object uid = super.workerAdd(message);

		return uid;

	}

	public Object workerAdd(Message message, String rawString) throws Exception
	{
		
		getHeadercacheMutex();
		
		if (alreadyLoaded == false)
		{
			if (headerFile.exists())
			{
				try
				{
					load(null);
				}
				catch (Exception ex)
				{
					System.out.println("exception: " + ex.getMessage());
					ex.printStackTrace();

					System.out.println("recreating index file");
					recreateIndex();

				}

				//fireTreeNodeUpdate();
				alreadyLoaded = true;
			}
		}
		
		releaseHeadercacheMutex();		

		message.setRawString(rawString);

		Object uid = super.workerAdd(message);

		return uid;
	}

	/********************************************************************/

	protected Message uidRemove(Object uid) throws Exception
	{
		
		
		if (!alreadyLoaded)
		{
			getHeaderList(null);
		}
		
		

		Integer uidInt = (Integer) uid;

		String rawString = loadMessage(uidInt);


		Message removed = super.uidRemove(uid);

		removed.setRawString(rawString);

		File file = new File(messageFile.toString() + uidInt);
		file.delete();

		return removed;
	}

	protected int getMessageFileCount()
	{
		File[] list = directoryFile.listFiles();
		int count = 0;

		System.out.println(list.length);

		for (int i = 0; i < Array.getLength(list); i++)
		{
			File file = list[i];
			String name = file.getName();
			int index = name.indexOf("message");

			if (index != -1)
			{
				count++;
			}
			else
			{
				System.out.println(name);
			}
		}

		return count;
	}

	public void recreateIndex()
	{
		removeAll();

		//System.out.println("directory: "+ directoryFile.toString() );

		// first, create vector v for all message files
		File[] list = directoryFile.listFiles();
		Vector v = new Vector();

		for (int i = 0; i < Array.getLength(list); i++)
		{
			File file = list[i];
			File renamedFile;
			String name = file.getName();
			int index = name.indexOf("message");

			if (index != -1)
			{
				// message file found
				String number = name.substring(index + 7, name.length());
				//                Integer numberString = new Integer( number );
				//System.out.println("number: "+ number );

				if ((file.exists()) && (file.length() > 0))
				{
					renamedFile = new File(file.getParentFile(), file.getName() + '~');
					file.renameTo(renamedFile);
					v.add(renamedFile);
				}

				//System.out.println("v index: "+ v.indexOf( file ) );
			}

			/*
			else
			{
			    // header file found
			    headerFile.delete();
			}
			*/

		}

		// rename a message files
		// the message file index should be the same like the vector index
		/*
		    for ( int i=0; i<v.size(); i++ )
		    {
		
		    try {
			int n = Integer.parseInt( file.getName() );
		    }
		    catch( NumberFormatException e )
			{
			    e.printStackTrace();
			}
		
		    // rename message file
		}
		*/

		// parse all message files to recreate the header cache
		Rfc822Parser parser = new Rfc822Parser();
		ColumbaHeader header;

		for (int i = 0; i < v.size(); i++)
		{
			File file = (File) v.get(i);
			file.renameTo(new File(file.getParentFile(), "message" + i));

			try
			{
				String source = loadMessage(i);

				header = parser.parseHeader(source);

				Message m = new Message(header);
				ColumbaHeader h = m.getHeader();

				h.set("columba.flags.recent", new Boolean(false));

				Integer sizeInt = new Integer(source.length());
				int size = Math.round(sizeInt.intValue() / 1024);
				h.set("columba.size", new Integer(size));

				//m.setHost( item.getHost() );
				h.set("columba.host", new String(""));

				Date date = DateParser.parseString((String) h.get("date"));
				//System.out.println("date1: "+ h.get("date") );
				//m.setDate( date );
				h.set("columba.date", date);
				Date date2 = (Date) h.get("columba.date");

				//System.out.println("date2: "+ h.get("columba.date") );

				String shortFrom = (String) header.get("From");
				if (shortFrom != null)
				{
					if (shortFrom.indexOf("<") != -1)
					{
						shortFrom = shortFrom.substring(0, shortFrom.indexOf("<"));
						if (shortFrom.length() > 0)
						{
							if (shortFrom.startsWith("\""))
								shortFrom = shortFrom.substring(1, shortFrom.length() - 1);
							if (shortFrom.endsWith("\""))
								shortFrom = shortFrom.substring(0, shortFrom.length() - 1);
						}

					}
					else
						shortFrom = shortFrom;

					//m.setShortFrom( shortFrom );
					h.set("columba.from", shortFrom);
				}
				else
				{
					//m.setShortFrom("");
					h.set("columba.from", new String(""));
				}

				String priority = (String) header.get("X-Priority");
				if (priority != null)
				{
					//m.setPriority( prio );
					h.set("columba.priority", new Integer(priority));
				}
				else
				{
					//m.setPriority( 3 );
					h.set("columba.priority", new Integer(3));
				}

				String attachment = (String) header.get("Content-Type");
				if (attachment != null)
				{
					attachment = attachment.toLowerCase();

					if (attachment.indexOf("multipart") != -1)
					{
						//m.setAttachment(true);
						h.set("columba.attachment", new Boolean(true));
					}
					else
					{
						h.set("columba.attachment", new Boolean(false));
						//m.setAttachment(false);
					}
				}
				else
				{
					h.set("columba.attachment", new Boolean(false));
					//m.setAttachment(false);
				}

				super.add(m, source, new Integer(i));

				nextUid = i + 1;

			}
			catch (Exception ex)
			{
				System.out.println("recreateIndex exception: " + ex.getMessage());
				ex.printStackTrace();
			}

		}

		//fireFolderUpdate();
	}

	protected void cleanUpIndex() throws Exception
	{
		Message message;
		Object[] uids = getUids();
		File src;
		File dest;

		// do we really have to sort ?
		//  nextUid is increased every time
		//  we only get holes
		Arrays.sort(uids);

		for (int i = 0; i < count(); i++)
		{
			Integer uidInt = (Integer) uids[i];

			if (uidInt.intValue() != i)
			{
				System.out.println("trying to rename" + i);

				message = uidGet(uids[i]);
				src = new File(messageFile.toString() + uidInt);

				dest = new File(messageFile.toString() + i);

				src.renameTo(dest);

				message.setUID(new Integer(i));
			}
		}

	}

	// saves Message-Headers in separat File
	public void save() throws Exception
	{
		if (alreadyLoaded == false)
			return;

		// this has to called only if the uid becomes higher than Integer allows
		//cleanUpIndex();

		//System.out.println("saving headerfile: "+ headerFile.toString() );
		HeaderTableItem v =
			mainInterface.config.getMainFrameOptionsConfig().getHeaderTableItem();

		FileOutputStream istream = new FileOutputStream(headerFile.getPath());
		ObjectOutputStream p = new ObjectOutputStream(istream);

		//int count = getMessageFileCount();
		int count = count();

		p.writeInt(count);

		ColumbaHeader h;
		Message message;
		System.out.println("save count:" + count);
		for (int i = 0; i < count; i++)
		{
			p.writeInt(i + 1);

			message = get(i);
			if (message == null)
			{
				System.out.println("there appears to be a messag file,");
				System.out.println("but associated message index in folder");
				throw new Exception("columbafolder->save() == message is null!");
			}

			h = get(i).getHeader();

			p.writeObject(get(i).getUID());

			//p.writeObject( get(i).getHeader().get("Status") );
			//p.writeObject( get(i).getFlags() );

			Flags flags = get(i).getFlags();

			p.writeBoolean(flags.getSeen());
			p.writeBoolean(flags.getAnswered());
			p.writeBoolean(flags.getFlagged());
			p.writeBoolean(flags.getDeleted());
			p.writeBoolean(flags.getDraft());
			p.writeBoolean(flags.getRecent());

			p.writeObject(h.get("columba.date"));

			p.writeObject(h.get("columba.size"));

			p.writeObject(h.get("columba.from"));

			p.writeObject(h.get("columba.attachment"));

			p.writeObject(h.get("columba.priority"));

			p.writeObject(h.get("columba.host"));

			/*
			p.writeObject( get(i).getDate() );
			
			p.writeInt( get(i).getSize() );
			
			p.writeObject( get(i).getShortFrom() );
			
			p.writeObject( new Boolean( get(i).getAttachment() ) );
			
			p.writeInt( get(i).getPriority() );
			
			
			p.writeObject( get(i).getHost() );
			*/

			String column;
			HeaderItem item;
			Object o;
			for (int j = 0; j < v.count(); j++)
			{
				column = (String) v.getName(j);
				o = get(i).getHeader().get(column);
				p.writeObject(o);
			}
		}

		//p.flush();
		p.close();

	}

	// load Message-Headers
	public void load(SwingWorker worker) throws Exception
	{
		//System.out.println("loading headerfile: "+ headerFile.toString() );

		FileInputStream istream = new FileInputStream(headerFile.getPath());
		ObjectInputStream p = new ObjectInputStream(istream);

		int capacity = p.readInt();

		System.out.println("file capacity:" + capacity);

		//int capacity = getMessageFileCount();

		if (capacity != getMessageFileCount())
		{
			// messagebox headercache-file is corrupted
			System.out.println("Messagebox headercache-file is corrupted!");
			recreateIndex();
			return;
		}

		System.out.println("worker: " + worker);

		Integer uid;

		//System.out.println("Number of Messages : " + capacity);

		if (worker != null)
			worker.setProgressBarMaximum(capacity);

		for (int i = 1; i <= capacity; i++)
		{
			if (worker != null)
				worker.setProgressBarValue(i);

			Message message = new Message();
			ColumbaHeader h = message.getHeader();

			// read current number of message
			p.readInt();

			uid = (Integer) p.readObject();
			message.setUID(uid);

			//Integer status = (Integer) p.readObject();
			//message.getHeader().set( "Status", status );
			//message.setFlags( (Flags) p.readObject() );
			Flags flags = message.getFlags();

			flags.setSeen(p.readBoolean());
			flags.setAnswered(p.readBoolean());
			flags.setFlagged(p.readBoolean());
			flags.setDeleted(p.readBoolean());
			flags.setDraft(p.readBoolean());
			flags.setRecent(p.readBoolean());
			//message.setFlags( flags );

			Date date = (Date) p.readObject();
			h.set("columba.date", date);

			h.set("columba.size", p.readObject());

			String from = (String) p.readObject();
			h.set("columba.from", from);

			Boolean b = (Boolean) p.readObject();
			h.set("columba.attachment", b);

			//int priority = p.readInt();
			h.set("columba.priority", (Integer) p.readObject());

			String host = (String) p.readObject();
			h.set("columba.host", host);

			/*
			Date date = (Date) p.readObject();
			message.setDate( date );
			
			int size = p.readInt();
			message.setSize( size );
			
			message.setShortFrom( (String) p.readObject() );
			
			Boolean b = (Boolean) p.readObject();
			message.setAttachment( b.booleanValue() );
			
			int priority = p.readInt();
			message.setPriority( priority );
			
			String host = (String) p.readObject();
			message.setHost( host );
			*/

			HeaderTableItem v =
				mainInterface.config.getMainFrameOptionsConfig().getHeaderTableItem();
			String column;
			Object o;
			for (int j = 0; j < v.count(); j++)
			{
				column = (String) v.getName(j);
				o = p.readObject();

				if (o == null)
				{
				}
				else if (o instanceof String)
				{
					String value = (String) o;
					h.set(column, value);
				}
				else if (o instanceof Integer)
				{
					Integer value = (Integer) o;
					h.set(column, value);
				}
				else if (o instanceof Boolean)
				{
					Boolean value = (Boolean) o;
					h.set(column, value);
				}
				else if (o instanceof Date)
				{
					Date value = (Date) o;
					h.set(column, value);
				}

			}

			super.add(message, null, uid);

			nextUid = uid.intValue() + 1;

			/*
			        if( nextUid <= uid.intValue() ) {
			            nextUid = uid.intValue()+1;
			        }
			*/

		}

		// close stream
		p.close();

	}

	/*
	public void parseHeaders( Vector v ) throws Exception
	{
	    //System.out.println("---------------------> parsing headers");
	
	    try
	    {
	    for ( int i=0; i<v.size(); i++ )
	    {
	        Message message = (Message) v.get(i);
	        ColumbaHeader h = message.getHeader();
	
	        if  ( h.get("date") != null )
	        {
	            //System.out.println("---------------------> parsing date");
	            //System.out.println("date: " +h.get("date") );
	            Date date = DateParser.parseString( (String) h.get( "date" ) );
	            h.set("columba.date", date);
	            //System.out.println("date: " +h.get("columba.date") );
	        }
	
	        if ( h.get("columba.priority") != null )
	        {
	            String str = (String) h.get("columba.priority");
	            h.set("columba.priority", new Integer(str) );
	        }
	
	        if ( h.get("columba.size") != null )
	        {
	            String str = (String) h.get("columba.size");
	            h.set("columba.size", new Integer(str) );
	        }
	
	        if ( h.get("columba.attachment") != null )
	        {
	            String str = (String) h.get("columba.attachment");
	            h.set("columba.attachment", new Boolean(str) );
	        }
	
	        if ( h.get("columba.fetchstate") != null )
	        {
	            String str = (String) h.get("columba.fetchstate");
	            h.set("columba.fetchstate", new Boolean(str) );
	        }
	
	        if ( h.get("columba.flags.seen") != null )
	        {
	            String str = (String) h.get("columba.flags.seen");
	            h.set("columba.flags.seen", new Boolean( str ) );
	        }
	
	        if ( h.get("columba.flags.recent") != null )
	        {
	            String str = (String) h.get("columba.flags.recent");
	            h.set("columba.flags.recent", new Boolean( str ) );
	        }
	
	        if ( h.get("columba.flags.flagged") != null )
	        {
	            String str = (String) h.get("columba.flags.flagged");
	            h.set("columba.flags.flagged", new Boolean( str ) );
	        }
	
	        if ( h.get("columba.flags.expunged") != null )
	        {
	            String str = (String) h.get("columba.flags.expunged");
	            h.set("columba.flags.expunged", new Boolean( str ) );
	        }
	
	        if ( h.get("columba.flags.draft") != null )
	        {
	            String str = (String) h.get("columba.flags.draft");
	            h.set("columba.flags.draft", new Boolean( str ) );
	        }
	
	        if ( h.get("columba.flags.answered") != null )
	        {
	            String str = (String) h.get("columba.flags.answered");
	            h.set("columba.flags.answered", new Boolean( str ) );
	        }
	
	    }
	    }
	    catch ( Exception ex )
	    {
	        throw new Exception("parseheader: "+ ex.getMessage() );
	    }
	
	}
	*/

	/*
	public void loadxml() throws Exception
	{
	     System.out.println("load file: "+headerFile.getPath() );
	
	     try
	     {
	
	         HeaderCacheReader reader = new HeaderCacheReader();
	
	         Vector v =reader.open( headerFile.getPath() );
	
	         reader.close();
	
	         parseHeaders( v );
	
	         //System.out.println("-------------------->ready");
	
	         for ( int i=0; i<v.size(); i++ )
	         {
	             Message m = (Message) v.get(i);
	             ColumbaHeader header = m.getHeader();
	             Integer uid = new Integer( (String) header.get("uid") );
	             super.add( m, null, uid );
	         }
	
	
	
	
	     }
	     catch ( Exception ex )
	     {
	         System.out.println( ex.getMessage() );
	         ex.printStackTrace();
	     }
	
	
	}
	*/

	/*
	public void savexml() throws Exception
	{
	
	     if ( alreadyLoaded == false ) return;
	
	     cleanUpIndex();
	
	     System.out.println("save file: "+headerFile.getPath() );
	
	     try
	     {
	
	         HeaderCacheWriter writer = new HeaderCacheWriter();
	
	         writer.open( headerFile.getPath() );
	
	         writer.startDocument();
	
	         for ( int i=0; i<count(); i++ )
	         {
	             Message message = get(i);
	             ColumbaHeader header = message.getHeader();
	             Hashtable hash = header.getHashtable();
	
	             writer.writeHash( hash );
	         }
	
	         writer.endDocument();
	
	         writer.close();
	
	     }
	     catch ( IOException ex )
	     {
	         System.out.println( ex.getMessage() );
	     }
	}
	*/

	/*********************** filter support ***************************/

	public boolean exists(Object uid) throws Exception
	{
		Message m = super.uidGet(uid);
		if (m == null)
			return false;
		else
			return true;
	}

	public Vector processHeaderCriteria(
		Object[] uids,
		String headerField,
		String pattern,
		String condition)
		throws Exception
	{
		Vector v = new Vector();

		for (int i = 0; i < uids.length; i++)
		{
			if (uids[i] == null)
				continue;
			if (exists(uids[i]) == false)
				continue;

			Message message = (Message) uidGet(uids[i]);
			if (message == null)
				continue;

			ColumbaHeader header = message.getHeader();

			String headerItem = (String) header.get(headerField);

			if (headerItem == null)
			{
				// try to parse the message

				String rawString = null;
				try
				{
					rawString = loadMessage((Integer) uids[i]);
				}
				catch (Exception ex)
				{
					System.out.println("exception: " + ex.getMessage());
					ex.printStackTrace();
				}

				if (rawString != null)
				{
					message =
						(Message) new Rfc822Parser().parse(
							rawString,
							true,
							header,
							MimeTypeParser.PARSE_MESSAGE);
					header = message.getHeader();
					headerItem = (String) header.get(headerField);
				}

				if (headerItem == null)
					continue;
			}

			headerItem = headerItem.toLowerCase();

			if (condition.equalsIgnoreCase("contains"))
			{
				if (headerItem.indexOf(pattern.toLowerCase()) != -1)
					v.add(uids[i]);
			}
			else if (condition.equalsIgnoreCase("contains not"))
			{
				if (headerItem.indexOf(pattern.toLowerCase()) == -1)
					v.add(uids[i]);
			}
			else if (condition.equalsIgnoreCase("is"))
			{
				if (headerItem.equalsIgnoreCase(pattern))
					v.add(uids[i]);
			}
			else if (condition.equalsIgnoreCase("is not"))
			{
				if (!headerItem.equalsIgnoreCase(pattern))
					v.add(uids[i]);
			}
			else if (condition.equalsIgnoreCase("begins with"))
			{
				if (headerItem.startsWith(pattern.toLowerCase()))
					v.add(uids[i]);
			}
			else if (condition.equalsIgnoreCase("ends with"))
			{
				if (headerItem.endsWith(pattern.toLowerCase()))
					v.add(uids[i]);
			}

		}

		return v;
	}

	public Operator getOperator()
	{
		return operator_;
	}

	public void add(Folder child)
	{
		super.add(child);
		child.setOperator(operator_);
	}

	protected synchronized void getHeadercacheMutex()
	{
		while (headercacheMutex)
		{
			try
			{
				wait();
			}
			catch (InterruptedException e)
			{
			}
		}

		headercacheMutex = true;
	}

	protected synchronized void releaseHeadercacheMutex()
	{
		headercacheMutex = false;
		notify();
	}

}