// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.folder.columba;

import org.columba.gui.statusbar.*;

import org.columba.message.*;

import org.columba.folder.*;
import org.columba.config.*;
import org.columba.gui.message.*;
import org.columba.gui.attachment.*;
import org.columba.gui.header.*;
import org.columba.gui.util.*;

import java.util.*;
import java.lang.reflect.*;

public class ColumbaWorkerFolder extends Worker {
	Message message;
	FolderOperation folderOp;
	ColumbaOperator columbaRoot;

	public Object construct() {

		if (op.type == Operation.FOLDER) {
			folderOp = (FolderOperation) op;
			columbaRoot = (ColumbaOperator) root;
			channel = folderOp.channel;
		}

		//System.out.println( "FolderOp : "+op.mode );

		switch (op.mode) {
			case (Operation.FOLDER_APPLYFILTER) :
				{

					try {
						folderOp.source.applyFilter(folderOp.uidList);
					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						setCancel(true);

						e.printStackTrace();
					}

					break;
				}
			case (Operation.FOLDER_ADD) :
				{
					

					if( waitForWrite() ) folderOp.channel.write("FOLDER_ADD");

					break;
				}
			case (Operation.FOLDER_REMOVE) :
				{
					try {
						folderOp.source.removeFolder();
					} catch (Exception ex) {
						ex.printStackTrace();
						
						setCancel( true );
					}

					if( waitForWrite() ) folderOp.channel.write("FOLDER_REMOVE");

					break;
				}
			case (Operation.FOLDER_RENAME) :
				{
					String name = folderOp.name;

					try {
						folderOp.source.renameFolder(name);
					} catch (Exception ex) {
						ex.printStackTrace();
						
						setCancel( true );
					}

					if( waitForWrite() ) folderOp.channel.write(name);

					break;
				}
			case (Operation.MARK_READ) :
				{
					/*
					FolderEvent event = new FolderEvent( folderOp.source, folderOp.uidList, FolderEvent.MARK, Operation.MARK_READ  );
					columbaRoot.mainInterface.crossbar.fireFolderChanged( event, folderOp.source );
					*/

					int opLength = Array.getLength(folderOp.uidList);
					Folder folder = folderOp.source;
					//ImapFolder folder = (ColFolder) folderOp.source;
					try {
						for (int i = 0; i < opLength; i++) {
							folder.workerMark(folderOp.uidList[i], FolderOp.READ);

						}

						TreeNodeEvent updateEvent2 =
							new TreeNodeEvent(folderOp.source, TreeNodeEvent.UPDATE);
						columbaRoot.mainInterface.crossbar.fireTreeNodeChanged(updateEvent2);

						//folderOp.source.renameFolder( name );
					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						setCancel(true);
						e.printStackTrace();
					}

					if( waitForWrite() ) folderOp.channel.write(folderOp.uidList);

					break;
				}
			case (Operation.MARK_FLAGGED) :
				{
					int opLength = Array.getLength(folderOp.uidList);
					ColumbaFolder folder = (ColumbaFolder) folderOp.source;
					//ImapFolder folder = (ImapFolder) folderOp.source;
					try {
						for (int i = 0; i < opLength; i++) {
							folder.workerMark(folderOp.uidList[i], FolderOp.FLAGGED);
							//FolderEvent event = new FolderEvent( this, folderOp.uidList[i], FolderEvent.MARK, Operation.MARK_FLAGGED  );
							//columbaRoot.mainInterface.crossbar.fireFolderChanged( event, folderOp.source );
						}
						//folderOp.source.renameFolder( name );
					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						setCancel(true);
						e.printStackTrace();
					}
					
					if( waitForWrite() ) folderOp.channel.write(folderOp.uidList);

					break;
				}
			case (Operation.MARK_EXPUNGED) :
				{
					int opLength = Array.getLength(folderOp.uidList);
					Folder folder = folderOp.source;
					//ImapFolder folder = (ImapFolder) folderOp.source;
					try {
						for (int i = 0; i < opLength; i++) {
							folder.workerMark(folderOp.uidList[i], FolderOp.EXPUNGED);
							//FolderEvent event = new FolderEvent( this, folderOp.uidList[i], FolderEvent.MARK, Operation.MARK_EXPUNGED  );
							//columbaRoot.mainInterface.crossbar.fireFolderChanged( event, folderOp.source );
						}
						//folderOp.source.renameFolder( name );
					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						setCancel(true);
						e.printStackTrace();
					}

					if( waitForWrite() ) folderOp.channel.write(folderOp.uidList);

					break;
				}
			case (Operation.MARK_ANSWERED) :
				{
					int opLength = Array.getLength(folderOp.uidList);
					Folder folder = folderOp.source;
					//ImapFolder folder = (ImapFolder) folderOp.source;
					try {
						for (int i = 0; i < opLength; i++) {
							folder.workerMark(folderOp.uidList[i], FolderOp.ANSWERED);
							//FolderEvent event = new FolderEvent( this, folderOp.uidList[i], FolderEvent.MARK, Operation.MARK_ANSWERED  );
							//columbaRoot.mainInterface.crossbar.fireFolderChanged( event, folderOp.source );
						}
						//folderOp.source.renameFolder( name );
					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						setCancel(true);
						e.printStackTrace();
					}

					if( waitForWrite() ) folderOp.channel.write(folderOp.uidList);

					break;
				}

		}

		unregister();

		return null;
	}

}