// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.folder.columba;

import org.columba.gui.statusbar.*;

import org.columba.message.Message;

import org.columba.gui.message.*;
import org.columba.gui.attachment.*;
import org.columba.gui.header.*;
import org.columba.gui.util.*;

import org.columba.folder.*;

import java.util.*;
import java.lang.reflect.*;

public class ColumbaWorkerIntern extends Worker
{
	Message message;
	FolderOperation folderOp;

	public Object construct()
	{
		int opLength;

		if (op.type == Operation.FOLDER)
		{
			folderOp = (FolderOperation) op;
			channel = folderOp.channel;
		}

		opLength = Array.getLength(folderOp.uidList);

		//        setText( folderOp.toString() );

		setProgressBarMaximum(opLength);
		setProgressBarValue(0);

		//System.out.println( folderOp.mode );

		switch (folderOp.mode)
		{

			/* Copy - Operazion */

			case (Operation.COPY) :
				{
					for (int i = 0; i < opLength; i++)
					{
						try
						{
							if (folderOp.source.exists(folderOp.uidList[i]) == true)
							{

								Message message =
									(Message) folderOp.source.workerGet(folderOp.uidList[i]).clone();

								folderOp.drain.workerAdd(message);
								FolderEvent event = new FolderEvent(folderOp.drain, message, FolderEvent.ADD);
								root.mainInterface.crossbar.fireFolderChanged(event, folderOp.drain);

							}

							setProgressBarValue(i + 1);
						}
						catch (Exception e)
						{
							ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
							dialog.showDialog(e);

							e.printStackTrace();

							setCancel(true);

							break;
						}
					}

					TreeNodeEvent updateEvent =
						new TreeNodeEvent(folderOp.source, TreeNodeEvent.UPDATE);
					root.mainInterface.crossbar.fireTreeNodeChanged(updateEvent);

					break;
				}

				/* Move - Operation */

			case (Operation.MOVE) :
				{
					// in crossbar we already used folder-workerviewremove
					// => the message are not visible for the table anymore
					// => we send a signal to the table to update his view

					FolderEvent event =
						new FolderEvent(folderOp.source, folderOp.uidList, FolderEvent.REMOVE);
					root.mainInterface.crossbar.fireFolderChanged(event, folderOp.source);

					TreeNodeEvent updateEvent =
						new TreeNodeEvent(folderOp.source, TreeNodeEvent.UPDATE);
					root.mainInterface.crossbar.fireTreeNodeChanged(updateEvent);

					for (int i = 0; i < opLength; i++)
					{
						try
						{
							if (folderOp.source.exists(folderOp.uidList[i]) == true)
							{
								//Message message = (Message) folderOp.source.workerRemove(folderOp.uidList[i]);
								Message message = (Message) folderOp.source.workerGet(folderOp.uidList[i]);
								folderOp.source.workerRemove(folderOp.uidList[i]);

								folderOp.drain.workerAdd(message);
								event = new FolderEvent(folderOp.drain, message, FolderEvent.ADD);
								root.mainInterface.crossbar.fireFolderChanged(event, folderOp.drain);

							}

							setProgressBarValue(i + 1);
						}
						catch (Exception e)
						{
							ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
							dialog.showDialog(e);

							e.printStackTrace();

							setCancel(true);

							break;
						}
					}

					TreeNodeEvent updateEvent2 =
						new TreeNodeEvent(folderOp.drain, TreeNodeEvent.UPDATE);
					root.mainInterface.crossbar.fireTreeNodeChanged(updateEvent2);

					break;
				}

				/* Delete - Operation */

			case (Operation.DELETE) :
				{
					// in crossbar we already used folder-workerviewremove
					// => the message are not visible for the table anymore
					// => we send a signal to the table to update his view

					FolderEvent event =
						new FolderEvent(folderOp.source, folderOp.uidList, FolderEvent.REMOVE);
					root.mainInterface.crossbar.fireFolderChanged(event, folderOp.source);

					TreeNodeEvent updateEvent =
						new TreeNodeEvent(folderOp.source, TreeNodeEvent.UPDATE);
					root.mainInterface.crossbar.fireTreeNodeChanged(updateEvent);

					for (int i = 0; i < opLength; i++)
					{
						try
						{
							if (folderOp.source.exists(folderOp.uidList[i]) == true)
							{
								folderOp.source.workerRemove(folderOp.uidList[i]);
							}

							setProgressBarValue(i + 1);
						}
						catch (Exception e)
						{
							ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
							dialog.showDialog(e);

							e.printStackTrace();

							setCancel(true);

							break;
						}
					}

					break;
				}

		}

		unregister();

		return null;
	}

}