// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.folder.composer;

import org.columba.gui.statusbar.*;

import org.columba.message.*;
import org.columba.parser.*;
import org.columba.folder.*;
import org.columba.config.*;
import org.columba.gui.message.*;
import org.columba.gui.attachment.*;
import org.columba.gui.header.*;
import org.columba.gui.util.*;
import org.columba.gui.composer.*;
import org.columba.composer.*;
import org.columba.coder.*;

import java.io.*;
import java.util.*;
import java.lang.reflect.*;

/**
 * @version 	1.0
 * @author
 */
public class ComposerWorker extends Worker
{
	ComposerOperation composerOp;
	ComposerOperator composerRoot;

	public Object construct()
	{

		if (op.type == Operation.COMPOSER)
		{
			composerOp = (ComposerOperation) op;
			composerRoot = (ComposerOperator) root;
		}

		//System.out.println( "FolderOp : "+op.mode );

		switch (op.mode)
		{
			case (Operation.COMPOSER_REPLY) :
				{
					while (!composerOp.channel.canRead(this))
					{
						try
						{
							System.out.println("Going to sleep..");

							sleep();
						}
						catch (InterruptedException e)
						{
							e.printStackTrace();
						}
					}

					Message message = (Message) composerOp.channel.read();

					ComposerFrame composer = new ComposerFrame(root.mainInterface);

					String sender;
					String subject = (String) message.getHeader().get("Subject");

					if ( isAlreadyReply(subject) == false )	
						composer.setSubject("Re: " + subject);
					else
					    composer.setSubject(subject);

					sender = (String) message.getHeader().get("Reply-To");
					if (sender == null)
						sender = (String) message.getHeader().get("From");

					composer.setTo(sender);

					String messageId = (String) message.getHeader().get("Message-Id");
					if (messageId != null)
					{
						composer.addHeaderItem("In-Reply-To", messageId);

						String references = (String) message.getHeader().get("References");
						if (references != null)
						{
							references = references + " " + messageId;
							composer.addHeaderItem("References", references);

						}
					}

					composer.setProfile(
						(String) message.getHeader().get("columba.host"),
						(String) message.getHeader().get("To"));

					MimePart bodyPart = message.getBodyPart();
					Decoder decoder =
						CoderRouter.getDecoder(bodyPart.getHeader().contentTransferEncoding);
					StringWriter decodedBody = new StringWriter();

					decoder.setupDecoder(new StringReader(bodyPart.getBody()), decodedBody);
					decoder.run();

					composer.setInlineText(decodedBody.toString());

					break;
				}
			case (Operation.COMPOSER_REPLY_ALL) :
				{
					while (!composerOp.channel.canRead(this))
					{
						try
						{
							System.out.println("Going to sleep..");

							sleep();
						}
						catch (InterruptedException e)
						{
							e.printStackTrace();
						}
					}

					Message message = (Message) composerOp.channel.read();

					ComposerFrame composer = new ComposerFrame(root.mainInterface);

					String sender;
					String subject = (String) message.getHeader().get("Subject");
	
					if ( isAlreadyReply(subject) == false )	
						composer.setSubject("Re: " + subject);
					else
					    composer.setSubject(subject);
					    
					sender = (String) message.getHeader().get("Reply-To");
					if (sender == null)
						sender = (String) message.getHeader().get("From");
					String to = (String) message.getHeader().get("To");
					StringBuffer buf = new StringBuffer();
					buf.append(sender);
					buf.append(",");
					buf.append(to);

					composer.setTo(buf.toString());

					String messageId = (String) message.getHeader().get("Message-Id");
					if (messageId != null)
					{
						composer.addHeaderItem("In-Reply-To", messageId);

						String references = (String) message.getHeader().get("References");
						if (references != null)
						{
							references = references + " " + messageId;
							composer.addHeaderItem("References", references);

						}
					}

					composer.setProfile(
						(String) message.getHeader().get("columba.host"),
						(String) message.getHeader().get("To"));

					MimePart bodyPart = message.getBodyPart();
					Decoder decoder =
						CoderRouter.getDecoder(bodyPart.getHeader().contentTransferEncoding);
					StringWriter decodedBody = new StringWriter();

					decoder.setupDecoder(new StringReader(bodyPart.getBody()), decodedBody);
					decoder.run();

					composer.setInlineText(decodedBody.toString());

					break;
				}
			case (Operation.COMPOSER_REPLY_MAILINGLIST) :
				{
					while (!composerOp.channel.canRead(this))
					{
						try
						{
							System.out.println("Going to sleep..");

							sleep();
						}
						catch (InterruptedException e)
						{
							e.printStackTrace();
						}
					}

					Message message = (Message) composerOp.channel.read();

					ComposerFrame composer = new ComposerFrame(root.mainInterface);

					String sender = null;
					String subject = (String) message.getHeader().get("Subject");
		
					if ( isAlreadyReply(subject) == false )	
						composer.setSubject("Re: " + subject);
					else
					    composer.setSubject(subject);
					    
					sender = (String) message.getHeader().get("List-Post");
					if (sender != null)
					{
						// example: List-Post: <mailto:frd@localhost>

						// we have to remove the "mailto:" substring

						int pos = 0;
						StringBuffer list = new StringBuffer(sender);
						while (pos < list.length())
						{
							char ch = list.charAt(pos);
							if (ch == '<')
							{
								pos++;
								char ch2 = '<';
								while (ch2 != ':')
								{
									System.out.println("char:" + ch2);
									if (pos >= list.length())
										break;

									ch2 = list.charAt(pos);
									list.deleteCharAt(pos);
								}

								if (list.charAt(pos + 1) == ':')
									list.deleteCharAt(pos + 1);
							}
							pos++;
						}

						sender = list.toString();
					}
					else
						sender = (String) message.getHeader().get("Reply-To");

					if (sender == null)
						sender = (String) message.getHeader().get("From");

					System.out.println("sender: " + sender);

					composer.setTo(sender);

					String messageId = (String) message.getHeader().get("Message-Id");
					if (messageId != null)
					{
						composer.addHeaderItem("In-Reply-To", messageId);

						String references = (String) message.getHeader().get("References");
						if (references != null)
						{
							references = references + " " + messageId;
							composer.addHeaderItem("References", references);

						}
					}

					composer.setProfile(
						(String) message.getHeader().get("columba.host"),
						(String) message.getHeader().get("To"));

					MimePart bodyPart = message.getBodyPart();
					Decoder decoder =
						CoderRouter.getDecoder(bodyPart.getHeader().contentTransferEncoding);
					StringWriter decodedBody = new StringWriter();

					decoder.setupDecoder(new StringReader(bodyPart.getBody()), decodedBody);
					decoder.run();

					composer.setInlineText(decodedBody.toString());

					break;
				}
			case (Operation.COMPOSER_REPLY_AS_ATTACHMENT) :
				{
					while (!composerOp.channel.canRead(this))
					{
						try
						{
							System.out.println("Going to sleep..");

							sleep();
						}
						catch (InterruptedException e)
						{
							e.printStackTrace();
						}
					}

					Message message = (Message) composerOp.channel.read();

					ComposerFrame composer = new ComposerFrame(root.mainInterface);

					String sender;
					System.out.println("header:" + message.getHeader());
					System.out.println("subject:" + message.getHeader().get("Subject"));
					String subject = (String) message.getHeader().get("Subject");

					if ( isAlreadyReply(subject) == false )
						composer.setSubject("Re: " + subject);
					else
					    composer.setSubject(subject);
					    
					sender = (String) message.getHeader().get("Reply-To");
					if (sender == null)
						sender = (String) message.getHeader().get("From");

					composer.setTo(sender);

					String messageId = (String) message.getHeader().get("Message-Id");
					if (messageId != null)
					{
						composer.addHeaderItem("In-Reply-To", messageId);

						String references = (String) message.getHeader().get("References");
						if (references != null)
						{
							references = references + " " + messageId;
							composer.addHeaderItem("References", references);

						}
					}

					composer.setProfile(
						(String) message.getHeader().get("columba.host"),
						(String) message.getHeader().get("To"));

					if (message.getRawString() != null)
					{
						MimeHeader header = new MimeHeader();
						header.contentType = new String("Message");
						header.contentSubtype = new String("Rfc822");

						composer.addMimePart(
							new MimePart(header, new StringReader(message.getRawString())));
					}

					break;
				}
			case (Operation.COMPOSER_FORWARD) :
				{
					while (!composerOp.channel.canRead(this))
					{
						try
						{
							System.out.println("Going to sleep..");

							sleep();
						}
						catch (InterruptedException e)
						{
							e.printStackTrace();
						}
					}

					Message message = (Message) composerOp.channel.read();

					ComposerFrame composer = new ComposerFrame(root.mainInterface);

					MimeHeader header = new MimeHeader();
					String sender;
					String subject = (String) message.getHeader().get("Subject");

					composer.setSubject("Fwd: " + subject);

					composer.setProfile(
						(String) message.getHeader().get("columba.host"),
						(String) message.getHeader().get("To"));

					header.contentType = new String("Message");
					header.contentSubtype = new String("Rfc822");

					composer.addMimePart(
						new MimePart(header, new StringReader(message.getRawString())));

					break;
				}
			case (Operation.COMPOSER_FORWARD_INLINE) :
				{
					while (!composerOp.channel.canRead(this))
					{
						try
						{
							System.out.println("Going to sleep..");

							sleep();
						}
						catch (InterruptedException e)
						{
							e.printStackTrace();
						}
					}

					Message message = (Message) composerOp.channel.read();

					ComposerFrame composer = new ComposerFrame(root.mainInterface);

					String sender;
					String subject = (String) message.getHeader().get("Subject");

					composer.setSubject("Fwd: " + subject);

					composer.setProfile(
						(String) message.getHeader().get("columba.host"),
						(String) message.getHeader().get("To"));

					MimePart bodyPart = message.getBodyPart();
					Decoder decoder =
						CoderRouter.getDecoder(bodyPart.getHeader().contentTransferEncoding);
					StringWriter decodedBody = new StringWriter();

					decoder.setupDecoder(new StringReader(bodyPart.getBody()), decodedBody);
					decoder.run();

					composer.setInlineText(decodedBody.toString());

					break;
				}
			case (Operation.COMPOSER_OPEN) :
				{
					while (!composerOp.channel.canRead(this))
					{
						try
						{
							System.out.println("Going to sleep..");

							sleep();
						}
						catch (InterruptedException e)
						{
							e.printStackTrace();
						}
					}

					Message message = (Message) composerOp.channel.read();

					ComposerFrame composer = new ComposerFrame(root.mainInterface);

					String sender;

					String subject = (String) message.getHeader().get("Subject");
					if (subject != null)
						composer.setSubject(subject);

					String reply = (String) message.getHeader().get("Reply-To");
					if (reply != null)
						composer.addHeaderItem("Reply-To", reply);

					sender = (String) message.getHeader().get("To");
					if (sender != null)
						composer.setTo(sender);

					String messageId = (String) message.getHeader().get("Message-Id");
					if (messageId != null)
					{
						composer.addHeaderItem("In-Reply-To", messageId);

						String references = (String) message.getHeader().get("References");
						if (references != null)
						{
							composer.addHeaderItem("References", references);

						}
					}

					String host = (String) message.getHeader().get("columba.host");
					if (host != null)
						composer.addHeaderItem("columba.host", host);

					composer.composerInterface.composerEditor.selectProfile(
						composer.composerInterface.config.getAccountList().hostGetAccountName(
							(String) message.getHeader().get("columba.host")));

					Message parsedMessage =
						new Rfc822Parser().parse(
							message.getRawString(),
							true,
							message.getHeader(),
							MimeTypeParser.SHOW_MESSAGE);
					if (parsedMessage.getMimePartCollection().count() >= 1)
					{

						MimePart bodyPart = parsedMessage.getMimePartCollection().get(0);

						Decoder decoder =
							CoderRouter.getDecoder(bodyPart.getHeader().contentTransferEncoding);
						StringWriter decodedBody = new StringWriter();

						decoder.setupDecoder(new StringReader(bodyPart.getBody()), decodedBody);
						decoder.run();

						composer.setBodyText(decodedBody.toString());

						if (parsedMessage.getMimePartCollection().count() == 1)
						{

							break;
						}

						for (int i = 1; i < parsedMessage.getMimePartCollection().count(); i++)
						{
							MimePart mp = parsedMessage.getMimePartCollection().get(i);
							MimeHeader header = mp.getHeader();
							decoder = CoderRouter.getDecoder(header.contentTransferEncoding);

							StringWriter strWriter = new StringWriter();
							decoder.setupDecoder(new StringReader(mp.getBody()), strWriter);

							decoder.run();
							mp.setBody(strWriter.toString());
							composer.addMimePart(mp);

						}

					}

					break;
				}

			case (Operation.COMPOSER_SAVEDRAFT) :
				{
					int uid = composerOp.frame.composerInterface.composerEditor.getAccountUid();

					Message message =
						composerOp.frame.composerInterface.messageComposer.compose(uid);

					// this method works only for SendableMessage-Objects
					/*
					message.setRecipients(
						composerOp.frame.composerInterface.composerHeader.getRCPTVector());
					*/

					// add message to temp-folder
					try
					{
						composerOp.frame.composerInterface.composerFolder.workerAdd(message);
					}
					catch (Exception e)
					{
						System.out.println(e);
					}

					// move it from temp-folder to draft-folder
					AccountItem item =
						composerOp.frame.composerInterface.config.getAccountList().uidGet(uid);
					SpecialFoldersItem folderItem = item.getSpecialFoldersItem();
					String str = folderItem.getDrafts();
					int destUid = Integer.parseInt(str);
					Folder destFolder =
						composerOp.frame.composerInterface.treeViewer.getFolderTree().getFolder(
							destUid);
					Folder srcFolder = composerOp.frame.composerInterface.composerFolder;
					Object uids[] = srcFolder.getUids();

					FolderOperation op =
						new FolderOperation(Operation.MOVE, 0, uids, srcFolder, destFolder);
					composerOp.frame.composerInterface.crossbar.operate(op);

					break;
				}
			case (Operation.COMPOSER_SAVETEMPLATE) :
				{
					int uid = composerOp.frame.composerInterface.composerEditor.getAccountUid();

					Message message =
						composerOp.frame.composerInterface.messageComposer.compose(uid);

					// this method works only for SendableMessage-Objects
					/*
					message.setRecipients(
						composerOp.frame.composerInterface.composerHeader.getRCPTVector());
					*/

					// add message to temp-folder
					try
					{
						composerOp.frame.composerInterface.composerFolder.workerAdd(message);
					}
					catch (Exception e)
					{
						System.out.println(e);
					}

					// move it from temp-folder to template-folder
					AccountItem item =
						composerOp.frame.composerInterface.config.getAccountList().uidGet(uid);
					SpecialFoldersItem folderItem = item.getSpecialFoldersItem();
					String str = folderItem.getTemplates();
					int destUid = Integer.parseInt(str);
					Folder destFolder =
						composerOp.frame.composerInterface.treeViewer.getFolderTree().getFolder(
							destUid);
					Folder srcFolder = composerOp.frame.composerInterface.composerFolder;
					Object uids[] = srcFolder.getUids();

					FolderOperation op =
						new FolderOperation(Operation.MOVE, 0, uids, srcFolder, destFolder);
					composerOp.frame.composerInterface.crossbar.operate(op);

					break;
				}

			case (Operation.COMPOSER_SEND) :
				{
					ComposerInterface composerInterface = composerOp.frame.composerInterface;
					SendableMessage message;

					String content;

					int uid = composerInterface.composerEditor.getAccountUid();

					message = composerInterface.messageComposer.compose(uid);

					message.setRecipients(composerInterface.composerHeader.getRCPTVector());

					try
					{
						System.out.println("----------------->message:"+message);
						
						composerInterface.composerFolder.workerAdd(message);
					}
					catch (Exception e)
					{
						System.out.println(e);
					}

					composerInterface.crossbar.operate(
						new OutboxOperation(
							Operation.SMTP_SENDCOMPOSED,
							10,
							composerInterface.composerFolder.getUids(),
							composerInterface.composerFolder));

					composerInterface.composerFrame.dispose();

					break;
				}

			case (Operation.COMPOSER_SENDLATER) :
				{
					ComposerInterface composerInterface = composerOp.frame.composerInterface;
					SendableMessage message;

					String content;

					int uid = composerInterface.composerEditor.getAccountUid();

					message = composerInterface.messageComposer.compose(uid);

					message.setRecipients(composerInterface.composerHeader.getRCPTVector());

					try
					{
						composerInterface.composerFolder.workerAdd(message);
					}
					catch (Exception e)
					{
						System.out.println(e);
					}

					Folder srcFolder = composerOp.frame.composerInterface.composerFolder;
					Object uids[] = srcFolder.getUids();

					Folder destFolder =
						composerOp.frame.composerInterface.treeViewer.getFolderTree().getFolder(103);

					FolderOperation op =
						new FolderOperation(Operation.MOVE, 0, uids, srcFolder, destFolder);
					composerOp.frame.composerInterface.crossbar.operate(op);

					composerInterface.composerFrame.dispose();

					break;
				}

		}

		unregister();
		return null;
	}
	
	protected boolean isAlreadyReply( String subject )
	{
		String str = subject.toLowerCase();
		
		if ( str.startsWith("re:") == true ) return true;
		
		int index = str.indexOf("re:");
		if ( index != -1 ) return true;
		
		return false;
	}
}