// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.folder.encryption;

import org.columba.parser.*;
import org.columba.folder.*;
import org.columba.pgp.*;
import org.columba.message.*;
import org.columba.config.*;

import org.columba.gui.message.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */
public class EncryptionWorker extends Worker
{
    EncryptionOperation encryptionOp;
    EncryptionOperator encryptionRoot;

    public Object construct()
    {
        System.out.println("EncryptionWorker started...");

        if ( op.type == Operation.ENCRYPTION )
        {
            encryptionOp = (EncryptionOperation) op;
            encryptionRoot = (EncryptionOperator) root;
            channel = encryptionOp.channel;
        }

        switch ( op.mode )
        {
            case Operation.DECRYPT:
            {
                PGPController util = root.mainInterface.pgpController;

                ColumbaHeader header = null;
                try
                {
                    header = encryptionOp.source.uidGetHeader( encryptionOp.uid );
                }
                catch ( Exception ex )
                {
                    ex.printStackTrace();
                }


                String to = (String) header.get("To");
                PGPItem pgpItem=null;
                if ( to != null )
                {
                    AccountList list = root.mainInterface.config.getAccountList();
                    pgpItem = list.getPGPItem(to);
                }

                if ( pgpItem == null )
                {
                    System.out.println("-----------------> pgp item == null");
                    break;
                }


                String source = null;
                try
                {
                    source = encryptionOp.source.uidToString( encryptionOp.uid, null );
                }
                catch ( Exception ex )
                {
                    ex.printStackTrace();
                }

                if ( source != null )
                {
                    String decryptedMessage = util.decrypt( source, pgpItem );
		    if ( decryptedMessage == null ) break;
                    //MimePartCollection mc = new MimeMultipartParser().parseDecrypt( decryptedMessage  );

                    //Message message = new Message( header );
                    //message.setMimePartCollection( mc );

		    Message message = new Message();
		    message = new Rfc822Parser().parse(decryptedMessage, true, message.getHeader(),1);

                    Object uid = null;
                    try
                    {
                        uid = encryptionRoot.tempFolder.add( message, source );
                    }
                    catch (Exception ex )
                    {
                        ex.printStackTrace();
                    }

		    root.mainInterface.messageViewer.setSecurityIndicator( SecurityIndicator.DECRYPTION_SUCCESS );

                    root.mainInterface.crossbar.operate(
                        new GuiOperation(Operation.MESSAGEBODY, 4, encryptionRoot.tempFolder, uid) );


                }

                break;
            }
            case Operation.VERIFY:
            {

                PGPController util = root.mainInterface.pgpController;

                ColumbaHeader header = null;
                try
                {
                    header = encryptionOp.source.uidGetHeader( encryptionOp.uid );
                }
                catch ( Exception ex )
                {
                    ex.printStackTrace();
                }


                String to = (String) header.get("To");
                PGPItem pgpItem=null;
                if ( to != null )
                {
                    AccountList list = root.mainInterface.config.getAccountList();
                    pgpItem = list.getPGPItem(to);
                }

                if ( pgpItem == null )
                {
                    System.out.println("-----------------> pgp item == null");
                    break;
                }


                String source = null;
                try
                {
                    source = encryptionOp.source.uidToString( encryptionOp.uid, null );
                }
                catch ( Exception ex )
                {
                    ex.printStackTrace();
                }

                if ( source != null )
                {
                    MimePartCollection mp = new MimeMultipartEncryptedParser().parseDecrypt( source  );

                    //Message message = new Message( header );
                    //message.setMimePartCollection( mp );

		    if ( mp.count() == 2 )
		    {
			MimePart part = mp.get(1);
                        MimeHeader h = part.getHeader();
                        String contentType = h.getContentType();
                        String contentSubtype = h.getContentSubtype();
			
			String signatureBody = part.getBody();
			System.out.println("signature body:\n"+signatureBody);

			MimePart messagePart = mp.get(0);			
			MimeHeader messageHeader = messagePart.getHeader();

			StringBuffer messageSource = new StringBuffer();
			messageSource.append( messageHeader.getHeader() );
			String messageBody = messagePart.getBody();
			messageSource.append("\n\n");
			messageSource.append( messageBody );
			System.out.println("message body:\n"+messageSource);

			if ( contentSubtype.equals("pgp-signature") )
                        {
                            System.out.println("found pgp-signature!");

                            // at this point we have to verify the message signature with PGP                                                      
                            util.verify( messageSource.toString(), signatureBody, pgpItem );                            
                        }
		    }

		    /*
                    for ( int i=0; i<mp.count(); i++ )
                    {
                        MimePart part = mp.get(i);
                        MimeHeader h = part.getHeader();
                        String contentType = h.getContentType();
                        String contentSubtype = h.getContentSubtype();

                        if ( contentSubtype.equals("pgp-signature") )
                        {
                            System.out.println("found pgp-signature!");

                            // at this point we have to verify the message signature with PGP
                            if ( i>=1 )
                            {
                                MimePart messagePart = mp.get(0);
                                String body = messagePart.getBody();
				System.out.println("bodypart:\n"+body);

                                util.verify( source, part.getBody(), pgpItem );
                            }

                        }
                    }
		    */


                }



                break;
            }

        }

        unregister();

        return null;
    }
}
