// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.folder.gui;

import org.columba.gui.statusbar.*;

import org.columba.message.*;

import org.columba.gui.message.*;
import org.columba.gui.attachment.*;
import org.columba.gui.header.*;
import org.columba.config.*;
import org.columba.folder.*;
import org.columba.folder.imap.*;
import org.columba.folder.columba.*;
import org.columba.folder.temp.*;
import org.columba.coder.*;
import org.columba.util.*;
import org.columba.gui.util.*;

import javax.swing.*;
import java.util.*;
import java.io.*;
import javax.swing.plaf.basic.*;

public class GuiWorker extends Worker {
	GuiOperation guiOp;
	GuiOperator guiRoot;
	static File lastDir = null;

	public Object construct() {
		System.out.println("GuiWorker started...");

		if (op.type == Operation.GUI) {
			guiOp = (GuiOperation) op;
			guiRoot = (GuiOperator) root;
			channel = guiOp.channel;
		}

		switch (op.mode) {
			case (Operation.MESSAGEBODY) :
				{
					Message message = null;
					
					if( waitForRead() ) message = (Message) guiOp.channel.read();
					
					if (message == null)
						break;

					try {
						guiRoot.getMessageViewer().setFolder(guiOp.source);
						guiRoot.getMessageViewer().setUid(guiOp.uid);
						guiRoot.getMessageViewer().showMessage(message);

						System.out.println("guiop uid = " + guiOp.uid);

						if (guiOp.source instanceof TempFolder)
							break;

						ColumbaHeader header = message.getHeader();
						Flags flags = header.getFlags();
						boolean b = flags.getSeen();

						if (b == false)
							guiRoot.markAsReadTimer.restart(guiOp.source, guiOp.uid);
					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						e.printStackTrace();

						setCancel(true);

						//guiOp.channel.cancel();

						break;
					}

					break;
				}
			case (Operation.MESSAGESOURCE) :
				{
					String rawText = null;

					if( waitForRead() ) rawText = (String) guiOp.channel.read();
					if (rawText == null)
						break;

					guiRoot.getMessageViewer().showMessageSource(rawText);

					break;
				}
			case (Operation.ATTACHMENTLIST) :
				{
					MimePartCollection mimePartCollection = null;
					
					if( waitForRead() ) mimePartCollection = (MimePartCollection) guiOp.channel.read();

					if (mimePartCollection == null) {
						guiRoot.showAttachmentViewer(false);
						break;
					}

					if (mimePartCollection.count() > 1) {
						guiRoot.showAttachmentViewer(
							guiRoot.getAttachmentViewer().setMimeCollection(mimePartCollection));
						guiRoot.getAttachmentViewer().setFolder(guiOp.source);
						guiRoot.getAttachmentViewer().setUid(guiOp.uid);
					} else {
						guiRoot.showAttachmentViewer(false);
					}

					break;
				}
			case (Operation.ATTACHMENT) :
				{
					MimePart part = null;
					
					if( waitForRead() ) part = (MimePart) guiOp.channel.read();

					switch (guiOp.submode) {
						case Operation.OPEN_WITH :
							{
								Decoder decoder;
								MimeHeader header;

								header = part.getHeader();
								File tempFile = null;
								try {
									File filename = part.getFile();
									if (filename != null) {
										//System.out.println("filename:"+filename);
										tempFile = TempFileStore.createTempFile(filename.toString());
									} else
										tempFile = TempFileStore.createTempFile();

									decoder = CoderRouter.getDecoder(header.contentTransferEncoding);

									decoder.setupDecoder(
										new StringReader(part.getBody()),
										new FileWriter(tempFile));

									setText("Running Decoder ... ");
									System.out.println("decoder");

									decoder.run();

								} catch (Exception ex) {
									ex.printStackTrace();
								}

								guiRoot.getAttachmentViewer().openWith(part, tempFile);

								break;
							}
						case Operation.OPEN :
							{
								Decoder decoder;
								MimeHeader header;

								header = part.getHeader();
								File tempFile = null;
								try {
									File filename = part.getFile();
									if (filename != null) {
										System.out.println("filename:" + filename);
										tempFile = TempFileStore.createTempFile(filename.toString());
									} else
										tempFile = TempFileStore.createTempFile();

									decoder = CoderRouter.getDecoder(header.contentTransferEncoding);

									decoder.setupDecoder(
										new StringReader(part.getBody()),
										new FileWriter(tempFile));

									setText("Running Decoder ... ");
									System.out.println("decoder");

									decoder.run();

								} catch (Exception ex) {
									ex.printStackTrace();
								}

								guiRoot.getAttachmentViewer().open(part, tempFile);

								break;
							}
						case Operation.SAVE_AS :
							{
								System.out.println("save as command");

								File tempFile = null;

								String fileName = part.getHeader().getContentParameter("name");
								if (fileName == null)
									fileName = part.getHeader().getDispositionParameter("filename");

								cFileChooser fileChooser;

								if (lastDir == null)
									fileChooser = new cFileChooser();
								else
									fileChooser = new cFileChooser(lastDir);

								cFileFilter fileFilter = new cFileFilter();
								fileFilter.acceptFilesWithProperty(cFileFilter.FILEPROPERTY_FILE);

								fileChooser.setDialogTitle("Save Attachment as ...");
								if (fileName != null)
									fileChooser.forceSelectedFile(new File(fileName));

								fileChooser.setSelectFilter(fileFilter);
								int returnVal = fileChooser.showSaveDialog(guiRoot.mainInterface.mainFrame);

								if (returnVal == JFileChooser.APPROVE_OPTION) {
									tempFile = fileChooser.getSelectedFile();
								}
								if (tempFile == null)
									break;

								lastDir = tempFile.getParentFile();

								if (tempFile.exists()) {
									Object[] options = { "OK", "CANCEL" };
									JOptionPane.showOptionDialog(
										null,
										"Overwrite File?",
										"Warning",
										JOptionPane.DEFAULT_OPTION,
										JOptionPane.WARNING_MESSAGE,
										null,
										options,
										options[0]);
								}

								Decoder decoder;
								MimeHeader header;

								header = part.getHeader();
								try {
									decoder = CoderRouter.getDecoder(header.contentTransferEncoding);

									decoder.setupDecoder(
										new StringReader(part.getBody()),
										new FileWriter(tempFile));

									setText("Running Decoder ... ");

									System.out.println("decoder");

									decoder.run();

								} catch (Exception ex) {
									ex.printStackTrace();
								}

								//guiRoot.getAttachmentViewer().show( part, tempFile);

								break;
							}
					}

					//HeaderList headerList = (HeaderList) guiOp.channel.read();
					//guiRoot.getHeaderTableViewer().setHeaderList( headerList );

					break;
				}
			case (Operation.HEADERLIST) :
				{
					HeaderList headerList = null;
					if( waitForRead() ) headerList = (HeaderList) guiOp.channel.read();
					
					guiRoot.getHeaderTableViewer().setHeaderList(headerList);

					break;
				}

			case (Operation.FOLDER_ADD) :
				{
					String folderPath = null;
					if( waitForRead() ) folderPath = (String) guiOp.channel.read();
				
					FolderItem folderItem = guiOp.source.getFolderItem();

					if (folderItem.getType().equals("imap")) {

						ImapFolder imapFolder = (ImapFolder) guiOp.source;
						ImapRootFolder parent = imapFolder.getImapRootFolder();
						guiRoot.getTreeViewer().getFolderTree().addImapFolder(
							parent,
							folderPath,
							parent.getImapItem(),
							parent,
							parent.getAccountUid());
					} else if (folderItem.getType().equals("imaproot")) {
						ImapRootFolder parent = (ImapRootFolder) guiOp.source;

						guiRoot.getTreeViewer().getFolderTree().addImapFolder(
							parent,
							guiOp.name,
							parent.getImapItem(),
							parent,
							parent.getAccountUid());
					} else if (folderItem.getType().equals("columba")) {
						ColumbaFolder parent = (ColumbaFolder) guiOp.source;
						guiRoot.getTreeViewer().getFolderTree().addUserFolder(parent, guiOp.name);
					}

					break;
				}
			case (Operation.FOLDER_RENAME) :
				{
					String name = null;
					if( waitForRead() ) name = (String) guiOp.channel.read();
					
					if( name == null ) break;
					guiOp.source.setName(name);

					TreeNodeEvent updateEvent2 =
						new TreeNodeEvent(guiOp.source, TreeNodeEvent.STRUCTURE_CHANGED);
					guiRoot.mainInterface.crossbar.fireTreeNodeChanged(updateEvent2);

					break;
				}
			case (Operation.FOLDER_REMOVE) :
				{
					String name = null;
					if( waitForRead() ) name = (String) guiOp.channel.read();
					
					if( name == null ) break;
	
					guiOp.source.removeFromParent();

					TreeNodeEvent updateEvent2 =
						new TreeNodeEvent(guiOp.source.getParent(), TreeNodeEvent.STRUCTURE_CHANGED);
					guiRoot.mainInterface.crossbar.fireTreeNodeChanged(updateEvent2);

					break;
				}
			case (Operation.MARK_READ) :
			case (Operation.MARK_FLAGGED) :
			case (Operation.MARK_EXPUNGED) :
			case (Operation.MARK_ANSWERED) :
				{
					Object[] uidList = null;
					if( waitForRead() ) uidList = (Object[]) guiOp.channel.read();
					
					guiRoot.mainInterface.headerTableViewer.backupSelection();

					break;
				}
			case (Operation.DECRYPT) :
				{
					String raw = null;
					if( waitForRead() ) raw = (String) guiOp.channel.read();

					System.out.println("raw:\n" + raw);
					break;
				}
			case (Operation.VERIFY) :
				{
					System.out.println("not yet implemented");

					break;
				}

		}

		unregister();

		return null;
	}

}