// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.folder.imap;

import org.columba.message.*;
import org.columba.protocol.Imap4;
import org.columba.config.*;
import org.columba.parser.*;
import org.columba.util.*;
import org.columba.gui.util.*;
import org.columba.main.*;
import org.columba.gui.tree.util.*;
import org.columba.gui.header.util.*;

import org.columba.folder.*;

import java.lang.reflect.Array;
import java.io.*;
import java.util.*;
import org.columba.gui.header.*;
import org.columba.filter.*;
import javax.swing.event.*;
import javax.swing.*;

import org.columba.config.*;


public class ImapFolder extends RemoteFolder {
	private MainInterface mainInterface;
	private Imap4 imap;
	private ImapItem item;
	//private boolean select=false;
	//private boolean fetch=false;
	private Imap4Parser parser;
	private StringBuffer cache;

	private ImapRootFolder parent;

	private Object aktMessageUid;
	private Message aktMessage;

	public ImapFolder(
		AdapterNode node,
		FolderItem folderItem,
		ImapItem item,
		ImapRootFolder parent,
		MainInterface mainInterface) {
		super(node, folderItem, mainInterface);

		this.mainInterface = mainInterface;
		this.item = item;
		this.parent = parent;

		// this file caches all parsed header info
		headerFile = new File(directoryFile, "header");

		if (headerFile.exists()) {
			try {
				load();
			} catch (Exception ex) {
				System.out.println("Error while loading Folder: " + ex.getMessage());
				
				// clean up everything
				removeAll();
			}

		}

		imap = null;

		setChanged(true);

	}

	public boolean isSelected(SwingWorker worker) throws Exception {
		if (parent == null) {
			parent = getImapRootFolder();
			imap = parent.getImapServerConnection();
		} else {
			imap = parent.getImapServerConnection();
		}

		if (parent.getState() == Imap4.STATE_SELECTED) {
			if (parent.getSelectedFolderPath().equals(getPath()))
				return true;
			else {
				select(worker);
				return false;
			}
		} else if (parent.getState() == Imap4.STATE_AUTHENTICATE) {
			select(worker);
			return false;
		} else {
			// we are in Imap4.STATE_NONAUTHENTICATE
			getImapRootFolder().login(worker);
			select(worker);

			return false;
		}

	}

	private Message parseMessage(String headerString) {
		parser = new Imap4Parser();

		ColumbaHeader h = parser.parseHeader(headerString.toString());

		Message message = new Message(h);
		parser.parseFlags(imap.answer, message);
		h = message.getHeader();

		int size = -1;

		Integer octetString = parser.parseSize(imap.answer);
		size = Math.round(octetString.intValue() / 1024);
		if (size == 0)
			size = 1;

		h.set("columba.size", new Integer(size));
		//message.setSize( size  );
		//message.setHost( item.getHost() );
		h.set("columba.host", item.getHost());

		if (message.getHeader().get("Date") instanceof String) {
			Date date = DateParser.parseString((String) message.getHeader().get("Date"));
			h.set("columba.date", date);
			//message.setDate( date );
		}

		String shortFrom = (String) h.get("From");
		if (shortFrom != null) {
			if (shortFrom.indexOf("<") != -1) {
				shortFrom = shortFrom.substring(0, shortFrom.indexOf("<"));
				if (shortFrom.length() > 0) {
					if (shortFrom.startsWith("\""))
						shortFrom = shortFrom.substring(1, shortFrom.length() - 1);
					if (shortFrom.endsWith("\""))
						shortFrom = shortFrom.substring(0, shortFrom.length() - 1);
				}

			} else
				shortFrom = shortFrom;

			h.set("columba.from", shortFrom);
			//message.setShortFrom( shortFrom );
		} else {
			//message.setShortFrom("");
			h.set("columba.from", new String(""));
		}

		String priority = (String) h.get("X-Priority");
		if (priority != null) {
			int prio = -1;

			if (priority.indexOf("1") != -1) {
				prio = 1;

			} else if (priority.indexOf("2") != -1) {
				prio = 2;
			} else if (priority.indexOf("3") != -1) {
				prio = 3;
			} else if (priority.indexOf("4") != -1) {
				prio = 4;
			} else if (priority.indexOf("5") != -1) {
				prio = 5;
			}

			//message.setPriority( prio );
			h.set("columba.priority", new Integer(prio));
		} else {
			//message.setPriority( 3 );
			h.set("columba.priority", new Integer(3));
		}

		String attachment = (String) h.get("Content-Type");
		if (attachment != null) {
			attachment = attachment.toLowerCase();

			if (attachment.indexOf("multipart") != -1) {
				//message.setAttachment(true);
				h.set("columba.attachment", new Boolean(true));
			} else {
				h.set("columba.attachment", new Boolean(false));
				//message.setAttachment(false);
			}
		} else {
			h.set("columba.attachment", new Boolean(false));
			//message.setAttachment(false);
		}

		return message;
	}

	private void fetchAllHeader(Vector v, SwingWorker worker) throws Exception {

		boolean answer = false;
		ColumbaHeader h = new ColumbaHeader();
		StringBuffer buffer = new StringBuffer();
		int j = 0;
		int i = 0;
		int max = getMessageFolderInfo().getExists();

		worker.setProgressBarMaximum(max);

		// fetch headers, size, flags
		imap.sendString("UID FETCH 1:* (Rfc822.HEADER Rfc822.SIZE FLAGS)");

		StringBuffer headerString = new StringBuffer();
		i = imap.getAnswer();
		//int j=0;

		while (i != Imap4.ANSWER_OK) {
			worker.setText("Fetching headers from " + getName() + " ..");
			if (imap.answer.startsWith("*")) {
				headerString = new StringBuffer();
			} else if (imap.answer.equals("")) {
				i = imap.getAnswer();

				//Flags flags = parser.parseFlags( imap.answer );

				headerString.append("\n");

				Message message = parseMessage(headerString.toString());
				message.setUID(v.get(j));

				try {
					super.add(message, null, message.getUID());
				} catch (Exception ex) {
					System.out.println("imapfolder: " + ex.getMessage());
				}

				worker.setProgressBarValue(j + 1);

				j++;
			} else {
				headerString.append(imap.answer + "\n");
			}

			i = imap.getAnswer();
		}

	}

	private void fetchHeader(Object uid) throws Exception {

		boolean answer = false;
		ColumbaHeader h = new ColumbaHeader();
		StringBuffer buffer = new StringBuffer();
		int j = 0;
		int i = 0;

		// fetch headers, size, flags
		imap.sendString("UID FETCH " + uid + " (Rfc822.HEADER Rfc822.SIZE FLAGS)");

		StringBuffer headerString = new StringBuffer();
		i = imap.getAnswer();
		//int j=0;

		while (i != Imap4.ANSWER_OK) {

			if (imap.answer.startsWith("*")) {
				headerString = new StringBuffer();
			} else if (imap.answer.equals("")) {
				i = imap.getAnswer();

				headerString.append("\n");

				Message message = parseMessage(headerString.toString());
				message.setUID(uid);

				try {
					super.add(message, null, message.getUID());
				} catch (Exception ex) {
					System.out.println("imapfolder: " + ex.getMessage());
				}

			} else {
				headerString.append(imap.answer + "\n");
			}

			i = imap.getAnswer();
		}

	}

	private boolean existsLocally(String uid, int startIndex) throws Exception {
		/*
		for (int i = startIndex; i < count(); i++) {

			Message message = get(i);
			String str = (String) message.getUID();
			//System.out.println("local message uid: "+ str );
			if (uid.equals(str)) {
				//System.out.println("server uid exists locally");
				return true;
			}

		}
		*/
		
		if ( exists( uid ) ) 
			return true;
		else
			return false;
	}

	private boolean existsRemotely(String uid, int startIndex, Vector uidList)
		throws Exception {
		for (int i = startIndex; i < uidList.size(); i++) {
			String serverUID = (String) uidList.get(i);

			//System.out.println("server message uid: "+ serverUID );
			if (uid.equals(serverUID)) {
				//System.out.println("local uid exists remotely");
				return true;
			}
		}

		return false;
	}

	// synchronize local cache with server
	private void synchronize(Vector uidList, SwingWorker worker) throws Exception {

		/*
		// FIXME - synchronize method has bugs
		int lastSeen = count();

		if (lastSeen == 0) {
			// this is the first time we fetch headers => get all headers
			fetchAllHeader(uidList, worker);
			return;
		}
		*/
		
		String serverUID = null;
		String localUID = null;

		worker.setText("Fetching headers from " + getName() + " ..");

		for (int i = 0; i < count(); i++) {
			Message message = get(i);
			localUID = (String) message.getUID();

			if (existsRemotely(localUID, 0, uidList)) {
				// local uid exists on server, leave it like this

			} else {
				// local uid does not exist on server, remove uid
				try {
					super.workerRemove(localUID);

				} catch (Exception e) {
					System.out.println("error while removeing: " + e.getMessage());
				}

			}

		}

		worker.setProgressBarMaximum(uidList.size());
		worker.setProgressBarValue(0);
		for (int i = 0; i < uidList.size(); i++) {
			serverUID = (String) uidList.get(i);

			if (existsLocally(serverUID, 0)) {
				// server uid already exists

			} else {
				// new message found
				fetchHeader(serverUID);

			}
			worker.incProgressBarValue(1);
		}

	}

	private Vector fetchUIDList(SwingWorker worker) throws Exception {

		Vector v = new Vector();
		String buffer = new String();

		if (worker != null) {
			worker.setText("Fetching UID list " + getName());

			worker.setProgressBarMaximum(getMessageFolderInfo().getExists());
		}

		// download all UIDs for unique identification
		// later actions only use UID for message identification
		// fetch all UIDs
		buffer = imap.getUIDL(worker);

		// parse UID list
		parser = new Imap4Parser();
		v = parser.parseUids(buffer);

		return v;
	}
	
	
	private Vector fetchUIDList(String messageSet, SwingWorker worker) throws Exception {

		Vector v = new Vector();
		String buffer = new String();

		if (worker != null) {
			worker.setText("Fetching UID list " + getName());

			worker.setProgressBarMaximum(getMessageFolderInfo().getExists());
		}

		// download all UIDs for unique identification
		// later actions only use UID for message identification
		// fetch all UIDs
		buffer = imap.getUIDL(messageSet, worker);

		// parse UID list
		parser = new Imap4Parser();
		v = parser.parseUids(buffer);

		return v;
	}

	public ImapRootFolder getImapRootFolder() {
		Folder folder = (Folder) getParent();
		while (folder.getUid() != 100) {
			// System.out.println("name: "+ folder.getName() );

			FolderItem item = folder.getFolderItem();
			String type = item.getType();

			if (type.equals("imaproot")) {
				parent = (ImapRootFolder) folder;
				// System.out.println(" folder found");
				return (ImapRootFolder) folder;
			}

			folder = (Folder) folder.getParent();
		}

		return null;
	}

	public HeaderList getHeaderList(SwingWorker worker) throws Exception {
		

		isSelected(worker);

		MessageFolderInfo info = getMessageFolderInfo();
		int exists = info.getExists();
		if (exists == 0) {
			

			removeAll();

			
			return new HeaderList();
		}
		
		// fetch UID list
		Vector v = fetchUIDList(worker);

		

		synchronize(v, worker);

		HeaderList list = new HeaderList();

		for (int i = 0; i < exists; i++) {
			list.add(get(i).getHeader(), getUid(i));
		}

		return list;
	}

	
	public Object add(Message message, String s) throws Exception {
		setChanged(true);

		getMessageFolderInfo().incExists();
		if (message.getFlags().getSeen() == false)
			getMessageFolderInfo().incUnseen();
		if (message.getFlags().getRecent() == true)
			getMessageFolderInfo().incRecent();

		int size = message.getSize();

		//System.out.println("imap->add");

		StringBuffer buffer = new StringBuffer();

		cache = new StringBuffer();

		int i = 0;

		isSelected(null);

		if (s == null)
			System.out.println("string is null");
		imap.append(getPath(), s);

		return null;
	}

	private void uidsetAktMessage(Object uid) throws Exception {
		aktMessageUid = uid;
		aktMessage = uidGet(uid);

		imap.sendString("UID FETCH " + uid + " (BODYSTRUCTURE)");

		StringBuffer bodyStructureBuffer = new StringBuffer();
		int i = imap.getAnswer();

		while (i != Imap4.ANSWER_OK) {
			bodyStructureBuffer.append(imap.answer + "\n");
			i = imap.getAnswer();
		}

		parser = new Imap4Parser();
		aktMessage.setMimePartCollection(
			parser.parseBodyStructure3(bodyStructureBuffer.toString()));
	}

	public Message uidGetParsedMessage(Object uid) throws Exception {
		if ((uid == aktMessageUid) && (aktMessage != null)) {
			aktMessageUid = uid;

			System.out.println("-------------->uidGet: using aktmessage");
			return aktMessage;
		} else
			return uidGet(uid);
	}

	/*
	
	    public MimePart uidgetBodyPart( Object uid, SwingWorker worker ) throws Exception
	    {
		MimePartCollection mimeCollection = uidgetMimePartCollection( uid );
	
		int bodyPartIndex;
	
		if( mimeCollection == null ) {
		    MimePart bodyPart = new MimePart();
	
		    StringBuffer buffer = new StringBuffer();
		    
		    cache = new StringBuffer();
		    
		    int i = 0;
		    
		    if ( worker != null )
			{
			    worker.setText(" Fetching body...");
			    worker.startTimer();
			}
		    
		    // fetch all UIDs
		    imap.sendString("UID FETCH "+ uid +" BODY[0]");
		    
	
	            buffer = new StringBuffer();
	            i = imap.getAnswer();
	
	            if ( i==Imap4.ANSWER_MORE )
	            {
	
	              while  ( i != Imap4.ANSWER_OK )
	              {
	                  if ( i == Imap4.ANSWER_MORE )
	                  {
	
	                      //System.out.println("cache: "+ imap.answer );
	                      cache.append( imap.answer+"\n");
	
	                      //Flags flags = changeFlags( cache.toString() );
	
	                  }
	                  else
	                  {
	                      //System.out.println("buffer: "+ imap.answer );
	                      buffer.append( imap.answer+"\n" );
	                  }
	
	                  i = imap.getAnswer();
	              }
	            }
	            else if ( i==Imap4.ANSWER_OK )
	            {}
	
	
		    if ( worker != null )
	            	worker.stopTimer();
	          //System.out.println("body fetch completed successfully");
	
		    bodyPart.setSource( new StringReader( buffer.toString() ));
	
		}
		else {
		    String contentType;
	
		    for( int i=0; i<mimeCollection.count(); i++ ) {
			contentType = mimeCollection.get(i).getHeader().getContentType();
			if( contentType.indexOf( "text" ) != -1 ) {
			    bodyPartIndex = i+1;
			    break;
			}
		    }
		}
	
		return uidgetMimePart( uid, bodyPartIndex );
	
	
	
	        //System.out.println("result: "+ result );
	
	        try
	        {
	            if ( result.length() == 0 )
	            {
	                // this message body has only one line
	                result = cache.toString();
	
	                int end = result.lastIndexOf("\"");
	                int start = result.indexOf("\"");
	
	                if ( ( start != -1 ) && ( end != -1 ) )
	                   result = result.substring( start+1 , end-1 );
	            }
	            else
	            {
	                int end = result.lastIndexOf(")");
	
	                if ( end != -1 )
	                 result = result.substring( 0, end-1 );
			 result = result + "\n";
	            }
	
	        }
	        catch ( StringIndexOutOfBoundsException ex)
	        {
	            System.out.println("parsing error: "+ex.getMessage() );
	
	            result = buffer.toString()+cache.toString();
	
	        }
		
		bodyPart.setSource( new StringReader( result ) );
	
	        return bodyPart;
	    }
	
	*/

	public String uidToString(Object uid, SwingWorker worker) throws Exception {
		//System.out.println("imapfolder->uidToString");

		StringBuffer buffer = new StringBuffer();

		cache = new StringBuffer();

		isSelected(worker);

		if (worker != null) {
			worker.setText(" Fetching message source...");
			worker.startTimer();
		}

		String result = imap.fetchMessageSource(uid, worker);

		if (worker != null)
			worker.stopTimer();

		return result;
	}

	public MimePartCollection uidgetMimePartCollection(Object uid)
		throws Exception {
		if (aktMessageUid != uid) {
			uidsetAktMessage(uid);
		}

		return aktMessage.getMimePartCollection();
	}

	public int uidgetMimePartCount(Object uid) throws Exception {
		if (aktMessageUid != uid) {
			uidsetAktMessage(uid);
		}

		// look for attachments:
		Boolean attachBoolean =
			(Boolean) aktMessage.getHeader().get("columba.attachment");
		boolean attachments = attachBoolean.booleanValue();
		if (attachments == false)
			return 0;

		return aktMessage.getMimePartCount();
	}

	public MimePart uidgetMimePart(Object uid, int number, SwingWorker worker)
		throws Exception {

		if (aktMessageUid != uid) {
			uidgetMimePartCollection(uid);
		}

		String result = imap.fetchMimePart(uid, number, worker);

		MimePart part = aktMessage.getMimePartCollection().get(number);
		if (part != null)
			part.setBody(result);

		return part;
	}

	public void remove(int index) throws Exception {
		System.out.println("imapfolder->remove: not yet implemented");
	}

	public String getPath() {
		StringBuffer path = new StringBuffer();
		path.append(getName());
		Folder child = this;
		

		while (true) {
			child = (Folder) child.getParent();
			if (child instanceof ImapRootFolder)
				break;

			

			String n = child.getName();

			path.insert(0, n + getImapRootFolder().getDelimiter() );

		}

		return path.toString();
	}

	public boolean select(SwingWorker worker) throws Exception {
		

		if (worker != null)
			worker.setText("Selecting folder...");

		// close previously selected folder
		//  -> remove \DELETED-marked messages
		imap.close();


		/*
		// remove messages from local cache
		Object uids[] = getUids();
		for ( int i=0; i<uids.length; i++ )
		{
			Message message = uidGet( uids[i] );
			Flags flags = message.getFlags();
			String uid = (String) message.getUID();
			
			if ( flags.getDeleted() == true ) uidRemove(uid);
		}
		*/	
		
		
		
		boolean answer = false;

		

		String path = getPath();

		

		answer = imap.select(path);

		if (answer == false)
			throw new ImapException("Error while trying to select folder:\n" + imap.answer);

		

		String result = imap.getResult();
		
		
		messageFolderInfo = Imap4Parser.parseMessageFolderInfo(result);

		

		return answer;
	}

	// saves Message-Headers in separat File
	public void save() throws Exception {
		/*
		if ( getState() == 2 )
		   imap.close();
		*/

		System.out.println("saving headerfile: " + headerFile.toString());
		
		HeaderTableItem v =
			mainInterface.config.getMainFrameOptionsConfig().getHeaderTableItem();

		FileOutputStream istream = new FileOutputStream(headerFile.getPath());
		ObjectOutputStream p = new ObjectOutputStream(istream);

		p.writeInt(count());

		Rfc822Header h;

		for (int i = 0; i < count(); i++) {
			p.writeInt(i + 1);
			h = get(i).getHeader();

			p.writeObject(get(i).getUID());

			//p.writeObject( get(i).getHeader().get("Status") );
			//p.writeObject( get(i).getFlags() );

			Flags flags = get(i).getFlags();

			p.writeBoolean(flags.getSeen());
			p.writeBoolean(flags.getAnswered());
			p.writeBoolean(flags.getFlagged());
			p.writeBoolean(flags.getDeleted());
			p.writeBoolean(flags.getDraft());
			p.writeBoolean(flags.getRecent());

			p.writeObject(h.get("columba.date"));

			p.writeObject(h.get("columba.size"));

			p.writeObject(h.get("columba.from"));

			p.writeObject(h.get("columba.attachment"));

			p.writeObject(h.get("columba.priority"));

			p.writeObject(h.get("columba.host"));
			/*
			p.writeObject( get(i).getDate() );
			
			p.writeInt( get(i).getSize() );
			
			p.writeObject( get(i).getShortFrom() );
			
			p.writeObject( new Boolean( get(i).getAttachment() ) );
			
			p.writeInt( get(i).getPriority() );
			
			
			p.writeObject( get(i).getHost() );
			*/

			String column;
			HeaderItem item;
			Object o;
			for (int j = 0; j < v.count(); j++) {
				column = (String) v.getName(j);
				o = get(i).getHeader().get(column);
				p.writeObject(o);
			}
		}

		p.flush();
		p.close();

	}

	// load Message-Headers from  separat File instead of using the mbox-file

	public void load() throws Exception {

		System.out.println("loading headerfile: "+ headerFile.toString() );

		FileInputStream istream = new FileInputStream(headerFile.getPath());
		ObjectInputStream p = new ObjectInputStream(istream);

		int capacity = p.readInt();
		int number;

		for (int i = 1; i <= capacity; i++) {
			Message message = new Message();
			Rfc822Header h = message.getHeader();

			// read current number of message
			number = p.readInt();

			String str = (String) p.readObject();
			message.setUID(str);

			Flags flags = message.getFlags();

			flags.setSeen(p.readBoolean());
			flags.setAnswered(p.readBoolean());
			flags.setFlagged(p.readBoolean());
			flags.setDeleted(p.readBoolean());
			flags.setDraft(p.readBoolean());
			flags.setRecent(p.readBoolean());
			//message.setFlags( flags );

			Date date = (Date) p.readObject();
			h.set("columba.date", date);

			h.set("columba.size", p.readObject());

			String from = (String) p.readObject();
			h.set("columba.from", from);

			Boolean b = (Boolean) p.readObject();
			h.set("columba.attachment", b);

			//int priority = p.readInt();
			h.set("columba.priority", (Integer) p.readObject());

			String host = (String) p.readObject();
			h.set("columba.host", host);

			HeaderTableItem v =
				mainInterface.config.getMainFrameOptionsConfig().getHeaderTableItem();
			String column;
			Object o;
			for (int j = 0; j < v.count(); j++) {
				column = (String) v.getName(j);
				o = p.readObject();

				if (o == null) {
				} else if (o instanceof String) {
					String value = (String) o;
					h.set(column, value);
				} else if (o instanceof Integer) {
					Integer value = (Integer) o;
					h.set(column, value);
				} else if (o instanceof Boolean) {
					Boolean value = (Boolean) o;
					h.set(column, value);
				} else if (o instanceof Date) {
					Date value = (Date) o;
					h.set(column, value);
				}

			}
			add(message, null, message.getUID());
			/*
			getMessageFolderInfo().incExists();
			if (!flags.getSeen())
				getMessageFolderInfo().incUnseen();
			if (flags.getRecent())
				getMessageFolderInfo().incRecent();
				*/

		}

		// close stream
		p.close();

	}

	/*
	public void parseHeaders( Vector v ) throws Exception
	{
	    //System.out.println("---------------------> parsing headers");
	
	    try
	    {
	    for ( int i=0; i<v.size(); i++ )
	    {
	        Message message = (Message) v.get(i);
	        Rfc822Header h = message.getHeader();
	
	        if  ( h.get("date") != null )
	        {
	            //System.out.println("---------------------> parsing date");
	            //System.out.println("date: " +h.get("date") );
	            Date date = DateParser.parseString( (String) h.get( "date" ) );
	            h.set("columba.date", date);
	            //System.out.println("date: " +h.get("columba.date") );
	        }
	
	        if ( h.get("columba.priority") != null )
	        {
	            String str = (String) h.get("columba.priority");
	            h.set("columba.priority", new Integer(str) );
	        }
	
	        if ( h.get("columba.size") != null )
	        {
	            String str = (String) h.get("columba.size");
	            h.set("columba.size", new Integer(str) );
	        }
	
	        if ( h.get("columba.attachment") != null )
	        {
	            String str = (String) h.get("columba.attachment");
	            h.set("columba.attachment", new Boolean(str) );
	        }
	
	        if ( h.get("columba.fetchstate") != null )
	        {
	            String str = (String) h.get("columba.fetchstate");
	            h.set("columba.fetchstate", new Boolean(str) );
	        }
	
	        if ( h.get("columba.flags.seen") != null )
	        {
	            String str = (String) h.get("columba.flags.seen");
	            h.set("columba.flags.seen", new Boolean( str ) );
	        }
	
	        if ( h.get("columba.flags.recent") != null )
	        {
	            String str = (String) h.get("columba.flags.recent");
	            h.set("columba.flags.recent", new Boolean( str ) );
	        }
	
	        if ( h.get("columba.flags.flagged") != null )
	        {
	            String str = (String) h.get("columba.flags.flagged");
	            h.set("columba.flags.flagged", new Boolean( str ) );
	        }
	
	        if ( h.get("columba.flags.expunged") != null )
	        {
	            String str = (String) h.get("columba.flags.expunged");
	            h.set("columba.flags.expunged", new Boolean( str ) );
	        }
	
	        if ( h.get("columba.flags.draft") != null )
	        {
	            String str = (String) h.get("columba.flags.draft");
	            h.set("columba.flags.draft", new Boolean( str ) );
	        }
	
	        if ( h.get("columba.flags.answered") != null )
	        {
	            String str = (String) h.get("columba.flags.answered");
	            h.set("columba.flags.answered", new Boolean( str ) );
	        }
	
	    }
	    }
	    catch ( Exception ex )
	    {
	        throw new Exception("parseheader: "+ ex.getMessage() );
	    }
	
	}
	*/

	/*
	public void loadxml() throws Exception
	{
	     System.out.println("filename: "+headerFile.getPath() );
	
	     try
	     {
	         HeaderCacheReader reader = new HeaderCacheReader();
	
	         Vector v =reader.open( headerFile.getPath() );
	
	         reader.close();
	
	         parseHeaders( v );
	
	         //System.out.println("-------------------->ready");
	
	         for ( int i=0; i<v.size(); i++ )
	         {
	             Message m = (Message) v.get(i);
	             Rfc822Header header = m.getHeader();
	             Integer uid = new Integer( (String) header.get("uid") );
	             super.add( m, null, uid );
	         }
	
	
	
	     }
	     catch ( Exception ex )
	     {
	         System.out.println( ex.getMessage() );
	         ex.printStackTrace();
	     }
	
	
	}
	*/

	/*
	public void savexml() throws Exception
	{
	
	
	     System.out.println("filename: "+headerFile.getPath() );
	
	     try
	     {
	         HeaderCacheWriter writer = new HeaderCacheWriter();
	
	         writer.open( headerFile.getPath() );
	
	         writer.startDocument();
	
	         for ( int i=0; i<count(); i++ )
	         {
	             Message message = get(i);
	             Rfc822Header header = message.getHeader();
	             Hashtable hash = header.getHashtable();
	
	             writer.writeHash( hash );
	         }
	
	         writer.endDocument();
	
	         writer.close();
	     }
	     catch ( IOException ex )
	     {
	         System.out.println( ex.getMessage() );
	     }
	}
	*/

	public void expunge() throws Exception {
		boolean b = imap.expunge();

		if (b == false)
			throw new ImapException("imapfolder->expunge: " + imap.getResult());

	}

	public boolean addFolder(String path) throws Exception {
		boolean b = imap.create(path);

		if (b == false) {
			throw new ImapException(
				"Error while trying to add new folder:\n" + imap.getResult());
		}

		return b;
	}

	public boolean removeFolder() throws Exception {
		
		
		// close selected folder
		//  if we don't do this, we get a "already in use by another process"-exception
		//  and can't remove the folder
		imap.close();
		
		getImapRootFolder().unsubscribe( getPath() );
		
		
		boolean b = imap.delete(getPath());

		if (b == false) {
			throw new ImapException(
				"Error while trying to remove folder:\n" + imap.getResult());
		}

		return b;
	}

	public boolean renameFolder(String name) throws Exception {
		boolean b = imap.rename(getName(), name);

		if (b == false) {
			throw new ImapException(
				"Error while trying to rename folder:\n" + imap.getResult());
		}

		return b;
	}

	public boolean internMessageCopy(Object[] uids, ImapFolder destFolder)
		throws Exception {

		boolean b = true;
		
		
		/*
		for (int i = 0; i < uids.length; i++) 
		{
			String uid = (String) uids[i];
			b = imap.internCopy(uid, destFolder.getPath());
		}
		*/
		
		String messageSet = Imap4Parser.createMessageSet(uids);
		b = imap.internCopy( messageSet, destFolder.getPath() );

		return b;
	}
	
	
	public boolean internMessageRemove(Object[] uids) throws Exception {
		
		String messageSet = Imap4Parser.createMessageSet(uids);
		
		imap.store( messageSet, "+FLAGS", "(\\Deleted)");
		
		for (int i = 0; i < uids.length; i++) {

			super.workerRemove(uids[i]);

		}
		
		

		return true;
	}
	
	/*************************** Worker Methods ***************************/

	public void workerViewRemove(Object[] uids) throws Exception {

		/*
		for (int i = 0; i < uids.length; i++) {

			workerMark((String) uids[i], FolderOp.EXPUNGED);

		}
		*/

		super.workerViewRemove(uids);

	}

	public void workerViewMark(Object[] uids, int variant) throws Exception {
		switch (variant) {
			case FolderOp.READ :
				{
					for (int i = 0; i < Array.getLength(uids); i++) {
						markAsRead(uids[i]);
					}
					break;
				}
			case FolderOp.EXPUNGED :
				{
					for (int i = 0; i < Array.getLength(uids); i++) {
						markAsExpunged(uids[i]);
					}
					break;
				}
			case FolderOp.FLAGGED :
				{
					for (int i = 0; i < Array.getLength(uids); i++) {
						markAsFlagged(uids[i]);
					}
					break;
				}
			case FolderOp.ANSWERED :
				{
					for (int i = 0; i < Array.getLength(uids); i++) {
						markAsAnswered(uids[i]);
					}
					break;
				}

		}

		//fireTreeNodeUpdate();
	}

	public Message workerRemove(Object uid) throws Exception {
		
		
		workerMark((String) uid, FolderOp.EXPUNGED);
		
		return super.workerRemove(uid);

		
	}

	public void workerMark(Object uid, int variant) throws Exception {

		boolean b = false;

		switch (variant) {
			case FolderOp.READ :
				{

					b = imap.store(uid.toString(), "+FLAGS", "(\\Seen)");

					break;
				}
			case FolderOp.EXPUNGED :
				{

					b = imap.store(uid.toString(), "+FLAGS", "(\\Deleted)");

					break;
				}
			case FolderOp.FLAGGED :
				{

					b = imap.store(uid.toString(), "+FLAGS", "(\\Flagged)");

					break;
				}
			case FolderOp.ANSWERED :
				{

					b = imap.store(uid.toString(), "+FLAGS", "(\\Answered)");

					break;
				}

		}

	}

	public Operator getOperator() {
		return getImapRootFolder().getOperator();
	}

	protected Vector convertIndexToUid( Vector v ) throws Exception
	{
	    if ( v.size() == 0 ) return v;

		String messageSet = Imap4Parser.createMessageSet( v.toArray() );
		// fetch a new UIDlist
		Vector result = fetchUIDList(messageSet, null);
		/*
		for ( int i=0; i<v.size(); i++ )
		{
			int index = Integer.parseInt( (String) v.get(i) );
			
			Object uid = uids.get(index-1);
			result.add( uid );
		}
		*/
				
		return result;
	}
	

	public Vector processHeaderCriteria(
		Object[] uids,
		String headerField,
		String pattern,
		String condition)
		throws Exception
	{
		Vector v = new Vector();
		String messageSet = Imap4Parser.createMessageSet(v.toArray());
		System.out.println("messageSet:" + messageSet);

		boolean c = false;
		if (condition.equalsIgnoreCase("contains"))
		{
			c = true;
		}
		else if (condition.equalsIgnoreCase("contains not"))
		{
			c = false;
		}
		else if (condition.equalsIgnoreCase("is"))
		{
			c = true;
		}
		else if (condition.equalsIgnoreCase("is not"))
		{
			c = false;
		}
		else if (condition.equalsIgnoreCase("begins with"))
		{
			c = true;
		}
		else if (condition.equalsIgnoreCase("ends with"))
		{
			c = true;
		}
		
		Vector result = null;
		result = imap.searchHeader(messageSet, headerField, pattern, c);

		if (result != null)
			result = convertIndexToUid(result);

		return result;
	}
	
	
	 public Vector processBodyCriteria( Object[] uids, String pattern, String condition ) throws Exception
        {
        	Vector v = new Vector();
		String messageSet = Imap4Parser.createMessageSet(v.toArray());
		System.out.println("messageSet:" + messageSet);

		boolean c = false;
		if (condition.equalsIgnoreCase("contains"))
		{
			c = true;
		}
		else if (condition.equalsIgnoreCase("contains not"))
		{
			c = false;
		}
		
		
		Vector result = null;
		result = imap.searchBody(messageSet, pattern, c);

		if (result != null)
			result = convertIndexToUid(result);

		return result;
		
           
        }
        
	public Vector processDateCriteria( Object[] uids, String pattern, String condition ) throws Exception
        {
            Vector v = new Vector();

            java.text.DateFormat df = java.text.DateFormat.getDateTimeInstance( java.text.DateFormat.SHORT,
                                                                                java.text.DateFormat.SHORT );
            Date searchPattern = null;
            try
            {
                searchPattern = df.parse( pattern );
            }
            catch ( java.text.ParseException ex )
            {
                System.out.println("exception: "+ ex.getMessage() );
                ex.printStackTrace();

                return new Vector();
            }
            
            String messageSet = Imap4Parser.createMessageSet(v.toArray());
		System.out.println("messageSet:" + messageSet);

		boolean c = false;
		if (condition.equalsIgnoreCase("before"))
		{
			c = true;
		}
		else if (condition.equalsIgnoreCase("after"))
		{
			c = false;
		}
		
		
		Vector result = null;
		result = imap.searchDate(messageSet, pattern, c);

		if (result != null)
			result = convertIndexToUid(result);

		return result;
        }
        
        
        public Vector processSizeCriteria( Object[] uids, String pattern, String condition ) throws Exception
        {
        	Vector v = new Vector();
		String messageSet = Imap4Parser.createMessageSet(v.toArray());
		System.out.println("messageSet:" + messageSet);

		boolean c = false;
		if (condition.equalsIgnoreCase("smaller"))
		{
			c = true;
		}
		else if (condition.equalsIgnoreCase("bigger"))
		{
			c = false;
		}
		
		
		Vector result = null;
		result = imap.searchSize(messageSet, pattern, c);

		if (result != null)
			result = convertIndexToUid(result);

		return result;
        }
        
        public Vector processFlagsCriteria( Object[] uids, String pattern, String condition ) throws Exception
        {
        	Vector v = new Vector();
		String messageSet = Imap4Parser.createMessageSet(v.toArray());
		System.out.println("messageSet:" + messageSet);

		boolean c = false;
		if (condition.equalsIgnoreCase("is"))
		{
			c = true;
		}
		else if (condition.equalsIgnoreCase("is not"))
		{
			c = false;
		}
		
		
		Vector result = null;
		result = imap.searchFlags(messageSet, pattern, c);

		if (result != null)
			result = convertIndexToUid(result);

		return result;
        }
        
        
        public Vector processPriorityCriteria( Object[] uids, String pattern, String condition ) throws Exception
        {

            Integer searchPattern = null;
            if ( pattern.equalsIgnoreCase("Highest") )
            {
                searchPattern = new Integer(1);
            }
            else if ( pattern.equalsIgnoreCase("High") )
            {
                searchPattern = new Integer(2);
            }
            else if ( pattern.equalsIgnoreCase("Normal") )
            {
                searchPattern = new Integer(3);
            }
            else if ( pattern.equalsIgnoreCase("Low") )
            {
                searchPattern = new Integer(4);
            }
            else if ( pattern.equalsIgnoreCase("Lowest") )
            {
                searchPattern = new Integer(5);
            }
            
            
            return processHeaderCriteria( uids, "X-Priority", searchPattern.toString(), condition );
        }
}

