// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.folder.imap;

import org.columba.message.*;
import org.columba.protocol.Imap4;
import org.columba.config.*;
import org.columba.parser.*;
import org.columba.util.*;
import org.columba.gui.util.*;
import org.columba.main.*;
import org.columba.gui.tree.util.*;
import org.columba.gui.config.subscribe.SubscribeTreeNode;

import org.columba.folder.*;

import java.io.*;
import java.util.*;
import org.columba.gui.header.*;
import org.columba.filter.*;
import javax.swing.event.*;
import javax.swing.*;
import java.net.*;

public class ImapRootFolder extends RemoteFolder
{
	private MainInterface mainInterface;
	private Imap4 imap;
	private ImapItem item;
	//private boolean select=false;
	private boolean fetch = false;
	private Imap4Parser parser;
	private StringBuffer cache;
	private int state;
	private int accountUid;
	private Vector lsubList;

	private String delimiter = new String();
	//    private ImapOperator operator;

	public ImapRootFolder(
		AdapterNode node,
		FolderItem folderItem,
		ImapItem item,
		int accountUid,
		MainInterface mainInterface)
	{
		super(node, folderItem, mainInterface);

		this.mainInterface = mainInterface;
		this.item = item;
		this.accountUid = accountUid;

		state = 0;

		lsubList = null;

		imap = new Imap4();

	}

	public String getDelimiter()
	{
		return delimiter;
	}

	public boolean isLogin(SwingWorker worker) throws Exception
	{
		if (getState() == Imap4.STATE_AUTHENTICATE)
			return true;
		else
		{
			// we are in Imap4.STATE_NONAUTHENTICATE

			login(worker);
			return false;
		}

	}

	public void clearLSubList()
	{
		lsubList = null;
	}

	public int getAccountUid()
	{
		return accountUid;
	}

	public ImapItem getImapItem()
	{
		return item;
	}

	public Imap4 getImapServerConnection()
	{
		return imap;
	}

	private boolean isSubscribed(ImapFolder folder, Vector lsubList)
	{
		String folderPath = folder.getPath();

		for (int i = 0; i < lsubList.size(); i++)
		{

			String path = (String) lsubList.get(i);

			if (folderPath.equalsIgnoreCase(path))
				return true;
		}

		return false;
	}

	private void removeUnsubscribedFolders(Folder child)
	{
		if (!(child instanceof ImapRootFolder))
		{
			if (child.getChildCount() == 0)
			{
				Folder parent = (Folder) child.getParent();
				child.removeFromParent();
				System.out.println("folder removed:" + parent);

				removeUnsubscribedFolders(parent);
			}
		}
	}

	public void removeUnsubscribedFolders(Folder parent, Vector lsubList)
	{
		for (int i = 0; i < parent.getChildCount(); i++)
		{
			if (!(parent.getChildAt(i) instanceof ImapFolder))
				continue;

			ImapFolder child = (ImapFolder) parent.getChildAt(i);

			if (child.getChildCount() == 0)
			{
				// if folder is not subscribed: remove folder
				if (!isSubscribed(child, lsubList))
				{
					//removeUnsubscribedFolders(child);
					child.removeFromParent();
					i--;
				}
			}
			else
				removeUnsubscribedFolders(child, lsubList);
		}
	}

	private void addFolder(Folder folder, String name, Vector lsubList)
	{

		
		if (name.indexOf(delimiter) != -1)
		{
			
			
			String subchild = name.substring(0, name.indexOf(delimiter));
			ImapFolder subFolder = (ImapFolder) folder.getChild(subchild);

			if (subFolder == null)
			{

				// folder does not exist, create new folder
				subFolder =
					(ImapFolder) mainInterface.treeViewer.getFolderTree().addImapFolder(
						folder,
						subchild,
						item,
						this,
						accountUid);

				//subFolder.getFolderItem().setMessageFolder("false");
			}

			addFolder(
				subFolder,
				name.substring(name.indexOf(delimiter) + 1, name.length()),
				lsubList);
		}
		else
		{ 
		if (folder.getChild(name) == null)
			mainInterface.treeViewer.getFolderTree().addImapFolder(
				folder,
				name,
				item,
				this,
				accountUid);
		}

	}

	private void addSubscribedFolders(Vector lsubList) throws Exception
	{
		boolean inbox = false;
		boolean answer;

		for (int i = 0; i < lsubList.size(); i++)
		{
			String name = (String) lsubList.get(i);

			if (name.toLowerCase().equalsIgnoreCase("inbox"))
				inbox = true;

			addFolder(this, name, lsubList);

		}

		if (inbox == false)
		{
			answer = imap.flist("", "");
			//System.out.println("trying to parse flist");
			String str = imap.getResult();
			//System.out.println("str: "+ str );
			str = str.toLowerCase();
			//System.out.println("str: "+ str );

			if (str.indexOf("inbox") != -1)
			{
				Folder childFolder = (Folder) getChild("Inbox");
				if (childFolder == null)
				{
					mainInterface.treeViewer.getFolderTree().addImapFolder(
						this,
						"Inbox",
						item,
						this,
						accountUid);
				}
			}
			else
			{
				System.out.println("string inbox not found");
			}
		}
	}

	protected void lsubsub(Vector lsubList, Vector v) throws Exception
	{

		//lsubList.addAll(v);

		for (int i = 0; i < v.size(); i++)
		{
			String path = (String) v.get(i);
			lsubList.add(path);

			System.out.println("path:" + path);

			boolean answer = imap.flsub(path + "/*");

			if (imap.getResult().length() > 0)
			{
				Vector temp = Imap4Parser.parseLsub(imap.getResult());

				if (temp.size() > 0)
					lsubsub(lsubList, temp);
			}

		}

	}

	/**
	 * 
	 * this method is called to get the initial folder-list
	 *  ( this happens when doppel-clicking the imap root-folder
	 * */
	public void lsub(SwingWorker worker) throws Exception
	{

		boolean answer;
		lsubList = new Vector();
		Vector v = new Vector();

		isLogin(worker);

		if (worker != null)
			worker.setText("Retrieve folder listing...");

		if (worker != null)
			worker.startTimer();

		answer = imap.flsub("*");
		//System.out.println("trying to parse lsub");
		String result = imap.getResult();		
		
		delimiter = Imap4Parser.parseDelimiter(result);
		System.out.println("delimiter="+delimiter);
		
		lsubList = Imap4Parser.parseLsub(result);
		
		

		//lsubsub(lsubList, v);

		v = new Vector();
		v.add("INBOX");

		lsubsub(lsubList, v);

		// add subscribed folders

		addSubscribedFolders(lsubList);

		// remove unsubscribed folders

		removeUnsubscribedFolders(this, lsubList);

		if (worker != null)
			worker.stopTimer();

	}

	/**
	 * 
	 *  this method is called by the subscribe/unsubscribe dialog
	 * 
	 * 
	 **/
	public Vector getLSubList() throws IOException
	{
		if (lsubList != null)
			return lsubList;

		boolean answer;
		lsubList = new Vector();

		try
		{

			answer = imap.flsub("*");
			//System.out.println("trying to parse lsub");

			lsubList = Imap4Parser.parseLsub(imap.getResult());

		}
		catch (IOException ex)
		{
			//System.out.println("imapfolder->lsub: "+ ex.getMessage() );
			throw new IOException("ImapRootFolder->getLSubList: " + ex.getMessage());
		}

		return lsubList;
	}

	public boolean subscribe(String path) throws IOException
	{
		boolean answer = false;

		try
		{
			answer = imap.subscribe(path);
		}
		catch (IOException ex)
		{
			//System.out.println("imapfolder->lsub: "+ ex.getMessage() );
			throw new IOException("ImapRootFolder->subscribe: " + ex.getMessage());
		}

		return answer;
	}

	public boolean unsubscribe(String path) throws IOException
	{
		boolean answer = false;
		try
		{
			answer = imap.unsubscribe(path);
		}
		catch (IOException ex)
		{
			//System.out.println("imapfolder->lsub: "+ ex.getMessage() );
			throw new IOException("ImapRootFolder->unsubscribe: " + ex.getMessage());
		}

		return answer;
	}

	/**
	 * 
	 *  this method is called by the subscribe/unsubscribe dialog
	 * 
	 * 
	 **/
	public Vector getList(SubscribeTreeNode treeNode) throws IOException
	{
		StringBuffer buf = new StringBuffer(treeNode.getName());
		SubscribeTreeNode node = (SubscribeTreeNode) treeNode.getParent();
		while (node != null)
		{
			if (node.getName().equals("root"))
				break;

			buf.insert(0, node.getName() + "/");
			node = (SubscribeTreeNode) node.getParent();
		}

		String name = buf.toString();

		Vector v = new Vector();

		boolean answer;

		try
		{

			answer = imap.flist(name.trim() + "/%", "");
			//System.out.println("trying to parse list");
			v = Imap4Parser.parseList(imap.getResult());

		}
		catch (IOException ex)
		{
			//System.out.println("imapfolder->lsub: "+ ex.getMessage() );
			throw new IOException("ImapRootFolder->getList: " + ex.getMessage());
		}

		return v;
	}

	/**
	 * 
	 *  this method is called by the subscribe/unsubscribe dialog
	 * 
	 * 
	 **/
	public Vector getList() throws Exception
	{
		Vector v = new Vector();

		boolean answer;

		isLogin(null);

		answer = imap.flist("*", "");
		// System.out.println("trying to parse list");
		v = Imap4Parser.parseList(imap.getResult());

		return v;
	}

	public void login(SwingWorker worker) throws Exception
	{

		if (worker != null)
		{
			worker.setText(" Login " + item.getHost());
			worker.startTimer();
		}

		PasswordDialog dialog = null;

		boolean answer = false;
		boolean cancel = false;
		boolean first = true;

		boolean openport = false;

		try
		{
			openport = imap.openPort(item.getHost());
		}
		catch (Exception e)
		{
			if (e instanceof SocketException)
				throw new ImapException(e.getMessage());
			e.printStackTrace();
		}

		if (openport == true)
		{
			while (cancel == false)
			{
				if (first == true)
				{

					if (item.getPassword().length() != 0)
					{
						answer = imap.login(item.getUser(), item.getPassword());
						break;
					}

					first = false;
				}

				Vector loginMethods = new Vector();
				loginMethods.add(new String("LOGIN"));

				dialog = new PasswordDialog();
				dialog.showDialog(
					item.getHost(),
					item.getUser(),
					item.getPassword(),
					loginMethods,
					new String("LOGIN"),
					item.isSavePassword());

				char[] name;

				if (dialog.success() == true)
				{
					// ok pressed
					name = dialog.getPassword();
					String password = new String(name);
					String user = dialog.getUser();
					boolean save = dialog.getSave();

					answer = imap.login(user, password);

					if (answer == true)
					{
						cancel = true;

						item.setUser(user);

						item.setSavePassword(save);

						if (save == true)
							item.setPassword(password);

					}

					else
						cancel = false;
				}
				else
				{
					cancel = true;
					answer = false;
					// cancel pressed
				}
			}

		}
		else
		{
			answer = false;
		}

	}

	public boolean addFolder(String path) throws Exception
	{
		isLogin(null);

		boolean b = imap.create(path);

		if (b == false)
		{
			throw new ImapException(imap.getResult());
		}

		return b;
	}

	public void setState(int i)
	{
		getImapServerConnection().setState(i);
	}

	public int getState()
	{
		return getImapServerConnection().getState();
	}

	public String getSelectedFolderPath()
	{
		return getImapServerConnection().getSelectedFolderPath();
	}

}