// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.folder.imap;

import org.columba.gui.statusbar.*;

import org.columba.message.*;

import org.columba.folder.*;
import org.columba.config.*;
import org.columba.gui.message.*;
import org.columba.gui.attachment.*;
import org.columba.gui.header.*;
import org.columba.gui.util.*;

import java.util.*;
import java.lang.reflect.*;

public class ImapWorkerFolder extends Worker {
	Message message;
	FolderOperation folderOp;
	ImapOperator imapRoot;

	public Object construct() {

		if (op.type == Operation.FOLDER) {
			folderOp = (FolderOperation) op;
			imapRoot = (ImapOperator) root;
			channel = folderOp.channel;
		}

		//System.out.println( "FolderOp : "+op.mode );

		switch (op.mode) {
			case (Operation.FOLDER_APPLYFILTER) :
				{

					try {
						folderOp.source.applyFilter(folderOp.uidList);
					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						setCancel(true);

						e.printStackTrace();
					}

					break;
				}
			case (Operation.FOLDER_ADD) :
				{

					try {
						FolderItem item = folderOp.source.getFolderItem();

						if (item.getType().equals("imap")) {
							// imap folder
							ImapFolder imapFolder = (ImapFolder) folderOp.source;
							String path = imapFolder.getPath();

							String newPath = path + "/" + folderOp.name;

							boolean result = imapFolder.addFolder(newPath);

							if (result == true) {
								boolean result2 = imapFolder.getImapRootFolder().subscribe(newPath);

								if (result2 == true)
									if( waitForWrite() ) folderOp.channel.write(newPath);
								else
									setCancel(true);
							} else
								setCancel(true);

						} else {
							// imap root folder

							ImapRootFolder parent = (ImapRootFolder) folderOp.source;

							String newPath = folderOp.name;

							boolean result = parent.addFolder(newPath);

							if (result == true) {
								boolean result2 = parent.subscribe(newPath);
								if (result2 == true)
									if( waitForWrite() ) folderOp.channel.write(newPath);
								else
									setCancel(true);
							} else
								setCancel(true);

						}

					} catch (Exception ex) {
						if (ex instanceof ImapException) {
							NotifyDialog dialog = new NotifyDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						} else {
							ExceptionDialog dialog = new ExceptionDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						}

						
						setCancel(true);
												
						
					}

					
					break;
				}
			case (Operation.FOLDER_REMOVE) :
				{
					boolean result = false;
					
					try {
						
							result = folderOp.source.removeFolder();
					} catch (Exception ex) {
						if (ex instanceof ImapException) {
							NotifyDialog dialog = new NotifyDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						} else {
							ExceptionDialog dialog = new ExceptionDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						}

						
						setCancel(true);
						break;
					}
					
					if( !result ) {
						setCancel( true );
						break;
					}

					if( waitForWrite() ) folderOp.channel.write("FOLDER_REMOVE");

					break;
				}
			case (Operation.FOLDER_RENAME) :
				{
					String name = folderOp.name;

					try {
						folderOp.source.renameFolder(name);
					} catch (Exception ex) {
						if (ex instanceof ImapException) {
							NotifyDialog dialog = new NotifyDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						} else {
							ExceptionDialog dialog = new ExceptionDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						}

						setCancel(true);
					}

					if( waitForWrite() ) folderOp.channel.write(name);

					break;
				}
			case (Operation.MARK_READ) :
				{
					int opLength = Array.getLength(folderOp.uidList);
					ImapFolder folder = (ImapFolder) folderOp.source;
					try {
						for (int i = 0; i < opLength; i++) {

							folder.workerMark(folderOp.uidList[i], FolderOp.READ);
							/*
							FolderEvent event = new FolderEvent( this, folderOp.uidList[i], FolderEvent.MARK, Operation.MARK_READ  );
							imapRoot.mainInterface.crossbar.fireFolderChanged( event, folderOp.source );
							*/
						}

					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						e.printStackTrace();

						setCancel(true);

						break;

					}

					if( waitForWrite() ) folderOp.channel.write(folderOp.uidList);

					break;
				}
			case (Operation.MARK_FLAGGED) :
				{
					int opLength = Array.getLength(folderOp.uidList);
					ImapFolder folder = (ImapFolder) folderOp.source;
					try {
						for (int i = 0; i < opLength; i++) {
							folder.workerMark(folderOp.uidList[i], FolderOp.FLAGGED);
							//FolderEvent event = new FolderEvent( this, folderOp.uidList[i], FolderEvent.MARK, Operation.MARK_FLAGGED  );
							//imapRoot.mainInterface.crossbar.fireFolderChanged( event, folderOp.source );
						}
						//folderOp.source.renameFolder( name );
					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						e.printStackTrace();

						setCancel(true);

						break;

					}

					if( waitForWrite() ) folderOp.channel.write(folderOp.uidList);

					break;
				}
			case (Operation.MARK_EXPUNGED) :
				{
					int opLength = Array.getLength(folderOp.uidList);
					ImapFolder folder = (ImapFolder) folderOp.source;
					try {
						for (int i = 0; i < opLength; i++) {
							folder.workerMark(folderOp.uidList[i], FolderOp.EXPUNGED);
							//FolderEvent event = new FolderEvent( this, folderOp.uidList[i], FolderEvent.MARK, Operation.MARK_EXPUNGED  );
							//imapRoot.mainInterface.crossbar.fireFolderChanged( event, folderOp.source );
						}
						//folderOp.source.renameFolder( name );
					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						e.printStackTrace();

						setCancel(true);

						break;

					}

					if( waitForWrite() ) folderOp.channel.write(folderOp.uidList);

					break;
				}
			case (Operation.MARK_ANSWERED) :
				{
					int opLength = Array.getLength(folderOp.uidList);
					ImapFolder folder = (ImapFolder) folderOp.source;
					try {
						for (int i = 0; i < opLength; i++) {
							folder.workerMark(folderOp.uidList[i], FolderOp.ANSWERED);
							//FolderEvent event = new FolderEvent( this, folderOp.uidList[i], FolderEvent.MARK, Operation.MARK_ANSWERED  );
							//imapRoot.mainInterface.crossbar.fireFolderChanged( event, folderOp.source );
						}
						//folderOp.source.renameFolder( name );
					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						e.printStackTrace();

						setCancel(true);

						break;
					}

					if( waitForWrite() ) folderOp.channel.write(folderOp.uidList);

					break;
				}
			case Operation.FOLDER_LIST :
				{
					ImapRootFolder parent = (ImapRootFolder) folderOp.source;
					try {
						parent.lsub(this);
					} catch (Exception ex) {
						if (ex instanceof ImapException) {
							NotifyDialog dialog = new NotifyDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						} else {
							ExceptionDialog dialog = new ExceptionDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						}

						setCancel(true);

					}

					if( waitForWrite() ) folderOp.channel.write("FOLDER_LIST");

					break;
				}
				case Operation.FOLDER_REFRESHLIST :
				{
					ImapRootFolder parent = (ImapRootFolder) folderOp.source;
					Vector nodeList=null;
					Vector lsubList=null;
					try {
						parent.clearLSubList();

        				nodeList = parent.getList();
        				lsubList = parent.getLSubList();
						
					} catch (Exception ex) {
						if (ex instanceof ImapException) {
							NotifyDialog dialog = new NotifyDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						} else {
							ExceptionDialog dialog = new ExceptionDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						}

						
						setCancel(true);

						break;
					}

					
					
					folderOp.dialog.setList( nodeList, lsubList );

					break;
				}
				case Operation.FOLDER_SUBSCRIBE :
				{
					ImapRootFolder parent = (ImapRootFolder) folderOp.source;
					
					try {
						parent.subscribe( folderOp.node.getDirectoryPath() );

        				
						
					} catch (Exception ex) {
						if (ex instanceof ImapException) {
							NotifyDialog dialog = new NotifyDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						} else {
							ExceptionDialog dialog = new ExceptionDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						}

						
						setCancel(true);

						break;
					}

					
					folderOp.node.setSubscribeState( true );
            		folderOp.dialog.repaintTable();
					

					break;
				}
				case Operation.FOLDER_UNSUBSCRIBE :
				{
					ImapRootFolder parent = (ImapRootFolder) folderOp.source;
					
					try {
						parent.unsubscribe( folderOp.node.getDirectoryPath() );

        				
						
					} catch (Exception ex) {
						if (ex instanceof ImapException) {
							NotifyDialog dialog = new NotifyDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						} else {
							ExceptionDialog dialog = new ExceptionDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						}

						
						setCancel(true);

						break;
					}

					
					folderOp.node.setSubscribeState( false );
            		folderOp.dialog.repaintTable();
					

					break;
				}
				case Operation.FOLDER_SUBLIST :
				{
					ImapRootFolder parent = (ImapRootFolder) folderOp.source;
					Vector nodeList=null;
					Vector lsubList=null;
					try {
						parent.clearLSubList();

        				nodeList = parent.getList( folderOp.node );
        				lsubList = parent.getLSubList();
						
					} catch (Exception ex) {
						if (ex instanceof ImapException) {
							NotifyDialog dialog = new NotifyDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						} else {
							ExceptionDialog dialog = new ExceptionDialog(imapRoot.mainInterface);
							dialog.showDialog(ex);
						}

						
						setCancel(true);

						break;
					}

					
					folderOp.dialog.addTreeNodes(folderOp.node, nodeList, lsubList);
					

					break;
				}
		}

		unregister();

		return null;
	}

}