// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.folder.imap;

import org.columba.gui.statusbar.*;

import org.columba.message.*;

import org.columba.folder.*;

import org.columba.gui.message.*;
import org.columba.gui.attachment.*;
import org.columba.gui.header.*;
import org.columba.gui.util.*;

import java.util.*;

public class ImapWorkerGui extends Worker {
	Message message;
	FolderOperation folderOp;
	ImapOperator imapRoot;

	public Object construct() {

		if (op.type == Operation.FOLDER) {
			folderOp = (FolderOperation) op;
			imapRoot = (ImapOperator) root;
			channel = folderOp.channel;
		}

		//System.out.println( "FolderOp : "+op.mode );

		switch (op.mode) {
			case (Operation.MESSAGEBODY) :
				{

					setText("Fetching message body...");

					MimePart bodyPart = null;

					try {
						message = folderOp.source.uidGet(folderOp.uid);
						MimePartCollection mimeCollection =
							folderOp.source.uidgetMimePartCollection(folderOp.uid);

						for (int i = 0; i < mimeCollection.count(); i++) {
							if (mimeCollection.get(i).getHeader().contentType.equals("text")) {
								bodyPart = folderOp.source.uidgetMimePart(folderOp.uid, i, this);

								break;
							}
						}

						message.setBodyPart(bodyPart);

						//message.setBodyPart( folderOp.source.uidgetBodyPart( folderOp.uid, this ) );
					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						e.printStackTrace();

						setCancel(true);

						break;

					}
					
					if( waitForWrite() ) folderOp.channel.write(message);

					break;
				}
			case (Operation.MESSAGESOURCE) :
				{

					setText("Fetching message source...");

					String rawText = null;
					try {
						rawText = folderOp.source.uidToString(folderOp.uid, this);

					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						e.printStackTrace();

						setCancel(true);

						break;

					}
					
					if( waitForWrite() ) folderOp.channel.write(rawText);

					break;
				}
			case (Operation.ATTACHMENTLIST) :
				{

					MimePartCollection collection = null;
					try {
						collection = folderOp.source.uidgetMimePartCollection(folderOp.uid);
					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						e.printStackTrace();

						setCancel(true);

						break;

					}

					if( waitForWrite() ) folderOp.channel.write(collection);

					break;
				}
			case (Operation.HEADERLIST) :
				{

					HeaderList headerList = null;

					try {
						headerList = folderOp.source.getHeaderList(this);
					} catch (Exception e) {
						if (e instanceof ImapException) {
							NotifyDialog dialog = new NotifyDialog(imapRoot.mainInterface);
							dialog.showDialog(e);
						} else {
							ExceptionDialog dialog = new ExceptionDialog(imapRoot.mainInterface);
							dialog.showDialog(e);

						}
						
						setCancel( true );
						
						break;						
					}

					if ( waitForWrite() ) {
						folderOp.channel.write(headerList);

						TreeNodeEvent updateEvent2 =
							new TreeNodeEvent(folderOp.source, TreeNodeEvent.UPDATE);
						root.mainInterface.crossbar.fireTreeNodeChanged(updateEvent2);
					}
					
					break;
				}
			case (Operation.ATTACHMENT) :
				{

					setText("Fetching attachment...");

					MimePart part = null;
					try {
						part = folderOp.source.uidgetMimePart(folderOp.uid, folderOp.part, this);

					} catch (Exception e) {
						ExceptionDialog dialog = new ExceptionDialog(root.mainInterface);
						dialog.showDialog(e);

						e.printStackTrace();

						setCancel(true);

						break;

					}
					if( waitForWrite() ) folderOp.channel.write(part);

					break;
				}
		}

		unregister();

		return null;
	}

}