// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.folder.outbox;

import org.columba.config.*;
import org.columba.protocol.*;
import org.columba.message.*;
import org.columba.main.*;
import org.columba.gui.tree.util.*;
import org.columba.composer.*;
import org.columba.gui.util.PasswordDialog;
import org.columba.util.*;
import org.columba.parser.*;

import org.columba.folder.*;
import org.columba.folder.columba.*;

import java.io.*;
import java.util.*;
import javax.swing.JOptionPane;

public class OutboxFolder extends ColumbaFolder
{

	private SendListManager[] sendListManager = new SendListManager[2];
	private int actSender;
	private boolean isSending;

	public OutboxFolder(
		AdapterNode node,
		FolderItem item,
		MainInterface mainInterface)
	{
		super(node, item, mainInterface);

		sendListManager[0] = new SendListManager();
		sendListManager[1] = new SendListManager();
		actSender = 0;

		isSending = false;
		
		if (headerFile.exists())
		{
			try
			{
				load(null);
			}
			catch (Exception ex)
			{
				System.out.println("exception: " + ex.getMessage());
				ex.printStackTrace();

				System.out.println("recreating index file");
				recreateIndex();
			}
		}
		else
		{
			recreateIndex();

		}
		
		alreadyLoaded = true;
	}

	private void swapListManagers() throws Exception
	{
		// copy lost Messages
		System.out.println(
			"Sizes : "
				+ sendListManager[actSender].count()
				+ " - "
				+ sendListManager[1
				- actSender].count());

		while (sendListManager[actSender].hasMoreMessages())
		{
			sendListManager[1
				- actSender].add(
					(SendableMessage) uidGet(sendListManager[actSender].getNextUid()));
		}

		// swap
		actSender = 1 - actSender;

		System.out.println(
			"Sizes : "
				+ sendListManager[actSender].count()
				+ " - "
				+ sendListManager[1
				- actSender].count());

	}

	/*
	public synchronized void sendAll() throws Exception
	{
	    System.out.println("Flushing Outbox");
	
	    isSending = true;
	
	    swapListManagers();
	
	    System.out.println("Flushing Outbox");
	
	    sendMessages();
	}
	*/

	public void stoppedSending()
	{
		isSending = false;
	}

	/*
	public void sendMessages()
	{
	    System.out.println("Sending all Messages");
	
	
	    SwingWorker sendWorker;
	    final OutboxFolder outFolder = this;
	
	    sendWorker = new SwingWorker()
	        {
	            int accountUid = -1;
	            boolean open = false;
	
	            ESmtp smtp;
	
	            AccountItem account;
	            IdentityItem identity;
	            SmtpItem smtpItem;
	            Folder sentFolder;
	            String fromAddress;
	            String host;
	
	            private void setAccount( int uid )
	            {
	                System.out.println( "AccountUID = "+uid );
	
	                if( accountUid == uid ) return;
	
	                if( open ) closeConnection();
	
	                open = openConnection( uid );
	
	                accountUid = uid;
	            }
	
	            private boolean openConnection( int accountUid )
	            {
	                String username;
	                String password;
	
	                int smtpMode;
	                boolean authenticate;
	                boolean cont = false;
	
	                PasswordDialog passDialog =
	                    new PasswordDialog();
	
	
	                  // Init Values
	
	                account = mainInterface.config.getAccountList()
	                    .uidGet( accountUid );
	
	                identity = account.getIdentityItem();
	                fromAddress = identity.getAddress();
	
	                smtpItem = account.getSmtpItem();
	                host = smtpItem.getHost();
	
	                SpecialFoldersItem specialFoldersItem = account.getSpecialFoldersItem();
	                Integer i = new Integer( specialFoldersItem.getSent() );
	                sentFolder = mainInterface.treeViewer.getFolderTree().getFolder( i.intValue() );
	
	                authenticate = account.getSmtpItem().getESmtp().equals("true");
	
	                  // Start login procedure
	
	                smtp = new ESmtp( host, smtpItem.getPort() );
	
	                setText("Opening Port to "+host);
	
	                try {
	                    smtpMode = smtp.openPort();
	                }
	                catch( Exception e ) {
	
	                    JOptionPane.showMessageDialog( null, e,
	                                                   "Error while Sending",
	                                                   JOptionPane.ERROR_MESSAGE );
	
	                    return false;
	                }
	
	                if( (smtpMode == ESmtp.ESMTP) & authenticate ) {
	
	                    setText("Authenticating");
	
	                    username = account.getSmtpItem().getUser();
	                    password = account.getSmtpItem().getPassword();
	
	
	                    if ( ( username.length() == 0 ) || ( password.length() == 0 ) )
	                    {
	                        System.out.println("username or password not specificed");
	
	                        passDialog.showDialog( host, username,
	                                               password, account.getSmtpItem().isSavePassword() );
	
	                        if( passDialog.success() )
	                        {
	                            System.out.println("username or password set");
	                            username = passDialog.getUser();
	                            password = new String( passDialog.getPassword() );
	
	                        }
	                        else
	                        {
	                            return false;
	                        }
	                    }
	
	                    while( !cont )
	                    {
	                        if ( getCancel() == true ) break;
	
	                        cont = true;
	
	
	                        try {
	                            smtp.authenticate( username, password );
	                        }
	                        catch( Exception e )
	                        {
	                            cont = false;
	                            passDialog.showDialog( host, username,
	                                                   password, account.getSmtpItem().isSavePassword() );
	
	                            if( !passDialog.success() ) return false;
	                            else
	                            {
	                                username = passDialog.getUser();
	                                password = new String( passDialog.getPassword() );
	                            }
	
	                        }
	                    }
	
	                    account.getSmtpItem().setUser( username );
	                    Boolean bool = new Boolean( passDialog.getSave() );
	                    account.getSmtpItem().setSavePassword( bool.toString() );
	
	                    if ( passDialog.getSave() == true )
	                        account.getSmtpItem().setPassword( password );
	                }
	
	                return true;
	
	            }
	
	            private void closeConnection()
	            {
	                  // Close Port
	
	                setText("Closing Connection to "+host);
	
	                try {
	                    smtp.closePort();
	                        }
	                catch( Exception e ) {
	                    JOptionPane.showMessageDialog( null, e,
	                                                   "Error while Sending",
	                                                   JOptionPane.ERROR_MESSAGE );
	
	                }
	
	            }
	
	
	
	            public Object construct()
	            {
	                SendableMessage message;
	                Object[] uids = new Object[1];
	
	                int index;
	
	                try
	                {
	                    getHeaderList( this );
	                }
	                catch ( Exception ex )
	                {
	                    System.out.println("Error while loading Outbox Folder: "+ ex.getMessage() );
	                }
	
	
	                System.out.println( sendListManager[actSender].count() );
	
	                while( sendListManager[actSender].hasMoreMessages() ) {
	
	                message=null;
	                try
	                {
	                    message = (SendableMessage) outFolder.uidGet(
	                        sendListManager[actSender].getNextUid() );
	                }
	                catch ( Exception ex )
	                {
	                    ex.printStackTrace();
	                }
	
	                      // Is Message existing
	
	                    if( message == null ) continue;
	
	                    System.out.println( "Message UID = "+message.getUID() );
	
	                    setAccount( message.getAccountUid() );
	
	                      // Is Connection open
	
	                    if( !open )
	                    {
	                        sendListManager[1-actSender].add( message );
	                        continue;
	                    }
	
	
	                      // Connection is open
	
	                    if ( getCancel() == false )
	                    {
	                        setText("Sending Message");
	
	
	                        try {
	                            smtp.setupMessage( fromAddress , message.getRecipients() );
				
	
	                            smtp.sendMessage( message.getRawString());
	                        }
	                        catch( Exception e ) {
	                            JOptionPane.showMessageDialog( null, e,
	                                                           "Error while Sending",
	                                                           JOptionPane.ERROR_MESSAGE );
	                            sendListManager[1-actSender].add( message );
	                            continue;
	                        }
	
	
	
	                        uids[0] = message.getUID();
	
	                        System.out.println("--------------------------->Moving Message "+uids[0]);
	
			    // OLD CLASS
			    
	                        //mainInterface.folderOperator.operate(
	                        //    new FolderOp( FolderOp.MOVE, outFolder, sentFolder, uids ) );
			    
	
	                    }
	                }
	
	
	                unregister();
	                return null;
	            }
	
	            public void finished()
	            {
	                outFolder.stoppedSending();
	            }
	        };
	
	    sendWorker.register( mainInterface.taskManager );
	    sendWorker.start();
	
	}
	*/
	
	
	

	// saves Message-Headers in separat File
	public void save() throws Exception
	{
		
		

		//System.out.println("saving headerfile: "+ headerFile.toString() );
		HeaderTableItem v =
			mainInterface.config.getMainFrameOptionsConfig().getHeaderTableItem();

		FileOutputStream istream = new FileOutputStream(headerFile.getPath());
		ObjectOutputStream p = new ObjectOutputStream(istream);

		p.writeInt(count());

		Rfc822Header h;

		for (int i = 0; i < count(); i++)
		{
			p.writeInt(i + 1);
			h = get(i).getHeader();

			p.writeObject(get(i).getUID());

			//p.writeObject( get(i).getHeader().get("Status") );
			//p.writeObject( get(i).getFlags() );

			Flags flags = get(i).getFlags();

			p.writeBoolean(flags.getSeen());
			p.writeBoolean(flags.getAnswered());
			p.writeBoolean(flags.getFlagged());
			p.writeBoolean(flags.getDeleted());
			p.writeBoolean(flags.getDraft());
			p.writeBoolean(flags.getRecent());

			p.writeObject(h.get("columba.date"));

			p.writeObject(h.get("columba.size"));

			p.writeObject(h.get("columba.from"));

			p.writeObject(h.get("columba.attachment"));

			p.writeObject(h.get("columba.priority"));

			p.writeObject(h.get("columba.host"));

			// Additionals for SendableMessage

			p.writeInt(((SendableMessage) get(i)).getAccountUid());

			p.writeObject(((SendableMessage) get(i)).getRecipients());

			String column;
			HeaderItem item;
			Object o;
			for (int j = 0; j < v.count(); j++)
			{
				column = (String) v.getName(j);
				o = get(i).getHeader().get(column);
				p.writeObject(o);
			}
		}

		p.flush();
		p.close();

		/*
		  }
		  catch (IOException ex)
		  {
		  System.out.println("error while writing to file: "+ex.getMessage() );
		
		  }
		*/

	}

	// load Message-Headers
	public void load(SwingWorker worker) throws Exception
	{

		FileInputStream istream = new FileInputStream(headerFile.getPath());
		ObjectInputStream p = new ObjectInputStream(istream);

		int capacity = p.readInt();
		Integer uid;

		
		
		System.out.println("file capacity:" + capacity);

		//int capacity = getMessageFileCount();

		if (capacity != getMessageFileCount())
		{
			// messagebox headercache-file is corrupted
			System.out.println("Messagebox headercache-file is corrupted!");
			recreateIndex();
			return;
		}
		
		for (int i = 1; i <= capacity; i++)
		{
			SendableMessage message = new SendableMessage();
			Rfc822Header h = message.getHeader();

			// read current number of message
			p.readInt();

			uid = (Integer) p.readObject();
			message.setUID(uid);

			//Integer status = (Integer) p.readObject();
			//message.getHeader().set( "Status", status );
			//message.setFlags( (Flags) p.readObject() );
			Flags flags = message.getFlags();

			flags.setSeen(p.readBoolean());
			flags.setAnswered(p.readBoolean());
			flags.setFlagged(p.readBoolean());
			flags.setDeleted(p.readBoolean());
			flags.setDraft(p.readBoolean());
			flags.setRecent(p.readBoolean());
			//message.setFlags( flags );

			Date date = (Date) p.readObject();
			h.set("columba.date", date);

			h.set("columba.size", p.readObject());

			String from = (String) p.readObject();
			h.set("columba.from", from);

			Boolean b = (Boolean) p.readObject();
			h.set("columba.attachment", b);

			//int priority = p.readInt();
			h.set("columba.priority", (Integer) p.readObject());

			String host = (String) p.readObject();
			h.set("columba.host", host);

			/*
			Date date = (Date) p.readObject();
			message.setDate( date );
			
			int size = p.readInt();
			message.setSize( size );
			
			message.setShortFrom( (String) p.readObject() );
			
			Boolean b = (Boolean) p.readObject();
			message.setAttachment( b.booleanValue() );
			
			int priority = p.readInt();
			message.setPriority( priority );
			
			String host = (String) p.readObject();
			message.setHost( host );
			*/

			// Additionals for SendableMessage

			int accountUid = p.readInt();
			message.setAccountUid(accountUid);

			Vector recipients = (Vector) p.readObject();
			message.setRecipients(recipients);

			HeaderTableItem v =
				mainInterface.config.getMainFrameOptionsConfig().getHeaderTableItem();
			String column;
			Object o;
			for (int j = 0; j < v.count(); j++)
			{
				column = (String) v.getName(j);
				o = p.readObject();

				if (o == null)
				{
				}
				else if (o instanceof String)
				{
					String value = (String) o;
					h.set(column, value);
				}
				else if (o instanceof Integer)
				{
					Integer value = (Integer) o;
					h.set(column, value);
				}
				else if (o instanceof Boolean)
				{
					Boolean value = (Boolean) o;
					h.set(column, value);
				}
				else if (o instanceof Date)
				{
					Date value = (Date) o;
					h.set(column, value);
				}

			}

			super.add(message, null, uid);
			sendListManager[actSender].add(message);

			if (nextUid <= uid.intValue())
			{
				nextUid = uid.intValue() + 1;
			}

		}

		// close stream
		p.close();

		/*
		  catch (Exception ex)
		  {
		  System.out.println("loadMessageHeaders: error while trying to load Headers:  "+ex.getMessage() );
		  ex.printStackTrace();
		
		  }
		*/

	}

}