// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.folder.outbox;

import org.columba.gui.statusbar.*;
import org.columba.message.*;
import org.columba.folder.*;
import org.columba.protocol.*;
import org.columba.config.*;
import org.columba.composer.*;
import org.columba.gui.util.*;
import javax.swing.JOptionPane;

import java.util.*;
import java.lang.reflect.Array;
import java.net.*;

public class OutboxWorkerIntern extends Worker
{
	SendListManager sendListManager = new SendListManager();
	OutboxOperator outboxRoot;
	AccountItem account;
	IdentityItem identity;
	SmtpItem smtpItem;
	int sentFolder;
	String fromAddress;
	String host;
	ESmtp smtp;

	OutboxOperation smtpOp;

	public Object construct()
	{

		Vector sentList = new Vector();

		outboxRoot = (OutboxOperator) root;
		smtpOp = (OutboxOperation) op;

		int opLength = 0;

		System.out.println("OutboxWorkerIntern started...");

		if (smtpOp.uidList != null)
		{
			opLength = Array.getLength(smtpOp.uidList);
			setProgressBarMaximum(opLength + 1);
		}

		setText("Sending Messages");

		setProgressBarValue(0);

		switch (smtpOp.mode)
		{
			/* Send - Operation */

			case (Operation.SMTP_SENDALL) :
				{
					smtpOp.uidList = outboxRoot.outboxFolder.getUids();
					smtpOp.mode = Operation.SMTP_SEND;
					opLength = Array.getLength(smtpOp.uidList);
					setProgressBarMaximum(opLength + 1);
				}

			case (Operation.SMTP_SEND) :
				{

					int actAccountUid = -1;
					boolean open = false;

					for (int i = 0; i < opLength; i++)
					{

						try
						{

							/* Sort Messages with AccountID */

							if (outboxRoot.outboxFolder.exists(smtpOp.uidList[i]) == true)
							{
								SendableMessage message =
									(SendableMessage) outboxRoot.outboxFolder.uidGetParsedMessage(
										smtpOp.uidList[i]);
								sendListManager.add(message);
							}
							setProgressBarValue(1);

							/* For every Account send messages */

							while (sendListManager.hasMoreMessages())
							{
								SendableMessage message = sendListManager.getNextMessage();

								/* Seperate the Accounts */

								if (message.getAccountUid() != actAccountUid)
								{
									/* Move sent messages in sentFolders */
									if (sentList.size() != 0)
									{
										outboxRoot.moveSent(sentList.toArray(), sentFolder);
										sentList.clear();
									}

									actAccountUid = message.getAccountUid();

									/* Try Opening Connection */
									open = openConnection(actAccountUid);
								}

								/* Is connection to smtpserver of actual account open ? */

								if (open)
								{
									/* Send Message */

									try
									{
										smtp.setupMessage(fromAddress, message.getRecipients());

										smtp.sendMessage(message.getRawString());

										sentList.add(message.getUID());
									}
									catch (Exception e)
									{
										e.printStackTrace();
										NotifyDialog dialog = new NotifyDialog(root.mainInterface);
										dialog.showDialog(e);
										/*
										JOptionPane.showMessageDialog(
											null,
											e,
											"Error while Sending",
											JOptionPane.ERROR_MESSAGE);
										*/
									}
								}

								setProgressBarValue(i + 2);
							}

						}
						catch (Exception ex)
						{
							if (ex instanceof UnknownHostException)
							{
								Exception ex1 =
									new Exception("Unknown host: " + ex.getMessage() + "\nAre you  online?");
								NotifyDialog dialog = new NotifyDialog(root.mainInterface);
								dialog.showDialog(ex1);

							}
							else
							{
								NotifyDialog dialog = new NotifyDialog(root.mainInterface);
								dialog.showDialog(ex);
							}
							ex.printStackTrace();
						}
					}
				}
		}

		if (sentList.size() > 0)
		{
			outboxRoot.moveSent(sentList.toArray(), sentFolder);
		}

		unregister();

		return null;
	}

	private boolean openConnection(int accountUid)
	{
		String username;
		String password;
		String method;
		
		int smtpMode;
		boolean authenticate;
		boolean cont = false;

		PasswordDialog passDialog = new PasswordDialog();

		// Init Values

		account = outboxRoot.getAccountItem(accountUid);

		identity = account.getIdentityItem();
		fromAddress = identity.getAddress();

		smtpItem = account.getSmtpItem();
		host = smtpItem.getHost();

		SpecialFoldersItem specialFoldersItem = account.getSpecialFoldersItem();
		Integer i = new Integer(specialFoldersItem.getSent());
		sentFolder = i.intValue();

		authenticate = account.getSmtpItem().getESmtp().equals("true");

		if (authenticate == false)
		{
			boolean popbeforesmtp = account.getSmtpItem().getPopBeforeSmtp();

			if (popbeforesmtp == true)
			{
				// no esmtp - use POP3-before-SMTP instead
				try
				{
					pop3Authentification();
				}
				catch (Exception e)
				{
					if (e instanceof UnknownHostException)
					{
						Exception ex =
							new Exception("Unknown host: " + e.getMessage() + "\nAre you  online?");
						NotifyDialog dialog = new NotifyDialog(root.mainInterface);
						dialog.showDialog(ex);

					}
					else
					{
						NotifyDialog dialog = new NotifyDialog(root.mainInterface);
						dialog.showDialog(e);
					}

					return false;
				}
			}

		}

		// find host
		setText("Opening Port to " + host);
		try
		{
			smtp = new ESmtp(host, smtpItem.getPort());

		}
		catch (Exception e)
		{
			if (e instanceof UnknownHostException)
			{
				Exception ex =
					new Exception("Unknown host: " + e.getMessage() + "\nAre you  online?");

			}
			else
			{
				NotifyDialog dialog = new NotifyDialog(root.mainInterface);
				dialog.showDialog(e);
			}

			return false;
		}

		// Start login procedure

		try
		{

			smtpMode = smtp.openPort();
		}
		catch (Exception e)
		{
			NotifyDialog dialog = new NotifyDialog(root.mainInterface);
			dialog.showDialog(e);

			return false;
		}

		//if ((smtpMode == ESmtp.ESMTP) & authenticate)
		if (authenticate)
		{

			setText("Authenticating");

			username = account.getSmtpItem().getUser();
			password = account.getSmtpItem().getPassword();
			method = account.getSmtpItem().getLoginMethod();

			if ((username.length() == 0) || (password.length() == 0))
			{
				

				Vector loginMethods = new Vector();
				loginMethods.add(new String("PLAIN"));
				loginMethods.add(new String("LOGIN"));
				

				passDialog.showDialog(
					host,
					username,
					password,
					loginMethods,
					method,
					account.getSmtpItem().isSavePassword());

				if (passDialog.success())
				{
					
					username = passDialog.getUser();
					password = new String(passDialog.getPassword());
					method = passDialog.getLoginMethod();

				}
				else
				{
					return false;
				}
			}

			while (!cont)
			{
				if (getCancel() == true)
					break;

				cont = true;

				try
				{
					smtp.authenticate(username, password, method);
				}
				catch (Exception e)
				{
					cont = false;

					Vector loginMethods = new Vector();
					loginMethods.add(new String("PLAIN"));
					loginMethods.add(new String("LOGIN"));

					passDialog.showDialog(
						host,
						username,
						password,
						loginMethods,
						method,
						account.getSmtpItem().isSavePassword());

					if (!passDialog.success())
						return false;
					else
					{
						username = passDialog.getUser();
						password = new String(passDialog.getPassword());
						method = passDialog.getLoginMethod();
					}

				}
			}

			account.getSmtpItem().setUser(username);
			Boolean bool = new Boolean(passDialog.getSave());
			account.getSmtpItem().setSavePassword(bool.toString());
			account.getSmtpItem().setLoginMethod(method);

			if (passDialog.getSave() == true)
			{				
				account.getSmtpItem().setPassword(password);
			}
		}

		return true;

	}

	private void closeConnection()
	{
		// Close Port

		setText("Closing Connection to " + host);

		try
		{
			smtp.closePort();
		}
		catch (Exception e)
		{
			NotifyDialog dialog = new NotifyDialog(root.mainInterface);
			dialog.showDialog(e);

		}

	}

	private void pop3Authentification() throws Exception
	{
		String password = new String("");
		String user = new String("");
		String method = new String("");
		boolean save = false;
		boolean login = false;
		boolean cancel = false;
		PopItem item = account.getPopItem();
		PasswordDialog dialog = null;

		while ((login == false) && (cancel == false))
		{
			if (item.getPassword().length() == 0)
			{
				dialog = new PasswordDialog();

				Vector loginMethods = new Vector();
				loginMethods.add(new String("USER"));
				loginMethods.add(new String("APOP"));

				dialog.showDialog(
					item.getHost(),
					item.getUser(),
					item.getPassword(),
					loginMethods,
					item.getLoginMethod(),
					item.isSavePassword());

				char[] name;

				if (dialog.success() == true)
				{
					// ok pressed
					name = dialog.getPassword();
					password = new String(name);
					user = dialog.getUser();
					save = dialog.getSave();
					method = dialog.getLoginMethod();

					//System.out.println("pass:<"+password+">");
					cancel = false;
				}
				else
				{
					// cancel pressed
					cancel = true;
				}
			}
			else
			{
				password = item.getPassword();
				user = item.getUser();
				save = item.isSavePassword();
				method = item.getLoginMethod();
			}

			if (cancel == false)
			{
				//System.out.println("trying to login");
				setText(item.getHost() + " : Login...");

				startTimer();
				// authenticate

				Pop3 pop3Connection = new Pop3();
				// open socket, query for host
				pop3Connection.openPort(
					item.getHost(),
					(new Integer(item.getPort())).intValue());

				pop3Connection.setLoginMethod(method);
				login = pop3Connection.login(user, password);
				stopTimer();

				if (login == false)
				{
					NotifyDialog d = new NotifyDialog(root.mainInterface);
					d.showDialog("Authentification failed");
					if (dialog != null)
						item.setUser(dialog.getUser());
				}
				item.setPassword("");

			}

		}

		if (login)
		{
			item.setUser(user);
			item.setSavePassword(save);
			item.setLoginMethod(method);

			if (save)
			{

				// save plain text password in config file
				// this is a security risk !!!
				item.setPassword(password);
			}
		}
	}

}