// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.folder.temp;

import org.columba.folder.*;
import org.columba.message.*;
import org.columba.coder.*;
import org.columba.util.*;
import org.columba.config.*;

import java.io.*;
import org.columba.main.*;


public class TempFolder extends Folder
{
    private int nextUid;
    static private int nextFolderUid = -1;
    static private TempOperator tempOperator = null;

    public TempFolder( MainInterface mainInterface )
    {
        super( null, null );
        uid =  nextFolderUid-- ;
        nextUid = 0;
	if( tempOperator == null ) {
	    System.out.println( "TempOperator ready." );
	    tempOperator = new TempOperator( mainInterface  );
	    tempOperator.start();
	}
    }


    public Object add(Message message, String s) throws Exception
    {
        return super.add(message, s,  calculateNewUid() );
    }


    protected Object calculateNewUid()
    {
        return new Integer( nextUid++ );
    }

      /*********** Worker Methods ******************/

    public void workerViewMark( Object[] uids, int variant )
    {}

    public void fireTreeNodeUpdate()
    {}

    public MimePart uidgetMimePart( Object uid, int number, SwingWorker worker ) throws Exception
    {
        Message aktMessage = uidGet( uid );

        return aktMessage.getMimePartCollection().get( number );
    }

    public MimePartCollection uidgetMimePartCollection( Object uid ) throws Exception
    {
        Message aktMessage = uidGet( uid );

        return aktMessage.getMimePartCollection();
    }

    public int uidgetMimePartCount( Object uid ) throws Exception
    {
        Message aktMessage = uidGet( uid );

        MimePartCollection mimeparts = aktMessage.getMimePartCollection();

        int count = mimeparts.count();

        if( count == 1 ) {
            MimeHeader mimeheader = mimeparts.get( 0 ).getHeader();
            if( mimeheader.getContentType().equals("text") &
                mimeheader.getContentSubtype().equals("plain") ) {
                count--;
            }
        }

        return count;
    }

    public String uidgetBodyText( Object uid, SwingWorker worker ) throws Exception
    {

        Message aktMessage = uidGet( uid );
        MimePartCollection aktCollection;
        MimePart aktPart;

        aktCollection = aktMessage.getMimePartCollection();

        for(int i=0; i < aktCollection.count(); i++) {
            aktPart = aktCollection.get(i);
            if( aktPart.getHeader().contentType.equals("text") ) {
                Decoder decoder = CoderRouter.getDecoder(
                    aktPart.getHeader().contentTransferEncoding);

                StringWriter decodedBody = new StringWriter();

                decoder.setupDecoder( new StringReader( aktPart.getBody() ), decodedBody );
                decoder.run();

                return decodedBody.toString();
            }
        }

	return new String();
    }

    public Operator getOperator()
    {
	return tempOperator;
    }

}
