// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.folder.virtual;

import org.columba.message.*;
import org.columba.filter.*;
import org.columba.util.*;
import org.columba.gui.tree.util.*;
import org.columba.gui.tree.*;
import org.columba.config.*;
import org.columba.main.*;

import org.columba.folder.*;

import java.io.*;
import java.util.Vector;
import java.lang.reflect.Array;


public class VirtualFolder extends Folder
{

    //private MainInterface mainInterface;

    private Search searchFilter;

    public VirtualFolder( AdapterNode node, FolderItem item )
    {
        super( node, item );
    }



    public HeaderList getHeaderList( SwingWorker worker ) throws Exception
    {

	// NOT YET IMPLEMENTED

        /*
        Folder searchFolder = mainInterface.treeViewer.getSelected();
        AdapterNode searchNode = searchFolder.getNode();


        Search search = new Search( searchNode.getChild("search"), this, mainInterface );
        search.applySearch();
        */

        searchFilter.applySearch();

        HeaderList list = new HeaderList();

        for( int i=0; i < count(); i++ ) {
	    list.add( get(i).getHeader(), getUid(i) );
	}

        return list;


    }

    public void setSearchFilter( Search s )
    {
        searchFilter = s;
    }

    public Search getSearchFilter()
    {
        return searchFilter;
    }

    public Message get( int i ) throws Exception
    {
        VirtualMessage vMessage = (VirtualMessage) super.get(i);

        Folder parentFolder = vMessage.getParentFolder();
        Object parentUid = vMessage.getParentUID();


        Message message = parentFolder.uidGet( parentUid );

        return message;
    }

    public VirtualMessage getVirtualMessage( int index ) throws Exception
    {
        VirtualMessage vMessage = (VirtualMessage) get(index);

        return vMessage;
    }

    public VirtualMessage uidGetVirtualMessage( Object uid ) throws Exception
    {
	VirtualMessage vMessage = (VirtualMessage) super.uidGet( uid );

	return vMessage;
    }

    public Message uidGet( Object uid ) throws Exception
    {
        VirtualMessage vMessage = (VirtualMessage) super.uidGet( uid );

        Folder parentFolder = vMessage.getParentFolder();
        Object parentUid = vMessage.getParentUID();

        Message message = parentFolder.uidGet( parentUid );

        return message;
    }

    public void add(Folder f, Object uid, int index) throws Exception
    {
        VirtualMessage m = new VirtualMessage( f, uid, index );

        super.add( m, null, uid  );

        //fireTreeNodeUpdate();
    }

    public String uidgetBodyText( Object uid , SwingWorker worker ) throws Exception
    {
        VirtualMessage vMessage = (VirtualMessage) super.uidGet( uid );

        Folder parentFolder = vMessage.getParentFolder();
        Object parentUid = vMessage.getParentUID();

        return parentFolder.uidgetBodyText( parentUid, worker );
    }


    public String uidToString( Object uid, SwingWorker worker ) throws Exception
    {
        VirtualMessage vMessage = (VirtualMessage) super.uidGet( uid );

        Folder parentFolder = vMessage.getParentFolder();
        Object parentUid = vMessage.getParentUID();

        return parentFolder.uidToString( parentUid, worker );
    }

    /*
    public String toString( int i )
    {
        VirtualMessage vMessage = (VirtualMessage) getMessageCollection().get(i);

        Folder parentFolder = vMessage.getParentFolder();
        int parentIndex = vMessage.getParentIndex();

        return parentFolder.toString( parentIndex );

    }
*/
      /*************************** Worker Methods ***************************/

    public Message workerGet( Object uid ) throws Exception
    {
        VirtualMessage vMessage = (VirtualMessage) super.uidGet( uid );

        Folder parentFolder = vMessage.getParentFolder();
        Object parentUid = vMessage.getParentUID();

        return parentFolder.workerGet( parentUid );
    }

    public Message workerRemove( Object uid ) throws Exception
    {
        VirtualMessage vMessage = (VirtualMessage) super.workerRemove( uid );

        Folder parentFolder = vMessage.getParentFolder();
        Object parentUid = vMessage.getParentUID();

        return parentFolder.workerRemove( parentUid );
    }

    public void workerViewRemove( Object[] uids ) throws Exception
    {
        VirtualMessage vMessage;
        Folder parentFolder;
        Object[] parentUid = new Object[1];

        for( int i=0; i<Array.getLength(uids); i++)
        {
            //vMessage = (VirtualMessage) uidGet( uids[i] );
            vMessage = (VirtualMessage) super.uidGet( uids[i] );
            parentFolder = vMessage.getParentFolder();
            parentUid[0] = vMessage.getParentUID();

            parentFolder.workerViewRemove( parentUid );
        }
        super.workerViewRemove( uids );
    }



      /**********************************************************************/

    /*
    public void remove( int i ) throws Exception
    {
        getMessageFolderInfo().decExists();

        if ( get(i).getFlags().getRecent() == true );
	{
	    getMessageFolderInfo().decRecent();
	}

        if ( get(i).getFlags().getSeen() == false )
	{
	    getMessageFolderInfo().decUnseen();
	}

        VirtualMessage vMessage = (VirtualMessage) get(i);
        Object uid = (Object) vMessage.getUID();
        uidRemove( uid );
        Folder parentFolder = vMessage.getParentFolder();
        int parentIndex = vMessage.getParentIndex();

        parentFolder.remove( parentIndex );


        for ( int j=i; j < count(); j++ )
        {
            VirtualMessage m = (VirtualMessage) get(i);

            Folder pf = m.getParentFolder();
            int in = m.getParentIndex();

            if ( pf.equals( parentFolder ) )
            {
                System.out.println("same folder -> going to dec index "+ j);
                m.decParentIndex();

            }
            else
            {
                System.out.println("no match "+ j);
            }


        }

    }
    */

/*
    public Integer[] getUids( int[] rows )
    {
        Integer[] uids = new Integer[rows.length];
        Integer uid;
        for ( int i=0; i<rows.length; i++ )
        {
            VirtualMessage message = getVirtualMessage( rows[i] );
            uid = ( Integer ) message.getUID();

            uids[i] = uid;
        }

        return uids;
    }
*/

    public int uidgetMimePartCount( Object uid ) throws Exception
    {

        VirtualMessage vMessage = (VirtualMessage) super.uidGet( uid );

        Folder parentFolder = vMessage.getParentFolder();
        Object parentUid = vMessage.getParentUID();

        int count = parentFolder.uidgetMimePartCount( parentUid );


        return count;
    }

    public MimePart uidgetMimePart( Object uid, int number, SwingWorker worker ) throws Exception
    {

        VirtualMessage vMessage = (VirtualMessage) super.uidGet( uid );

        Folder parentFolder = vMessage.getParentFolder();
        Object parentUid = vMessage.getParentUID();

        MimePart part = parentFolder.uidgetMimePart( parentUid, number, worker );

        return part;
    }

    public MimePartCollection uidgetMimePartCollection( Object uid ) throws Exception
    {
        VirtualMessage vMessage = (VirtualMessage) super.uidGet( uid );

        Folder parentFolder = vMessage.getParentFolder();
        Object parentUid = vMessage.getParentUID();

        MimePartCollection mc = parentFolder.uidgetMimePartCollection( parentUid );


        return mc;
    }


    public Operator getOperator()
    {
	return operator_;
    }

    public void add( Folder child )
    {
	super.add(child);
	child.setOperator( operator_ );
    }

}



