// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.folder.virtual;

import org.columba.gui.statusbar.*;
import org.columba.main.*;

import org.columba.folder.*;

import org.columba.gui.message.*;
import org.columba.gui.attachment.*;
import org.columba.gui.header.*;

import org.columba.message.*;

import java.util.*;

/**
 *
 * VirtualOperator is just an abstraction layer on top of the
 * other Operators. It just gets all Operations and 
 * sends them to the apropriate Operators
 **/
public class VirtualOperator extends Operator
{
    public VirtualOperator(MainInterface mainInterface)
    {
	super( mainInterface );
	maxOut = 1; // 1 Thread parallel

	// WorkerTable
	

	try {
	workerTable.put( new Integer( 0x0000 ), Class.forName( "org.columba.folder.FolderWorkerIO" ));
	workerTable.put( new Integer( 0x0100 ), Class.forName( "org.columba.folder.virtual.VirtualWorkerIntern" ));
	workerTable.put( new Integer( 0x0200 ), Class.forName( "org.columba.folder.virtual.VirtualWorkerFolder" ));
	workerTable.put( new Integer( 0x0300 ), Class.forName( "org.columba.folder.virtual.VirtualWorkerGui" ));
        workerTable.put( new Integer( 0x0500 ), Class.forName( "org.columba.folder.virtual.VirtualWorkerFolder" ));
	}
	catch( ClassNotFoundException e )
	{
	    System.out.println( e );
	}

   }


    protected Folder getParentFolder( VirtualFolder virtualFolder, Object virtualUid ) throws Exception
    {
	VirtualMessage message = (VirtualMessage) virtualFolder.uidGetVirtualMessage(virtualUid);

	Folder parentFolder = message.getParentFolder();
	
	return parentFolder;
    }

    protected Object getParentUid( VirtualFolder virtualFolder, Object virtualUid ) throws Exception
    {		    
	VirtualMessage message = (VirtualMessage)  virtualFolder.uidGetVirtualMessage(virtualUid);

	Object parentUid = message.getParentUID();
	
	return parentUid;
    }

    
    
    
    // take every operation and send them again to the crossbar
    // after replacing the right variables
    public synchronized void operate( Operation op )
    {
	try
        {
	    if ( op.type == Operation.GUI )
	    {
		if ( op.mode == Operation.HEADERLIST ) 
		    {
			addOp(op);
			return;
		    }

		GuiOperation guiOp = (GuiOperation) op;		

		System.out.println("uid: "+guiOp.uid );
		System.out.println("folder: "+guiOp.source );

		// this is the virtual folder
		Folder virtualFolder = guiOp.source;
		Object virtualUid = guiOp.uid;
		
		Folder parentFolder = getParentFolder( (VirtualFolder) virtualFolder, virtualUid );
		Object parentUid = getParentUid( (VirtualFolder) virtualFolder, virtualUid );

		guiOp.source = parentFolder;
		guiOp.uid = parentUid;

		mainInterface.crossbar.operate( guiOp );			

	    }
	else if ( op.type == Operation.FOLDER )
	    {
		if( op.mode == Operation.FOLDER_REMOVE ) 
		{
		    addOp(op);
		    return;
		}
		else if( op.mode == Operation.FOLDER_RENAME ) 
		{
		    addOp(op);
		    return;
		}
		else if( op.mode == Operation.FOLDER_ADD ) 
		{
		    addOp(op);
		    return;
		}
		else
		    {
		FolderOperation folderOp = (FolderOperation) op;

		Folder virtualFolder = folderOp.source;
		Object[] virtualUids = folderOp.uidList;
		
		for ( int i=0; i<virtualUids.length; i++ )
		{
		    Folder parentFolder = getParentFolder( (VirtualFolder) virtualFolder, virtualUids[i] );
		    Object parentUid = getParentUid( (VirtualFolder) virtualFolder, virtualUids[i] );

				    
		    folderOp.source = parentFolder;

		    Object[] uidList = new Object[1];
		    uidList[0] = parentUid;
		    folderOp.uidList = uidList;

		    mainInterface.crossbar.operate( folderOp );			
		}
		    }
	    }
	}
	catch ( Exception ex )
	{
	    ex.printStackTrace();
	}
    }



}

