package org.columba.gui.addressbook;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Vector;

import org.columba.config.*;
import org.columba.gui.util.*;
import org.columba.gui.addressbook.util.*;


public class EditGroupDialog implements ActionListener
{
    private JDialog dialog;

    private AddressbookXmlConfig config;
    private JList addressbook;
    private JList list;
    private JButton addButton;
    private JButton removeButton;
    private JLabel nameLabel;
    private JTextField nameTextField;
    private AdapterNode groupNode;

    private DefaultListModel members;


    private JButton cancelButton;
    private JButton okButton;
    private AddressbookInterface addressbookInterface;

    private AddressbookListRenderer renderer;

    public EditGroupDialog( AddressbookInterface i, AddressbookXmlConfig config, AdapterNode groupNode)
    {
        dialog = DialogStore.getDialog();

        this.addressbookInterface = i;

        this.config = config;
        this.groupNode = groupNode;


        renderer = new AddressbookListRenderer();


        init();
    }

    protected void init()
    {
        dialog.getContentPane().setLayout( new BorderLayout() );


        JPanel panel = new JPanel();
        panel.setLayout( new BoxLayout( panel, BoxLayout.X_AXIS ) );

        CPanel leftPanel = new CPanel( " Addressbook " );
        leftPanel.setLayout( new BoxLayout( leftPanel, BoxLayout.Y_AXIS ) );

        JPanel labelPanel = new JPanel();
	labelPanel.setLayout( new BoxLayout( labelPanel, BoxLayout.X_AXIS ) );
          //labelPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
	nameLabel = new JLabel (" Group Name:");
	labelPanel.add( nameLabel );
	labelPanel.add( Box.createHorizontalGlue() );
        nameTextField = new JTextField ( 18 );
        nameTextField.setMaximumSize( new java.awt.Dimension( 150,25) );
        if ( groupNode != null )
        {
            AdapterNode nameNode = groupNode.getChild("name");
            nameTextField.setText( nameNode.getValue() );
        }

	labelPanel.add( nameTextField );
          //labelPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );

        leftPanel.add( labelPanel );

        leftPanel.add( Box.createRigidArea( new java.awt.Dimension(0,10) ) );

        Vector v = config.getList();
        if ( groupNode != null )
        {
            GroupItem item = config.getGroupItem( groupNode );


            for ( int i=0; i<v.size(); i++ )
            {
                AdapterNode parent = (AdapterNode) v.get(i);

                AdapterNode uidNode = parent.getChild("uid");
                int uid = (new Integer( uidNode.getValue() ) ).intValue();

                if ( uid == item.getUid() ) v.removeElementAt( i );

            }
        }

        addressbook = new JList( v );
        addressbook.setCellRenderer( renderer  );

        JScrollPane scrollPane = new JScrollPane( addressbook );
        leftPanel.add( scrollPane );


        panel.add( leftPanel );

        panel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

        panel.add( Box.createHorizontalGlue() );


        JPanel middlePanel = new JPanel();
        middlePanel.setLayout( new BoxLayout( middlePanel, BoxLayout.Y_AXIS ) );

        middlePanel.add( Box.createVerticalGlue() );

        addButton = new JButton("->");
        addButton.addActionListener( this );
        addButton.setActionCommand( "ADD" );
        middlePanel.add( addButton );
        removeButton = new JButton("<-");
        removeButton.addActionListener( this );
        removeButton.setActionCommand( "REMOVE" );
        middlePanel.add( removeButton );

        middlePanel.add( Box.createVerticalGlue() );

        panel.add( middlePanel );

        CPanel rightPanel = new CPanel( " Group Members " );

        members = new DefaultListModel();
        if ( groupNode != null )
        {
            GroupItem item = config.getGroupItem( groupNode );
            if ( item != null ) System.out.println("item found") ;

            Vector ve = item.getListNodes();
            int uid;
            AdapterNode node, itemNode;

            for ( int i=0; i<ve.size(); i++ )
            {
                node = ( AdapterNode ) ve.get(i);
                uid = (new Integer( node.getValue() ) ).intValue();
                itemNode = config.getNode( uid );

                members.addElement( itemNode );
            }
        }

        list = new JList( members );
        list.setCellRenderer( renderer);
        JScrollPane toPane = new JScrollPane( list );
        rightPanel.add( toPane );

        panel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

        panel.add( Box.createHorizontalGlue() );

        panel.add( rightPanel );

        dialog.getContentPane().add( panel, BorderLayout.CENTER );

        JPanel bottomPanel = new JPanel();
        cancelButton = new JButton( "Cancel" );
        cancelButton.setActionCommand("CANCEL");
        cancelButton.addActionListener( this );
        okButton = new JButton( "Ok" );
        okButton.setActionCommand("OK");
        okButton.addActionListener( this );

        bottomPanel.add( cancelButton );
        bottomPanel.add( okButton );


        dialog.getContentPane().add( bottomPanel, BorderLayout.SOUTH );

        dialog.setSize( new Dimension( 640, 480 ) );



        java.awt.Dimension dim = new Dimension( 640, 480 );

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

        dialog.setLocation(screenSize.width/2 - dim.width/2, screenSize.height/2 - dim.height/2);

        dialog.setVisible(true);
    }



     public void actionPerformed( ActionEvent e )
        {
            String command;

            command = e.getActionCommand();

            if ( command.equals("CANCEL") )
            {
                dialog.setVisible( false );
            }
            else if ( command.equals("OK") )
            {
                if ( nameTextField.getText().length() == 0 )
                {
                    JOptionPane.showMessageDialog( addressbookInterface.frame,
                                                           "You must enter a name for the group!" );
                    return;
                }

                if ( groupNode != null )
                {
                      // edit old group
                    GroupItem item = config.getGroupItem( groupNode );
                    Vector v = item.getListNodes();
                    Object[] array = members.toArray();
                    AdapterNode child, actUid;
                    int actUidInt;
                    boolean hit = false;


                      // remove old items
                    for ( int i=0; i<v.size(); i++ )
                    {
                        child = (AdapterNode) v.get(i);
                        actUidInt = (new Integer( child.getValue() ) ).intValue();

                        hit = false;

                        for ( int j=0; j<array.length; j++)
                        {
                            AdapterNode node = (AdapterNode) array[j];
                            AdapterNode uidNode = node.getChild("uid");
                            int uid = (new Integer( uidNode.getValue() ) ).intValue();

                            if ( actUidInt == uid ) hit = true;

                        }

                        if ( hit == false )
                        {
                              // this item does not exist anymore
                            child.remove();
                        }
                    }

                      // now add new items
                    for ( int j=0; j<array.length; j++)
                    {
                        AdapterNode node = (AdapterNode) array[j];
                        AdapterNode uidNode = node.getChild("uid");
                        int uid = (new Integer( uidNode.getValue() ) ).intValue();


                        item.add( uid );

                    }

                    item.setName( nameTextField.getText() );

                }
                else
                {
                      // create new group
                    AdapterNode adpNode = config.addGroup( nameTextField.getText() );
                    GroupItem item = config.getGroupItem( adpNode );


                    Object[] array = members.toArray();
                    for ( int j=0; j<array.length; j++)
                    {
                        AdapterNode node = (AdapterNode) array[j];
                        AdapterNode uidNode = node.getChild("uid");
                        int uid = (new Integer( uidNode.getValue() ) ).intValue();


                        item.add( uid );

                    }
                }

                addressbookInterface.table.update();
                addressbookInterface.tree.update();

                dialog.setVisible( false );

            }
            else if ( command.equals("ADD") )
            {
                int[] array = addressbook.getSelectedIndices();
                ListModel model = addressbook.getModel();
                AdapterNode node;

                for ( int j=0; j<array.length; j++)
                {
                    node = (AdapterNode) model.getElementAt( array[j] );

                    members.addElement( node );
                }
            }
            else if ( command.equals("REMOVE") )
            {
                int[] array = list.getSelectedIndices();
                for ( int j=0; j<array.length; j++)
                {
                    members.removeElementAt( array[j] );
                }
            }
        }


}
