package org.columba.gui.addressbook.util;

import javax.swing.*;
import javax.swing.table.*;
import java.awt.*;

import org.columba.gui.addressbook.*;
import org.columba.config.*;
import org.columba.gui.header.util.*;



public class AddressbookTable
{
    private AddressbookInterface addressbookInterface;
    private AddressbookXmlConfig config;
    
    private JTable table;
    private AddressbookTableModel addressbookModel;
    private GroupTableModel groupModel;
    
    public JScrollPane scrollPane;
    private AdapterNode node;
    
    
    public AddressbookTable( AddressbookInterface i )
    {
        this.addressbookInterface = i;

        config = i.mainInterface.config.getAddressbookConfig();
        
        addressbookModel = new AddressbookTableModel();
        groupModel = new GroupTableModel( config );
        
        table = new JTable( addressbookModel );
        table.setIntercellSpacing( new Dimension(0,0) );
        table.setShowGrid(false);
        table.setAutoResizeMode( table.AUTO_RESIZE_NEXT_COLUMN );
        
                
        setupRenderer();
        
        
        scrollPane = new JScrollPane( table );
        scrollPane.getViewport().setBackground( Color.white );
    }

    public void setupRenderer()
    {
        TableColumn tc = table.getColumn( "First Name" );
        tc.setCellRenderer( new CommonRenderer( true ) );
        tc.setHeaderRenderer( new AddressbookCommonHeaderRenderer( true, "First Name" ) );
        tc = table.getColumn( "Last Name" );
        tc.setHeaderRenderer( new AddressbookCommonHeaderRenderer( true, "Last Name" ) );
        tc.setCellRenderer( new CommonRenderer( true ) );
        tc = table.getColumn( "Address" );
        tc.setHeaderRenderer( new AddressbookCommonHeaderRenderer( true, "Address" ) );
        tc.setCellRenderer( new CommonRenderer( true ) );
        tc = table.getColumn( "Display Name" );
        tc.setHeaderRenderer( new AddressbookCommonHeaderRenderer( true, "Display Name" ) );
        tc.setCellRenderer( new DisplayNameRenderer( true ) );
        
    }
    


    public void setNode( AdapterNode n )
    {
        if ( n.getName().equals("addressbook") )
        {
            
            addressbookModel.setNode( n );
            addressbookModel.update();
            table.setModel( addressbookModel );
            setupRenderer();
            
            
            node = n.getChild("list");
        }
        else
        {
            groupModel.setNode( n );
            groupModel.update();

            table.setModel( groupModel );
            setupRenderer();
            
            node = n;
            
        }
        
    }

    public void update()
    {
        if ( isGroup() == true )
        {
            groupModel.update();
              //table.setModel( groupModel );
        }
        else
        {
            addressbookModel.update();
              //table.setModel( addressbookModel );
        }
        
    }

    protected boolean isGroup()
    {
        if ( node.getName().equals("group") )
            return true;
        else
            return false;
    }
    
    
    public AdapterNode[] getSelectedNodes()
    {
        int[] rows = table.getSelectedRows();
        AdapterNode[] nodes = new AdapterNode[rows.length];

        if ( isGroup() == true )
        {
            for ( int i=0; i<rows.length; i++ )
            {
                GroupItem item = config.getGroupItem( node );
                nodes[i] = item.getNode( rows[i] );                
            }
        }
        else
        {
            for ( int i=0; i<rows.length; i++ )
            {
                nodes[i] = node.getChild( rows[i] );
            }
            
        }
        

        return nodes;
    }

        
    public AdapterNode getSelectedNode()
    {
        AdapterNode child = null;

        int row = table.getSelectedRow();

        if ( row != -1 )
        {
            if ( isGroup() == true  )
            {
                GroupItem item = config.getGroupItem( node );
                
                int uid = item.getUid( row );

                child = config.getNode( uid );
                
            }
            else
            {    
                child = node.getChild(row);
            }
        }
        

        return child;
    }
    
    public void remove()
    {
        AdapterNode[] nodes = getSelectedNodes();
        
        
        for ( int i=0; i<nodes.length; i++ )
        {
            nodes[i].remove();
        }

          //if ( isGroup() == true ) groupModel.setNode( node );
        
        update();

        addressbookInterface.tree.update();
        
    }

    public AdapterNode getPropertiesNode()
    {
        AdapterNode child = null;

        int row = table.getSelectedRow();

        if ( row != -1 )
        {
            if ( node.getName().equals("group") )
            {
                GroupItem item = config.getGroupItem( node );
                int uid = item.getUid( row );

                child = config.getNode( uid );
                
            }
            else
            {    
                child = node.getChild(row);
            }
        }
        

        return child;
    }
    
    
            
}

