package org.columba.gui.addressbook.util;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import java.awt.*;
import org.columba.gui.addressbook.*;
import org.columba.config.*;
import java.util.Vector;

public class AddressbookTree  implements TreeSelectionListener
{
    private AddressbookTreeNode root;
    private JTree tree;
    private AddressbookInterface addressbookInterface;
    private AddressbookTreeModel model;
    
    public JScrollPane scrollPane;
    
    public AddressbookTree( AddressbookInterface i )
    {
        this.addressbookInterface = i;
        
        root = new AddressbookTreeNode(null,"Root");

        model = new AddressbookTreeModel( root );
        
        tree = new JTree( model );
        tree.setPreferredSize( new Dimension( 200,300 ) );
        tree.setShowsRootHandles( true );
        tree.setBorder( BorderFactory.createEmptyBorder( 2,2,2,2 ) );
        tree.addTreeSelectionListener( this );
        
        
        AddressbookTreeCellRenderer renderer =
            new AddressbookTreeCellRenderer( true );
                                             
        tree.setCellRenderer(renderer);
        
        update();
        
        scrollPane = new JScrollPane( tree );

    }

    public JTree getTree()
    {
        return tree;
    }

    public void removeAll()
    {
        root.removeAllChildren();
    }
    
    public void update()
    {
        removeAll();
        tree.setRootVisible( true );

        model = new AddressbookTreeModel( root );

        AddressbookXmlConfig config = addressbookInterface.mainInterface.config.getAddressbookConfig();
        AdapterNode node, child;
	for ( int i=0 ; i < config.count() ; i++ )
	    {
                node = config.getAddressbookNode(i);
                System.out.println("found addressbook");

                AddressbookTreeNode treeNode = add( node);
                
                AddressbookItem item = config.getAddressbookItem( node );
                Vector v = item.getGroupList();
                
                for ( int j=0; j<v.size(); j++ )
                {
                    AdapterNode child2 = (AdapterNode) v.get(j);
                    String name = child2.getChild("name").getValue();
                    
                    AddressbookTreeNode n = new AddressbookTreeNode( child2, name );
                    
                    treeNode.add( n );
                }
	    }

        tree.setModel( model );
        
        tree.expandRow(0);
        tree.setRootVisible( false );
    }

    public AddressbookTreeNode add( AdapterNode node )
    {
        String name = node.getChild("name").getValue();
        
        AddressbookTreeNode child = new AddressbookTreeNode( node, name);

        if ( root.getChildCount() == 0 )
        {
            tree.setRootVisible( true );
            root.add( child );
            tree.expandRow( 0 );
            tree.setRootVisible( false );
        }
        else
        {
            root.add( child );
        }
        
        
        return child;
    }

    public void valueChanged(TreeSelectionEvent e)
    {
        AddressbookTreeNode node = (AddressbookTreeNode) tree.getLastSelectedPathComponent();
        if (node == null) return;

        if ( node.getName().equals("Root") ) return;

        AdapterNode adpNode = node.getNode();
        

        addressbookInterface.table.setNode( adpNode );
    }
    
}
