// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.attachment;

import org.columba.message.*;
import org.columba.folder.*;
import org.columba.parser.*;
import org.columba.util.*;
import org.columba.gui.action.*;
import org.columba.message.*;
import org.columba.coder.*;
import org.columba.gui.util.*;
import org.columba.main.*;
import org.columba.gui.tree.util.*;
import org.columba.gui.util.*;
import org.columba.gui.mimetype.*;
import org.columba.folder.temp.*;
import org.columba.gui.attachment.menu.*;
import org.columba.gui.attachment.action.*;

import java.io.*;
import javax.swing.text.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.*;


/**
 * This class shows the attachmentlist
 *
 *
 * @version 1.0
 * @author Timo Stich
 */

public class AttachmentViewer
{

    private MainInterface mainInterface;

    public JScrollPane scrollPane;
    private boolean ready = true;
    private SwingWorker worker;



    private IconPanel attachmentPanel;
    private MimePartCollection collection;

    private Folder folder;
      //private int actIndex;
    private Object actUid;

    private Object uid;

    private TempFolder subMessageFolder;

    private boolean inline;

    private AttachmentMenu menu;
    private AttachmentActionListener actionListener;

    public AttachmentViewer(MainInterface mainInterface)
        {
            super();
	    this.mainInterface = mainInterface;

            actionListener = new AttachmentActionListener( mainInterface, this );

            menu = new AttachmentMenu( this );

            MouseListener popupListener = new PopupListener();

            attachmentPanel = new IconPanel();
            attachmentPanel.setDoubleClickActionCommand( "OPEN" );
            attachmentPanel.addActionListener( getActionListener() );
            attachmentPanel.addMouseListener(popupListener);

            scrollPane = new JScrollPane( attachmentPanel );
            scrollPane.getViewport().setBackground( Color.white );
            //attachmentPanel.setPreferredSize(new Dimension(600, 50));
        }


    public synchronized void setFolder( Folder folder )
    {
        this.folder = folder;
    }

    public synchronized void setUid( Object uid )
    {
        this.uid = uid;
    }

    public Folder getFolder()
    {
        return folder;
    }

    public Object getUid()
    {
        return uid;
    }


    public synchronized boolean setMimeCollection( MimePartCollection collection )
    {
        String contentType;
        String contentSubtype;
        String text = new String();
	boolean output = false;

        attachmentPanel.removeAll();

        this.collection = collection;

	if ( collection == null ) return true;

	for ( int i=0; i<collection.count(); i++ ) {
	    MimePart mp = collection.get(i);

	    contentType = mp.getHeader().contentType;
	    contentSubtype = mp.getHeader().contentSubtype;

	    if( mp.getFile() != null ) {
		text = mp.getFile().getName()+" ("+contentType+"/"+contentSubtype+")";
	    }
	    else {
		text = contentType+"/"+contentSubtype;
	    }

	    ImageIcon icon = ImageLoader.getImageIcon(mp.getHeader().contentType
						      +"/"+mp.getHeader().contentSubtype,
						      "brokenimage");
	    attachmentPanel.add( icon, text);
	    output = true;
	}

	return output;
    }


    public void openWith( MimePart part, File tempFile )
    {
        MimeHeader header = part.getHeader();

        MimeTypeViewer viewer = new MimeTypeViewer( mainInterface );
        viewer.openWith( header, tempFile );

    }

    public void open( MimePart part, File tempFile )
    {
        MimeHeader header = part.getHeader();

        if( header.getContentType().toLowerCase().indexOf("message") != -1 )
        {
            inline = true;
            openInlineMessage( part, tempFile );
        }
        else
        {
            inline = false;
            MimeTypeViewer viewer = new MimeTypeViewer( mainInterface );
            viewer.open( header, tempFile );
        }


    }


    protected void openInlineMessage( MimePart part, File tempFile )
    {
        subMessageFolder = new TempFolder( mainInterface );
        uid = null;

            try
            {
                Message subMessage = new Rfc822Parser().parse(part.getBody(), true, null,1);
                uid = subMessageFolder.add( subMessage, part.getBody() );
		

            }
            catch ( Exception ex )
            {
                System.out.println("attachment viewer: "+ex.getMessage() );
            }

           

            mainInterface.crossbar.operate( new GuiOperation(Operation.MESSAGEBODY, 4, subMessageFolder, uid) );
    }



    private int countSelected()
    {
        return attachmentPanel.countSelected();
    }


    public int[] getSelection()
    {
        return attachmentPanel.getSelection();
    }

    public AttachmentActionListener getActionListener()
    {
        return actionListener;
    }

    private JPopupMenu getPopupMenu()
    {
        return menu.getPopupMenu();
    }

    class PopupListener extends MouseAdapter
    {
        public void mousePressed(MouseEvent e)
            {
                maybeShowPopup(e);
            }

        public void mouseReleased(MouseEvent e)
            {
                maybeShowPopup(e);
            }

        private void maybeShowPopup(MouseEvent e)
            {
                if(e.isPopupTrigger() )
                {
                    if( attachmentPanel.countSelected() == 0 ) {
                        attachmentPanel.select( e.getPoint(), 0 );
                    }
                    getPopupMenu().show(e.getComponent(),  e.getX(), e.getY());
                }
            }
    }




}




