// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.composer;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

import org.columba.message.*;
import org.columba.util.*;
import org.columba.gui.util.*;
import org.columba.gui.composer.action.*;


public class ComposerAttachment implements KeyListener
{
    private ComposerInterface composerInterface;
    private JScrollPane scroller;
    private JPopupMenu contextMenu;
    private IconPanel attachment;
    private JFileChooser fileChooser;

    private MimePartCollection collection;


    public ComposerAttachment(ComposerInterface iface)
        {
            composerInterface = iface;

            attachment = new IconPanel();
            attachment.setMaximumSize( new Dimension( 200,80 ));

            scroller = new JScrollPane( attachment );
            scroller.setPreferredSize( new Dimension( 0,80 ) );
            scroller.setMaximumSize( new Dimension( 200,80 ));

            createContextMenu();

            fileChooser = new JFileChooser();

            collection = new MimePartCollection();
        }



    public Component getListPane()
        {
            return scroller;
        }

    public void removeFile()
        {
            int index = attachment.getSelected();

            if(  index != -1 )
            {
                collection.remove( index );
                attachment.removeSelected();
            }

        }

    public MimePartCollection getAttachmentCollection()
        {
            return collection;
        }

    public int countAttachments()
        {
            return collection.count();
        }

    public void attachFile( )
        {

            int returnValue;
            File[] files;

            fileChooser.setDialogTitle("Attach File");
            fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
            fileChooser.setMultiSelectionEnabled(true);
            returnValue = fileChooser.showOpenDialog( attachment );
            if( returnValue == JFileChooser.APPROVE_OPTION )
            {
                files = fileChooser.getSelectedFiles();

                for ( int i=0; i<files.length; i++ )
                {
                    File file = files[i];

                    MimeHeader header = new MimeHeader("application","octet-stream");
                    header.contentDisposition = "attachment";
                    header.dispositionParameter.put("filename",file.getName());
                    header.contentTransferEncoding = "quoted-printable";

                    try {
                        collection.add( new MimePart( header, new FileReader( file ) ) );
                        attachment.add( ImageLoader.getImageIcon("./org/columba/images/Home24.gif"),
                                    file.getName());
                        }
                    catch( IOException e ){
                           System.out.println( e );
                        }
                }

            }
        }

    public void add( MimePart part )
    {
        collection.add( part );
        attachment.add( ImageLoader.getImageIcon("./org/columba/images/Home24.gif"),
                        part.getName() );
    }



    public void keyPressed( KeyEvent k )
        {
            switch( k.getKeyCode() )
            {
                case( KeyEvent.VK_DELETE ) :
                    {
                        removeFile();
                        break;
                    }
            }
        }

    public void keyReleased( KeyEvent k )
        {}

    public void keyTyped( KeyEvent k )
        {}


    private void createContextMenu()
        {
            JMenuItem menuItem;

            contextMenu = new JPopupMenu();
            menuItem = new JMenuItem("Attach File");
            menuItem.addActionListener( composerInterface.composerActionListener  );
            menuItem.setActionCommand("ATTACH");
            contextMenu.add( menuItem );

            menuItem = new JMenuItem("Attach Message");
            menuItem.addActionListener( composerInterface.composerActionListener  );
            menuItem.setActionCommand("QUOTE");
            contextMenu.add( menuItem );

            menuItem = new JMenuItem("Remove");
            menuItem.addActionListener( composerInterface.composerActionListener  );
            menuItem.setActionCommand("REMOVE");
            contextMenu.add( menuItem );

            contextMenu.addSeparator();

            menuItem = new JMenuItem("Cancel");
            contextMenu.add( menuItem );

            PopupListener popupListener = new PopupListener( contextMenu );
	    attachment.addMouseListener( popupListener );

        }


}
