// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.gui.composer;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;

import org.columba.config.*;
import org.columba.gui.composer.action.*;
import org.columba.gui.composer.util.*;
import org.columba.gui.util.ImageLoader;

public class ComposerEditor
	implements FocusListener, ItemListener, DocumentListener
{
	private static final String[] prioritys =
		{ "Highest", "High", "Normal", "Low", "Lowest" };

	private JPanel editorPane;
	private JPanel subjectPane;

	private JTextPane editor;
	private JTextField subject;
	private JComboBox smtpList;
	private JComboBox priorityList;

	private UndoDocument message;

	private ComposerInterface composerInterface;

	private boolean encrypt;
	private boolean sign;

	public ComposerEditor(ComposerInterface iface)
	{
		composerInterface = iface;

		subject = new JTextField();
		//subject.setPreferredSize(new Dimension(50,20) );
		subject.addFocusListener(this);

		JLabel subjectLabel = new JLabel("Subject:");
		subjectLabel.setLabelFor(subject);

		smtpList = new JComboBox();
		smtpList.setRenderer(new AccountListRenderer());
		smtpList.addItemListener(this);

		AccountList config = composerInterface.config.getAccountList();

		for (int i = 0; i < config.count(); i++)
		{
			smtpList.addItem(config.get(i));
		}

		priorityList = new JComboBox(prioritys);
		priorityList.setSelectedIndex(2);

		subjectPane = createSubjectPane(subject);

		message = new UndoDocument();
		message.addDocumentListener( this );
		composerInterface.message = message;

		editor = new JTextPane();
		editor.setStyledDocument(message);
		editor.setEditable(true);

		editorPane = new JPanel();
		editorPane.setLayout(new BorderLayout());
		//editorPane.setBorder( BorderFactory.createEmptyBorder(5,5,2,5) );
		JScrollPane scrollPane = new JScrollPane(editor);
		scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		scrollPane.setWheelScrollingEnabled(true);
		editorPane.add(scrollPane, BorderLayout.CENTER);
		//editorPane.setPreferredSize(new Dimension( 0,250 ) );
	}

	public JPanel getEditorPane()
	{
		return editorPane;
	}

	public JEditorPane getEditor()
	{
		return editor;
	}

	public void setBodyText(String s)
	{
		getEditor().setText(s);
	}

	public JPanel getSubjectPane()
	{
		return subjectPane;
	}

	public String getPriority()
	{
		return new Integer(0).toString(priorityList.getSelectedIndex() + 1);
	}

	public String getBody()
	{
		return message.getText();
	}

	public String getSubject()
	{
		return subject.getText();
	}

	public void setSubject(String sub)
	{
		subject.setText(sub);
		composerInterface.composerFrame.setTitle(sub);
		subject.transferFocus();
	}

	public Component getSubjectComponent()
	{
		return subject;
	}

	public SmtpItem getSmtpItem()
	{
		AccountItem accountItem = (AccountItem) smtpList.getSelectedItem();

		SmtpItem smtpItem = accountItem.getSmtpItem();

		return smtpItem;
	}

	public String getFromAddress()
	{

		AccountItem accountItem = (AccountItem) smtpList.getSelectedItem();

		IdentityItem identityItem = accountItem.getIdentityItem();

		return identityItem.getAddress();

	}

	public String getSignature()
	{
		IdentityItem identity =
			((AccountItem) smtpList.getSelectedItem()).getIdentityItem();

		if (identity.getAttachSignature())
		{
			return identity.getSignature();
		}

		return null;
	}

	public boolean getSign()
	{
		return sign;
	}

	public void changeSign()
	{
		if (sign == true)
			sign = false;
		else
			sign = true;

	}

	public void changeEncrypt()
	{
		if (encrypt == true)
		{

			encrypt = false;
		}
		else
		{
			encrypt = true;
		}
	}

	public boolean getEncrypt()
	{
		return encrypt;
	}

	public void selectProfile(String account)
	{
		if (account != null)
			smtpList.setSelectedItem(account);
		else
			smtpList.setSelectedIndex(0);
	}

	public void selectProfile(int index)
	{
		if (index == -1)
			smtpList.setSelectedIndex(0);
		else
			smtpList.setSelectedIndex(index);
	}

	public String getProfile()
	{
		return ((AccountItem) smtpList.getSelectedItem()).getName();
	}

	public int getAccountUid()
	{
		return ((AccountItem) smtpList.getSelectedItem()).getUid();
	}

	private JPanel createSubjectPane(JTextField textField)
	{
		JLabel subjectLabel = new JLabel("Subject:");
		//subjectLabel.setForeground(Color.black);
		subjectLabel.setLabelFor(subject);

		JLabel smtpLabel = new JLabel("Identity:");
		//smtpLabel.setForeground( Color.black );

		smtpLabel.setLabelFor(smtpList);

		JLabel priorityLabel = new JLabel("Priority:");
		//priorityLabel.setForeground( Color.black );
		priorityLabel.setLabelFor(priorityList);

		GridBagLayout gridbag = new GridBagLayout();
		JPanel container = new JPanel();

		container.setLayout(gridbag);

		GridBagConstraints c = new GridBagConstraints();
		//c.insets = new Insets( 2,2,2,2 );

		c.anchor = GridBagConstraints.WEST;
		//c.fill = GridBagConstraints.NONE;      //reset to default
		//c.gridwidth = 4;     //next to Last
		//c.weightx = 0.0;                       //reset to default
		c.gridx = 0;
		c.gridy = 0;
		c.insets = new Insets(5, 5, 5, 5);
		gridbag.setConstraints(smtpLabel, c);
		container.add(smtpLabel);

		//c.anchor = GridBagConstraints.WEST;
		c.gridx = 1;
		c.weightx = 1.0;
		gridbag.setConstraints(smtpList, c);
		container.add(smtpList);

		//JPanel priorityPanel = new JPanel();

		/*
		JToggleButton signButton = new JToggleButton( ImageLoader.getImageIcon("sign","Host16") );
		signButton.setMargin( new Insets( 0,0,0,0 )  );
		signButton.setEnabled( false );
		JToggleButton encryptButton = new JToggleButton( ImageLoader.getImageIcon("encrypt","WebComponent16") );
		encryptButton.setMargin( new Insets( 0,0,0,0 )  );
		encryptButton.setEnabled( false );
		JToggleButton trashButton = new JToggleButton( ImageLoader.getImageIcon("trash","Delete16") );
		trashButton.setMargin( new Insets( 0,0,0,0 )  );
		trashButton.setEnabled( false );
		JToggleButton earthButton = new JToggleButton( ImageLoader.getImageIcon("trash","Bookmarks16") );
		earthButton.setMargin( new Insets( 0,0,0,0 )  );
		earthButton.setEnabled( false );
		
		
		priorityPanel.add( signButton );
		priorityPanel.add( encryptButton );
		priorityPanel.add( trashButton );
		*/

		c.gridwidth = GridBagConstraints.RELATIVE;
		c.gridx = 2;
		c.weightx = 0.0;
		gridbag.setConstraints(priorityLabel, c);
		container.add(priorityLabel);

		c.gridwidth = GridBagConstraints.REMAINDER;
		c.gridx = 3;
		gridbag.setConstraints(priorityList, c);
		container.add(priorityList);

		//priorityPanel.add( earthButton );

		/*
		        c.anchor = GridBagConstraints.WEST;
		        c.insets = new Insets(0,20,0,0);
		        c.gridwidth = GridBagConstraints.REMAINDER;
		        c.weightx = 1.0;
		        gridbag.setConstraints(priorityPanel, c);
		        container.add(priorityPanel);
		*/
		/*
		c.anchor = GridBagConstraints.EAST;
		//c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = GridBagConstraints.REMAINDER;     //end row
		c.weightx = 1.0;
		gridbag.setConstraints(priorityList, c);
		container.add(priorityList);
		*/

		/*
		JLabel filler = new JLabel("");
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = GridBagConstraints.REMAINDER;     //end row
		c.weightx = 1.0;
		gridbag.setConstraints(filler, c);
		container.add(filler);
		*/

		//c.insets = new Insets(0,0,0,0);
		//c.anchor = GridBagConstraints.EAST;
		//c.gridwidth = 1;     //end row

		//c.fill = GridBagConstraints.NONE;      //reset to default
		//c.weightx = 0.0;                       //reset to default
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = GridBagConstraints.RELATIVE;
		gridbag.setConstraints(subjectLabel, c);
		container.add(subjectLabel);

		//c.gridwidth = GridBagConstraints.REMAINDER;     //next to Last
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 1;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.weightx = 1.0;
		gridbag.setConstraints(textField, c);
		container.add(textField);

		return container;
	}

	public void focusGained(FocusEvent e)
	{
	}

	public void focusLost(FocusEvent e)
	{
		Object source = e.getSource();

		if (source == subject)
		{
			if (!subject.getText().equals(""))
			{
				composerInterface.composerFrame.setTitle(subject.getText());
			}
			else
			{
				composerInterface.composerFrame.setTitle("New Message");
			}

			return;
		}
	}

	public void itemStateChanged(ItemEvent e)
	{
		try
		{

			if (e.getStateChange() == ItemEvent.SELECTED)
			{
				AccountItem item = (AccountItem) e.getItem();

				composerInterface.identityInfoPanel.set(item);

			}
			else
			{
			}
		}
		catch (ClassCastException ex)
		{
			System.out.println(ex.getMessage());
		}

	}

	public void insertUpdate(DocumentEvent e)
	{
		//message.parseLine( getEditor().getCaretPosition() );	
		update();
	}

	public void removeUpdate(DocumentEvent e)
	{
		//message.parseLine( getEditor().getCaretPosition() );	
		update();
	}

	public void changedUpdate(DocumentEvent e)
	{
		update();
		//Plain text components don't fire these events
	}

	public void update()
	{
		/*
		message.readLock();
		message.parseLine( getEditor().getCaretPosition() );	
		message.readUnlock();
		*/
	}

}

class AccountListRenderer extends JLabel implements ListCellRenderer
{
	ImageIcon image1;
	ImageIcon image2;

	public AccountListRenderer()
	{
		setOpaque(true);
		image1 = ImageLoader.getImageIcon("pop", "Host16");
		image2 = ImageLoader.getImageIcon("imap", "WebComponent16");
	}

	public Component getListCellRendererComponent(
		JList list,
		Object value,
		int index,
		boolean isSelected,
		boolean cellHasFocus)
	{
		if (isSelected)
		{
			setBackground(list.getSelectionBackground());
			setForeground(list.getSelectionForeground());
		}
		else
		{
			setBackground(list.getBackground());
			setForeground(list.getForeground());
		}

		if (value != null)
		{
			AccountItem item = (AccountItem) value;
			String accountName = item.getName();
			IdentityItem identity = item.getIdentityItem();
			String address = identity.getAddress();
			String name = identity.getName();

			String result = accountName + ":   " + name + " <" + address + ">";

			setText(result);

			if (item.isPopAccount())
				setIcon(image1);
			else
				setIcon(image2);

		}

		return this;
	}

}