// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.gui.composer;

import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.BorderLayout;
import java.awt.Container;
import java.util.*;

import java.awt.event.*;
import java.io.*;

import javax.swing.*;
import javax.swing.text.*;

import org.columba.folder.temp.*;
import org.columba.config.*;
import org.columba.composer.*;
import org.columba.gui.composer.util.*;
import org.columba.message.*;
import org.columba.main.*;
import javax.swing.*;
import org.columba.gui.util.*;
import org.columba.gui.composer.action.*;
import org.columba.folder.*;

import javax.swing.border.*;
import java.awt.Dimension;
import java.awt.Toolkit;

public class ComposerFrame extends JFrame
{
	private JWindow composerWindow;

	private JSplitPane mainSplitPane;

	public ComposerInterface composerInterface;
	private ComposerActionListener composerActionListener;

	private WindowItem windowItem;

	public ComposerFrame(MainInterface mainInterface)
	{
		this.setTitle("New Message");
		//this.setSize(640,800);

		composerInterface = new ComposerInterface(mainInterface.config);
		composerInterface.mainInterface = mainInterface;
		

		composerInterface.identityInfoPanel = new IdentityInfoPanel();
		composerInterface.messageComposer = new MessageComposer(composerInterface);
		composerActionListener = new ComposerActionListener(composerInterface);
		composerInterface.composerEditor = new ComposerEditor(composerInterface);
		composerInterface.composerFrame = this;
		composerInterface.composerAttachment =
			new ComposerAttachment(composerInterface);
		composerInterface.composerHeader = new ComposerHeader(composerInterface);

		composerInterface.treeViewer = mainInterface.treeViewer;
		composerInterface.taskManager = mainInterface.taskManager;
		composerInterface.crossbar = mainInterface.crossbar;

		composerInterface.pgpController = mainInterface.pgpController;
		windowItem =
			composerInterface.config.getComposerOptionsConfig().getWindowItem();

		setupWindow();

		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				saveWindowPosition();
				dispose();
			}
		});

		/*  Set up ComposerFolder and Operator  */

		composerInterface.composerFolder = new TempFolder(mainInterface);

		java.awt.Point point = windowItem.getPoint();
		java.awt.Dimension dim = windowItem.getDimension();

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setSize(dim);

		this.setVisible(true);
	}

	public void saveWindowPosition()
	{
		java.awt.Dimension d = getSize();

		windowItem.setXPosition(0);
		windowItem.setYPosition(0);
		windowItem.setWidth(d.width);
		windowItem.setHeight(d.height);

		/*
		item.setMainSplitPane( mainSplitPane.getDividerLocation() );
		item.setRightSplitPane( rightSplitPane.getDividerLocation() );
		*/
	}

	private void setupWindow()
	{
		Container contentPane;

		composerWindow = new JWindow(this);

		contentPane = this.getContentPane();

		this.setJMenuBar(createMenubar());

		contentPane.setLayout(new BorderLayout());
		contentPane.add(createToolbar(), BorderLayout.NORTH);
		contentPane.add( createEditorPane(), BorderLayout.CENTER);
		
		/*
		JLabel statusBar = new JLabel("Composer");
		Border border = BorderFactory.createEtchedBorder(1);
		Border margin = new EmptyBorder(0,5,2,5);
		statusBar.setBorder( new CompoundBorder( margin, border ) );
		
		contentPane.add( statusBar, BorderLayout.SOUTH );
		*/
		
	}

	private JPanel createToolbar()
	{
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));

		JToolBar toolBar;
		CButton toolButton;

		toolBar = new JToolBar();
		toolBar.putClientProperty("JToolBar.isRollover", Boolean.TRUE);
		toolBar.setFloatable(false);

		toolBar.add(new JToolBar.Separator());

		toolButton = new CButton(composerInterface.composerActionListener.sendAction);
		toolBar.add(toolButton);

		toolButton =
			new CButton(composerInterface.composerActionListener.attachFileAction);
		toolBar.add(toolButton);

		toolBar.add(new JToolBar.Separator());
		toolBar.add(new JToolBar.Separator());

		toolButton = new CButton(composerInterface.composerActionListener.cutAction);
		toolBar.add(toolButton);

		toolButton = new CButton(composerInterface.composerActionListener.copyAction);
		toolBar.add(toolButton);

		toolButton = new CButton(composerInterface.composerActionListener.pasteAction);
		toolBar.add(toolButton);

		toolBar.add(new JToolBar.Separator());
		toolBar.add(new JToolBar.Separator());
		toolBar.add(new JToolBar.Separator());

		toolButton =
			new CButton(composerInterface.composerActionListener.addressbookAction);
		toolBar.add(toolButton);

		toolBar.add(new JToolBar.Separator());

		toolBar.add(new JToolBar.Separator());
		/*
		toolButton = new CButton(composerInterface.composerActionListener.exitAction);
		toolBar.add( toolButton );
		*/
		toolBar.add(Box.createHorizontalGlue());

		panel.add(toolBar);

		panel.add(composerInterface.identityInfoPanel);

		return panel;
	}

	private JMenuBar createMenubar()
	{
		JMenu fileMenu =
			new JMenu(ResourceLoader.getString("action/global","menu_file"));
		fileMenu.setMnemonic(KeyEvent.VK_F);

		CMenuItem newMenuItem;
		newMenuItem = new CMenuItem(composerInterface.composerActionListener.newAction);
		fileMenu.add(newMenuItem);

		fileMenu.addSeparator();

		CMenuItem saveMenuItem;
		saveMenuItem =
			new CMenuItem(composerInterface.composerActionListener.saveAction);
		fileMenu.add(saveMenuItem);

		CMenuItem saveAsMenuItem;
		saveAsMenuItem =
			new CMenuItem(composerInterface.composerActionListener.saveAsAction);
		fileMenu.add(saveAsMenuItem);

		CMenuItem saveDraftMenuItem;
		saveDraftMenuItem =
			new CMenuItem(composerInterface.composerActionListener.saveDraftAction);
		fileMenu.add(saveDraftMenuItem);
		
		CMenuItem saveTemplateMenuItem;
		saveTemplateMenuItem =
			new CMenuItem(composerInterface.composerActionListener.saveTemplateAction);
		fileMenu.add(saveTemplateMenuItem);
		

		fileMenu.addSeparator();

		CMenuItem sendMenuItem;
		sendMenuItem =
			new CMenuItem(composerInterface.composerActionListener.sendAction);
		fileMenu.add(sendMenuItem);

		CMenuItem sendLaterMenuItem;
		sendLaterMenuItem =
			new CMenuItem(composerInterface.composerActionListener.sendLaterAction);
		fileMenu.add(sendLaterMenuItem);

		fileMenu.addSeparator();

		JMenuItem printMenuItem = new JMenuItem("Print Message...");
		printMenuItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
		printMenuItem.setMnemonic(KeyEvent.VK_P);
		printMenuItem.setEnabled(false);

		fileMenu.add(printMenuItem);

		fileMenu.addSeparator();

		CMenuItem exitMenuItem;
		exitMenuItem =
			new CMenuItem(composerInterface.composerActionListener.exitAction);
		fileMenu.add(exitMenuItem);

		JMenu editMenu =
			new JMenu(ResourceLoader.getString("action/global","menu_edit"));
		editMenu.setMnemonic(KeyEvent.VK_E);

		CMenuItem undoMenuItem;
		undoMenuItem =
			new CMenuItem(composerInterface.composerActionListener.undoAction);
		editMenu.add(undoMenuItem);

		CMenuItem redoMenuItem;
		redoMenuItem =
			new CMenuItem(composerInterface.composerActionListener.redoAction);
		editMenu.add(redoMenuItem);

		editMenu.addSeparator();

		CMenuItem cutMenuItem;
		cutMenuItem = new CMenuItem(composerInterface.composerActionListener.cutAction);
		editMenu.add(cutMenuItem);

		CMenuItem copyMenuItem;
		copyMenuItem =
			new CMenuItem(composerInterface.composerActionListener.copyAction);
		editMenu.add(copyMenuItem);

		CMenuItem pasteMenuItem;
		pasteMenuItem =
			new CMenuItem(composerInterface.composerActionListener.pasteAction);
		editMenu.add(pasteMenuItem);

		CMenuItem deleteMenuItem;
		deleteMenuItem =
			new CMenuItem(composerInterface.composerActionListener.deleteAction);
		editMenu.add(deleteMenuItem);

		editMenu.addSeparator();

		CMenuItem selectAllMenuItem;
		selectAllMenuItem =
			new CMenuItem(composerInterface.composerActionListener.selectAllAction);
		editMenu.add(selectAllMenuItem);

		editMenu.addSeparator();

		JMenuItem menuItem =
			new JMenuItem(
				ResourceLoader.getString("action/global","menu_edit_find"),
				ImageLoader.getImageIcon("search_small", "org/columba/images/Find16.gif"));
		menuItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_F, ActionEvent.CTRL_MASK));
		menuItem.setMnemonic(KeyEvent.VK_F);

		menuItem.setEnabled(false);
		editMenu.add(menuItem);

		menuItem =
			new JMenuItem(
				ResourceLoader.getString("action/global","menu_edit_findagain"));

		menuItem.setMnemonic(KeyEvent.VK_G);
		menuItem.setAccelerator(
			KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.CTRL_MASK));
		menuItem.setEnabled(false);
		editMenu.add(menuItem);

		/*
		  menuItem = new JMenuItem( composerInterface.composerLabels.getString("menu_edit_replace")  );
		  menuItem.setEnabled( false );
		  editMenu.add(menuItem);
		*/

		JMenu viewMenu = new JMenu("View");
		viewMenu.setMnemonic(KeyEvent.VK_V);

		JCheckBoxMenuItem toButton = new JCheckBoxMenuItem("To");
		toButton.setSelected(true);
		toButton.setEnabled(false);
		viewMenu.add(toButton);

		JCheckBoxMenuItem ccButton = new JCheckBoxMenuItem("Cc");
		ccButton.setEnabled(false);
		ccButton.setSelected(true);
		viewMenu.add(ccButton);

		JCheckBoxMenuItem bccButton = new JCheckBoxMenuItem("Bcc");
		bccButton.setEnabled(false);
		bccButton.setSelected(true);
		viewMenu.add(bccButton);

		viewMenu.addSeparator();

		JCheckBoxMenuItem attachmentButton = new JCheckBoxMenuItem("Attachments");
		attachmentButton.setEnabled(false);
		attachmentButton.setSelected(true);
		viewMenu.add(attachmentButton);

		JMenu messageMenu =
			new JMenu(ResourceLoader.getString("action/mail","menu_message"));
		messageMenu.setMnemonic(KeyEvent.VK_M);

		CMenuItem addressbookMessageMenuItem;
		addressbookMessageMenuItem =
			new CMenuItem(composerInterface.composerActionListener.addressbookAction);
		messageMenu.add(addressbookMessageMenuItem);

		CMenuItem attachFileMenuItem;
		attachFileMenuItem =
			new CMenuItem(composerInterface.composerActionListener.attachFileAction);
		messageMenu.add(attachFileMenuItem);

		messageMenu.addSeparator();

		ButtonGroup group = new ButtonGroup();

		JMenu subMenu = new JMenu("Priority");
		JRadioButtonMenuItem priorityMenuItem = new JRadioButtonMenuItem("Lowest");
		subMenu.add(priorityMenuItem);
		group.add(priorityMenuItem);
		priorityMenuItem.setEnabled(false);

		priorityMenuItem = new JRadioButtonMenuItem("Low");
		priorityMenuItem.setEnabled(false);
		group.add(priorityMenuItem);
		subMenu.add(priorityMenuItem);
		priorityMenuItem = new JRadioButtonMenuItem("Normal");
		priorityMenuItem.setSelected(true);
		priorityMenuItem.setEnabled(false);
		group.add(priorityMenuItem);
		subMenu.add(priorityMenuItem);
		priorityMenuItem = new JRadioButtonMenuItem("High");
		priorityMenuItem.setEnabled(false);
		group.add(priorityMenuItem);
		subMenu.add(priorityMenuItem);
		priorityMenuItem = new JRadioButtonMenuItem("Highest");
		priorityMenuItem.setEnabled(false);
		group.add(priorityMenuItem);
		subMenu.add(priorityMenuItem);

		messageMenu.add(subMenu);

		messageMenu.addSeparator();

		JCheckBoxMenuItem signMenuItem =
			new JCheckBoxMenuItem(composerInterface.composerActionListener.signAction);
		signMenuItem.setSelected(false);
		signMenuItem.setEnabled(false);
		messageMenu.add(signMenuItem);

		JCheckBoxMenuItem encryptMenuItem =
			new JCheckBoxMenuItem(composerInterface.composerActionListener.encryptAction);
		encryptMenuItem.setSelected(false);
		//encryptMenuItem.setEnabled(false);
		messageMenu.add(encryptMenuItem);

		JMenuBar menuBar = new JMenuBar();
		menuBar.add(fileMenu);
		menuBar.add(editMenu);
		menuBar.add(viewMenu);
		menuBar.add(messageMenu);

		return menuBar;
	}

	private Container createEditorPane()
	{
		/*
		GridBagConstraints c = new GridBagConstraints();
		GridBagLayout gridbag = new GridBagLayout();

		JPanel editorPane = new JPanel();
		editorPane.setLayout(gridbag);
		c.insets = new Insets(2, 2, 2, 2);
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.gridheight = GridBagConstraints.RELATIVE;
		c.anchor = GridBagConstraints.NORTHWEST;
		c.weighty = 0.0;
		c.weightx = 1.0;

		gridbag.setConstraints(composerInterface.composerEditor.getSubjectPane(), c);
		*/
		JPanel editorPane = new JPanel();
		editorPane.setLayout( new BorderLayout() );
		
			
		JPanel subjectPanel = composerInterface.composerEditor.getSubjectPane();
		editorPane.add( subjectPanel, BorderLayout.NORTH );

		/*
		c.gridheight = GridBagConstraints.REMAINDER;
		c.fill = GridBagConstraints.BOTH;
		c.weighty = 1.0;
		gridbag.setConstraints(composerInterface.composerEditor.getEditorPane(), c);
		*/
		JPanel editorPanel = composerInterface.composerEditor.getEditorPane();
		
		editorPane.add(editorPanel, BorderLayout.CENTER);

		mainSplitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
		mainSplitPane.add(
			composerInterface.composerHeader.getTabbedPane(),
			JSplitPane.TOP);
		mainSplitPane.add(editorPane, JSplitPane.BOTTOM);
		mainSplitPane.setDividerSize(5);

		return mainSplitPane;
	}

	public void redrawSplitPane()
	{
		mainSplitPane.resetToPreferredSizes();
	}

	public void setSubject(String subject)
	{
		composerInterface.composerEditor.setSubject(subject);
	}

	public void setProfile(String host, String address)
	{
		
		composerInterface.composerEditor.selectProfile(
			composerInterface.config.getAccountList().hostGetAccountIndex(host, address));
	}

	public void setTo(String to, String host, String address)
	{
		composerInterface.composerHeader.setTo(to);

		setProfile(host, address);
	}
	
	public void addHeaderItem( String headerfield, String value )
	{
		composerInterface.composerHeader.addHeaderItem( headerfield, value);
	}
	
	public void addMimePart( MimePart mp )
	{
		composerInterface.composerAttachment.add( mp );
	}

	public void setTo(String to)
	{
		composerInterface.composerHeader.setTo(to);
	}
	
	
	public void setBodyText(String str)
	{
		composerInterface.composerEditor.setBodyText(str);
	}

	

	// FIXME - please
	// something funny is happening with attachments:
	// just open a message with pictures or ms word files with Open With Composer Action
	// then open message from outbox or sent-folder and examine the attachments....hmmmm
	public void openMessage(Folder folder, Object uid)
	{
		/*
		Message message = folder.uidGet( uid );
		Rfc822Header header = message.getHeader();
		String subject = (String) header.get("Subject");
		composerInterface.composerEditor.setSubject( subject );
		
		composerInterface.composerEditor.selectProfile(
		        composerInterface.config.getAccountList().hostGetAccountName( (String) message.getHeader().get("columba.host") )
		        );
		
		try
		{
		
		    String bodyText = folder.uidgetBodyText( uid, null );
		    composerInterface.composerEditor.setBodyText( bodyText );
		
		    int count = folder.uidgetMimePartCount(uid);
		    for ( int i=1; i<count; i++ )
		    {
		        MimePart part = folder.uidgetMimePart(uid, i );
		        MimeHeader mimeHeader = part.getHeader();
		        String contentType = mimeHeader.getContentType();
		        String contentSubtype = mimeHeader.getContentSubtype();
		        String encoding = mimeHeader.getContentTransferEncoding();
		        String body = part.getBody();
		
		        composerInterface.composerAttachment.add(
		            new MimePart( mimeHeader, new StringReader( body ) ) );
		    }
		}
		catch ( Exception ex )
		{
		    ex.printStackTrace();
		}
		
		*/
	}

	
	
	public void setInlineText( String text )
	{
		if ( text == null ) return;
		
		StringBuffer buf = new StringBuffer();
		StringTokenizer tok = new StringTokenizer( text, "\n" );
		while ( tok.hasMoreElements() )
		{
			buf.append("> ");
			buf.append( (String) tok.nextElement() );
			buf.append("\n");
		}
		
		composerInterface.composerEditor.setBodyText( buf.toString() );	
	}

}