// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.gui.composer;

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import org.columba.gui.composer.action.*;
import org.columba.gui.addressbook.util.*;
import org.columba.config.*;
import org.columba.gui.util.ImageLoader;

public class ComposerHeader
	implements ActionListener, FocusListener, KeyListener
{
	private static final int ANZ_TABS = 3;

	private JTabbedPane adressTabs;
	private JTabbedPane optionTabs;
	private JPanel tabbedPane;


	//private JList[] list = new JList[ANZ_TABS];
	private AddressbookListView[] list = new AddressbookListView[ANZ_TABS];
	private JButton[] addButton = new JButton[ANZ_TABS];
	private JButton[] removeButton = new JButton[ANZ_TABS];
	private JLabel label[] = new JLabel[ANZ_TABS];
	private AddressbookListModel[] listModel = new AddressbookListModel[ANZ_TABS];
	private JTextField inline[] = new JTextField[ANZ_TABS];
	private JButton buttonOK;
	private JButton buttonDEL;

	private ComposerInterface composerInterface;

	private Hashtable additionalHeaderItems;

	public ComposerHeader(ComposerInterface iface)
	{
		composerInterface = iface;

		tabbedPane = new JPanel();
		tabbedPane.setBorder(BorderFactory.createEmptyBorder(5, 5, 0, 5));
		tabbedPane.setLayout(new GridLayout(1, 2, 5, 0));

		adressTabs = new JTabbedPane(JTabbedPane.TOP);
		adressTabs.setPreferredSize(new Dimension(0, 150));

		label[0] = new JLabel("  To: ");
		label[1] = new JLabel("  Cc: ");
		label[2] = new JLabel("  Bcc: ");

		adressTabs.addTab(" TO ", createTab(0));
		adressTabs.addTab(" CC ", createTab(1));
		adressTabs.addTab(" BCC ", createTab(2));

		tabbedPane.add(adressTabs);

		optionTabs = new JTabbedPane(JTabbedPane.TOP);
		optionTabs.setPreferredSize(new Dimension(0, 150));

		optionTabs.addTab(
			" Attachments ",
			ImageLoader.getImageIcon("attachment_small", "Save16"),
			composerInterface.composerAttachment.getListPane());

		tabbedPane.add(optionTabs);

		additionalHeaderItems = new Hashtable();

	}

	public void addHeaderItem(String key, String value)
	{
		additionalHeaderItems.put(key.toLowerCase(), value);
	}

	public String getAdditionalHeaderItem(String key)
	{

		if (additionalHeaderItems.containsKey(key.toLowerCase()))
		{
			String result = (String) additionalHeaderItems.get(key.toLowerCase());

			return result;
		}

		return null;
	}

	public Container getTabbedPane()
	{
		return tabbedPane;
	}

	public AddressbookListView[] getList()
	{
		return list;
	}

	public AddressbookListModel[] getListModels()
	{
		return listModel;
	}

	private Container createTab(int i)
	{
		JPanel panel = new JPanel();
		//panel.setBorder( BorderFactory.createEmptyBorder( 1,1,1,1 ) );
		panel.setLayout(new BorderLayout());

		JPanel textPane = new JPanel();
		textPane.setLayout(new BoxLayout(textPane, BoxLayout.X_AXIS));
		textPane.setBorder(BorderFactory.createEmptyBorder(4, 0, 4, 0));

		textPane.add(label[i]);

		inline[i] = new JTextField();
		inline[i].setActionCommand("OK");
		inline[i].addActionListener(this);
		/*		
		inline[i].setNextFocusableComponent(
			composerInterface.composerEditor.getSubjectComponent());
		
		inline[i].addFocusListener(this);
		*/
				
		textPane.add(inline[i]);

		textPane.add(Box.createRigidArea(new java.awt.Dimension(6, 0)));

		addButton[i] =
			new JButton(ImageLoader.getImageIcon("add_small", "Properties16"));
		addButton[i].addActionListener(this);
		addButton[i].setActionCommand("OK");
		addButton[i].setMargin(new Insets(0, 0, 0, 0));
		textPane.add(addButton[i]);

		textPane.add(Box.createRigidArea(new java.awt.Dimension(4, 0)));

		removeButton[i] =
			new JButton(ImageLoader.getImageIcon("remove_small", "Cut16"));
		removeButton[i].addActionListener(this);
		removeButton[i].setActionCommand("REMOVE");
		removeButton[i].setMargin(new Insets(0, 0, 0, 0));
		textPane.add(removeButton[i]);

		textPane.add(Box.createRigidArea(new java.awt.Dimension(1, 0)));

		panel.add(textPane, BorderLayout.NORTH);

		JScrollPane listScroller;
		listModel[i] = new AddressbookListModel();

		//list[i] = new JList( listModel[i]);
		list[i] = new AddressbookListView(listModel[i]);
		//list[i].setCellRenderer( new AddressbookListRenderer() );
		list[i].getComponent().addKeyListener(this);
		listScroller = new JScrollPane(list[i].getComponent());
		listScroller.setPreferredSize(new Dimension(0, 150));

		panel.add(listScroller, BorderLayout.CENTER);

		return panel;
	}

	public int getToSize()
	{
		return listModel[0].getSize();
	}

	public String getToTextFieldString()
	{
		return inline[0].getText();
	}

	public String getCcTextFieldString()
	{
		return inline[1].getText();
	}

	public String getBccTextFieldString()
	{
		return inline[2].getText();
	}

	public String getToList()
	{
		AddressbookXmlConfig config = composerInterface.config.getAddressbookConfig();

		String output = ListParser.parse(config, listModel[0]);

		return output;
	}

	public int getCcSize()
	{
		return listModel[1].getSize();
	}
	
	public String getBccList()
	{
		AddressbookXmlConfig config = composerInterface.config.getAddressbookConfig();

		String output = ListParser.parse(config, listModel[2]);

		return output;

	}

	public String getCcList()
	{
		AddressbookXmlConfig config = composerInterface.config.getAddressbookConfig();

		String output = ListParser.parse(config, listModel[1]);

		return output;

	}

	public void setTo(String to)
	{
		AddressbookXmlConfig config = composerInterface.config.getAddressbookConfig();
		AdapterNode node = config.addContact(to);

		listModel[0].addElement(node);

	}

	
	public Vector getRCPTVector()
	{
		Vector output = new Vector();
		Enumeration aktEnum;
		Object aktAdress;

		AddressbookXmlConfig config = composerInterface.config.getAddressbookConfig();
		Vector v;
		for (int i = 0; i < 3; i++)
		{
			v = ListParser.parseVector(config, listModel[i]);
			output.addAll(v);
		}

		

		return output;
	}
	
	public void addAddress(int selected)
	{
		AddressbookXmlConfig config = composerInterface.config.getAddressbookConfig();

			String address = inline[selected].getText();

			if (AddressParser.isValid(address) == true)
			{
				AdapterNode node = config.addContact(address, "", "", address);
				System.out.println("added address: " + address);

				listModel[selected].addElement(node);
				inline[selected].setText("");
			}
			else
			{
				JOptionPane.showMessageDialog(
					composerInterface.composerFrame,
					"This is no valid address!");
			}
	}

	public void actionPerformed(ActionEvent e)
	{
		String command;
		int selected;

		command = e.getActionCommand();
		selected = adressTabs.getSelectedIndex();

		if (command.equals("OK"))
		{
			addAddress( selected );	
			
			
			

			
		}
		else if (command.equals("REMOVE"))
		{
			while (!list[selected].isSelectionEmpty())
			{
				listModel[selected].remove(list[selected].getSelectedIndex());
			}
		}

	}

	public void focusGained(FocusEvent e)
	{
	}

	public void focusLost(FocusEvent e)
	{
		/*
		Object source = e.getSource();
		AddressbookXmlConfig config = composerInterface.config.getAddressbookConfig();
		
		
		for(int i=0; i<ANZ_TABS; i++)
		{
		  if( source == inline[i] ) {
		      if( !inline[i].getText().equals("") )
		      {
		          String str = new String( inline[i].getText() );
		          if ( str.indexOf("@") != -1 )
		          {
		              AdapterNode node = config.addContact( str, "","", str );
		              listModel[i].addElement( node  );
		              inline[i].setText("");
		          }
		          else
		          {
		              JOptionPane.showMessageDialog( composerInterface.composerFrame,
		                                             "This is no valid address!" );
		          }
		          
		          
		          
		          
		      }
		      return;
		  }
		}
		*/
	}

	public void keyPressed(KeyEvent k)
	{
		int selected;

		selected = adressTabs.getSelectedIndex();

		switch (k.getKeyCode())
		{
			case (KeyEvent.VK_DELETE) :
				{
					while (!list[selected].isSelectionEmpty())
					{
						listModel[selected].remove(list[selected].getSelectedIndex());
					}
					break;
				}
		}
	}

	public void keyReleased(KeyEvent k)
	{
	}

	public void keyTyped(KeyEvent k)
	{
	}

}
