// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.gui.composer.action;

import java.awt.event.*;

import javax.swing.*;
import javax.swing.filechooser.*;
import java.util.*;

import org.columba.gui.composer.*;
import org.columba.gui.addressbook.util.*;
import org.columba.protocol.*;
import org.columba.message.*;
import org.columba.config.*;
import org.columba.folder.*;
import org.columba.folder.imap.*;
import org.columba.folder.columba.*;
import org.columba.folder.outbox.*;
import org.columba.gui.action.*;
import org.columba.gui.tree.util.*;
import org.columba.gui.addressbook.*;
import org.columba.util.*;
import org.columba.gui.util.*;
import org.columba.composer.*;
import java.awt.*;

public class ComposerActionListener implements ActionListener {
	private ComposerInterface composerInterface;

	public BasicAction undoAction;
	public BasicAction redoAction;
	public BasicAction attachFileAction;
	public BasicAction attachMessageAction;
	public BasicAction cutAction;
	public BasicAction copyAction;
	public BasicAction pasteAction;
	public BasicAction deleteAction;
	public BasicAction sendAction;
	public BasicAction sendLaterAction;
	public BasicAction newAction;
	public BasicAction saveAction;
	public BasicAction saveAsAction;
	public BasicAction saveDraftAction;
	public BasicAction saveTemplateAction;
	public BasicAction exitAction;
	public BasicAction addressbookAction;
	public BasicAction selectAllAction;
	public BasicAction signAction;
	public BasicAction encryptAction;

	public ComposerActionListener(ComposerInterface iface) {
		composerInterface = iface;
		composerInterface.composerActionListener = this;

		initActions();
	}

	private void initActions() {

		undoAction =
			new BasicAction(
				ResourceLoader.getString("action/global", "menu_edit_undo"),
				ResourceLoader.getString("action/global", "menu_edit_undo"),
				ResourceLoader.getString("action/global", "menu_edit_undo"),
				"UNDO",
				ImageLoader.getImageIcon("undo_small", "Undo16"),
				ImageLoader.getImageIcon("undo", "Undo24"),
				'U',
				KeyStroke.getKeyStroke(KeyEvent.VK_Z, ActionEvent.CTRL_MASK));
		undoAction.addActionListener(this);
		undoAction.setEnabled(true);

		redoAction =
			new BasicAction(
				ResourceLoader.getString("action/global", "menu_edit_redo"),
				ResourceLoader.getString("action/global", "menu_edit_redo"),
				ResourceLoader.getString("action/global", "menu_edit_redo"),
				"REDO",
				ImageLoader.getImageIcon("redo_small", "Redo16"),
				ImageLoader.getImageIcon("redo", "Redo24"),
				'R',
				KeyStroke.getKeyStroke(KeyEvent.VK_Y, ActionEvent.CTRL_MASK));
		redoAction.addActionListener(this);
		redoAction.setEnabled(true);

		cutAction =
			new BasicAction(
				ResourceLoader.getString("action/global", "menu_edit_cut"),
				ResourceLoader.getString("action/global", "menu_edit_cut"),
				ResourceLoader.getString("action/global", "menu_edit_cut"),
				"CUT",
				ImageLoader.getImageIcon("cut_small", "Cut16"),
				ImageLoader.getImageIcon("cut", "Cut24"),
				'T',
				KeyStroke.getKeyStroke(KeyEvent.VK_X, ActionEvent.CTRL_MASK));
		cutAction.addActionListener(this);
		cutAction.setEnabled(true);

		copyAction =
			new BasicAction(
				ResourceLoader.getString("action/global", "menu_edit_copy"),
				ResourceLoader.getString("action/global", "menu_edit_copy"),
				ResourceLoader.getString("action/global", "menu_edit_copy"),
				"COPY",
				ImageLoader.getImageIcon("copy_small", "Copy16"),
				ImageLoader.getImageIcon("copy", "Copy24"),
				'C',
				KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
		copyAction.addActionListener(this);
		copyAction.setEnabled(true);

		pasteAction =
			new BasicAction(
				ResourceLoader.getString("action/global", "menu_edit_paste"),
				ResourceLoader.getString("action/global", "menu_edit_paste"),
				ResourceLoader.getString("action/global", "menu_edit_paste"),
				"PASTE",
				ImageLoader.getImageIcon("paste_small", "Paste16"),
				ImageLoader.getImageIcon("paste", "Paste24"),
				'V',
				KeyStroke.getKeyStroke(KeyEvent.VK_V, ActionEvent.CTRL_MASK));

		pasteAction.addActionListener(this);
		pasteAction.setEnabled(false);

		attachFileAction =
			new BasicAction(
				ResourceLoader.getString("action/composer", "menu_message_attachFile"),
				ResourceLoader.getString("action/composer", "menu_message_attachFile"),
				ResourceLoader.getString("action/composer", "menu_message_attachFile"),
				"ATTACH",
				ImageLoader.getImageIcon("attachment_small", "Save16"),
				ImageLoader.getImageIcon("attachment", "Save24"),
				'N',
				null);
		attachFileAction.addActionListener(this);
		attachFileAction.setEnabled(true);

		attachMessageAction =
			new BasicAction(
				ResourceLoader.getString("action/composer", "menu_message_attachMessage"),
				ResourceLoader.getString("action/composer", "menu_message_attachMessage"),
				ResourceLoader.getString("action/composer", "menu_message_attachMessage"),
				"QUOTE",
				ImageLoader.getImageIcon("attachment_small", "Save16"),
				ImageLoader.getImageIcon("attachment", "Save24"),
				'N',
				null);
		attachMessageAction.addActionListener(this);
		attachMessageAction.setEnabled(true);

		sendAction =
			new BasicAction(
				ResourceLoader.getString("action/composer", "menu_file_send"),
				ResourceLoader.getString("action/composer", "menu_file_send"),
				ResourceLoader.getString("action/composer", "menu_file_send"),
				"SEND",
				ImageLoader.getImageIcon("send_small", "SendMail16"),
				ImageLoader.getImageIcon("send", "SendMail24"),
				'N',
				null);
		sendAction.addActionListener(this);
		sendAction.setEnabled(true);

		sendLaterAction =
			new BasicAction(
				ResourceLoader.getString("action/composer", "menu_file_sendlater"),
				ResourceLoader.getString("action/composer", "menu_file_sendlater"),
				ResourceLoader.getString("action/composer", "menu_file_sendlater"),
				"SENDLATER",
				null,
				null,
				'0',
				null);
		sendLaterAction.addActionListener(this);
		sendLaterAction.setEnabled(true);

		saveAction =
			new BasicAction(
				ResourceLoader.getString("action/composer", "menu_file_save"),
				ResourceLoader.getString("action/composer", "menu_file_save"),
				ResourceLoader.getString("action/composer", "menu_file_save"),
				"SAVE",
				ImageLoader.getImageIcon("save_small", "Save16"),
				ImageLoader.getImageIcon("save", "Save24"),
				'S',
				KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK));
		saveAction.addActionListener(this);
		saveAction.setEnabled(false);

		saveAsAction =
			new BasicAction(
				ResourceLoader.getString("action/composer", "menu_file_saveas"),
				ResourceLoader.getString("action/composer", "menu_file_saveas"),
				ResourceLoader.getString("action/composer", "menu_file_saveas"),
				"SAVEAS",
				ImageLoader.getImageIcon("saveas_small", "SaveAs16"),
				ImageLoader.getImageIcon("saveas", "SaveAs24"),
				'A',
				KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		saveAsAction.addActionListener(this);
		saveAsAction.setEnabled(false);

		saveDraftAction =
			new BasicAction(
				ResourceLoader.getString("action/composer", "menu_file_savedraft"),
				ResourceLoader.getString("action/composer", "menu_file_savedraft"),
				ResourceLoader.getString("action/composer", "menu_file_savedraft"),
				"SAVEDRAFT",
				null,
				null,
				'0',
				null);
		saveDraftAction.addActionListener(this);
		saveDraftAction.setEnabled(true);

		saveAsAction.setEnabled(false);

		saveTemplateAction =
			new BasicAction(
				ResourceLoader.getString("action/composer", "menu_file_savetemplate"),
				ResourceLoader.getString("action/composer", "menu_file_savetemplate"),
				ResourceLoader.getString("action/composer", "menu_file_savetemplate"),
				"SAVETEMPLATE",
				null,
				null,
				'0',
				null);
		saveTemplateAction.addActionListener(this);
		saveTemplateAction.setEnabled(true);

		newAction =
			new BasicAction(
				ResourceLoader.getString("action/composer", "menu_file_new"),
				ResourceLoader.getString("action/composer", "menu_file_new"),
				ResourceLoader.getString("action/composer", "menu_file_new"),
				"NEW",
				ImageLoader.getImageIcon("compose_small", "ComposeMail16"),
				ImageLoader.getImageIcon("compose", "ComposeMail24"),
				'M',
				KeyStroke.getKeyStroke(KeyEvent.VK_M, ActionEvent.CTRL_MASK));
		newAction.addActionListener(this);
		newAction.setEnabled(true);

		exitAction =
			new BasicAction(
				ResourceLoader.getString("action/global", "menu_file_close"),
				ResourceLoader.getString("action/global", "menu_file_close"),
				ResourceLoader.getString("action/global", "menu_file_close"),
				"EXIT",
				ImageLoader.getImageIcon("close_small", "Home16"),
				ImageLoader.getImageIcon("close", "Home24"),
				'C',
				KeyStroke.getKeyStroke(KeyEvent.VK_W, ActionEvent.CTRL_MASK));
		exitAction.addActionListener(this);
		exitAction.setEnabled(true);

		addressbookAction =
			new BasicAction(
				ResourceLoader.getString("action/composer", "menu_message_addressbook"),
				ResourceLoader.getString("action/composer", "menu_message_addressbook"),
				ResourceLoader.getString("action/composer", "menu_message_addressbook"),
				"ADDRESSBOOK",
				ImageLoader.getImageIcon("contact_small", "History16"),
				ImageLoader.getImageIcon("contact", "History24"),
				'B',
				null);
		addressbookAction.addActionListener(this);
		addressbookAction.setEnabled(true);

		deleteAction =
			new BasicAction(
				ResourceLoader.getString("action/global", "menu_edit_delete"),
				ResourceLoader.getString("action/global", "menu_edit_delete"),
				ResourceLoader.getString("action/global", "menu_edit_delete"),
				"DELETE",
				ImageLoader.getImageIcon("delete_small", "Delete16"),
				ImageLoader.getImageIcon("delete", "Delete24"),
				'D',
				KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
		deleteAction.addActionListener(this);
		deleteAction.setEnabled(true);

		selectAllAction =
			new BasicAction(
				ResourceLoader.getString("action/global", "menu_edit_selectall"),
				ResourceLoader.getString("action/global", "menu_edit_selectall"),
				ResourceLoader.getString("action/global", "menu_edit_selectall"),
				"SELECTALL",
				null,
				null,
				'A',
				KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
		selectAllAction.addActionListener(this);
		selectAllAction.setEnabled(false);

		signAction =
			new BasicAction(
				ResourceLoader.getString("action/composer", "menu_message_sign"),
				ResourceLoader.getString("action/composer", "menu_message_sign"),
				ResourceLoader.getString("action/composer", "menu_message_sign"),
				"SIGN",
				null,
				null,
				'0',
				null);
		signAction.addActionListener(this);
		signAction.setEnabled(true);

		encryptAction =
			new BasicAction(
				ResourceLoader.getString("action/composer", "menu_message_encrypt"),
				ResourceLoader.getString("action/composer", "menu_message_encrypt"),
				ResourceLoader.getString("action/composer", "menu_message_encrypt"),
				"ENCRYPT",
				ImageLoader.getImageIcon("encrypt_small", "Delete16"),
				ImageLoader.getImageIcon("encrypt", "Delete24"),
				'0',
				null);
		encryptAction.addActionListener(this);
		encryptAction.setEnabled(true);

	}

	protected boolean checkInformation() {

		if (composerInterface.composerHeader.getToSize() == 0) {
			String to = composerInterface.composerHeader.getToTextFieldString();
			if (to.length() == 0) {
				System.out.println("no recipient");
				NotifyDialog dialog = new NotifyDialog(composerInterface.mainInterface);
				dialog.showDialog("No recipients specified!");

				return false;
			} else {
				// add string to list
				if (AddressParser.isValid(to) == true) {
					composerInterface.composerHeader.addAddress(0);
				} else {
					NotifyDialog dialog = new NotifyDialog(composerInterface.mainInterface);
					dialog.showDialog("No valid email address specified!");
					return false;
				}
			}

		}

		String cc = composerInterface.composerHeader.getCcTextFieldString();
		if (cc.length() > 0) {

			// add string to list
			if (AddressParser.isValid(cc) == true) {
				composerInterface.composerHeader.addAddress(1);
			} else {
				NotifyDialog dialog = new NotifyDialog(composerInterface.mainInterface);
				dialog.showDialog("No valid email address specified!");
				return false;
			}
		}

		String bcc = composerInterface.composerHeader.getBccTextFieldString();
		if (bcc.length() > 0) {

			// add string to list
			if (AddressParser.isValid(bcc) == true) {
				composerInterface.composerHeader.addAddress(2);
			} else {
				NotifyDialog dialog = new NotifyDialog(composerInterface.mainInterface);
				dialog.showDialog("No valid email address specified!");
				return false;
			}
		}

		String subject = composerInterface.composerEditor.getSubject();
		if (subject.length() == 0) {
			subject = new String("no subject");
			SubjectDialog dialog = new SubjectDialog(composerInterface.composerFrame);
			dialog.showDialog(subject);
			if (dialog.success() == true)
				subject = dialog.getSubject();

			composerInterface.composerEditor.setSubject(subject);
		}

		return true;
	}

	public void actionPerformed(ActionEvent e) {
		String command;

		command = e.getActionCommand();

		if (command.equals("NEW")) {
			ComposerFrame frame = new ComposerFrame(composerInterface.mainInterface);

		}

		if (command.equals("SEND")) {

			/*
			SwingWorker sendWorker = new SendWorker(composerInterface);
			
			sendWorker.register(composerInterface.taskManager);
			sendWorker.start();
			*/
			if (checkInformation() == false)
				return;

			ComposerOperation op =
				new ComposerOperation(
					Operation.COMPOSER_SEND,
					0,
					composerInterface.composerFrame);

			composerInterface.crossbar.operate(op);

			return;
		}
		if (command.equals("SENDLATER")) {
			if (checkInformation() == false)
				return;

			ComposerOperation op =
				new ComposerOperation(
					Operation.COMPOSER_SENDLATER,
					0,
					composerInterface.composerFrame);

			composerInterface.crossbar.operate(op);

		}
		if (command.equals("SAVEDRAFT")) {
			if (checkInformation() == false)
				return;

			ComposerOperation op =
				new ComposerOperation(
					Operation.COMPOSER_SAVEDRAFT,
					0,
					composerInterface.composerFrame);

			composerInterface.crossbar.operate(op);

		}
		if (command.equals("SAVETEMPLATE")) {
			if (checkInformation() == false)
				return;

			ComposerOperation op =
				new ComposerOperation(
					Operation.COMPOSER_SAVETEMPLATE,
					0,
					composerInterface.composerFrame);

			composerInterface.crossbar.operate(op);

		}
		if (command.equals("ATTACH")) {
			composerInterface.composerAttachment.attachFile();
			return;
		}
		if (command.equals("QUOTE")) {
			return;
		}
		if (command.equals("REMOVE")) {
			composerInterface.composerAttachment.removeFile();
			return;
		}
		if (command.equals("EXIT")) {
			composerInterface.composerFrame.saveWindowPosition();
			composerInterface.composerFrame.dispose();
			return;
		}
		if (command.equals("UNDO")) {
			composerInterface.message.Undo();
			return;
		}
		if (command.equals("REDO")) {
			composerInterface.message.Redo();
			return;
		}
		if (command.equals("CUT")) {
			pasteAction.setEnabled(true);
			composerInterface.composerEditor.getEditor().cut();
			return;
		}
		if (command.equals("COPY")) {
			pasteAction.setEnabled(true);
			composerInterface.composerEditor.getEditor().copy();
			return;
		}
		if (command.equals("PASTE")) {
			composerInterface.composerEditor.getEditor().paste();
			return;
		}
		if (command.equals("ADDRESSBOOK")) {
			System.out.println("addressbook");
			SelectAddressDialog dialog =
				new SelectAddressDialog(
					composerInterface.composerFrame,
					composerInterface.config.getAddressbookConfig(),
					composerInterface.composerHeader.getListModels());
			dialog.setVisible(true);

		}
		if (command.equals("SIGN")) {
			System.out.println("sign");
			composerInterface.composerEditor.changeSign();
		}
		if (command.equals("ENCRYPT")) {
			System.out.println("encrypt");
			composerInterface.composerEditor.changeEncrypt();

		}

	}
}

/*
final class SendWorker extends SwingWorker
{
	SendableMessage message;
	ComposerInterface composerInterface;

	public SendWorker(ComposerInterface ci)
	{
		composerInterface = ci;
	}

	public Object construct()
	{

		String content;

		int uid = composerInterface.composerEditor.getAccountUid();

		message = composerInterface.messageComposer.compose(uid);

		message.setRecipients(composerInterface.composerHeader.getRCPTVector());

		unregister();
		return null;
	}

	public void finished()
	{
		try
		{
			composerInterface.composerFolder.workerAdd(message);
		}
		catch (Exception e)
		{
			System.out.println(e);
		}

		composerInterface.crossbar.operate(
			new OutboxOperation(
				Operation.SMTP_SENDCOMPOSED,
				10,
				composerInterface.composerFolder.getUids(),
				composerInterface.composerFolder));

		composerInterface.composerFrame.dispose();
	}
	
	
	
	
}
*/