package org.columba.gui.config.account;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

import org.columba.main.MainInterface;
import org.columba.config.*;
import org.columba.folder.*;
import org.columba.pop3.*;
import org.columba.gui.util.*;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

public class AccountDialog implements ActionListener
{
    private JDialog dialog;

    private MainInterface mainInterface;
    private AccountItem accountItem;

    private IdentityPanel identityPanel;
    private IncomingServerPanel incomingServerPanel;
    private OutgoingServerPanel outgoingServerPanel;
    private SecurityPanel securityPanel;
    private SpecialFoldersPanel specialFoldersPanel;

    private JButton okButton;
    private JButton cancelButton;
    private JButton helpButton;

    public AccountDialog( MainInterface mainInterface, AccountItem item)
    {
        dialog = DialogStore.getDialog();

        this.mainInterface = mainInterface;
        this.accountItem = item;


        dialog.addWindowListener(
            new WindowAdapter()
                {
                    public void windowClosing(WindowEvent e)
                    {
			dialog.setVisible(false);
                    }
                }
            );

	initComponents();
        dialog.pack();
        dialog.setVisible(true);
        //setSize( new Dimension( 600,500 ) );
    }


    protected void initComponents()
    {
        dialog.getContentPane().setLayout( new BorderLayout() );

        JTabbedPane tp = new JTabbedPane();
        tp.setBorder( BorderFactory.createEmptyBorder( 0,10,0,10 ) );
	tp.setTabLayoutPolicy( JTabbedPane.SCROLL_TAB_LAYOUT );

        IdentityItem identityItem = accountItem.getIdentityItem();
        identityPanel = new IdentityPanel( mainInterface, accountItem, identityItem );
        tp.add( ResourceLoader.getString("dialog/mail/account/accountdialog","identity"), identityPanel ); //$NON-NLS-1$

        if ( accountItem.isPopAccount() )
        {
            PopItem popItem = accountItem.getPopItem();

            incomingServerPanel = new IncomingServerPanel( mainInterface, accountItem, popItem );
            tp.add( ResourceLoader.getString("dialog/mail/account/accountdialog","incomingserverpop3"), incomingServerPanel ); //$NON-NLS-1$
        }
        else
        {
            ImapItem imapItem = accountItem.getImapItem();
            incomingServerPanel = new IncomingServerPanel( mainInterface, accountItem, imapItem );
            tp.add( ResourceLoader.getString("dialog/mail/account/accountdialog","incomingserverimap"), incomingServerPanel ); //$NON-NLS-1$
        }

        SmtpItem smtpItem = accountItem.getSmtpItem();
        outgoingServerPanel = new OutgoingServerPanel( mainInterface, smtpItem );
        tp.add( ResourceLoader.getString("dialog/mail/account/accountdialog","outgoingserver"), outgoingServerPanel ); //$NON-NLS-1$

        specialFoldersPanel = new SpecialFoldersPanel( mainInterface, accountItem, accountItem.getSpecialFoldersItem() );
        tp.add( ResourceLoader.getString("dialog/mail/account/accountdialog","specialfolders"), specialFoldersPanel ); //$NON-NLS-1$

        securityPanel = new SecurityPanel( mainInterface, accountItem.getPGPItem() );
        tp.add( ResourceLoader.getString("dialog/mail/account/accountdialog","security"), securityPanel ); //$NON-NLS-1$

        dialog.getContentPane().add( tp, BorderLayout.CENTER );



        JPanel bottom = new JPanel();
        bottom.setLayout( new BorderLayout() );
        //bottom.setLayout( new BoxLayout( bottom, BoxLayout.X_AXIS ) );
        bottom.setBorder( BorderFactory.createEmptyBorder( 10,10,10,10 ) );

        //bottom.add( Box.createHorizontalStrut());

        /*
        helpButton = new JButton("Help");
        helpButton.addActionListener( this );
        helpButton.setActionCommand( "HELP" );
        */

        cancelButton = new JButton(ResourceLoader.getString("dialog/global","cancel")); //$NON-NLS-1$ //$NON-NLS-2$
        cancelButton.addActionListener( this );
        cancelButton.setActionCommand("CANCEL"); //$NON-NLS-1$

        okButton = new JButton(ResourceLoader.getString("dialog/global","ok")); //$NON-NLS-1$ //$NON-NLS-2$
	okButton.addActionListener( this );
	okButton.setActionCommand("OK"); //$NON-NLS-1$
        okButton.setDefaultCapable(true);
        dialog.getRootPane().setDefaultButton( okButton );

        helpButton = new JButton(ResourceLoader.getString("dialog/global","help")); //$NON-NLS-1$ //$NON-NLS-2$


        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout( new GridLayout( 1,3,10,0 ) );
        buttonPanel.add( helpButton );
        buttonPanel.add( cancelButton );
        buttonPanel.add( okButton );

        //bottom.add( Box.createHorizontalGlue() );

        bottom.add( buttonPanel, BorderLayout.EAST );


        /*
        helpButton.setSize( okButton.getSize() );
        bottom.add( helpButton );


        */

	dialog.getContentPane().add( bottom, BorderLayout.SOUTH );
    }

    public void actionPerformed( ActionEvent e )
    {
        String action = e.getActionCommand();

        if ( action.equals("OK") ) //$NON-NLS-1$
        {

            identityPanel.updateComponents( false );
            incomingServerPanel.updateComponents( false );
            outgoingServerPanel.updateComponents( false );
            securityPanel.updateComponents( false );
            specialFoldersPanel.updateComponents( false );


            if ( accountItem.isPopAccount() )
            {
                // update pop3 fetch menu
                mainInterface.mainFrame.getMenu().updatePopServerMenu();

                int uid = accountItem.getUid();
                PopServer server = mainInterface.popServerCollection.uidGet(uid);
                server.restartTimer();

                mainInterface.popServerCollection.enableMailCheckIcon();
            }
            else
            {
                // update tree label
                int uid = accountItem.getUid();

                Folder folder = mainInterface.treeViewer.getFolderTree().getImapFolder( uid );
                folder.setName( accountItem.getName() );
            }

            dialog.setVisible( false );
        }
        else if ( action.equals("CANCEL") ) //$NON-NLS-1$
        {
            dialog.setVisible( false );
        }


    }

}
