// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.gui.config.account;

import javax.swing.*;
import javax.swing.border.*;

import javax.swing.Box;
import java.awt.*;

import org.columba.config.*;
import org.columba.gui.util.*;
import org.columba.message.*;
import org.columba.gui.tree.util.*;
import org.columba.folder.*;
import org.columba.gui.tree.*;
import org.columba.main.MainInterface;

public class IdentityPanel extends DefaultPanel
{

	private JLabel nameLabel;
	private JTextField nameTextField;
	private JLabel addressLabel;
	private JTextField addressTextField;
	private JLabel organisationLabel;
	private JTextField organisationTextField;
	private JLabel replyaddressLabel;
	private JTextField replyaddressTextField;
	private JLabel accountnameLabel;
	private JTextField accountnameTextField;

	private JCheckBox attachsignatureCheckBox;
	private JTextArea attachsignatureTextArea;
	private JButton okButton;
	private JButton cancelButton;
	private JButton helpButton;

	private IdentityItem identityItem;
	private AccountItem accountItem;

	//private ConfigFrame frame;

	public IdentityPanel(
		MainInterface mainInterface,
		AccountItem account,
		IdentityItem identity)
	{
		super(mainInterface);

		//this.frame = frame;
		this.accountItem = account;
		this.identityItem = identity;

		initComponents();

		updateComponents(true);

	}

	protected String getAccountName()
	{
		return accountnameTextField.getText();
	}

	protected String getAddress()
	{
		return addressTextField.getText();
	}

	protected void updateComponents(boolean b)
	{
		if (b)
		{
			accountnameTextField.setText(accountItem.getName());
			nameTextField.setText(identityItem.getName());
			addressTextField.setText(identityItem.getAddress());
			replyaddressTextField.setText(identityItem.getReplyAddress());
			organisationTextField.setText(identityItem.getOrganisation());
			attachsignatureTextArea.setText(identityItem.getSignature());

			if (identityItem.getAttachSignature() == true)
				attachsignatureCheckBox.setSelected(true);
			else
				attachsignatureCheckBox.setSelected(false);

		}
		else
		{
			if (nameTextField.getText() != null)
				identityItem.setName(nameTextField.getText());
			identityItem.setAddress(addressTextField.getText());
			identityItem.setOrganisation(organisationTextField.getText());
			identityItem.setReplyAddress(replyaddressTextField.getText());
			identityItem.setSignature(attachsignatureTextArea.getText());

			if (attachsignatureCheckBox.isSelected() == true)
				identityItem.setAttachSignature(true);
			else
				identityItem.setAttachSignature(false);

			accountItem.setName(accountnameTextField.getText());
		}
	}

	protected void initComponents()
	{
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		JPanel accountPanel = new JPanel();
		accountPanel.setBorder(BorderFactory.createEmptyBorder(0, 0, 10, 0));
		accountPanel.setLayout(new BoxLayout(accountPanel, BoxLayout.X_AXIS));
		//accountPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );
		accountnameLabel =
			new JLabel(
				ResourceLoader.getString("dialog/mail/account/identitypanel", "accountname"));
		//$NON-NLS-1$
		accountPanel.add(accountnameLabel);

		accountPanel.add(Box.createRigidArea(new java.awt.Dimension(5, 0)));
		accountPanel.add(Box.createHorizontalGlue());

		accountnameTextField = new JTextField(22);
		//accountnameTextField.setMaximumSize( new java.awt.Dimension( 150, 25 ) );
		accountPanel.add(accountnameTextField);

		//accountPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

		add(accountPanel, BorderLayout.NORTH);

		//add( Box.createRigidArea( new java.awt.Dimension(0,10) ) );

		JPanel mainMiddle = new JPanel();
		mainMiddle.setLayout(new BorderLayout());
		//mainMiddle.setBorder( BorderFactory.createTitledBorder("Identity") );
		JPanel middle = new JPanel();
		middle.setBorder(
			javax.swing.BorderFactory.createTitledBorder(
				javax.swing.BorderFactory.createEtchedBorder(),
				ResourceLoader.getString("dialog/mail/account/identitypanel", "identity")));
		//$NON-NLS-1$
		//middle.setBorder( BorderFactory.createTitledBorder("Identity") );
		middle.setLayout(new BorderLayout());

		JPanel innerPanel = new JPanel();
		innerPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		innerPanel.setLayout(new BorderLayout());

		JPanel leftPanel = new JPanel();
		leftPanel.setLayout(new GridLayout(0, 1, 50, 5));

		nameLabel =
			new JLabel(
				ResourceLoader.getString("dialog/mail/account/identitypanel", "yourname"));
		//$NON-NLS-1$
		leftPanel.add(nameLabel);
		addressLabel =
			new JLabel(
				ResourceLoader.getString("dialog/mail/account/identitypanel", "address"));
		//$NON-NLS-1$
		leftPanel.add(addressLabel);
		replyaddressLabel =
			new JLabel(
				ResourceLoader.getString(
					"dialog/mail/account/identitypanel",
					"replytoaddress"));
		//$NON-NLS-1$
		leftPanel.add(replyaddressLabel);
		organisationLabel =
			new JLabel(
				ResourceLoader.getString("dialog/mail/account/identitypanel", "organisation"));
		//$NON-NLS-1$
		leftPanel.add(organisationLabel);
		attachsignatureCheckBox =
			new JCheckBox(
				ResourceLoader.getString(
					"dialog/mail/account/identitypanel",
					"attachthissignature"));
		//$NON-NLS-1$
		//leftPanel.add( attachsignatureCheckBox );
		//leftPanel.add( Box.createVerticalGlue() );

		JPanel rightPanel = new JPanel();
		rightPanel.setLayout(new GridLayout(0, 1, 50, 5));

		nameTextField = new JTextField(30);
		rightPanel.add(nameTextField);
		addressTextField = new JTextField(30);
		rightPanel.add(addressTextField);
		replyaddressTextField = new JTextField(30);
		rightPanel.add(replyaddressTextField);
		organisationTextField = new JTextField(30);
		rightPanel.add(organisationTextField);
		//attachsignatureTextArea = new JTextArea("",5,30);
		//attachsignatureTextArea.setEnabled(true);
		//attachsignatureTextArea.setAlignmentY( Component.TOP_ALIGNMENT );
		//attachsignatureTextArea.setPreferredSize( new java.awt.Dimension( 150,25) );
		//JScrollPane scrollPane = new JScrollPane( attachsignatureTextArea );
		//rightPanel.add( attachsignatureTextArea );

		//rightPanel.add( Box.createVerticalGlue() );

		innerPanel.add(leftPanel, BorderLayout.CENTER);
		innerPanel.add(rightPanel, BorderLayout.EAST);

		JPanel attachsignaturePanel = new JPanel();
		attachsignaturePanel.setLayout(new BorderLayout());
		attachsignaturePanel.setBorder(BorderFactory.createEmptyBorder(10, 0, 0, 0));
		//attachsignaturePanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
		attachsignatureCheckBox =
			new JCheckBox(
				ResourceLoader.getString(
					"dialog/mail/account/identitypanel",
					"attachthissignature"));
		//$NON-NLS-1$
		//attachsignatureCheckBox.setAlignmentY( Component.TOP_ALIGNMENT );

		attachsignaturePanel.add(attachsignatureCheckBox, BorderLayout.NORTH);
		//attachsignaturePanel.add( Box.createHorizontalGlue() );
		attachsignatureTextArea = new JTextArea("", 8, 15); //$NON-NLS-1$
		//attachsignatureTextArea.setEnabled(true);
		//attachsignatureTextArea.setAlignmentY( Component.TOP_ALIGNMENT );
		//attachsignatureTextArea.setPreferredSize( new java.awt.Dimension( 150,25) );
		JScrollPane scrollPane = new JScrollPane(attachsignatureTextArea);
		attachsignaturePanel.add(scrollPane, BorderLayout.CENTER);
		//attachsignaturePanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );

		innerPanel.add(attachsignaturePanel, BorderLayout.SOUTH);
		middle.add(innerPanel, BorderLayout.CENTER);
		//middle.add( Box.createVerticalGlue(), BorderLayout.SOUTH );

		mainMiddle.add(middle, BorderLayout.NORTH);

		add(mainMiddle, BorderLayout.CENTER);

		//middle.add( Box.createRigidArea( new java.awt.Dimension(0,5) ) );

		/*
		JPanel labelPanel = new JPanel();
		labelPanel.setLayout( new BoxLayout( labelPanel, BoxLayout.X_AXIS ) );
		labelPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
		
		labelPanel.add( nameLabel );
		labelPanel.add( Box.createHorizontalGlue() );
		nameTextField = new JTextField ( 18 );
		nameTextField.setMaximumSize( new java.awt.Dimension( 150,25) );
		labelPanel.add( nameTextField );
		labelPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
		
		//middle.add( labelPanel );
		
		//middle.add( Box.createRigidArea( new java.awt.Dimension(0,5) ) );
		*/

		/*
		JPanel addressPanel = new JPanel();
		addressPanel.setLayout( new BoxLayout( addressPanel, BoxLayout.X_AXIS ) );
		addressPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
		addressLabel = new JLabel ( "E-Message:");
		addressPanel.add( addressLabel );
		addressPanel.add( Box.createHorizontalGlue() );
		addressTextField = new JTextField ( 18 );
		addressTextField.setMaximumSize( new java.awt.Dimension( 150,25) );
		addressPanel.add( addressTextField );
		addressPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
		//middle.add( addressPanel );
		
		//middle.add( Box.createRigidArea( new java.awt.Dimension(0,5) ) );
		*/

		/*
		JPanel replyaddressPanel = new JPanel();
		replyaddressPanel.setLayout( new BoxLayout( replyaddressPanel, BoxLayout.X_AXIS ) );
		replyaddressPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
		replyaddressLabel = new JLabel ("Reply-to Address");
		replyaddressPanel.add( replyaddressLabel );
		replyaddressPanel.add( Box.createHorizontalGlue() );
		replyaddressTextField = new JTextField ( 18 );
		replyaddressTextField.setMaximumSize( new java.awt.Dimension( 150, 25 ) );
		replyaddressPanel.add( replyaddressTextField );
		replyaddressPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
		*/
		//middle.add( replyaddressPanel );

		//middle.add( Box.createRigidArea( new java.awt.Dimension(0,5) ) );

		/*
		JPanel organisationPanel = new JPanel();
		organisationPanel.setLayout( new BoxLayout( organisationPanel, BoxLayout.X_AXIS ) );
		organisationPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
		organisationLabel = new JLabel ("Organisation:");
		organisationPanel.add( organisationLabel );
		organisationPanel.add( Box.createHorizontalGlue() );
		organisationTextField = new JTextField ( 18 );
		organisationTextField.setMaximumSize( new java.awt.Dimension( 150,25) );
		organisationPanel.add( organisationTextField );
		organisationPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
		//middle.add( organisationPanel );
		*/

		//middle.add( Box.createRigidArea( new java.awt.Dimension(0,5) ) );

		/*
		JPanel attachsignaturePanel = new JPanel();
		attachsignaturePanel.setLayout( new BoxLayout( attachsignaturePanel, BoxLayout.X_AXIS ) );
		attachsignaturePanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
		attachsignatureCheckBox = new JCheckBox("Attach this signature:");
		attachsignatureCheckBox.setAlignmentY( Component.TOP_ALIGNMENT );
		
		
		attachsignaturePanel.add( attachsignatureCheckBox );
		attachsignaturePanel.add( Box.createHorizontalGlue() );
		attachsignatureTextArea = new JTextArea("",5,15);
		attachsignatureTextArea.setEnabled(true);
		attachsignatureTextArea.setAlignmentY( Component.TOP_ALIGNMENT );
		attachsignatureTextArea.setPreferredSize( new java.awt.Dimension( 150,25) );
		JScrollPane scrollPane = new JScrollPane( attachsignatureTextArea );
		attachsignaturePanel.add( scrollPane  );
		attachsignaturePanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
		*/

	}

	public boolean isFinished()
	{
		boolean result = true;

		String name = getAccountName();
		String address = getAddress();

		if (name.length() == 0)
		{
			result = false;
			JOptionPane.showMessageDialog(
				mainInterface.mainFrame,
				ResourceLoader.getString("dialog/mail/account/identitypanel", "namelabel"));
			//$NON-NLS-1$
		}
		else if (address.length() == 0)
		{
			result = false;
			JOptionPane.showMessageDialog(
				mainInterface.mainFrame,
				ResourceLoader.getString("dialog/mail/account/identitypanel", "addresslabel"));
			//$NON-NLS-1$
		}
		else
		{
			result = true;
		}

		return result;
	}

	protected void makeChanges(boolean newAccount)
	{

		String panelName = accountItem.getName();
		//FIXME
		//frame.getSelected().setName( panelName );

		if (accountItem.isPopAccount() == true)
		{
			//PopItem item = accountItem.getPopItem();
			//item.setFolderName( accountItem.getName() );

			if (newAccount == true)
			{
				mainInterface.popServerCollection.add(accountItem);
				System.out.println("add popserver");
			}

			mainInterface.mainFrame.getMenu().updatePopServerMenu();

		}
		else
		{
			ImapItem item = accountItem.getImapItem();

			if (newAccount == true)
			{
				System.out.println("create Imap Folder");

				Folder parentFolder =
					mainInterface.treeViewer.getFolderTree().addImapRootFolder(
						accountItem.getName(),
						item,
						accountItem.getUid());

				//FIXME
				/*
				TreeNodeList list = new TreeNodeList("/");
				frame.mainInterface.treeViewer.addFolder( list, accountItem.getName(),
				                                          "system", "false", "imaproot",
				                                          "false", "false", "false",accountItem.getUid() );
				
				list = new TreeNodeList("/"+accountItem.getName() );
				frame.mainInterface.treeViewer.addFolder( list, "Inbox",
				                                          "system", "true", "imap",
				                                          "false", "false", "false",accountItem.getUid() );
				                                          */

			}
			else
			{
				int uid = accountItem.getUid();

				Folder folder = mainInterface.treeViewer.getFolderTree().getImapFolder(uid);
				folder.setName(accountItem.getName());

			}

		}

	}
}