// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.config.account;

/*
 * PopPanel.java
 *
 * Created on 1. November 2000, 23:39
 */

import javax.swing.*;
import javax.swing.border.*;

import java.awt.*;
import java.awt.event.*;

import org.columba.gui.util.*;
import org.columba.config.*;
import org.columba.folder.*;
import org.columba.message.*;
import org.columba.gui.tree.util.*;
import org.columba.main.MainInterface;

/**
 *
 * @author  freddy
 * @version
 */
public class IncomingServerPanel extends DefaultPanel implements ActionListener
{

    private JLabel loginLabel;
    private JTextField loginTextField;
    private JLabel passwordLabel;
    private JTextField passwordTextField;
    private JLabel hostLabel;
    private JTextField hostTextField;
    private JLabel portLabel;
    private JTextField portTextField;
    private JLabel typeLabel;
    private JComboBox typeComboBox;
    private JButton okButton;
    private JButton cancelButton;
    private JButton helpButton;
    private JCheckBox storePasswordCheckBox;

    private PopAttributPanel popPanel;
    private ImapAttributPanel imapPanel;

    private PopItem popItem=null;
    private ImapItem imapItem=null;
    private AccountItem accountItem;

    //private ConfigFrame frame;

    public IncomingServerPanel(MainInterface mainInterface, AccountItem account, PopItem item)
    {
        super(mainInterface);
        //super( frame, item );

	//this.frame = frame;
        this.accountItem = account;
	this.popItem = item;

        initComponents ();

        updateComponents( true );

    }

    public IncomingServerPanel(MainInterface mainInterface, AccountItem account, ImapItem item)
    {
        //super( frame, item );
        super(mainInterface);
	//this.frame = frame;
            this.accountItem = account;
	this.imapItem = item;


        initComponents ();

        updateComponents( true );
    }


    public String getHost()
    {
        return hostTextField.getText();
    }

    public String getLogin()
    {
        return loginTextField.getText();
    }


    public boolean isPopAccount()
    {
	if ( popItem != null ) return true;
	else return false;
    }

    /*
    public String getDestinationFolder()
    {
        return popPanel.getDestinationFolder();
    }
    */

    protected void updateComponents( boolean b )
    {

        if ( b )
        {
            if ( isPopAccount() )
            {
                loginTextField.setText ( popItem.getUser() );
                passwordTextField.setText ( popItem.getPassword() );
                hostTextField.setText ( popItem.getHost() );
                portTextField.setText ( popItem.getPort() );

                if ( popItem.isSavePassword() )
                   storePasswordCheckBox.setSelected(true);

                popPanel.updateComponents( true );

            }
            else
            {

                loginTextField.setText ( imapItem.getUser() );
                passwordTextField.setText ( imapItem.getPassword() );
                hostTextField.setText ( imapItem.getHost() );
                portTextField.setText ( imapItem.getPort() );


                if ( imapItem.isSavePassword() )
                   storePasswordCheckBox.setSelected(true);

                imapPanel.updateComponents( true );
            }
        }
        else
        {
            if ( isPopAccount() )
            {
                popItem.setUser( loginTextField.getText() );
                popItem.setHost( hostTextField.getText() );
                popItem.setPassword( passwordTextField.getText() );
                popItem.setPort( portTextField.getText() );

                if ( storePasswordCheckBox.isSelected() == true )
                   popItem.setSavePassword("true"); //$NON-NLS-1$
                else
                   popItem.setSavePassword("false"); //$NON-NLS-1$

                popPanel.updateComponents( false );
            }
            else
            {

                imapItem.setUser( loginTextField.getText() );
                imapItem.setHost( hostTextField.getText() );
                imapItem.setPassword( passwordTextField.getText() );
                imapItem.setPort( portTextField.getText() );

                if ( storePasswordCheckBox.isSelected() == true )
                   imapItem.setSavePassword("true"); //$NON-NLS-1$
                else
                   imapItem.setSavePassword("false"); //$NON-NLS-1$

                imapPanel.updateComponents( false );
            }


        }
    }


    protected void initComponents ()
    {

        setLayout( new BorderLayout() );

        setBorder( BorderFactory.createEmptyBorder( 10,10,10,10 ) );





        loginLabel = new JLabel ();
	loginLabel.setText (ResourceLoader.getString("dialog/mail/account/incomingserverpanel", "Login")); //$NON-NLS-1$
        loginTextField = new JTextField (30);

        passwordLabel = new JLabel ();
	passwordLabel.setText (ResourceLoader.getString("dialog/mail/account/incomingserverpanel", "Password")); //$NON-NLS-1$
        passwordTextField = new JTextField (30);


        hostLabel = new JLabel ();
	hostLabel.setText (ResourceLoader.getString("dialog/mail/account/incomingserverpanel", "Host")); //$NON-NLS-1$

        hostTextField = new JTextField (30);


        portLabel = new JLabel ();
	portLabel.setText (ResourceLoader.getString("dialog/mail/account/incomingserverpanel", "Port")); //$NON-NLS-1$
        portTextField = new JTextField (30);


        JPanel mainMiddle = new JPanel();
        mainMiddle.setLayout( new BorderLayout() );

	JPanel middle = new JPanel();
        middle.setBorder( javax.swing.BorderFactory.createTitledBorder(
                   javax.swing.BorderFactory.createEtchedBorder(),
                   ResourceLoader.getString("dialog/mail/account/incomingserverpanel", "_Server_Settings") ) ); //$NON-NLS-1$

        middle.setLayout( new BorderLayout() );

        JPanel innerPanel = new JPanel();
        innerPanel.setBorder( BorderFactory.createEmptyBorder(10,10,10,10) );
        innerPanel.setLayout( new BorderLayout() );

        JPanel leftPanel = new JPanel();
        leftPanel.setLayout( new GridLayout(0,1,50,5) );


        leftPanel.add( loginLabel );
        leftPanel.add( hostLabel );
        leftPanel.add( portLabel );


        JPanel rightPanel = new JPanel();
        rightPanel.setLayout( new GridLayout(0,1,50,5) );

        rightPanel.add( loginTextField );
        rightPanel.add( hostTextField );
        rightPanel.add( portTextField );


        innerPanel.add( leftPanel, BorderLayout.CENTER );
        innerPanel.add( rightPanel, BorderLayout.EAST );

        middle.add( innerPanel, BorderLayout.CENTER );

        mainMiddle.add( middle, BorderLayout.NORTH );

        add( mainMiddle, BorderLayout.NORTH );

        JPanel mainBottom;

	if ( isPopAccount() == true )
	    {
                //typeComboBox.setSelectedIndex( 0 );

		popPanel = new PopAttributPanel( mainInterface, popItem );
		mainBottom = popPanel;
	    }
	else
	    {
                //typeComboBox.setSelectedIndex( 1 );

		imapPanel = new ImapAttributPanel( mainInterface, imapItem );
		mainBottom = imapPanel ;
	    }



	add( mainBottom, BorderLayout.CENTER );


        String title = null;
        if ( isPopAccount() )
           title = new String(ResourceLoader.getString("dialog/mail/account/incomingserverpanel", "_POP3_Security")); //$NON-NLS-1$
        else
            title = new String(ResourceLoader.getString("dialog/mail/account/incomingserverpanel", "_IMAP4_Security")); //$NON-NLS-1$

        JPanel bottomPanel = new JPanel();
        bottomPanel.setLayout( new BorderLayout() );
        bottomPanel.setBorder( javax.swing.BorderFactory.createTitledBorder(
                   javax.swing.BorderFactory.createEtchedBorder(),
                   title ) );


        innerPanel = new JPanel();
        GridBagLayout layout = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        innerPanel.setLayout( layout );
        innerPanel.setBorder( BorderFactory.createEmptyBorder(10,10,10,10) );

        bottomPanel.add( innerPanel, BorderLayout.NORTH );

        add( bottomPanel, BorderLayout.SOUTH );


        JPanel esmtp3Panel = new JPanel();
	esmtp3Panel.setLayout( new BoxLayout( esmtp3Panel, BoxLayout.X_AXIS ) );
        storePasswordCheckBox = new JCheckBox ();
	storePasswordCheckBox.setText (ResourceLoader.getString("dialog/mail/account/incomingserverpanel", "Store_password_in_configuration_file")); //$NON-NLS-1$
	storePasswordCheckBox.setAlignmentX(Component.LEFT_ALIGNMENT);
        esmtp3Panel.add( storePasswordCheckBox );
	esmtp3Panel.add( Box.createHorizontalGlue() );
        c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx = 0;
        c.weightx = 1.0;
        c.anchor = GridBagConstraints.WEST;
        layout.setConstraints( esmtp3Panel, c);
        innerPanel.add( esmtp3Panel );


        JPanel securePanel = new JPanel();
	securePanel.setLayout( new BoxLayout( securePanel, BoxLayout.X_AXIS ) );
        JCheckBox secureCheckBox = new JCheckBox ();
	secureCheckBox.setText (ResourceLoader.getString("dialog/mail/account/incomingserverpanel", "Use_SSL_for_secure_connection")); //$NON-NLS-1$
	secureCheckBox.setEnabled(false);
	secureCheckBox.setAlignmentX(Component.LEFT_ALIGNMENT);
	securePanel.add( secureCheckBox );
	securePanel.add( Box.createHorizontalGlue() );
        c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx = 0;
        c.weightx = 1.0;
        c.anchor = GridBagConstraints.WEST;
        layout.setConstraints( securePanel, c);
        innerPanel.add( securePanel );



    }


    public void actionPerformed(ActionEvent e)
    {
	String action  = e.getActionCommand();

        if ( action.equals("SERVER") ) //$NON-NLS-1$
        {
            System.out.println("selection changed");

        }
    }



    public void makeChanges( boolean newAccount )
    {

          // change incoming server tab data
        if ( accountItem.isPopAccount() )
        {
              //PopItem item = accountItem.getPopItem();
              //item.setFolderName( accountItem.getName() );

            updateComponents( false );


            mainInterface.mainFrame.getMenu().updatePopServerMenu();

              /*
            if ( newAccount )

                 frame.mainInterface.popServerCollection.add( accountItem );


                 frame.mainInterface.mainFrame.getMenu().updatePopServerMenu();
              */


        }
        else
        {


            updateComponents( false );



            /*
            System.out.println("Imap Folder");

            if ( newAccount )
            {

              frame.mainInterface.treeViewer.addFolder( null, accountItem.getName(),
                                                    "system", "false", "imaproot", accountItem.getUid() );

                TreeNodeList list = new TreeNodeList("/"+accountItem.getName() );
                frame.mainInterface.treeViewer.addFolder( list, "Inbox",
                                                    "system", "true", "imap", accountItem.getUid() );



                  //newAccount = false;
            }
            */
        }
    }

    public boolean isFinished()
    {
        boolean result = false;


        String host = getHost();
        String login = getLogin();


        if ( host.length() == 0 )
        {
            JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                           ResourceLoader.getString("dialog/mail/account/incomingserverpanel", "You_have_to_enter_a_host_name")); //$NON-NLS-1$
        }
        else if ( login.length() == 0 )
        {
            JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                           ResourceLoader.getString("dialog/mail/account/incomingserverpanel", "You_have_to_enter_a_login_name")); //$NON-NLS-1$
        }
        else if ( isPopAccount() )
        {
            /*
            String folder = getDestinationFolder();

            Folder f = mainInterface.treeViewer.getFolder( new TreeNodeList( folder ) );

            if ( f == null )
            {
                JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                               "You have to enter a correct destination folder!");
            }
            else
            {
                  //saveIncomingData();
                result = true;
            }
            */
        }
        else
        {
              //saveIncomingData();
            result = true;
        }


          /*
            AccountTreeNode child0 = (AccountTreeNode) getSelected().getChildAt(0);
            tree.nodeChanged( child0, child0.getName() );
          */

        return result;
    }

}
