// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.gui.config.account;

/*
 * PopPanel.java
 *
 * Created on 1. November 2000, 23:39
 */

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import org.columba.config.*;
import org.columba.gui.util.*;
import org.columba.folder.*;
import org.columba.message.*;
import org.columba.gui.tree.util.*;
import org.columba.main.MainInterface;

/**
 *
 * @author  freddy
 * @version
 */
public class OutgoingServerPanel
	extends DefaultPanel
	implements ActionListener {

	private JButton okButton;
	private JButton cancelButton;
	private JButton helpButton;

	private JLabel hostLabel;
	private JTextField hostTextField;
	private JLabel portLabel;
	private JTextField portTextField;
	private JRadioButton esmtpRadioButton;
	private JLabel loginLabel;
	private JTextField loginTextField;
	private JRadioButton popbeforesmtpRadioButton;
	
	private JCheckBox needAuthCheckBox;
	/*
	private JLabel placecopyLabel;
	private JTextField placecopyTextField;
	private JComboBox placecopyComboBox;
	*/

	private JCheckBox bccyourselfCheckBox;
	private JLabel bccanotherLabel;
	private JTextField bccanotherTextField;
	private JButton selectButton;
	private JCheckBox storePasswordCheckBox;

	private SmtpItem item;
	//private ConfigFrame frame;

	public OutgoingServerPanel(MainInterface mainInterface, SmtpItem item) {
		super(mainInterface);
		//super( frame, item );

		//this.frame = frame;
		this.item = item;

		initComponents();

		updateComponents(true);

	}

	public String getHost() {
		return hostTextField.getText();
	}

	public String getLogin() {
		return loginTextField.getText();
	}

	public boolean isESmtp() {
		return esmtpRadioButton.isSelected();
	}

	/*
	public String getSentFolder()
	{
	    return placecopyTextField.getText();
	}
	*/

	protected void updateComponents(boolean b) {

		if (b) {

			hostTextField.setText(item.getHost());

			portTextField.setText(new Integer(0).toString(item.getPort()));

			loginTextField.setText(item.getUser());

			if (item.isSavePassword())
				storePasswordCheckBox.setSelected(true);

			if ((item.isESmtp() == true) || (item.getPopBeforeSmtp() == true)) {
				needAuthCheckBox.setSelected(true);
				if (item.isESmtp()) {
					esmtpRadioButton.setSelected(true);
					popbeforesmtpRadioButton.setSelected(false);
					storePasswordCheckBox.setEnabled(true);
					loginLabel.setEnabled(true);
					loginTextField.setEnabled(true);
				} else {
					//esmtpRadioButton.setSelected(false);
					popbeforesmtpRadioButton.setSelected(true);
					esmtpRadioButton.setSelected(false);
					storePasswordCheckBox.setEnabled(false);
					loginLabel.setEnabled(false);
					loginTextField.setEnabled(false);
				}
			} else {
				needAuthCheckBox.setSelected(false);
				esmtpRadioButton.setEnabled(false);
				storePasswordCheckBox.setEnabled(false);
				loginLabel.setEnabled(false);
				loginTextField.setEnabled(false);
				popbeforesmtpRadioButton.setEnabled(false);
			}

			/*
			int uid = item.getUid();
			Folder folder = mainInterface.treeViewer.getFolder( uid );
			String treepath = folder.getTreePath();
			placecopyTextField.setText( treepath );
			*/

			if (item.isBccYourself() == true)
				bccyourselfCheckBox.setSelected(true);

			bccanotherTextField.setText(item.getBccAnother());

		} else {
			item.setUser(loginTextField.getText());

			if (storePasswordCheckBox.isSelected())
				item.setSavePassword("true"); //$NON-NLS-1$
			else
				item.setSavePassword("false"); //$NON-NLS-1$

			item.setPort(portTextField.getText());

			item.setHost(hostTextField.getText());

			if ( needAuthCheckBox.isSelected() )
			{
			if (esmtpRadioButton.isSelected()) {
				item.setESmtp("true"); //$NON-NLS-1$
				item.setPopBeforeSmtp(false);
			} else {
				item.setESmtp("false"); //$NON-NLS-1$
				item.setPopBeforeSmtp(true);
			}
			}
			else
			{
				item.setPopBeforeSmtp(false);	
				item.setESmtp("false"); //$NON-NLS-1$
			}

			if (bccyourselfCheckBox.isSelected())
				item.setBccYourself("true"); //$NON-NLS-1$
			else
				item.setBccYourself("false"); //$NON-NLS-1$

			//item.setPlaceCopy( placecopyTextField.getText() );

			/*
			String treepath = placecopyTextField.getText();
			TreeNodeList list = new TreeNodeList( treepath );
			Folder folder = mainInterface.treeViewer.getFolder( list );
			int uid = folder.getUid();
			
			item.setUid( uid );
			*/
		}
	}

	protected void initComponents() {
		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		hostLabel = new JLabel(ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "Host")); //$NON-NLS-1$
		hostTextField = new JTextField(30);
		portLabel = new JLabel(ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "Port")); //$NON-NLS-1$
		portTextField = new JTextField(30);

		JPanel mainMiddle = new JPanel();
		mainMiddle.setLayout(new BorderLayout());
		JPanel middle = new JPanel();
		middle.setBorder(
			javax.swing.BorderFactory.createTitledBorder(
				javax.swing.BorderFactory.createEtchedBorder(),
				ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "_Server_Settings"))); //$NON-NLS-1$
		middle.setLayout(new BorderLayout());

		JPanel innerPanel = new JPanel();
		innerPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		innerPanel.setLayout(new BorderLayout());

		JPanel leftPanel = new JPanel();
		leftPanel.setLayout(new GridLayout(0, 1, 50, 5));

		leftPanel.add(hostLabel);
		leftPanel.add(portLabel);

		JPanel rightPanel = new JPanel();
		rightPanel.setLayout(new GridLayout(0, 1, 50, 5));

		rightPanel.add(hostTextField);
		rightPanel.add(portTextField);

		innerPanel.add(leftPanel, BorderLayout.CENTER);
		innerPanel.add(rightPanel, BorderLayout.EAST);

		middle.add(innerPanel, BorderLayout.CENTER);

		mainMiddle.add(middle, BorderLayout.NORTH);

		add(mainMiddle, BorderLayout.NORTH);

		JPanel panel = new JPanel();
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		panel.setLayout(layout);

		innerPanel.add(panel, BorderLayout.SOUTH);

		JPanel authPanel = new JPanel();
		GridBagLayout authLayout = new GridBagLayout();
		authPanel.setLayout( authLayout );
		
		needAuthCheckBox = new JCheckBox(ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "Server_needs_authentification")); //$NON-NLS-1$
		needAuthCheckBox.setActionCommand("AUTH"); //$NON-NLS-1$
		needAuthCheckBox.addActionListener(this);
		c = new GridBagConstraints();
		c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx = 0;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 1.0;
        c.anchor = GridBagConstraints.WEST;
        authLayout.setConstraints( needAuthCheckBox, c);
		authPanel.add( needAuthCheckBox );

		ButtonGroup group = new ButtonGroup();

		popbeforesmtpRadioButton = new JRadioButton(ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "POP-before-SMTP_(SMTP-AFTER-POP)")); //$NON-NLS-1$
		popbeforesmtpRadioButton.setActionCommand("POPBEFORESMTP"); //$NON-NLS-1$
		popbeforesmtpRadioButton.addActionListener(this);
		group.add( popbeforesmtpRadioButton );
		c = new GridBagConstraints();
		c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx = 0;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 1.0;
        c.anchor = GridBagConstraints.WEST;
        c.insets = new Insets(0,20,0,0);
        authLayout.setConstraints( popbeforesmtpRadioButton, c);
		authPanel.add( popbeforesmtpRadioButton );
		
		JPanel esmtpPanel = new JPanel();
		esmtpPanel.setLayout(new BoxLayout(esmtpPanel, BoxLayout.X_AXIS));
		esmtpRadioButton = new JRadioButton(ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "Always_use_name_and_password_(ESMTP)")); //$NON-NLS-1$
		esmtpRadioButton.setActionCommand("ESMTP"); //$NON-NLS-1$
		esmtpRadioButton.addActionListener(this);
		group.add( esmtpRadioButton );
		esmtpPanel.add(esmtpRadioButton);
		esmtpPanel.add(Box.createHorizontalGlue());
		c = new GridBagConstraints();
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.gridx = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.WEST;
		c.insets = new Insets(0,20,0,0);
		authLayout.setConstraints(esmtpPanel, c);
		authPanel.add(esmtpPanel);

		JPanel esmtp2Panel = new JPanel();
		esmtp2Panel.add(Box.createRigidArea(new java.awt.Dimension(20, 0)));
		esmtp2Panel.setLayout(new BoxLayout(esmtp2Panel, BoxLayout.X_AXIS));
		loginLabel = new JLabel(ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "Login")); //$NON-NLS-1$
		loginLabel.setEnabled(false);
		esmtp2Panel.add(loginLabel);

		esmtp2Panel.add(Box.createHorizontalGlue());
		esmtp2Panel.add(Box.createRigidArea(new java.awt.Dimension(10, 0)));

		loginTextField = new JTextField(item.getUser(), 30);
		loginTextField.setEnabled(false);
		esmtp2Panel.add(loginTextField);
		esmtp2Panel.add(Box.createRigidArea(new java.awt.Dimension(10, 0)));
		c = new GridBagConstraints();
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.gridx = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.WEST;
		c.insets = new Insets(0,20,0,0);
		authLayout.setConstraints(esmtp2Panel, c);
		authPanel.add(esmtp2Panel);

		c = new GridBagConstraints();
		c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx = 0;
        c.anchor = GridBagConstraints.WEST;
        c.weightx = 1.0;
        c.anchor = GridBagConstraints.WEST;
        layout.setConstraints( authPanel, c);
		panel.add(authPanel);

		JPanel middlePanel = new JPanel();
		middlePanel.setLayout(new BorderLayout());
		middlePanel.setBorder(
			javax.swing.BorderFactory.createTitledBorder(
				javax.swing.BorderFactory.createEtchedBorder(),
				ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "_When_sending_messages,_automatically"))); //$NON-NLS-1$

		innerPanel = new JPanel();
		layout = new GridBagLayout();
		c = new GridBagConstraints();
		innerPanel.setLayout(layout);
		innerPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		middlePanel.add(innerPanel, BorderLayout.NORTH);

		add(middlePanel, BorderLayout.CENTER);

		/*
		JPanel placecopyPanel = new JPanel();
		placecopyPanel.setLayout( new BoxLayout( placecopyPanel, BoxLayout.X_AXIS ) );
		placecopyLabel = new JLabel ("Place a copy in:");
		placecopyPanel.add (placecopyLabel);
		placecopyPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );
		placecopyPanel.add( Box.createHorizontalGlue() );
		placecopyTextField = new JTextField( 30 );
		placecopyPanel.add (placecopyTextField);
		placecopyPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );
		selectButton = new JButton(" .. ");
		selectButton.setMargin( new Insets( 0,0,0,0 ) );
		selectButton.addActionListener( this );
		selectButton.setActionCommand("SELECT");
		placecopyPanel.add( selectButton );
		placecopyPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );
		c.gridx = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = GridBagConstraints.REMAINDER;
		layout.setConstraints( placecopyPanel, c);
		innerPanel.add( placecopyPanel );
		*/

		JPanel bccyourselfPanel = new JPanel();
		bccyourselfPanel.setLayout(new BoxLayout(bccyourselfPanel, BoxLayout.X_AXIS));
		bccyourselfCheckBox = new JCheckBox(ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "Bcc_to_yourself")); //$NON-NLS-1$
		bccyourselfCheckBox.setEnabled(false);
		bccyourselfPanel.add(bccyourselfCheckBox);
		bccyourselfPanel.add(Box.createHorizontalGlue());
		c.gridx = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = GridBagConstraints.REMAINDER;
		layout.setConstraints(bccyourselfPanel, c);
		innerPanel.add(bccyourselfPanel);

		JPanel bccanotherPanel = new JPanel();
		bccanotherPanel.setLayout(new BoxLayout(bccanotherPanel, BoxLayout.X_AXIS));
		bccanotherLabel = new JLabel(ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "Bcc_another_address")); //$NON-NLS-1$
		bccanotherLabel.setEnabled(false);
		bccanotherPanel.add(bccanotherLabel);
		bccanotherPanel.add(Box.createHorizontalGlue());
		bccanotherPanel.add(Box.createRigidArea(new java.awt.Dimension(10, 0)));
		bccanotherTextField = new JTextField(30);
		bccanotherTextField.setEnabled(false);
		bccanotherPanel.add(bccanotherTextField);
		bccanotherPanel.add(Box.createRigidArea(new java.awt.Dimension(10, 0)));
		c.gridx = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.WEST;
		c.gridwidth = GridBagConstraints.REMAINDER;
		layout.setConstraints(bccanotherPanel, c);
		innerPanel.add(bccanotherPanel);

		JPanel bottomPanel = new JPanel();
		bottomPanel.setLayout(new BorderLayout());
		bottomPanel.setBorder(
			javax.swing.BorderFactory.createTitledBorder(
				javax.swing.BorderFactory.createEtchedBorder(),
				ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "_SMTP_Security"))); //$NON-NLS-1$

		innerPanel = new JPanel();
		layout = new GridBagLayout();
		c = new GridBagConstraints();
		innerPanel.setLayout(layout);
		innerPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		bottomPanel.add(innerPanel, BorderLayout.NORTH);

		add(bottomPanel, BorderLayout.SOUTH);

		JPanel esmtp3Panel = new JPanel();
		esmtp3Panel.setLayout(new BoxLayout(esmtp3Panel, BoxLayout.X_AXIS));
		storePasswordCheckBox = new JCheckBox();
		storePasswordCheckBox.setText(ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "Store_password_in_configuration_file")); //$NON-NLS-1$
		storePasswordCheckBox.setEnabled(false);
		storePasswordCheckBox.setAlignmentX(Component.LEFT_ALIGNMENT);
		esmtp3Panel.add(storePasswordCheckBox);
		esmtp3Panel.add(Box.createHorizontalGlue());
		c = new GridBagConstraints();
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.gridx = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.WEST;
		layout.setConstraints(esmtp3Panel, c);
		innerPanel.add(esmtp3Panel);

		JPanel securePanel = new JPanel();
		securePanel.setLayout(new BoxLayout(securePanel, BoxLayout.X_AXIS));
		JCheckBox secureCheckBox = new JCheckBox();
		secureCheckBox.setText(ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "Use_SSL_for_secure_connection")); //$NON-NLS-1$
		secureCheckBox.setEnabled(false);
		secureCheckBox.setAlignmentX(Component.LEFT_ALIGNMENT);
		securePanel.add(secureCheckBox);
		securePanel.add(Box.createHorizontalGlue());
		c = new GridBagConstraints();
		c.gridwidth = GridBagConstraints.REMAINDER;
		c.gridx = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.WEST;
		layout.setConstraints(securePanel, c);
		innerPanel.add(securePanel);
	}

	public void actionPerformed(ActionEvent e) {
		String action = e.getActionCommand();


		if (action.equals("ESMTP")) {
			if (esmtpRadioButton.isSelected() == true) {
				storePasswordCheckBox.setEnabled(true);
				loginLabel.setEnabled(true);
				loginTextField.setEnabled(true);
			} else {
				storePasswordCheckBox.setEnabled(false);
				loginLabel.setEnabled(false);
				loginTextField.setEnabled(false);
			}

		}
		
		else if ( action.equals("AUTH") )
		{
			esmtpRadioButton.setEnabled(true);
				
				popbeforesmtpRadioButton.setEnabled(true);
				
			if ( needAuthCheckBox.isSelected() == true )
			{
				if (esmtpRadioButton.isSelected() == true) {
				storePasswordCheckBox.setEnabled(true);
				loginLabel.setEnabled(true);
				loginTextField.setEnabled(true);
				
				popbeforesmtpRadioButton.setSelected(false);
			} else {
				storePasswordCheckBox.setEnabled(false);
				loginLabel.setEnabled(false);
				loginTextField.setEnabled(false);
				
				popbeforesmtpRadioButton.setSelected(true);
			}	
				
			}
			else
			{
				esmtpRadioButton.setEnabled(false);
				storePasswordCheckBox.setEnabled(false);
				loginLabel.setEnabled(false);
				loginTextField.setEnabled(false);
				popbeforesmtpRadioButton.setEnabled(false);
			}
		}
		else if (action.equals("POPBEFORESMTP")) {
			if (esmtpRadioButton.isSelected() == true) {
				storePasswordCheckBox.setEnabled(true);
				loginLabel.setEnabled(true);
				loginTextField.setEnabled(true);
			} else {
				storePasswordCheckBox.setEnabled(false);
				loginLabel.setEnabled(false);
				loginTextField.setEnabled(false);
			}

		}

	}

	/*
	public void saveOutgoingData()
	{
	    // update account-outgoingServer xml-config
	  outgoingServerPanel.updateComponents( false );
	}
	*/

	public boolean isFinished() {
		boolean result = false;

		String host = getHost();
		boolean esmtp = isESmtp();

		//String sentFolder = getSentFolder();
		//Folder f = mainInterface.treeViewer.getFolder( new TreeNodeList( sentFolder ) );

		if (host.length() == 0) {
			JOptionPane.showMessageDialog(
				mainInterface.mainFrame,
				ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "You_have_to_enter_a_host_name")); //$NON-NLS-1$
		}
		/*
		else if ( f == null )
		{
		    JOptionPane.showMessageDialog( mainInterface.mainFrame,
		                                   "You have to enter a correct Sent Folder!");
		}
		*/
		else if (esmtp == true) {
			String login = getLogin();

			if (login.length() == 0) {

				JOptionPane.showMessageDialog(
					mainInterface.mainFrame,
					ResourceLoader.getString("dialog/mail/account/outgoingserverpanel", "You_have_to_enter_a_login_name")); //$NON-NLS-1$
			} else {
				//saveOutgoingPanel();
				result = true;
			}
		} else {
			//saveOutgoingData();
			result = true;
		}

		/*
		  AccountTreeNode child1 = (AccountTreeNode) getSelected().getChildAt(1);
		  tree.nodeChanged( child1, child1.getName() );
		*/

		return result;

	}

}