// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.config.account;


import javax.swing.*;
import javax.swing.border.*;

import javax.swing.Box;
import java.awt.*;
import java.awt.event.*;
import java.io.File;

import org.columba.config.*;
import org.columba.gui.util.*;
import org.columba.message.*;
import org.columba.gui.tree.util.*;
import org.columba.folder.*;
import org.columba.gui.tree.*;
import org.columba.main.MainInterface;


public class SecurityPanel extends DefaultPanel implements ActionListener
{

    private JLabel idLabel;
    private JTextField idTextField;
    private JLabel typeLabel;
    private JComboBox typeComboBox;
    private JLabel pathLabel;
    private JButton pathButton;

    private JCheckBox enableCheckBox;
    private JCheckBox alwaysSignCheckBox;
    private JCheckBox alwaysEncryptCheckBox;


    private PGPItem item;


    public SecurityPanel( MainInterface mainInterface, PGPItem item)
    {
        super(mainInterface);


        this.item = item;

        initComponents ();

        updateComponents( true );

	//enableCheckBox.setEnabled(false);
    }

    protected void updateComponents( boolean b )
    {
	
        if ( b )
        {
            idTextField.setText( item.getId() );
            pathButton.setText( item.getPath() );


            enableCheckBox.setSelected( item.getEnabled() );

            alwaysSignCheckBox.setSelected( item.getAlwaysSign() );
            alwaysEncryptCheckBox.setSelected( item.getAlwaysEncrypt() );
	    
            enablePGP( enableCheckBox.isSelected() );

        }
        else
        {
            item.setId( idTextField.getText() );
            item.setPath( pathButton.getText() );


            item.setEnabled( enableCheckBox.isSelected() );

            item.setAlwaysSign( alwaysSignCheckBox.isSelected() );
            item.setAlwaysEncrypt( alwaysEncryptCheckBox.isSelected() );



        }
    }






    protected void initComponents ()
    {
        GridBagLayout layout = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();

	setLayout( new BorderLayout() );
        setBorder( BorderFactory.createEmptyBorder( 10,10,10,10 ) );


        JPanel mainMiddle = new JPanel();
        mainMiddle.setLayout( new BorderLayout() );

	JPanel middle = new JPanel();
        middle.setBorder( javax.swing.BorderFactory.createTitledBorder(
                   javax.swing.BorderFactory.createEtchedBorder(),ResourceLoader.getString("dialog/mail/account/securitypanel", "_PGP_(Pretty_Good_Privacy)_Options") ) ); //$NON-NLS-1$
        middle.setLayout( new BorderLayout() );


        JPanel enablePanel = new JPanel();
        enablePanel.setBorder( BorderFactory.createEmptyBorder(0,10,0,10) );
	enablePanel.setLayout( new BoxLayout( enablePanel, BoxLayout.X_AXIS ) );
        enableCheckBox = new JCheckBox (ResourceLoader.getString("dialog/mail/account/securitypanel", "enable_PGP_Support")); //$NON-NLS-1$
	enableCheckBox.setAlignmentX(Component.LEFT_ALIGNMENT);
        enableCheckBox.setActionCommand("ENABLE");
        enableCheckBox.addActionListener( this );
	enablePanel.add( enableCheckBox );
	enablePanel.add( Box.createHorizontalGlue() );
        layout = new GridBagLayout();
        c = new GridBagConstraints();
        c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx = 0;
        c.weightx = 1.0;
        c.anchor = GridBagConstraints.WEST;
        layout.setConstraints( enablePanel, c);
        add( enablePanel, BorderLayout.NORTH );

        JPanel innerPanel = new JPanel();
        innerPanel.setBorder( BorderFactory.createEmptyBorder(10,10,10,10) );
        innerPanel.setLayout( new BorderLayout() );

        JPanel leftPanel = new JPanel();
        leftPanel.setLayout( new GridLayout(0,1,50,5) );

        idLabel = new JLabel (ResourceLoader.getString("dialog/mail/account/securitypanel", "User_ID")); //$NON-NLS-1$
        leftPanel.add( idLabel );
        typeLabel = new JLabel ( ResourceLoader.getString("dialog/mail/account/securitypanel", "PGP_Version")); //$NON-NLS-1$
        leftPanel.add( typeLabel );
        pathLabel = new JLabel (ResourceLoader.getString("dialog/mail/account/securitypanel", "Path_to_Binary")); //$NON-NLS-1$
        leftPanel.add( pathLabel );

        JPanel rightPanel = new JPanel();
        rightPanel.setLayout( new GridLayout(0,1,50,5) );

        idTextField = new JTextField ( 30 );
        rightPanel.add( idTextField );
        typeComboBox = new JComboBox();
        //typeComboBox.setMargin( new Insets( 0,0,0,0 ) );
        typeComboBox.insertItemAt("GnuPG",0 );
        typeComboBox.insertItemAt("PGP2",1 );
        typeComboBox.insertItemAt("PGP5",2 );
        typeComboBox.insertItemAt("PGP6",3 );
        typeComboBox.setSelectedIndex(0);
        typeComboBox.setEnabled(false);

        rightPanel.add( typeComboBox );
        pathButton = new JButton();
        //pathButton.setMargin( new Insets( 0,0,0,0 ) );
        pathButton.setActionCommand("PATH");
        pathButton.addActionListener( this );
        rightPanel.add( pathButton );

        innerPanel.add( leftPanel, BorderLayout.CENTER );
        innerPanel.add( rightPanel, BorderLayout.EAST );

        middle.add( innerPanel, BorderLayout.NORTH );

        JPanel alwaysPanel  = new JPanel();
        alwaysPanel.setLayout( new BoxLayout( alwaysPanel, BoxLayout.Y_AXIS )  );
        alwaysPanel.setBorder( BorderFactory.createEmptyBorder(0,0,10,0) );

        JPanel alwaysSignPanel = new JPanel();
        alwaysSignPanel.setBorder( BorderFactory.createEmptyBorder(0,10,0,10) );
	alwaysSignPanel.setLayout( new BoxLayout( alwaysSignPanel, BoxLayout.X_AXIS ) );
        alwaysSignCheckBox = new JCheckBox (ResourceLoader.getString("dialog/mail/account/securitypanel", "Always_sign_when_sending_messages")); //$NON-NLS-1$
	alwaysSignCheckBox.setAlignmentX(Component.LEFT_ALIGNMENT);
	alwaysSignPanel.add( alwaysSignCheckBox );
	alwaysSignPanel.add( Box.createHorizontalGlue() );
        layout = new GridBagLayout();
        c = new GridBagConstraints();
        c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx = 0;
        c.weightx = 1.0;
        c.anchor = GridBagConstraints.WEST;
        layout.setConstraints( alwaysSignPanel, c);
        alwaysPanel.add( alwaysSignPanel );

        JPanel alwaysEncryptPanel = new JPanel();
        alwaysEncryptPanel.setBorder( BorderFactory.createEmptyBorder(0,10,0,10) );
	alwaysEncryptPanel.setLayout( new BoxLayout( alwaysEncryptPanel, BoxLayout.X_AXIS ) );
        alwaysEncryptCheckBox = new JCheckBox (ResourceLoader.getString("dialog/mail/account/securitypanel", "Always_encrypt_when_sending_messages")); //$NON-NLS-1$
	alwaysEncryptCheckBox.setAlignmentX(Component.LEFT_ALIGNMENT);
	alwaysEncryptPanel.add( alwaysEncryptCheckBox );
	alwaysEncryptPanel.add( Box.createHorizontalGlue() );
        layout = new GridBagLayout();
        c = new GridBagConstraints();
        c = new GridBagConstraints();
        c.gridwidth = GridBagConstraints.REMAINDER;
        c.gridx = 0;
        c.weightx = 1.0;
        c.anchor = GridBagConstraints.WEST;
        layout.setConstraints( alwaysEncryptPanel, c);
        alwaysPanel.add( alwaysEncryptPanel  );


        middle.add( alwaysPanel, BorderLayout.CENTER );

        mainMiddle.add( middle, BorderLayout.NORTH);

        add( mainMiddle, BorderLayout.CENTER );


    }

    public void enablePGP( boolean b )
    {
        if ( b )
            {
                //typeComboBox.setEnabled( true );
                idTextField.setEnabled( true );
                idLabel.setEnabled( true );
                typeLabel.setEnabled( true );
                pathLabel.setEnabled( true );
                pathButton.setEnabled( true );
                alwaysSignCheckBox.setEnabled( true );
                alwaysEncryptCheckBox.setEnabled( true );
            }
            else
            {
                //typeComboBox.setEnabled( false );
                idTextField.setEnabled( false );
                idLabel.setEnabled( false );
                typeLabel.setEnabled( false );
                pathLabel.setEnabled( false );
                pathButton.setEnabled( false );
                alwaysSignCheckBox.setEnabled( false );
                alwaysEncryptCheckBox.setEnabled( false );
            }
    }

    public void actionPerformed( ActionEvent e )
    {
        String action = e.getActionCommand();

        if ( action.equals("ENABLE") )
        {
            enablePGP( enableCheckBox.isSelected() );
        }
        else if ( action.equals("PATH") )
        {
            JFileChooser fileChooser = new JFileChooser();
            File aktFile;

            fileChooser.setDialogTitle(ResourceLoader.getString("dialog/mail/account/securitypanel", "PGP_Binary")); //$NON-NLS-1$
            fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

            int returnVal = fileChooser.showDialog( mainInterface.mainFrame, ResourceLoader.getString("dialog/mail/account/securitypanel", "Select_File") ); //$NON-NLS-1$

            if( returnVal == JFileChooser.APPROVE_OPTION )
            {
                File file = fileChooser.getSelectedFile();
                pathButton.setText( file.getPath() );
            }
        }
    }



    public boolean isFinished()
    {
        boolean result = true;

        /*
        String name = getAccountName();
        String address = getAddress();

        if ( name.length() == 0 )
        {
            result = false;
            JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                           "You have to enter a name for this account!");
        }
        else if ( address.length() == 0 )
        {
            result = false;
            JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                           "You have to enter your address!");
        }
        else
        {
            result = true;
        }
        */
        return result;
    }


    protected void makeChanges( boolean newAccount )
    {
        /*
        String panelName = accountItem.getName();
        //FIXME
        //frame.getSelected().setName( panelName );

        if ( accountItem.isPopAccount() == true )
        {
              //PopItem item = accountItem.getPopItem();
              //item.setFolderName( accountItem.getName() );

            if ( newAccount == true )
            {
                mainInterface.popServerCollection.add( accountItem );
                System.out.println("add popserver");
            }


            mainInterface.mainFrame.getMenu().updatePopServerMenu();



        }
        else
        {
            ImapItem item = accountItem.getImapItem();

            if ( newAccount == true )
            {
                System.out.println("create Imap Folder");


                Folder parentFolder = mainInterface.treeViewer.addImapRootFolder( accountItem.getName(),
                                                                                        item,
                                                                                        accountItem.getUid() );



            }
            else
            {
                int uid = accountItem.getUid();

                Folder folder = mainInterface.treeViewer.getImapFolder( uid );
                folder.setName( accountItem.getName() );

            }


        }



        */

    }
}











