// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.config.account;


import javax.swing.*;
import javax.swing.border.*;

import javax.swing.Box;
import java.awt.*;
import java.awt.event.*;
import java.io.File;

import org.columba.config.*;
import org.columba.gui.util.*;
import org.columba.message.*;
import org.columba.gui.tree.util.*;
import org.columba.folder.*;
import org.columba.gui.tree.*;
import org.columba.main.MainInterface;


public class SpecialFoldersPanel extends DefaultPanel implements ActionListener
{

    
    private JLabel trashLabel;
    private JButton trashButton;
    

    private JLabel draftsLabel;
    private JButton draftsButton;

    private JLabel templatesLabel;
    private JButton templatesButton;

    private JLabel sentLabel;
    private JButton sentButton;

    private JLabel inboxLabel;
    private JButton inboxButton;

    private SpecialFoldersItem item;
    private AccountItem accountItem;

    public SpecialFoldersPanel( MainInterface mainInterface, AccountItem accountItem, SpecialFoldersItem item)
    {
        super(mainInterface);


        this.item = item;
        this.accountItem = accountItem;

        initComponents ();

        updateComponents( true );

    }

    protected String getPath( String uid )
    {
        Integer u = new Integer(uid);

        Folder f = mainInterface.treeViewer.getFolderTree().getFolder(u.intValue());

        if ( f == null ) return ""; //$NON-NLS-1$

        return f.getTreePath();
    }

    protected String getUid( String treePath )
    {
        TreeNodeList list = new TreeNodeList( treePath );
        Folder f = mainInterface.treeViewer.getFolderTree().getFolder( list );

        if ( f == null ) return ""; //$NON-NLS-1$

        Integer i = new Integer( f.getUid() );

        return i.toString();
    }

    protected boolean isPopAccount()
    {
        return accountItem.isPopAccount();
    }


    protected void updateComponents( boolean b )
    {
        if ( b )
        {
        	if ( !isPopAccount() )
            	trashButton.setText( getPath( item.getTrash() ) );

            draftsButton.setText( getPath( item.getDrafts() ) );
            templatesButton.setText( getPath( item.getTemplates() ) );
            sentButton.setText( getPath( item.getSent() ) );

            if ( isPopAccount() ) inboxButton.setText( getPath( item.getInbox() ) );
        }
        else
        {
        	if ( !isPopAccount() )
            	item.setTrash( getUid( trashButton.getText() ) );

            item.setDrafts( getUid( draftsButton.getText() ) );
            item.setTemplates( getUid( templatesButton.getText() ) );
            item.setSent( getUid( sentButton.getText() ) );

            if ( isPopAccount() ) item.setInbox( getUid( inboxButton.getText() ) );
        }
    }






	protected void initComponents()
	{
		GridBagLayout layout = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();

		setLayout(new BorderLayout());
		setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));

		JPanel mainMiddle = new JPanel();
		mainMiddle.setLayout(new BorderLayout());

		JPanel middle = new JPanel();
		middle.setBorder(
			javax.swing.BorderFactory.createTitledBorder(
				javax.swing.BorderFactory.createEtchedBorder(),
				ResourceLoader.getString("dialog/mail/account/specialfolderspanel", "_Special_Folders_Location"))); //$NON-NLS-1$
		middle.setLayout(new BorderLayout());

		JPanel innerPanel = new JPanel();
		innerPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		innerPanel.setLayout(new BorderLayout());

		JPanel leftPanel = new JPanel();
		leftPanel.setLayout(new GridLayout(0, 1, 50, 5));

		inboxLabel = new JLabel(ResourceLoader.getString("dialog/mail/account/specialfolderspanel", "Inbox_Folder")); //$NON-NLS-1$
		if (isPopAccount())
			leftPanel.add(inboxLabel);
		draftsLabel = new JLabel(ResourceLoader.getString("dialog/mail/account/specialfolderspanel", "Drafts_Folder")); //$NON-NLS-1$
		leftPanel.add(draftsLabel);
		templatesLabel = new JLabel(ResourceLoader.getString("dialog/mail/account/specialfolderspanel", "Templates_Folder")); //$NON-NLS-1$
		leftPanel.add(templatesLabel);
		sentLabel = new JLabel(ResourceLoader.getString("dialog/mail/account/specialfolderspanel", "Sent_Folder")); //$NON-NLS-1$
		leftPanel.add(sentLabel);
		if (!isPopAccount())
		{
			trashLabel = new JLabel(ResourceLoader.getString("dialog/mail/account/specialfolderspanel", "Trash_Folder")); //$NON-NLS-1$
			leftPanel.add(trashLabel);
		}

		JPanel rightPanel = new JPanel();
		rightPanel.setLayout(new GridLayout(0, 1, 50, 5));

		inboxButton = new JButton();
		inboxButton.setActionCommand("INBOX"); //$NON-NLS-1$
		inboxButton.addActionListener(this);
		if (isPopAccount())
			rightPanel.add(inboxButton);

		draftsButton = new JButton();
		draftsButton.setActionCommand("DRAFTS"); //$NON-NLS-1$
		draftsButton.addActionListener(this);
		rightPanel.add(draftsButton);

		templatesButton = new JButton();
		templatesButton.setActionCommand("TEMPLATES"); //$NON-NLS-1$
		templatesButton.addActionListener(this);
		rightPanel.add(templatesButton);

		sentButton = new JButton();
		sentButton.setActionCommand("SENT"); //$NON-NLS-1$
		sentButton.addActionListener(this);
		rightPanel.add(sentButton);

		if (!isPopAccount())
		{
			trashButton = new JButton();
			trashButton.setActionCommand("TRASH"); //$NON-NLS-1$
			trashButton.addActionListener(this);
			rightPanel.add(trashButton);
		}

		innerPanel.add(leftPanel, BorderLayout.CENTER);
		innerPanel.add(rightPanel, BorderLayout.EAST);

		middle.add(innerPanel, BorderLayout.NORTH);

		mainMiddle.add(middle, BorderLayout.NORTH);

		add(mainMiddle, BorderLayout.CENTER);

	}


    public void actionPerformed( ActionEvent e )
    {
        String action = e.getActionCommand();


        
        if ( action.equals("TRASH") ) //$NON-NLS-1$
        {
            SelectFolderDialog dialog = mainInterface.treeViewer.getSelectFolderDialog();
            

            if ( dialog.success() )
            {
                Folder selectedFolder = dialog.getSelectedFolder();
                String path = selectedFolder.getTreePath();

                trashButton.setText( path );

                //int uid = selectedFolder.getUid();
                //item.setTrash( new Integer(uid).toString() );
            }
        }
        else if ( action.equals("INBOX") ) //$NON-NLS-1$
        {
            SelectFolderDialog dialog = mainInterface.treeViewer.getSelectFolderDialog();

            if ( dialog.success() )
            {
                Folder selectedFolder = dialog.getSelectedFolder();
                String path = selectedFolder.getTreePath();

                inboxButton.setText( path );

                //int uid = selectedFolder.getUid();
                //item.setInbox( new Integer(uid).toString() );
            }
        }
        else if ( action.equals("DRAFTS") ) //$NON-NLS-1$
        {
            SelectFolderDialog dialog = mainInterface.treeViewer.getSelectFolderDialog();

            if ( dialog.success() )
            {
                Folder selectedFolder = dialog.getSelectedFolder();
                String path = selectedFolder.getTreePath();

                draftsButton.setText( path );

                //int uid = selectedFolder.getUid();
                //item.setDrafts( new Integer(uid).toString() );
            }
        }
        else if ( action.equals("TEMPLATES") ) //$NON-NLS-1$
        {
            SelectFolderDialog dialog = mainInterface.treeViewer.getSelectFolderDialog();

            if ( dialog.success() )
            {
                Folder selectedFolder = dialog.getSelectedFolder();
                String path = selectedFolder.getTreePath();

                templatesButton.setText( path );

                //int uid = selectedFolder.getUid();
                //item.setTemplates( new Integer(uid).toString() );
            }
        }
        else if ( action.equals("SENT") ) //$NON-NLS-1$
        {
            SelectFolderDialog dialog = mainInterface.treeViewer.getSelectFolderDialog();

            if ( dialog.success() )
            {
                Folder selectedFolder = dialog.getSelectedFolder();
                String path = selectedFolder.getTreePath();

                sentButton.setText( path );

                //int uid = selectedFolder.getUid();
                //item.setSent( new Integer(uid).toString() );
            }
        }
    }



    public boolean isFinished()
    {
        boolean result = true;

        /*
        String name = getAccountName();
        String address = getAddress();

        if ( name.length() == 0 )
        {
            result = false;
            JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                           "You have to enter a name for this account!");
        }
        else if ( address.length() == 0 )
        {
            result = false;
            JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                           "You have to enter your address!");
        }
        else
        {
            result = true;
        }
        */
        return result;
    }


    protected void makeChanges( boolean newAccount )
    {
        /*
        String panelName = accountItem.getName();
        //FIXME
        //frame.getSelected().setName( panelName );

        if ( accountItem.isPopAccount() == true )
        {
              //PopItem item = accountItem.getPopItem();
              //item.setFolderName( accountItem.getName() );

            if ( newAccount == true )
            {
                mainInterface.popServerCollection.add( accountItem );
                //System.out.println("add popserver");
            }


            mainInterface.mainFrame.getMenu().updatePopServerMenu();



        }
        else
        {
            ImapItem item = accountItem.getImapItem();

            if ( newAccount == true )
            {
                //System.out.println("create Imap Folder");


                Folder parentFolder = mainInterface.treeViewer.addImapRootFolder( accountItem.getName(),
                                                                                        item,
                                                                                        accountItem.getUid() );



            }
            else
            {
                int uid = accountItem.getUid();

                Folder folder = mainInterface.treeViewer.getImapFolder( uid );
                folder.setName( accountItem.getName() );

            }


        }



        */

    }
}











