package org.columba.gui.config.accountwizard;

import org.columba.gui.util.*;
import java.awt.*;
import java.awt.event.*;
import org.columba.config.*;
import org.columba.gui.tree.*;
import org.columba.gui.tree.util.*;
import org.columba.folder.*;
import org.columba.gui.config.account.*;

import javax.swing.*;

import org.columba.main.*;

public class AccountWizard implements ActionListener
{
	private WelcomePanel welcomePanel;
	private IdentityPanel identityPanel;
	private OutgoingServerPanel outgoingServerPanel;
	private IncomingServerPanel incomingServerPanel;
	private AdvancedPanel advancedPanel;
	private FinishPanel finishPanel;

	private MainInterface mainInterface;

	private boolean add;

	private JDialog dialog;

	public AccountWizard(MainInterface mainInterface)
	{
		this.mainInterface = mainInterface;

		int count = mainInterface.config.getAccountList().count();

		add = false;

		dialog = DialogStore.getDialog("Create a new account...");

		if (count == 0)
			init();
	}

	public AccountWizard(MainInterface mainInterface, boolean b)
	{
		this.mainInterface = mainInterface;

		add = b;

		dialog = DialogStore.getDialog("Create a new account...");

		if (b == true)
			addInit();
	}

	protected void addInit()
	{
		welcomePanel =
			new WelcomePanel(
				dialog,
				this,
				"Welcome",
				"Initial Welcome Screen",
				ImageLoader.getImageIcon("preferences", "Preferences24"));

		identityPanel =
			new IdentityPanel(
				dialog,
				this,
				"Account Wizard",
				"Specify your identity information",
				ImageLoader.getImageIcon("preferences", "Preferences24"),
				true);

		incomingServerPanel =
			new IncomingServerPanel(
				dialog,
				this,
				"Account Wizard",
				"Incoming Server Properties",
				ImageLoader.getImageIcon("preferences", "Preferences24"));
		identityPanel.setNext(incomingServerPanel);
		incomingServerPanel.setPrev(identityPanel);

		outgoingServerPanel =
			new OutgoingServerPanel(
				dialog,
				this,
				"Account Wizard",
				"Outgoing Server Properties",
				ImageLoader.getImageIcon("preferences", "Preferences24"),
				true);
		outgoingServerPanel.setPrev(incomingServerPanel);
		incomingServerPanel.setNext(outgoingServerPanel);

		advancedPanel =
			new AdvancedPanel(
				dialog,
				this,
				"Account Wizard",
				"Advanced Options",
				ImageLoader.getImageIcon("preferences", "Preferences24"),
				true);
		advancedPanel.setPrev(outgoingServerPanel);
		outgoingServerPanel.setNext(advancedPanel);

		//identityDialog.showDialog();
		//dialog.getContentPane().add( identityPanel );
		setPanel(identityPanel);

		java.awt.Dimension dim = dialog.getSize();

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

		dialog.setLocation(
			screenSize.width / 2 - dim.width / 2,
			screenSize.height / 2 - dim.height / 2);
		dialog.setVisible(true);
	}

	protected void init()
	{
		welcomePanel =
			new WelcomePanel(
				dialog,
				this,
				"Account Wizard",
				"Initial Welcome Screen",
				ImageLoader.getImageIcon("preferences", "Preferences24"));

		identityPanel =
			new IdentityPanel(
				dialog,
				this,
				"Account Wizard",
				"Specify your identity information",
				ImageLoader.getImageIcon("preferences", "Preferences24"));
		identityPanel.setPrev(welcomePanel);
		welcomePanel.setNext(identityPanel);

		incomingServerPanel =
			new IncomingServerPanel(
				dialog,
				this,
				"Account Wizard",
				"Incoming Server Properties",
				ImageLoader.getImageIcon("preferences", "Preferences24"));
		identityPanel.setNext(incomingServerPanel);
		incomingServerPanel.setPrev(identityPanel);

		outgoingServerPanel =
			new OutgoingServerPanel(
				dialog,
				this,
				"Account Wizard",
				"Outgoing Server Properties",
				ImageLoader.getImageIcon("preferences", "Preferences24"));
		outgoingServerPanel.setPrev(incomingServerPanel);
		incomingServerPanel.setNext(outgoingServerPanel);
		
		advancedPanel =
			new AdvancedPanel(
				dialog,
				this,
				"Account Wizard",
				"Advanced Options",
				ImageLoader.getImageIcon("preferences", "Preferences24")
				);
		advancedPanel.setPrev(outgoingServerPanel);
		outgoingServerPanel.setNext(advancedPanel);

		finishPanel =
			new FinishPanel(
				dialog,
				this,
				"Account Wizard",
				"Finished information gathering",
				ImageLoader.getImageIcon("preferences", "Preferences24"));
		advancedPanel.setNext(finishPanel);
		finishPanel.setPrev(advancedPanel);

		///welcomePanel.showDialog();
		setPanel(welcomePanel);
		java.awt.Dimension dim = dialog.getSize();

		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

		dialog.setLocation(
			screenSize.width / 2 - dim.width / 2,
			screenSize.height / 2 - dim.height / 2);
		dialog.setVisible(true);
	}

	public void setPanel(DefaultWizardPanel panel)
	{
		dialog.getContentPane().add(panel);
		dialog.validate();
		dialog.pack();
	}

	public void actionPerformed(ActionEvent e)
	{
		String action = e.getActionCommand();

		if (action.equals("CANCEL"))
		{
			//identityPanel.showDialog();
			//incomingServerPanel.showDialog();
			//outgoingServerPanel.showDialog();
		}
		else if (action.equals("ACCOUNT"))
		{
			dialog.setVisible(false);

			AccountItem item = finish();

			AccountDialog dialog = new AccountDialog(mainInterface, item);
		}
		else if (action.equals("FINISH"))
		{
			if (add == true)
				//outgoingServerPanel.hideDialog();
				dialog.setVisible(false);
			else
				//finishPanel.hideDialog();
				dialog.setVisible(false);

			//System.out.println("saving data");

			finish();

		}

	}

	public AccountItem finish()
	{
		AccountItem item;

		if (incomingServerPanel.isPopAccount())
			item = mainInterface.config.getAccountList().addEmptyAccount(true);
		else
			item = mainInterface.config.getAccountList().addEmptyAccount(false);

		//System.out.println("name: "+ identityDialog.getAccountName() );

		IdentityItem identity = item.getIdentityItem();

		identity.setName(identityPanel.getName());
		identity.setAddress(identityPanel.getAddress());

		String accountname = advancedPanel.getAccountName();
		item.setName(accountname);

		if (incomingServerPanel.isPopAccount())
		{
			PopItem pop = item.getPopItem();

			pop.setHost(incomingServerPanel.getHost());
			pop.setUser(incomingServerPanel.getLogin());

			mainInterface.popServerCollection.add(item);

			mainInterface.mainFrame.getMenu().updatePopServerMenu();

		}
		else
		{
			ImapItem imap = item.getImapItem();

			imap.setHost(incomingServerPanel.getHost());
			imap.setUser(incomingServerPanel.getLogin());


			Folder parentFolder =
				mainInterface.treeViewer.getFolderTree().addImapRootFolder(
					item.getName(),
					imap,
					item.getUid());

		}

		SmtpItem smtp = item.getSmtpItem();

		smtp.setHost(outgoingServerPanel.getHost());

		return item;

	}

}