// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.config.filter;

import org.columba.config.*;
import org.columba.message.*;
import org.columba.main.*;
import org.columba.gui.util.*;
import org.columba.gui.tree.util.*;
import org.columba.filter.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.border.*;
import javax.swing.tree.*;


import java.awt.*;
import java.awt.event.*;


import java.util.Vector;

class ActionList extends JPanel implements ActionListener
{

    private MainInterface mainInterface;
    private Config config;

    private Filter filter;

    private HeaderTableItem v;

    private Vector list;

    private JPanel panel;

    protected GridBagLayout gridbag = new GridBagLayout();
    protected GridBagConstraints c = new GridBagConstraints();


    public ActionList(MainInterface mainInterface,  Filter filter, JFrame frame)
    {
        super();

        this.mainInterface = mainInterface;
        this.config = mainInterface.config;
	this.filter = filter;



	list = new Vector();

        panel = new JPanel();
        JScrollPane scrollPane = new JScrollPane( panel );

        setLayout( new BorderLayout() );

        scrollPane.setPreferredSize( new Dimension( 200,1000 ) );
        add( scrollPane, BorderLayout.CENTER );

        update();

    }


    public void initComponents()
    {
    }

    public void updateComponents( boolean b )
    {
        if ( b == false )
        {

            for ( int i=0; i<list.size(); i++ )
            {
                DefaultActionRow row = (DefaultActionRow) list.get(i);
                row.updateComponents( false );
            }
        }
    }


    public void add()
    {



        boolean allowed = true;
        for ( int i=0; i<filter.getActionCount(); i++ )
        {
            FilterAction action = filter.getFilterAction(i);
            String name = action.getAction();


            if ( ( action.equals("move") ) || ( action.equals("delete") ) )
               allowed = false;


        }

        if ( allowed )
        {
            updateComponents( false );
            filter.addEmptyAction();
        }

        update();
    }

    public void remove( int i )
    {
        if ( filter.getActionCount() > 1 )
        {

            updateComponents( false );

            filter.removeAction(i);

            update();
        }

    }


    public void update()
    {
        panel.removeAll();
        list.clear();


        panel.setLayout(gridbag);


        /*
        c.fill = GridBagConstraints.NONE;
        c.gridx = GridBagConstraints.REMAINDER;
        c.weightx = 1.0;
        c.anchor = GridBagConstraints.NORTHWEST;
        */

        for ( int i=0; i< filter.getActionCount(); i++)
        {
            FilterAction action = filter.getFilterAction(i);
            int type = action.getActionInt();
            DefaultActionRow row = null;

            if ( type==0 )
            {
                // move
                row = new ActionRow( mainInterface, this, action );
            }
            else if ( type==1 )
            {
                // copy
                row = new ActionRow( mainInterface, this, action );
            }
            else if ( type==2 )
            {
                // mark as read
                row = new MarkActionRow( mainInterface, this, action );
            }
            else if ( type==3 )
            {
                // delete
                row = new MarkActionRow( mainInterface, this, action );
            }


            if ( row != null )
            {
                c.fill = GridBagConstraints.NONE;
                c.gridx = GridBagConstraints.RELATIVE;
                c.gridy = i;
                c.weightx = 1.0;
                c.anchor = GridBagConstraints.NORTHWEST;
                gridbag.setConstraints( row, c);

                list.add( row );
                panel.add( row );


                JButton removeButton = new JButton("Remove", ImageLoader.getImageIcon("", "Delete16") );
                removeButton.setActionCommand( new Integer(i).toString());
                final int index = i;
                removeButton.addActionListener( new ActionListener() {
                                                public void actionPerformed(ActionEvent e)
                                                {
                                                    remove(index);
                                                }
                                            }
                                            );

                c.gridx = GridBagConstraints.REMAINDER;
                c.anchor = GridBagConstraints.NORTHEAST;
                gridbag.setConstraints( removeButton, c );
                panel.add( removeButton );
            }
        }


        c.weighty = 1.0;
        Component box =  Box.createVerticalGlue();
        gridbag.setConstraints( box, c );
        panel.add( box );

        validate();
        repaint();

    }


    public void actionPerformed( ActionEvent e )
    {
        System.out.println("actionperformed");

        updateComponents( false );
        update();
    }

}









