// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.



package org.columba.gui.config.filter;

import org.columba.config.*;
import org.columba.message.*;
import org.columba.main.*;
import org.columba.filter.*;
import org.columba.gui.util.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class CriteriaList extends JPanel implements ActionListener
{

    private MainInterface mainInterface;
    private Config config;
    private Filter filter;


    private HeaderTableItem v;

    private Vector list;

    private JPanel panel;


    public CriteriaList(MainInterface mainInterface,  Filter filter )
    {
        super();

        this.mainInterface = mainInterface;
        this.config = mainInterface.config;
	this.filter = filter;

        list = new Vector();

        panel = new JPanel();
        JScrollPane scrollPane = new JScrollPane( panel );

        scrollPane.setPreferredSize( new Dimension( 300,2000 ) );
        setLayout( new BorderLayout() );


        add( scrollPane, BorderLayout.CENTER );

        update();
    }

    public void updateComponents( boolean b )
    {
        if ( b == false )
        {

            for ( int i=0; i<list.size(); i++ )
            {
                DefaultCriteriaRow row = ( DefaultCriteriaRow ) list.get(i);
                row.updateComponents( false );
            }
        }

    }


    public void add()
    {

        filter.addEmptyCriteria();

        updateComponents( false );

        update();

    }

    public void remove( int i )
    {

        FilterRule rule = filter.getFilterRule();

        if ( rule.count() > 1 )
        {
            updateComponents( false );

            filter.removeCriteria(i);

            update();
        }


    }



    public void update()
    {
        panel.removeAll();
        list.clear();

        System.out.println("CriteriaList -> update() ");

        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        panel.setLayout(gridbag);

        /*
        c.fill = GridBagConstraints.NONE;
        c.gridx = GridBagConstraints.RELATIVE;
        c.weightx = 1.0;
        c.anchor = GridBagConstraints.NORTHWEST;
        */

        FilterRule rule = filter.getFilterRule();

        for ( int i=0; i<rule.count(); i++ )
        {
            FilterCriteria criteria = rule.get(i);
            String type = criteria.getType();
            DefaultCriteriaRow column;

            c.fill = GridBagConstraints.NONE;
            c.gridx = GridBagConstraints.RELATIVE;
            c.gridy = i;
            c.weightx = 1.0;
            c.anchor = GridBagConstraints.NORTHWEST;

            if ( type.equalsIgnoreCase("Custom Headerfield") )
            {
                column = new CustomHeaderCriteriaRow( mainInterface, this, criteria );
                gridbag.setConstraints(column, c);
            }
            else if ( type.equalsIgnoreCase("Date") )
            {
                column = new DateCriteriaRow( mainInterface, this, criteria );
                gridbag.setConstraints(column, c);
            }
            else if ( type.equalsIgnoreCase("Size") )
            {
                column = new SizeCriteriaRow( mainInterface, this, criteria );
                gridbag.setConstraints(column, c);
            }
            else if ( type.equalsIgnoreCase("Flags") )
            {
                column = new FlagsCriteriaRow( mainInterface, this, criteria );
                gridbag.setConstraints(column, c);
            }
            else if ( type.equalsIgnoreCase("Priority") )
            {
                column = new PriorityCriteriaRow( mainInterface, this, criteria );
                gridbag.setConstraints(column, c);
            }
            else if ( type.equalsIgnoreCase("Body") )
            {
                column = new BodyCriteriaRow( mainInterface, this, criteria );
                gridbag.setConstraints(column, c);
            }
            else
            {
                column = new HeaderCriteriaRow( mainInterface, this, criteria );
                gridbag.setConstraints(column, c);
            }

            list.add( column );

            panel.add( column );


            JButton removeButton = new JButton("Remove", ImageLoader.getImageIcon("", "Delete16") );
            removeButton.setActionCommand( new Integer(i).toString());
            final int index = i;
            removeButton.addActionListener( new ActionListener() {
                                                public void actionPerformed(ActionEvent e)
                                                {
                                                    remove(index);
                                                }
                                            }
                                            );

            c.gridx = GridBagConstraints.REMAINDER;
            c.anchor = GridBagConstraints.NORTHEAST;
            gridbag.setConstraints( removeButton, c );
            panel.add( removeButton );
        }


        c.weighty = 1.0;
        Component box =  Box.createVerticalGlue();
        gridbag.setConstraints( box, c );
        panel.add( box );


        validate();
        repaint();

    }



    public void actionPerformed( ActionEvent e )
    {
        System.out.println("actionperformed");


        updateComponents( false );
        update();
    }
}






