// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.config.filter;


import javax.swing.*;
import java.awt.Component;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;


import org.columba.gui.util.*;
import org.columba.config.*;
import org.columba.message.*;
import org.columba.main.*;
import org.columba.filter.*;


public class FilterDialog implements ActionListener
{

    private JDialog dialog;

    private JLabel nameLabel;
    private JTextField nameTextField;


    private JButton addButton;
    private JButton removeButton;
    private JButton addActionButton;
    private JButton removeActionButton;

    private JButton okButton;
    private JButton cancelButton;
    private JButton helpButton;



    private Filter filter;
    private JFrame frame;
    private MainInterface mainInterface;
    private CriteriaList criteriaList;
    private ActionList actionList;

    private javax.swing.JComboBox condList;



    public FilterDialog( MainInterface mainInterface,  Filter filter )
    {
	dialog = DialogStore.getDialog();


	this.filter = filter;
	this.mainInterface = mainInterface;


        System.out.println("filternode name: "+ filter.getName() );


        initComponents ();

        updateComponents(true);


	dialog.pack();
	dialog.setSize( new java.awt.Dimension( 570,480 ) );

        dialog.setVisible(true);
    }



    private void initComponents ()
    {
	JPanel rootPanel = new JPanel();
        rootPanel.setLayout( new BoxLayout( rootPanel, BoxLayout.Y_AXIS ) );
        rootPanel.setBorder( BorderFactory.createEmptyBorder( 10,0,10,0 ) );

	JPanel namePanel = new JPanel();
        namePanel.setLayout( new BoxLayout( namePanel, BoxLayout.X_AXIS ) );
	namePanel.add( Box.createRigidArea( new java.awt.Dimension(15,0) ) );
	nameLabel = new JLabel ("Filter Description:");
        namePanel.add( nameLabel );

        namePanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
	namePanel.add( Box.createHorizontalGlue() );


        nameTextField = new JTextField( 22 );
        //nameTextField.setMaximumSize( new java.awt.Dimension( 150, 25 ) );
	namePanel.add( nameTextField );

	namePanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );

	rootPanel.add( namePanel );


	CPanel middleIfPanel = new CPanel(" If ");



	JPanel ifPanel = new JPanel();
	ifPanel.setLayout( new BoxLayout( ifPanel, BoxLayout.X_AXIS ) );

	addButton = new JButton("Add Criterion");
	addButton.addActionListener( this );
	addButton.setActionCommand("ADD_CRITERION");
	ifPanel.add( addButton );
        /*
	ifPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );
	removeButton = new JButton("Remove");
	removeButton.addActionListener( this );
	removeButton.setActionCommand("REMOVE_CRITERION");
	ifPanel.add( removeButton );
        */

	//ifPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

	ifPanel.add( Box.createHorizontalGlue() );

	nameLabel = new JLabel ("Execute actions");
	ifPanel.add( nameLabel );


	ifPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );


	String[] cond = { "if ALL criteria are met", "if ANY criteria are met" };
	condList = new javax.swing.JComboBox( cond );


	ifPanel.add( condList );


          //ifPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

	middleIfPanel.add( ifPanel );

        middleIfPanel.add( Box.createRigidArea( new java.awt.Dimension(0,10) ) );


	criteriaList = new CriteriaList( mainInterface, filter );
          //JScrollPane scrollPane = new JScrollPane( criteriaList );
	middleIfPanel.add( criteriaList );


	rootPanel.add( middleIfPanel );

	//rootPanel.add( Box.createRigidArea( new java.awt.Dimension(0,10) ) );

	CPanel middleThenPanel = new CPanel(" Then ");

	//middleThenPanel.add( Box.createRigidArea( new java.awt.Dimension(0,5) ) );


	JPanel thenPanel = new JPanel();
	thenPanel.setLayout( new BoxLayout( thenPanel, BoxLayout.X_AXIS ) );

	addActionButton = new JButton("Add Action");
	addActionButton.addActionListener( this );
	addActionButton.setActionCommand("ADD_ACTION");
	thenPanel.add( addActionButton );
        /*
	thenPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );
	removeActionButton = new JButton("Remove Action");
	removeActionButton.addActionListener( this );
	removeActionButton.setActionCommand("REMOVE_ACTION");
	thenPanel.add( removeActionButton );
        */

	//thenPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

	thenPanel.add( Box.createHorizontalGlue() );

	middleThenPanel.add( thenPanel );

	middleThenPanel.add( Box.createRigidArea( new java.awt.Dimension(0,10) ) );

	actionList = new ActionList( mainInterface, filter, frame );
	middleThenPanel.add( actionList );

	rootPanel.add( middleThenPanel );

	rootPanel.add( javax.swing.Box.createVerticalGlue() );

	rootPanel.add( Box.createRigidArea( new java.awt.Dimension(0,5) ) );

	JPanel bottom = new JPanel();
	bottom.setLayout( new javax.swing.BoxLayout(bottom, javax.swing.BoxLayout.X_AXIS) );

	bottom.add( javax.swing.Box.createHorizontalGlue() );

	okButton = new JButton("Ok");
	okButton.addActionListener( this );
	okButton.setActionCommand("OK");
	bottom.add( okButton );

	bottom.add( javax.swing.Box.createHorizontalGlue() );

	rootPanel.add( bottom );

	//rootPanel.add( Box.createRigidArea( new java.awt.Dimension(0,10) ) );

	dialog.getContentPane().add( rootPanel );


    }

    public void updateComponents( boolean b )
    {
        if ( b )
        {
              // set component values

            criteriaList.updateComponents( b );
            actionList.updateComponents( b );

              // filter description JTextField
            nameTextField.setText( filter.getName() );

            // all / match any JComboBox
            FilterRule filterRule = filter.getFilterRule();
            String value = filterRule.getCondition();
            if ( value.equals("matchall") )
                condList.setSelectedIndex( 0 );
            else
                condList.setSelectedIndex( 1 );
        }
        else
        {
              // get values from components

            criteriaList.updateComponents( b );
            actionList.updateComponents( b );


            filter.setName( nameTextField.getText() );

            int index = condList.getSelectedIndex();
            FilterRule filterRule = filter.getFilterRule();
            if ( index == 0 )
                filterRule.setCondition("matchall");
            else
                filterRule.setCondition("matchany");
        }

    }


    public void actionPerformed(ActionEvent e)
    {
	String action  = e.getActionCommand();


	if ( action.equals("OK") )
        {
              //System.out.println( "ok" );

	    updateComponents( false );

	    dialog.setVisible(false);
              //frame.listView.update();
        }
	else if ( action.equals("ADD_CRITERION") )
	    {
                  //System.out.println( "add" );

                criteriaList.add();

	    }
            /*
	else if ( action.equals("REMOVE_CRITERION") )
	    {
                  //System.out.println( "remove" );

                criteriaList.remove();

	    }
            */
	else if ( action.equals("ADD_ACTION") )
	    {
                  //System.out.println( "add" );
                actionList.add();

	    }
            /*
	else if ( action.equals("REMOVE_ACTION") )
	    {
                  //System.out.println( "remove" );
                actionList.remove();

	    }
            */

    }


}
