// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.config.search;


import javax.swing.*;
import java.awt.Component;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;
import java.util.*;


import org.columba.gui.util.*;
import org.columba.config.*;
import org.columba.message.*;
import org.columba.gui.config.filter.*;
import org.columba.filter.*;
import org.columba.gui.tree.*;
import org.columba.gui.tree.util.*;
import org.columba.folder.*;
import org.columba.folder.virtual.*;
import org.columba.main.*;




public class SearchFrame implements ActionListener
{
    private JDialog dialog;

    private JLabel folderLabel;
    private JLabel nameLabel;
    private JTextField folderTextField;


    private JButton addButton;
    //private JButton removeButton;
    private JButton selectButton;

    private JButton searchButton;
    private JButton closeButton;
    private JButton helpButton;

    private JCheckBox includeSubfolderButton;

    //private AdapterNode vFolderNode;
    private MainInterface mainInterface;

    private CriteriaList criteriaList;
    private VirtualFolder destFolder;

    //private FolderTreeNode folderTreeNode;

    private javax.swing.JComboBox condList;

    private Search searchFilter;

    public SearchFrame( MainInterface mainInterface, Folder folder )
    {
        dialog = DialogStore.getDialog( "Search Messages" );

        //this.folder = folder;
	//this.vFolderNode = vFolderNode;

	this.mainInterface = mainInterface;
        this.destFolder = (VirtualFolder) folder;
        this.searchFilter = destFolder.getSearchFilter();


        initComponents ();

        updateComponents(true);

		dialog.pack();
	

		java.awt.Dimension dim = new Dimension( 570,320 ) ;
		dialog.setSize( dim );
		
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

        dialog.setLocation(screenSize.width/2 - dim.width/2, screenSize.height/2 - dim.height/2);

        dialog.setVisible(true);
    }



    private void initComponents ()
    {

	JPanel rootPanel = new JPanel();
        rootPanel.setLayout( new BoxLayout( rootPanel, BoxLayout.Y_AXIS ) );
        rootPanel.setBorder( BorderFactory.createEmptyBorder( 10,0,10,0 ) );
          /*
            rootPanel.setLayout( new BoxLayout( rootPanel, BoxLayout.Y_AXIS ) );

            rootPanel.setBorder( javax.swing.BorderFactory.createTitledBorder(
            javax.swing.BorderFactory.createEtchedBorder(),
            " Search Messages ")
            );

            rootPanel.add( Box.createRigidArea( new java.awt.Dimension(0,5) ) );
          */

	JPanel namePanel = new JPanel();
        namePanel.setLayout( new BoxLayout( namePanel, BoxLayout.X_AXIS ) );
	namePanel.add( Box.createRigidArea( new java.awt.Dimension(15,0) ) );

	folderLabel = new JLabel ("Folder:");
        namePanel.add( folderLabel );

        namePanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

        folderTextField = new JTextField( 22 );
          //folderTextField.setMaximumSize( new java.awt.Dimension( 150, 25 ) );
	namePanel.add( folderTextField );

        namePanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

        selectButton = new JButton(" .. ");
        selectButton.setMargin( new Insets( 0,0,0,0 ) );
        selectButton.addActionListener( this );
        selectButton.setActionCommand("SELECT");

        namePanel.add( selectButton );

        namePanel.add( Box.createRigidArea( new java.awt.Dimension(20,0) ) );

        includeSubfolderButton = new JCheckBox("include Subfolders");
        includeSubfolderButton.setEnabled( true );

	namePanel.add( includeSubfolderButton );


	rootPanel.add( namePanel );
	//rootPanel.add( Box.createRigidArea( new java.awt.Dimension(0,10) ) );


	CPanel middleIfPanel = new CPanel(" If ");
          /*
            TitledBorder etched = javax.swing.BorderFactory.createTitledBorder(javax.swing.BorderFactory.createEtchedBorder(), " If ");
            middleIfPanel.setBorder( etched );
            middleIfPanel.setLayout( new BoxLayout( middleIfPanel, BoxLayout.Y_AXIS ) );

            middleIfPanel.add( Box.createRigidArea( new java.awt.Dimension(0,5) ) );
          */


	JPanel ifPanel = new JPanel();
	ifPanel.setLayout( new BoxLayout( ifPanel, BoxLayout.X_AXIS ) );

	addButton = new JButton("Add Criteria");
	addButton.addActionListener( this );
	addButton.setActionCommand("ADD_CRITERION");
	ifPanel.add( addButton );

        /*
	ifPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );
	removeButton = new JButton("Remove");
	removeButton.addActionListener( this );
	removeButton.setActionCommand("REMOVE_CRITERION");
	ifPanel.add( removeButton );
        */

	ifPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

	ifPanel.add( Box.createHorizontalGlue() );


	nameLabel = new JLabel ("Execute actions");
	ifPanel.add( nameLabel );


	ifPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );


	String[] cond = { "if ALL criteria are met", "if ANY criteria are met" };
	condList = new javax.swing.JComboBox( cond );


	ifPanel.add( condList );


	//ifPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

	middleIfPanel.add( ifPanel );

	middleIfPanel.add( Box.createRigidArea( new java.awt.Dimension(0,10) ) );



	criteriaList = new CriteriaList( mainInterface, searchFilter.getFilter() );
          //JScrollPane scrollPane = new JScrollPane( criteriaList );
	middleIfPanel.add( criteriaList );


	rootPanel.add( middleIfPanel );

	rootPanel.add( Box.createRigidArea( new java.awt.Dimension(0,10) ) );


	JPanel bottom = new JPanel();
	bottom.setLayout( new javax.swing.BoxLayout(bottom, javax.swing.BoxLayout.X_AXIS) );

	bottom.add( javax.swing.Box.createHorizontalGlue() );


	helpButton = new JButton("Help");
	helpButton.addActionListener( this );
	helpButton.setActionCommand("HELP");
        helpButton.setEnabled( false );
	bottom.add( helpButton );


	bottom.add( javax.swing.Box.createHorizontalGlue() );

        searchButton = new JButton("Search");
	searchButton.addActionListener( this );
	searchButton.setActionCommand("SEARCH");
	bottom.add( searchButton );

        bottom.add( javax.swing.Box.createHorizontalGlue() );

        closeButton = new JButton("Close");
	closeButton.addActionListener( this );
	closeButton.setActionCommand("CLOSE");
	bottom.add( closeButton );

        bottom.add( javax.swing.Box.createHorizontalGlue() );

	rootPanel.add( bottom );

	rootPanel.add( Box.createRigidArea( new java.awt.Dimension(0,10) ) );

	dialog.getContentPane().add( rootPanel );


    }

    public void updateComponents( boolean b )
    {
        if ( b )
        {
              // set component values

              /*
            AdapterNode filterNode = vFolderNode.getChild("filter");

            AdapterNode parentNode = filterNode.getChild("filterrule");
            AdapterNode childNode = parentNode.getChild("condition");
            */
            FilterRule filterRule = searchFilter.getFilter().getFilterRule();
            String value = filterRule.getCondition();
            if ( value.equals("matchall") )
                condList.setSelectedIndex( 0 );
            else
                condList.setSelectedIndex( 1 );

            //AdapterNode includeNode = vFolderNode.getChild("include");
            boolean value2 = searchFilter.isInclude();
            if ( value2 == true )
                includeSubfolderButton.setSelected( true );
            else
                includeSubfolderButton.setSelected( false );

                /*
            AdapterNode folderNode = vFolderNode.getChild("uid");
            String uidStr = folderNode.getValue();
            Integer i = new Integer( uidStr );
            */

            int uid = searchFilter.getUid();
            Folder f = mainInterface.treeViewer.getFolderTree().getFolder( uid );
            String treePath = f.getTreePath();

            folderTextField.setText( treePath );

            criteriaList.updateComponents( b );

        }
        else
        {
              // get values from components

              /*
            AdapterNode filterNode = vFolderNode.getChild("filter");


            AdapterNode parentNode = filterNode.getChild("filterrule");
            AdapterNode childNode = parentNode.getChild("condition");
            */

            FilterRule filterRule = searchFilter.getFilter().getFilterRule();
            int index = condList.getSelectedIndex();
            if ( index == 0 )
               filterRule.setCondition( "matchall" );
            else
                filterRule.setCondition( "matchany" ) ;


            //AdapterNode includeNode = vFolderNode.getChild("include");

            if ( includeSubfolderButton.isSelected() )
                searchFilter.setInclude("true");
            else
                searchFilter.setInclude("false");


            //AdapterNode folderNode = vFolderNode.getChild("uid");
            String path = folderTextField.getText();
            TreeNodeList list = new TreeNodeList( path );
            Folder folder = mainInterface.treeViewer.getFolderTree().getFolder( list );
            int uid = folder.getUid();
            searchFilter.setUid( uid );
            /*
            String uidStr = folderNode.getValue();
            Integer i = new Integer( uidStr );
            int uid = i.intValue();
            */
            //int uid = mainInterface.treeViewer.get( folderTextField.getText() );
            //folderNode.setValue( (new Integer(uid)).toString() );

            criteriaList.updateComponents( b );
        }

    }


    public void actionPerformed(ActionEvent e)
    {
	String action  = e.getActionCommand();


	if ( action.equals("CLOSE") )
        {

	    updateComponents( false );
            mainInterface.config.saveConfiguration();
	    dialog.setVisible(false);
        }
	else if ( action.equals("ADD_CRITERION") )
	    {
                  //System.out.println( "add" );

                criteriaList.add();

	    }
        else if ( action.equals("SELECT") )
	    {

                SelectFolderDialog dialog = mainInterface.treeViewer.getSelectFolderDialog();


                if ( dialog.success() )
                {
                    Folder folder = dialog.getSelectedFolder();
                    String path = folder.getTreePath();

                    folderTextField.setText( path );
                }

	    }
        else if ( action.equals("SEARCH") )
        {
	    updateComponents( false );


            //Search search = new Search( vFolderNode, destFolder, mainInterface );

            try
            {
                searchFilter.applySearch();
            }
            catch ( Exception ex )
            {
                ex.printStackTrace();
            }

            mainInterface.treeViewer.setSelected( destFolder );



	    System.out.println("searching messages");
        }


    }





}

