// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.config.subscribe;

import org.columba.config.*;
import org.columba.message.*;
import org.columba.main.*;
import org.columba.folder.*;
import org.columba.gui.util.*;


import java.awt.event.WindowEvent;
import java.awt.event.WindowAdapter;

import javax.swing.tree.*;
import javax.swing.event.*;
import java.util.*;
import java.awt.event.*;

import org.w3c.dom.*;
import java.awt.*;
import javax.swing.*;

public class SubscribeDialog implements ActionListener, TreeSelectionListener, TreeWillExpandListener
{
    private JDialog dialog;

    
    BorderLayout borderLayout1 = new BorderLayout();

    JPanel centerPanel = new JPanel();
    JPanel eastPanel = new JPanel();
    JTextField serverTextField = new JTextField(50);
    JLabel serverLabel = new JLabel();
    JButton closeButton = new JButton();
    JPanel jPanel1 = new JPanel();
    JButton subscribeButton = new JButton();
    JButton unsubscribeButton = new JButton();
    JButton refreshButton = new JButton();
    BorderLayout borderLayout3 = new BorderLayout();
    GridLayout gridLayout1 = new GridLayout();

    private SubscribeHeaderTableViewer table;

	private MainInterface mainInterface;
    public SubscribeDialog( MainInterface mainInterface )
    {
    	this.mainInterface = mainInterface;
        dialog = DialogStore.getDialog("Subscribe Folder Settings");


	dialog.addWindowListener(
            new WindowAdapter()
                {
                    public void windowClosing(WindowEvent e)
                    {
                        table.updateImapTree();
			dialog.setVisible(false);
                    }
                }
            );

        table = new SubscribeHeaderTableViewer( mainInterface );
        table.getTree().addTreeSelectionListener( this );
        table.getTree().addTreeWillExpandListener(this);

	initComponents();

        dialog.pack();
        

    }


	public void addTreeNodes(SubscribeTreeNode node, Vector nodeList, Vector lsubList )
	{
		table.getHeaderTableModel().addTreeNodes(node, nodeList, lsubList);
	}
	
    public void showDialog()
    {
        dialog.setVisible(true);
    }

	public void repaintTable()
	{
		table.repaintTable();
	}
	
	
    public void initComponents()
    {

        JPanel mainPanel = new JPanel();
        mainPanel.setLayout( new BorderLayout() );
        mainPanel.setBorder( BorderFactory.createEmptyBorder( 10,10,10,10 ) );
        dialog.getContentPane().add( mainPanel );

        closeButton.setText("closeButton");
        closeButton.setActionCommand("CLOSE");
        closeButton.addActionListener( this );
        subscribeButton.setText("Subscribe");
        subscribeButton.setActionCommand("SUBSCRIBE");
        subscribeButton.addActionListener( this );
        unsubscribeButton.setText("Unsubscribe");
        unsubscribeButton.setActionCommand("UNSUBSCRIBE");
        unsubscribeButton.addActionListener( this );
        subscribeButton.setEnabled( false );
        unsubscribeButton.setEnabled( false );
        refreshButton.setText("Refresh");
        refreshButton.setActionCommand("REFRESH");
        refreshButton.addActionListener( this );


        // top panel

        JPanel topPanel = new JPanel();
        GridBagLayout gridBagLayout = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        topPanel.setLayout( gridBagLayout );

        JPanel topBorderPanel = new JPanel();
        topBorderPanel.setLayout( new BorderLayout() );
        topBorderPanel.setBorder( BorderFactory.createEmptyBorder( 0,10,10,10 ) );
        topBorderPanel.add( topPanel, BorderLayout.WEST );
        mainPanel.add( topBorderPanel, BorderLayout.NORTH );



        c.weightx = 1.0;
        c.anchor = GridBagConstraints.WEST;
        c.insets = new Insets( 0,0,0,10 );
        gridBagLayout.setConstraints( serverLabel, c );
        serverLabel.setText("Server: ");
        topPanel.add(serverLabel, null);

        //c.anchor = GridBagConstraints.WEST;
        //c.fill = GridBagConstraints.NONE;
        c.gridwidth = GridBagConstraints.RELATIVE;
        c.insets = new Insets( 0,0,0,0 );
        c.fill = GridBagConstraints.HORIZONTAL;
        gridBagLayout.setConstraints( serverTextField, c );
        topPanel.add(serverTextField, null);

        Component glue = Box.createVerticalGlue();
        c.anchor = GridBagConstraints.EAST;
        c.gridwidth = GridBagConstraints.REMAINDER;
        //c.fill = GridBagConstraints.HORIZONTAL;
        gridBagLayout.setConstraints( glue, c );
        topPanel.add( glue );



        // east panel


        gridBagLayout = new GridBagLayout();
        c = new GridBagConstraints();
        eastPanel.setLayout( gridBagLayout );
        mainPanel.add( eastPanel, BorderLayout.EAST );


        c.fill = GridBagConstraints.HORIZONTAL;
        c.weightx = 1.0;
        c.gridwidth = GridBagConstraints.REMAINDER;
        gridBagLayout.setConstraints( subscribeButton, c );
        eastPanel.add( subscribeButton );

        Component strut1 = Box.createRigidArea( new Dimension(30,10) );
        gridBagLayout.setConstraints( strut1, c );
        eastPanel.add( strut1 );

        gridBagLayout.setConstraints( unsubscribeButton, c );
        eastPanel.add( unsubscribeButton );

        Component strut = Box.createRigidArea( new Dimension(30,20) );
        gridBagLayout.setConstraints( strut, c );
        eastPanel.add( strut );

        gridBagLayout.setConstraints( refreshButton, c );
        eastPanel.add( refreshButton );

        glue = Box.createVerticalGlue();
        c.fill = GridBagConstraints.BOTH;
        c.weighty = 1.0;
        gridBagLayout.setConstraints( glue, c );
        eastPanel.add( glue );

        c.gridheight = GridBagConstraints.REMAINDER;
        c.fill = GridBagConstraints.HORIZONTAL;
        c.weighty = 0;
        gridBagLayout.setConstraints( closeButton, c );
        eastPanel.add( closeButton );


        // centerpanel

        centerPanel = new JPanel();
        centerPanel.setLayout( new BorderLayout() );
        centerPanel.setBorder( BorderFactory.createEmptyBorder( 0,0,0,10 ) );
        centerPanel.add( table.scrollPane );

        mainPanel.add( centerPanel, BorderLayout.CENTER );


    }

    public void setFolder( Folder folder )
    {
        table.setFolder( folder );
        serverTextField.setText( folder.getName() );
        
        FolderOperation op = new FolderOperation( FolderOperation.FOLDER_REFRESHLIST,
        										  10,
        										  folder,
        										  this
        										 );
        mainInterface.crossbar.operate(op);
    }

    public void actionPerformed( ActionEvent event )
    {
        String action = event.getActionCommand();


        if ( action.equals("CLOSE") )
        {
            //table.updateImapTree();
            FolderOperation op = new FolderOperation( Operation.FOLDER_LIST,
                                                                  20,
                                                                  table.getFolder() );
            mainInterface.crossbar.operate(op);

            dialog.setVisible( false );

        }
        else if ( action.equals("REFRESH") )
        {
        	FolderOperation op = new FolderOperation(FolderOperation.FOLDER_REFRESHLIST,
        										 10,
        										 table.getFolder(),
        										 this
        										 );
        	mainInterface.crossbar.operate(op);
            //table.refresh();
        }
        else if ( action.equals("SUBSCRIBE") )
        {
        	SubscribeTreeNode node = table.getSelectedNode();
        	if ( node == null ) return;
        
        	FolderOperation op = new FolderOperation( FolderOperation.FOLDER_SUBSCRIBE,
        										  10,
        										  table.getFolder(),
        										  node,
        										  this
        										 );
        	mainInterface.crossbar.operate(op);
            //table.subscribe();
        }
        else if ( action.equals("UNSUBSCRIBE") )
        {
        	SubscribeTreeNode node = table.getSelectedNode();
        	if ( node == null ) return;
        
        	FolderOperation op = new FolderOperation( FolderOperation.FOLDER_UNSUBSCRIBE,
        										  10,
        										  table.getFolder(),
        										  node,
        										  this
        										 );
        	mainInterface.crossbar.operate(op);
            //table.unsubscribe();
        }
    }

	public void setList( Vector list, Vector subList )
	{
		table.getHeaderTableModel().setList( list, subList );
	}
	
    public void valueChanged(TreeSelectionEvent e)
    {
        SubscribeTreeNode node = table.getSelectedNode();
        if ( node == null )
        {
            subscribeButton.setEnabled( false );
            unsubscribeButton.setEnabled( false );
        }
        else
        {
            if ( node.getDirectory() == true )
            {
                // this folder is not selectable

                subscribeButton.setEnabled( false );
                unsubscribeButton.setEnabled( false );
            }
            else
            {


                if ( node.getSubscribeState() == true )
                {
                    subscribeButton.setEnabled( false );
                    unsubscribeButton.setEnabled( true );
                }
                else
                {
                    subscribeButton.setEnabled( true );
                    unsubscribeButton.setEnabled( false );
                }
            }
        }
    }


	 public void treeWillExpand(TreeExpansionEvent e) throws ExpandVetoException
    {
        System.out.println("Tree-will-expand event detected"+ e);


        TreePath path = e.getPath();
        int row = table.getTree().getRowForPath( path );
        table.getTable().setRowSelectionInterval( row,row );


        SubscribeTreeNode node = table.getSelectedNode();


        if ( node == null )
        {
            System.out.println("node is null");
            return;
        }
        
        FolderOperation op = new FolderOperation(FolderOperation.FOLDER_SUBLIST,
        										 10,
        										 table.getFolder(),
        										 node,
        										 this
        										 );
        	mainInterface.crossbar.operate(op);
        	
        /*

        try
        {
            Vector v = table.getFolder().getList( node );
            Vector lsubList = table.getFolder().getLSubList();
            table.getHeaderTableModel().addTreeNodes(node, v, lsubList);
        }
        catch ( Exception ex )
        {
            System.out.println( ex.getMessage() );
        }
		*/

    }



        // Required by TreeWillExpandListener interface.
    public void treeWillCollapse(TreeExpansionEvent e)
    {
            System.out.println("Tree-will-collapse event detected"+ e);
    }
}
