package org.columba.gui.config.themes;


import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

import org.columba.gui.util.*;
import org.columba.config.*;
import org.columba.gui.addressbook.*;
import org.columba.main.*;


import java.util.*;
import java.io.*;


public class ThemeDialog implements ActionListener
{
    private JDialog dialog;

    private JLabel mainFontLabel;
    private JTextField mainFontTextField;
    private JButton mainFontButton;

    private JLabel textFontLabel;
    private JTextField textFontTextField;
    private JButton textFontButton;

    private JButton themeButton;

    private JRadioButton nativeButton;
    private JRadioButton metalButton;
    private JRadioButton columbaButton;
    private ButtonGroup buttonGroup;

    private JComboBox selectorComboBox;
    private JComboBox iconsetComboBox;
    private JComboBox pulsatorComboBox;

    private JButton okButton;
    private JButton cancelButton;
    private JButton helpButton;

    private int status = -1;
    private int theme = -1;

    private Font mainFont;
    private Font textFont;
    private JFrame frame;

    private ThemeItem item;

    private MainInterface mainInterface;

    public ThemeDialog( MainInterface mainInterface )
    {
        dialog = DialogStore.getDialog();
	
	this.mainInterface = mainInterface;
	this.item = mainInterface.themeItem;

	mainFont = item.getMainFont();
	textFont = item.getTextFont();

	theme = item.getTheme();

        init( theme,mainFont,textFont );

    }

    protected void init( int theme, Font mainFont, Font textFont )
    {
        Container c = dialog.getContentPane();


        JPanel centerPanel = new JPanel();
        centerPanel.setLayout( new BoxLayout( centerPanel, BoxLayout.Y_AXIS ) );
        centerPanel.setBorder( BorderFactory.createEmptyBorder( 5,5,5,5 ) );



	JPanel themePanel = new JPanel();
	themePanel.setBorder( javax.swing.BorderFactory.createTitledBorder(
                   javax.swing.BorderFactory.createEtchedBorder(),
                   "Theme") );

        themePanel.setLayout( new BoxLayout( themePanel, BoxLayout.X_AXIS ) );
          //themePanel.add( Box.createRigidArea( new java.awt.Dimension(0,5) ) );


        java.util.Vector list = new java.util.Vector();
	list.add( "no theme" );
	list.add( "Java Look And Feel" );
        list.add( "Thin Columba" );
        list.add( "Contrast Columba" );
	selectorComboBox = new JComboBox( list );
	
	selectorComboBox.setSelectedIndex(theme);

	themePanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
        themePanel.add( selectorComboBox );
	
	themePanel.add( javax.swing.Box.createHorizontalGlue() );

	themePanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

	themeButton = new JButton("Advanced Options");


	selectorComboBox.setActionCommand("THEME");
	selectorComboBox.addActionListener( this );

	themePanel.add( themeButton );

	themePanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );

	centerPanel.add( themePanel );
        

        JPanel fontPanel = new JPanel();
	fontPanel.setLayout( new BoxLayout( fontPanel, BoxLayout.Y_AXIS ) );
	fontPanel.setBorder( javax.swing.BorderFactory.createTitledBorder(
                   javax.swing.BorderFactory.createEtchedBorder(),
                   "Fonts") );

	JPanel mainFontPanel = new JPanel();
        mainFontPanel.setLayout( new BoxLayout( mainFontPanel, BoxLayout.X_AXIS ) );
	mainFontPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
        //mainFontPanel.setAlignmentX(0);
	mainFontLabel = new JLabel ("MainFont:");
        mainFontPanel.add( mainFontLabel );

        mainFontPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

	mainFontTextField = new JTextField ( 22 );
        mainFontTextField.setFont( mainFont );
        mainFontTextField.setText( mainFont.getFontName() );
	mainFontPanel.add( mainFontTextField );

        mainFontPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

        mainFontButton = new JButton("Choose MainFont..");
        mainFontButton.setActionCommand("MAINFONT");
        mainFontButton.addActionListener( this );
        mainFontPanel.add( mainFontButton );


        mainFontPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );


	fontPanel.add( mainFontPanel );



        JPanel textFontPanel = new JPanel();
	textFontPanel.setLayout( new BoxLayout( textFontPanel, BoxLayout.X_AXIS ) );
	textFontPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
        //textFontPanel.setAlignmentX(0);
	textFontLabel = new JLabel ("TextFont:");
        textFontPanel.add( textFontLabel );

        textFontPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

	textFontTextField = new JTextField ( 22 );
        textFontTextField.setFont( textFont );
        textFontTextField.setText( textFont.getFontName() );
	textFontPanel.add( textFontTextField );

        textFontPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );

        textFontButton = new JButton("Choose TextFont..");
        textFontButton.setActionCommand("TEXTFONT");
        textFontButton.addActionListener( this );
        textFontPanel.add( textFontButton );


        textFontPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );

	fontPanel.add( textFontPanel );

        centerPanel.add( fontPanel );


	JPanel iconPanel = new JPanel();
	iconPanel.setBorder( javax.swing.BorderFactory.createTitledBorder(
                   javax.swing.BorderFactory.createEtchedBorder(),
                   "Theme") );

        iconPanel.setLayout( new BoxLayout( iconPanel, BoxLayout.Y_AXIS ) );
          //themePanel.add( Box.createRigidArea( new java.awt.Dimension(0,5) ) );


	JPanel iconsetPanel = new JPanel();
	iconsetPanel.setLayout( new BoxLayout( iconsetPanel, BoxLayout.X_AXIS ) );

        java.util.Vector v = new java.util.Vector();
	v.add( "Default" );
	File iconsetDirectory = new File( mainInterface.config.configDirectory
					  + "/iconsets" );
	if  ( iconsetDirectory.exists() )
	{
	    File[] fileList = iconsetDirectory.listFiles();
	    for ( int i=0; i<fileList.length; i++ )
	    {
		File file = fileList[i];
		String name = file.getName();
		int index = name.toLowerCase().indexOf(".jar");
		if (  index != -1 )		
		    v.add( name.substring(0,index)  );
	    }
	}
	
	iconsetComboBox = new JComboBox( v );
	iconsetComboBox.setSelectedItem( item.getIconset() );


	iconsetPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );

	
	JLabel iconsetLabel = new JLabel("Iconset:");
	iconsetPanel.add( iconsetLabel );

	iconsetPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );
	iconsetPanel.add( javax.swing.Box.createHorizontalGlue() );

	iconsetPanel.add( iconsetComboBox );

	iconsetPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );

	JPanel pulsatorPanel = new JPanel();
	pulsatorPanel.setLayout( new BoxLayout( pulsatorPanel, BoxLayout.X_AXIS ) );
	
	v = new java.util.Vector();
	v.add( "Default" );
	File pulsatorDirectory = new File( mainInterface.config.configDirectory
					  + "/pulsators" );
	if ( pulsatorDirectory.exists() )
	{
	    File[] pulsatorList = pulsatorDirectory.listFiles();
	    for ( int i=0; i<pulsatorList.length; i++ )
	    {
		File file = pulsatorList[i];
		String name = file.getName();
		int index = name.toLowerCase().indexOf(".jar");
		if (  index != -1 )		
		    v.add( name.substring(0,index)  );
	    }
	}

	pulsatorComboBox = new JComboBox( v );
	pulsatorComboBox.setSelectedItem( item.getPulsator() );

	pulsatorPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );
	//pulsatorsetPanel.add( javax.swing.Box.createHorizontalGlue() );

	JLabel pulsatorLabel = new JLabel("Pulsator:");
	pulsatorPanel.add( pulsatorLabel );

	pulsatorPanel.add( Box.createRigidArea( new java.awt.Dimension(5,0) ) );
	pulsatorPanel.add( javax.swing.Box.createHorizontalGlue() );

	pulsatorPanel.add( pulsatorComboBox );

	pulsatorPanel.add( Box.createRigidArea( new java.awt.Dimension(10,0) ) );

	iconPanel.add( iconsetPanel );

	iconPanel.add( pulsatorPanel );
	centerPanel.add( iconPanel );

        c.add( centerPanel, BorderLayout.CENTER );


        JPanel bottom = new JPanel();
	bottom.setLayout( new javax.swing.BoxLayout(bottom, javax.swing.BoxLayout.X_AXIS) );
       	bottom.setBorder( BorderFactory.createEmptyBorder( 10,10,10,10 ) );

	bottom.add( javax.swing.Box.createRigidArea( new java.awt.Dimension(10,0) ) );

	helpButton = new JButton("Help");
	helpButton.setEnabled( false );
	bottom.add( helpButton );

	bottom.add( javax.swing.Box.createHorizontalGlue() );

	cancelButton = new JButton("Cancel");
	cancelButton.addActionListener( this  );
	cancelButton.setActionCommand("CANCEL");
	bottom.add( cancelButton );

	bottom.add( javax.swing.Box.createRigidArea( new java.awt.Dimension(20,0) ) );

	okButton = new JButton("Ok");
	okButton.addActionListener( this  );
	okButton.setActionCommand("OK");
	bottom.add( okButton );

	bottom.add( javax.swing.Box.createRigidArea( new java.awt.Dimension(10,0) ) );

        c.add( bottom, BorderLayout.SOUTH );


          //setSize( new Dimension( 400, 300 ) );


          //java.awt.Dimension dim = new Dimension( 300, 300 )  ;

        dialog.pack();

        java.awt.Dimension dim = dialog.getSize() ;

        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();

        dialog.setLocation(screenSize.width/2 - dim.width/2, screenSize.height/2 - dim.height/2);

        dialog.setVisible(true);
    }

    public int getStatus()
    {
        return status;
    }

    public Font getMainFont()
    {
        return mainFont;
    }

    public Font getTextFont()
    {
        return textFont;
    }


    public int getTheme()
    {
        return selectorComboBox.getSelectedIndex();
    }



    public void actionPerformed( ActionEvent e )
    {
        String command;

        command = e.getActionCommand();

        if( command.equals("OK") )
        {
            status = 0;
	    
	    item.setPulsator( (String) pulsatorComboBox.getSelectedItem() );
	    item.setIconset( (String) iconsetComboBox.getSelectedItem() );

	    item.setTheme( getTheme() );

	    item.setTextFont( getTextFont() );
	    item.setMainFont( getMainFont() );

            dialog.dispose();
        }
        else if( command.equals("CANCEL") )
        {
            status = 1;
            dialog.dispose();
        }
	else if ( command.equals("THEME") )
	{
	    int index = selectorComboBox.getSelectedIndex();
	    if ( ( index == 2 ) || ( index == 3 ) )
	    {
		themeButton.setEnabled( true );
	    }
	    else
		themeButton.setEnabled( false );
	}
        else if( command.equals("MAINFONT") )
        {
            FontSelectionDialog fontDialog =  new FontSelectionDialog( null );
            fontDialog.showDialog();

            if( fontDialog.getStatus() == 0 )
            {
                mainFont = fontDialog.getSelectedFont();
                mainFontTextField.setFont( mainFont );
                mainFontTextField.setText( mainFont.getFontName() );
                dialog.pack();
                dialog.repaint();

            }

        }
        else if( command.equals("TEXTFONT") )
        {
            FontSelectionDialog fontDialog =  new FontSelectionDialog( null );
            fontDialog.showDialog();

            if( fontDialog.getStatus() == 0 )
            {
                textFont = fontDialog.getSelectedFont();
                textFontTextField.setFont( textFont );
                textFontTextField.setText( textFont.getFontName() );

                dialog.pack();
                dialog.repaint();
            }

        }
    }


}
