// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.header;



import org.columba.config.*;
import org.columba.gui.util.*;
import org.columba.gui.header.*;
import org.columba.gui.header.util.*;
import org.columba.gui.util.ImageLoader;

import javax.swing.*;
import java.awt.event.*;
import java.awt.*;
import java.util.*;




public class FilterToolbar extends JToolBar implements ActionListener
{
    public JToggleButton newButton;
    public JToggleButton oldButton;
    private JToggleButton answeredButton;
    private JToggleButton flaggedButton;
    private JToggleButton expungedButton;
    private JToggleButton draftButton;
    private JToggleButton attachmentButton;
    private JToggleButton secureButton;
    private JToggleButton threadButton;

    public JToggleButton searchButton;

    private JComboBox comboBox;
    private JTextField textField;

    private HeaderTableViewer headerTableViewer;


    private ResourceBundle toolbarLabels;

    public FilterToolbar( HeaderTableViewer headerTableViewer )
        {
            super();

	    this.headerTableViewer = headerTableViewer;

	    setMargin( new Insets(0,0,0,0) );

            

            addCButtons();

            setBorderPainted(false);
            setFloatable(false);

        }



    public void addCButtons()
        {

              //addSeparator();
            newButton = new JToggleButton( ImageLoader.getImageIcon( "unreadflag", "unreadmail") );
            newButton.addActionListener( this );
            newButton.setActionCommand("NEW");
            newButton.setSelected( true );
            newButton.setMargin( new Insets(0,0,0,0) );
            add( newButton );

            oldButton = new JToggleButton( ImageLoader.getImageIcon( "readflag", "readmail") );
            oldButton.addActionListener( this );
            oldButton.setActionCommand("OLD");
            oldButton.setSelected( true );
            oldButton.setMargin( new Insets(0,0,0,0) );
            add( oldButton );

	    addSeparator();

            answeredButton = new JToggleButton( ImageLoader.getImageIcon( "replyflag", "Redo16") );
            answeredButton.addActionListener( this );
            answeredButton.setActionCommand("ANSWERED");
            answeredButton.setMargin( new Insets(0,0,0,0) );
            answeredButton.setSelected( false );
            add(answeredButton);
	    flaggedButton = new JToggleButton( ImageLoader.getImageIcon( "flaggedflag", "Bookmarks16") );
            flaggedButton.setMargin( new Insets(0,0,0,0) );
            flaggedButton.addActionListener( this );
            flaggedButton.setActionCommand("FLAGGED");
            flaggedButton.setSelected( false );
	    add(flaggedButton);
            expungedButton = new JToggleButton( ImageLoader.getImageIcon( "expungeflag", "Delete16") );
            expungedButton.setMargin( new Insets(0,0,0,0) );
            expungedButton.addActionListener( this );
            expungedButton.setActionCommand("EXPUNGED");
            expungedButton.setSelected( false );
	    add(expungedButton);

            attachmentButton = new JToggleButton(  ImageLoader.getImageIcon("attachmentflag", "attachment") );
            attachmentButton.setMargin( new Insets(0,0,0,0) );
            attachmentButton.addActionListener( this );
            attachmentButton.setActionCommand("ATTACHMENT");
            attachmentButton.setSelected( false );
	    add(attachmentButton);

              //addSeparator();
            addSeparator();



	    addSeparator();

	    //HeaderTableItem list = mainInterface.config.getOptionsConfig().getHeaderTableItem();
            HeaderTableItem list = headerTableViewer.getHeaderTableItem();
            comboBox = new JComboBox();
            String name;

            for ( int i=0; i<list.count(); i++ )
            {
                name = list.getName(i);
                boolean enabled = list.getEnabled(i);

                if ( enabled == false ) continue;


                if ( ! ( name.equalsIgnoreCase("status") ||
                         name.equalsIgnoreCase("attachment") ||
                         name.equalsIgnoreCase("flagged") ||
                         name.equalsIgnoreCase("priority") ||
                         name.equalsIgnoreCase("date") ||
                         name.equalsIgnoreCase("size") ) )

                comboBox.addItem( name );

            }

	    comboBox.setMaximumSize( new java.awt.Dimension( 100, 25 ) );
            comboBox.setSelectedIndex(0);
            comboBox.addActionListener( this );
            comboBox.setActionCommand("COMBO");
            add( comboBox );

            addSeparator();

            textField = new JTextField(20);
            textField.addActionListener( this );
            textField.setActionCommand("TEXTFIELD");
            textField.addFocusListener( new FocusListener()
                {
                    public void focusGained(FocusEvent e)
                    {
                    }

                    public void focusLost(FocusEvent e)
                    {
                        TableModelFilteredView model = headerTableViewer.getTableModelFilteredView();
                        try
                        {
                            model.setPatternString( textField.getText() );
                        }
                        catch ( Exception ex )
                        {
                            ex.printStackTrace();
                        }
                    }
                }
                                 );
              //textField.setMaximumSize( new java.awt.Dimension( 600, 25 ) );

            add( textField );


	    addSeparator();

            searchButton = new JToggleButton("Activate Filtered View");
            searchButton.setMaximumSize( new java.awt.Dimension( 150, 25 ) );
            attachmentButton.setSelected( false );
	    searchButton.addActionListener( this );
            searchButton.setActionCommand("OK");
            add( searchButton  );

              //add( Box.createHorizontalGlue() );

            addSeparator();
	    addSeparator();

            threadButton = new JToggleButton(ImageLoader.getImageIcon("org/columba/images/Export16.gif") );
            threadButton.setEnabled( false );
            threadButton.setMargin( new Insets(0,0,0,0) );
	    add(threadButton);




                /*
                  secureCButton = new JButton(ImageLoader.getImageIcon("org/columba/images/secure.jpg") );
                  add(secureCButton);
                */

            addSeparator();



        }

    public void update()
    {
        headerTableViewer.getHeaderTableModel().update();
    }


    public void actionPerformed(ActionEvent e)
    {
	String action  = e.getActionCommand();

        try
        {

        //TableModelFilteredView model = mainInterface.headerTableViewer.getHeaderTable().getTableModelFilteredView();
        TableModelFilteredView model = headerTableViewer.getTableModelFilteredView();

	if ( action.equals("NEW") )
        {
            if ( model.getNewFlag() == true )
                model.setNewFlag( false );
            else
                model.setNewFlag( true );

            //mainInterface.headerTableViewer.getHeaderTable().getIndexedTableModel().update();
            //HeaderTableModel tableModel = mainInterface.headerTableViewer.getHeaderTable().getHeaderTableModel();
              //  tableModel.update();
              update();

        }
        else if ( action.equals("OLD") )
        {
            if ( model.getOldFlag() == true )
                model.setOldFlag( false );
            else
                model.setOldFlag( true );
            //mainInterface.headerTableViewer.getHeaderTable().getIndexedTableModel().update();
            //HeaderTableModel tableModel = mainInterface.headerTableViewer.getHeaderTable().getHeaderTableModel();
                //tableModel.update();
                update();
        }
        else if ( action.equals("ANSWERED") )
        {
            if ( model.getAnsweredFlag() == false )
                model.setAnsweredFlag( true );
            else
                model.setAnsweredFlag( false );
            //mainInterface.headerTableViewer.getHeaderTable().getIndexedTableModel().update();
            //HeaderTableModel tableModel = mainInterface.headerTableViewer.getHeaderTable().getHeaderTableModel();
              //  tableModel.update();
              update();
        }
        else if ( action.equals("FLAGGED") )
        {
            if ( model.getFlaggedFlag() == false )
                model.setFlaggedFlag( true );
            else
                model.setFlaggedFlag( false );

                //HeaderTableModel tableModel = mainInterface.headerTableViewer.getHeaderTable().getHeaderTableModel();
                //tableModel.update();
            //mainInterface.headerTableViewer.getHeaderTable().getIndexedTableModel().update();
            update();
        }
        else if ( action.equals("EXPUNGED") )
        {
            if ( model.getExpungedFlag() == false )
                model.setExpungedFlag( true );
            else
                model.setExpungedFlag( false );


          //HeaderTableModel tableModel = mainInterface.headerTableViewer.getHeaderTable().getHeaderTableModel();
                //tableModel.update();
            //mainInterface.headerTableViewer.getHeaderTable().getIndexedTableModel().update();
            update();
        }


        else if ( action.equals("ATTACHMENT") )
        {
            if ( model.getAttachmentFlag() == false )
                model.setAttachmentFlag( true );
            else
                model.setAttachmentFlag( false );

                //HeaderTableModel tableModel = mainInterface.headerTableViewer.getHeaderTable().getHeaderTableModel();
                //tableModel.update();
            //mainInterface.headerTableViewer.getHeaderTable().getIndexedTableModel().update();

            update();
        }

        else if ( action.equals("COMBO") )
        {
             JComboBox cb = (JComboBox) e.getSource();
             model.setPatternItem( (String)cb.getSelectedItem() );
        }
        else if ( action.equals("TEXTFIELD") )
        {
            model.setPatternString( textField.getText() );
            //HeaderTableModel tableModel = mainInterface.headerTableViewer.getHeaderTable().getHeaderTableModel();
               // tableModel.update();
            //mainInterface.headerTableViewer.getHeaderTable().getIndexedTableModel().update();
            update();
        }
        else if ( action.equals("OK") )
        {
            if ( model == null )
            {
                System.out.println("model is null");
                return;
            }
            if ( model.getDataFiltering() == true )
            {
                model.setDataFiltering( false );

                //HeaderTableModel tableModel = mainInterface.headerTableViewer.getHeaderTable().getHeaderTableModel();
                //tableModel.update();
                  ///mainInterface.headerTableViewer.getHeaderTable().getIndexedTableModel().recalculate();

                //mainInterface.headerTableViewer.getHeaderTable().getIndexedTableModel().update();
                update();
            }
            else
            {
              //HeaderTableModel tableModel = mainInterface.headerTableViewer.getHeaderTable().getHeaderTableModel();
              //tableModel.update();
                model.setDataFiltering( true );
                //mainInterface.headerTableViewer.getHeaderTable().getIndexedTableModel().update();
                update();
            }

        }
        }
        catch ( Exception ex )
        {
            ex.printStackTrace();
        }

    }



}











