// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.header;


import org.columba.message.*;
import org.columba.gui.util.*;
import org.columba.config.*;
import org.columba.main.*;
import org.columba.gui.header.util.*;
import org.columba.gui.header.action.*;
import org.columba.folder.*;
import org.columba.gui.header.treetable.*;






import javax.swing.table.AbstractTableModel;

import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;

import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.table.*;
import javax.swing.border.*;
import javax.swing.tree.*;

import java.io.*;
import java.text.*;
import java.util.*;




/**
 * This widget is a mix between a JTable and a JTree
 * ( we need the JTree for the Threaded viewing of mailing lists )
 *
 * @version 0.9.1
 * @author Frederik
 */
public class HeaderTable
{

    protected HeaderTableModel headerTableModel;

    private int selectedRow=0;
    private int update = 0;

    //private ListSelectionModel listSelectionModel;


    private String column;

    private JTreeTable table;

    private Vector tableModelPlugins;

    public HeaderTable( HeaderTableModel headerTableModel )
        {
            this.headerTableModel = headerTableModel;

            table = new JTreeTable( headerTableModel );
        }



    public void setTreeCellRenderer( TreeCellRenderer renderer )
    {
        table.setTreeCellRenderer( renderer );
    }

    public void registerRenderer( String name, TableCellRenderer cell, TableCellRenderer header, int size, boolean lockSize, int position )
    {
        TableColumn tc = null;

        try
        {
            tc = getTable().getColumn( name );
        }
        catch ( Exception ex )
        {
            System.out.println("headerTable->registerRenderer: "+ ex.getMessage() );
        }

        if ( tc==null ) return;

        if ( cell != null )
           tc.setCellRenderer( cell);

        if ( header != null )
           tc.setHeaderRenderer( header );

        if ( lockSize )
        {
            tc.setMaxWidth( size );
            tc.setMinWidth( size );
        }
        else
            tc.setPreferredWidth( size );

        try
        {
            int index = getTable().getColumnModel().getColumnIndex( name );
            getTable().getColumnModel().moveColumn( index, position );
        }
        catch ( Exception ex )
        {
            ex.printStackTrace();
        }
    }


    public void registerRenderer( String name, TableCellRenderer cell, TableCellRenderer header, int size, boolean lockSize)
    {
        TableColumn tc = null;

        try
        {
            tc = getTable().getColumn( name );
        }
        catch ( Exception ex )
        {
            System.out.println("headerTable->registerRenderer: "+ ex.getMessage() );
        }

        if ( tc==null ) return;

        if ( cell != null )
           tc.setCellRenderer( cell);

        if ( header != null )
           tc.setHeaderRenderer( header );

        if ( lockSize )
        {
            tc.setMaxWidth( size );
            tc.setMinWidth( size );
        }
        else
            tc.setPreferredWidth( size );


    }




    public MessageNode getSelectedNode()
    {

        MessageNode node = (MessageNode) table.getTree().getLastSelectedPathComponent();

        return node;
    }

    public int getSelectedRowCount()
    {
        int[] rows = table.getSelectedRows();

		if ( rows == null ) return 0;
		
        return rows.length;
    }

    public MessageNode[] getSelectedNodes()
    {
    	
        int[] rows=null;
        MessageNode[] nodes=null;

        rows = table.getSelectedRows();
        nodes = new MessageNode[ rows.length ];

        for ( int i=0; i<rows.length; i++ )
        {
            TreePath treePath = table.getTree().getPathForRow( rows[i] );
            nodes[i] = (MessageNode) treePath.getLastPathComponent();
			
        }


        return nodes;
    }



    public void addTreeSelectionListener( TreeSelectionListener l )
    {
        table.addTreeSelectionListener( l );
    }






    public int getRowCount()
        {
            return getTable().getRowCount();
        }

    public int getSelectedRow()
        {


            MessageNode node = (MessageNode) getTree().getLastSelectedPathComponent();
            if ( node == null ) return -1;
            TreePath treepath = new TreePath( getHeaderTableModel().getPathToRoot( node, 0) );
            if ( treepath == null ) return -1;
            int row = getTree().getRowForPath( treepath );



            return row;

        }

    public synchronized void setSelection( MessageNode[] nodes )
    {

		
        
        table.clearSelection();

		
		int[] rows = new int[nodes.length];
		
        for ( int i=0; i<nodes.length; i++ )
        {
            TreePath treePath = new TreePath( getHeaderTableModel().getPathToRoot( nodes[i], 0 ) );
            int row = getTree().getRowForPath( treePath );
            rows[i] = row;
            System.out.println("row:"+row);
            
        }

		getTree().addSelectionRows( rows );

		
		
    }


    public void addSelectionPath( TreePath path )
    {
        TreePath treePath = path;

        getTree().addSelectionPath(  treePath );

    }


    public void setSelectedRow(int row)
    {
        try
        {

            getTable().setRowSelectionInterval( row,row );
        }
        catch ( IllegalArgumentException ex )
        {
            ex.printStackTrace();
        }
    }




    public JTree getTree()
    {
        return table.getTree();

    }

    public JTable getTable()
        {
            return table;
        }


    public HeaderTableModel getHeaderTableModel()
    {
        return headerTableModel;
    }

}









