package org.columba.gui.header;

import java.io.IOException;
import java.io.File;
import java.util.*;

import javax.swing.SwingUtilities;
import javax.swing.tree.*;
import javax.swing.table.*;
import javax.swing.event.*;

import javax.swing.JTree;

import org.columba.folder.*;
import org.columba.config.*;
import org.columba.message.*;
import org.columba.gui.header.treetable.*;
import org.columba.gui.header.util.*;
import org.columba.pop3.*;

public class HeaderTableModel extends AbstractTreeTableModel
{

	protected boolean mutex;

	private HeaderTableItem item;

	private Folder folder;
	protected HeaderList headerList;

	private MessageCollection messageCollection;

	private MessageNode root;

	private MessageNode selectedMessageNode;

	private Vector tableModelPlugins;

	public HeaderTableModel(HeaderTableItem item)
	{
		super(null);
		this.item = item;

		tableModelPlugins = new Vector();

		root = new MessageNode("root", null);
		super.setRoot(root);

		mutex = false;

	}

	private synchronized void getMutex()
	{
		while (mutex)
		{
			try
			{
				wait();
			}
			catch (InterruptedException e)
			{
			}
		}

		mutex = true;
	}

	private synchronized void releaseMutex()
	{
		mutex = false;
		notify();
	}

	public void registerPlugin(TableModelPlugin plugin)
	{
		tableModelPlugins.add(plugin);
	}

	public Folder getFolder()
	{
		return folder;
	}

	public MessageCollection getMessageCollection()
	{
		return messageCollection;
	}

	public MessageNode getRootNode()
	{
		return root;
	}

	public MessageNode getSelectedMessageNode()
	{
		return selectedMessageNode;
	}

	public void setSelectedMessageNode(MessageNode node)
	{
		selectedMessageNode = node;
	}

	protected void removeNode(Object uid)
	{

		MessageNode node = findMessage(getRootNode(), uid);
		removeNodeFromParent(node);

		/*
		MutableTreeNode parent = (MutableTreeNode) node.getParent();
		if(parent == null) throw new IllegalArgumentException("node does not have a parent.");
		
		int[]            childIndex = new int[1];
		Object[]         removedArray = new Object[1];
		
		childIndex[0] = parent.getIndex(node);
		parent.remove(childIndex[0]);
		*/

		/*
		for ( int i=0; i<parent.getChildCount(); i++ )
		{
		    MessageNode node = (MessageNode) parent.getChildAt(i);
		    if ( node.isLeaf() == false ) removeNode( node, uid );
		
		    Object o = node.getUid();
		    if ( o.equals(uid) )
		    {
		MutableTreeNode parent2 = (MutableTreeNode) node.getParent();
		if(parent == null) throw new IllegalArgumentException("node does not have a parent.");
		
		int[]            childIndex = new int[1];
		Object[]         removedArray = new Object[1];
		
		childIndex[0] = parent2.getIndex(node);
		parent2.remove(childIndex[0]);
		
		        //removeNodeFromParent( node );
		        return;
		    }
		}
		*/

		//MessageNode child = (MessageNode) hashtable.get(uid);

		//removeNodeFromParent( child );

	}

	protected MessageNode findMessage(MessageNode parent, Object uid)
	{
		for (int i = 0; i < parent.getChildCount(); i++)
		{
			MessageNode child = (MessageNode) parent.getChildAt(i);
			Object u = child.getUid();

			if (u.equals(uid))
				return child;

			findMessage(child, uid);

		}

		return null;
	}

	public void markMessage(Object[] uids, int subMode)
	{

		for (int i = 0; i < uids.length; i++)
		{

			MessageNode node = findMessage(getRootNode(), uids[i]);

			if (node != null)
			{
				ColumbaHeader header = node.getHeader();
				switch (subMode)
				{
					case Operation.MARK_READ :
						{
							header.getFlags().setSeen(true);
							header.getFlags().setRecent(false);
							break;
						}
					case Operation.MARK_FLAGGED :
						{
							header.getFlags().setFlagged(true);
							break;
						}
					case Operation.MARK_EXPUNGED :
						{
							header.getFlags().setDeleted(true);
							break;
						}
					case Operation.MARK_ANSWERED :
						{
							header.getFlags().setAnswered(true);
							break;
						}
				}

				if (uids.length < 100)
					nodeChanged(node);
			}
		}

		if (uids.length >= 100)
			update();

	}

	public void removeMessage(Object[] uids)
	{
		if (uids.length > 100)
		{
			// recreate whole tablemodel

			for (int i = 0; i < uids.length; i++)
			{
				headerList.remove(uids[i]);
			}

			update();
		}
		else
		{
			// single operation per message

			for (int i = 0; i < uids.length; i++)
			{
				headerList.remove(uids[i]);
				removeNode(uids[i]);
			}
		}
	}

	public void addMessage(Message message)
	{

		//System.out.println("headertablemodel->addMessage()");

		int count;
		if (headerList != null)
			count = getRootNode().getChildCount();
		else
			count = 0;

		if (count == 0)
		{
			//System.out.println("------------------> first message");
			headerList = new HeaderList();
			//setFolder( getFolder() );
			//update();

			ColumbaHeader header = message.getHeader();
			Object uid = message.getUID();

			headerList.add(header, uid);

			MessageNode child = new MessageNode(header, uid);
			getRootNode().add(child);

			//hashtable.put( uid, child );

			nodeStructureChanged(getRootNode());
			return;
		}

		ColumbaHeader header = null;
		try
		{
			if (headerList != null)
			{

				header = message.getHeader();
				headerList.add(header, message.getUID());
			}

			MessageNode node = new MessageNode(header, message.getUID());
			setSelectedMessageNode(node);

			boolean result =
				getTableModelFilteredView().manipulateModel(TableModelPlugin.NODES_INSERTED);

			if (result == true)
			{
				boolean result2 =
					getTableModelThreadedView().manipulateModel(TableModelPlugin.NODES_INSERTED);

				if (result2 == true)
				{

				}
				else
				{
					int index = getTableModelSorter().getInsertionSortIndex(node);

					insertNodeInto(node, getRootNode(), index);

					//hashtable.put( message.getUID(), node );
				}
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}

	}

	public TreeNode[] getPathToRoot(TreeNode aNode, int depth)
	{

		TreeNode[] nodes = super.getPathToRoot(aNode, depth);

		return nodes;
	}

	public TableModelFilteredView getTableModelFilteredView()
	{
		return (TableModelFilteredView) tableModelPlugins.get(0);
	}

	public TableModelThreadedView getTableModelThreadedView()
	{
		return (TableModelThreadedView) tableModelPlugins.get(1);
	}

	public TableModelSorter getTableModelSorter()
	{
		return (TableModelSorter) tableModelPlugins.get(2);
	}

	public HeaderList getHeaderList()
	{
		return headerList;
	}

	public void update()
	{
		setHeaderList(headerList);
	}

	public void setHeaderList(HeaderList list)
	{
		root.removeAllChildren();

		headerList = list;

		if (list == null)
		{
			nodeStructureChanged(root);
			return;
		}

		if (list.count() == 0)
		{
			nodeStructureChanged(root);
			return;
		}

		try
		{

			boolean result =
				getTableModelFilteredView().manipulateModel(TableModelPlugin.STRUCTURE_CHANGE);

			if (result == false)
			{
				//hashtable = new Hashtable();

				for (int i = 0; i < headerList.count(); i++)
				{
					ColumbaHeader header = headerList.getHeader(i);
					Object uid = headerList.getUid(i);

					MessageNode child = new MessageNode(header, uid);

					//hashtable.put( uid, child );

					root.add(child);
				}
			}

			getTableModelThreadedView().manipulateModel(TableModelPlugin.STRUCTURE_CHANGE);

			getTableModelSorter().manipulateModel(TableModelPlugin.STRUCTURE_CHANGE);

		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}

		nodeStructureChanged(root);

	}

	/*
	public void setFolder( Folder folder )
	{
	this.folder = folder;
	
	    //System.out.println("headerTableModel->setFolder");
	
	
	
	root.removeAllChildren();
	
	
	
	    if ( folder == null )
	       {
	            nodeStructureChanged( root );
	            System.out.println("folder is null");
	            return;
	       }
	
	
	    try
	    {
	
	    boolean result = getTableModelFilteredView().manipulateModel( TableModelPlugin.STRUCTURE_CHANGE );
	
	    if ( result == false )
	    {
	        for ( int i=0; i<folder.count(); i++ )
	    {
		Message message = folder.get(i);
		Rfc822Header header = message.getHeader();
	            Object uid = folder.getUid(i);
		//String subject = (String) header.get("Subject");
	
		//System.out.println("subject("+i+"): " +subject );
	
		MessageNode child = new MessageNode( message, uid );
	
		//child.setUserObject( message );
		root.add( child );
	    }
	    }
	
	    getTableModelThreadedView().manipulateModel( TableModelPlugin.STRUCTURE_CHANGE );
	
	    getTableModelSorter().manipulateModel( TableModelPlugin.STRUCTURE_CHANGE );
	
	
	    }
	    catch ( Exception ex )
	    {
	        ex.printStackTrace();
	    }
	
	    //System.out.println("folder count: "+ folder.count() );
	
	
	
	
	
	    nodeStructureChanged( root );
	
	}
	*/

	public void setMessageCollection(MessageCollection mc)
	{
		this.messageCollection = mc;

		System.out.println("headerTableModel->setMessagecollection");

		root.removeAllChildren();

		if (messageCollection == null)
		{
			System.out.println("messagecollection is null");
			return;
		}

		try
		{
			boolean result =
				getTableModelFilteredView().manipulateModel(TableModelPlugin.STRUCTURE_CHANGE);

			if (result == false)
			{
				for (int i = 0; i < messageCollection.count(); i++)
				{
					Message message = messageCollection.get(i);
					Rfc822Header header = message.getHeader();
					Object uid = header.get("columba.pop3uid");
					//String subject = (String) header.get("Subject");

					//System.out.println("subject("+i+"): " +subject );

					MessageNode child = new MessageNode(message, uid);

					//child.setUserObject( message );
					root.add(child);
				}
			}

			getTableModelThreadedView().manipulateModel(TableModelPlugin.STRUCTURE_CHANGE);

			getTableModelSorter().manipulateModel(TableModelPlugin.STRUCTURE_CHANGE);

			System.out.println("mc count: " + messageCollection.count());

		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}

		nodeStructureChanged(root);

	}

	/***************************** treemodel interface ********************************/

	//
	// The TreeModel interface
	//

	/**
	 * Returns the number of children of <code>node</code>.
	 */
	public int getChildCount(Object node)
	{

		DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) node;

		int count = treeNode.getChildCount();

		return count;
	}

	/**
	 * Returns the child of <code>node</code> at index <code>i</code>.
	 */
	public Object getChild(Object node, int i)
	{

		DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) node;

		DefaultMutableTreeNode child = (DefaultMutableTreeNode) treeNode.getChildAt(i);

		return child;
	}

	/**
	 * Returns true if the passed in object represents a leaf, false
	 * otherwise.
	 */
	public boolean isLeaf(Object node)
	{
		//Message message = (Message) node;
		DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) node;

		boolean result;

		if (treeNode.getChildCount() == 0)
			result = true;
		else
			result = false;

		return result;
	}

	/**************************** treetablenode interface ******************************/
	//
	//  The TreeTableNode interface.
	//

	/**
	 * Returns the number of columns.
	 */
	public int getColumnCount()
	{
		return item.count();
	}

	/**
	 * Returns the name for a particular column.
	 */
	public String getColumnName(int column)
	{

		return item.getName(column);
	}

	public int getColumnNumber(String name)
	{

		for (int i = 0; i < getColumnCount(); i++)
		{
			//System.out.println("column name: "+ getColumnName(i) );
			if (name.indexOf(getColumnName(i)) != -1)
				return i;
		}

		return -1;
	}

	/**
	 * Returns the class for the particular column.
	 */
	public Class getColumnClass(int column)
	{

		//return getValueAt( getRoot() , column).getClass();

		//return cTypes[column];

		String name = getColumnName(column);

		if (name.equalsIgnoreCase("subject"))
		{
			return TreeTableModel.class;
		}
		else if (name.equalsIgnoreCase("date"))
		
			{
			return Date.class;
		}
		else if (name.equalsIgnoreCase("size"))
		
			{
			return Integer.class;
		}
		else if (name.equalsIgnoreCase("status"))
		
			{
			return org.columba.message.Flags.class;
		}
		else if (name.equalsIgnoreCase("flagged"))
		
			{
			return Boolean.class;
		}
		else if (name.equalsIgnoreCase("attachment"))
		
			{
			return Boolean.class;
		}
		else if (name.equalsIgnoreCase("priority"))
		
			{
			return Integer.class;
		}
		else if (name.equalsIgnoreCase("fetch"))
		
			{
			return Boolean.class;
		}
		else if (name.equalsIgnoreCase("delete"))
		
			{
			return Boolean.class;
		}
		else
			return String.class;

	}

	/**
	 * Returns the value of the particular column.
	 */
	public Object getValueAt(Object node, int col)
	
	{

		DefaultMutableTreeNode treeNode = (DefaultMutableTreeNode) node;
		ColumbaHeader header = null;

		if (treeNode.getUserObject() instanceof String)
			return new String("rootNode");
		else
		{
			//System.out.println("found message instance");
			header = (ColumbaHeader) treeNode.getUserObject();
		}

		if (header == null)
			return "";

		String column = getColumnName(col);

		//Message message = folder.get( row );
		//if ( message == null ) return "";

		if (column.equals("Status"))
		{
			return header.getFlags();
		}
		else if (column.equals("Flagged"))
		{
			return header.get("columba.flags.flagged");
		}
		else if (column.equals("Fetch"))
		{
			return (Boolean) header.get("columba.fetchstate");
		}
		/*
		else if ( column.equals("Delete") )
		{
		
		    return new Boolean( header.get("columba.fetchstate );
		}
		*/
		else if (column.equals("Date"))
		{
			//Date date = message.getDate();
			if (header.get("columba.date") instanceof Date)
			{
				Date date = (Date) header.get("columba.date");
				return date;
			}
			else
				return (String) header.get("columba.date");
		}
		else if (column.equals("Attachment"))
		{
			//Boolean b = new Boolean( message.getAttachment() );

			return (Boolean) header.get("columba.attachment");
		}

		else if (column.equals("Size"))
		{
			//Integer i = new Integer( message.getSize() );
			return (Integer) header.get("columba.size");

		}

		else if (column.equals("From"))
		{
			//String s = message.getShortFrom();

			return (String) header.get("columba.from");
		}
		else if (column.equals("Priority"))
		{
			//int s = message.getPriority();
			//return new Integer( s );

			return (Integer) header.get("columba.priority");
		}
		else
		{
			Object object = header.get(column);

			if (object == null)
				return new String("");
			return object;
		}

	}

}