// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.gui.header;

import org.columba.message.*;
import org.columba.folder.*;
import org.columba.folder.imap.*;
import org.columba.folder.columba.*;
import org.columba.folder.outbox.*;
import org.columba.folder.gui.*;

import org.columba.parser.*;
import org.columba.util.*;
import org.columba.config.*;

import org.columba.gui.message.*;
import org.columba.gui.util.*;
import org.columba.gui.action.*;
import org.columba.gui.tree.*;
import org.columba.gui.composer.*;
import org.columba.gui.themes.*;
import org.columba.filter.*;
import org.columba.main.*;

import org.columba.gui.header.util.*;
import org.columba.gui.header.action.*;
import org.columba.gui.header.treetable.*;
import org.columba.gui.header.menu.*;

import java.io.*;

import javax.swing.text.*;
import javax.swing.table.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.*;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.event.*;

import java.util.*;

/**
 * This class shows the messageheaderlist
 *
 *
 * @version 0.9.1
 * @author Frederik
 */

public class HeaderTableViewer
	extends JPanel
	implements FolderChangeListener, TreeSelectionListener
{

	private HeaderTableMenu menu;

	private HeaderTable headerTable;
	private HeaderTableModel headerTableModel;

	private MainInterface mainInterface;

	public JScrollPane scrollPane;
	private SwingWorker worker;
	private Folder folder;
	private HeaderList headerList;

	private FilterToolbar filterToolbar;

	private HeaderTableActionListener headerTableActionListener;

	private MessageNode[] messageNodes;

	private MessageNode node;
	private MessageNode oldNode;

	protected TableModelFilteredView tableModelFilteredView;
	protected HeaderTableModelSorter tableModelSorter;
	protected TableModelThreadedView tableModelThreadedView;

	private HeaderTableMouseListener headerTableMouseListener;
	private HeaderTableDnd headerTableDnd;
	private HeaderTableFocusListener headerTableFocusListener;
	private HeaderItemActionListener headerItemActionListener;
	private FilterActionListener filterActionListener;

	private HeaderTableItem headerTableItem;
	
	private boolean folderChanged = false;
	
	private int counter=1;

	public HeaderTableViewer(MainInterface mainInterface)
	{
		super();
		this.mainInterface = mainInterface;

		setLayout(new BorderLayout());

		headerTableItem = (HeaderTableItem) mainInterface.headerTableItem.clone();
		headerTableItem.removeEnabledItem();

		headerTableModel = new HeaderTableModel(headerTableItem);

		tableModelFilteredView = new TableModelFilteredView(headerTableModel);

		tableModelSorter = new HeaderTableModelSorter(headerTableModel);
		tableModelSorter.setWindowItem(mainInterface.mainFrameWindowItem);

		tableModelThreadedView = new TableModelThreadedView(headerTableModel);

		headerTable = new HeaderTable(headerTableModel);

		headerTableModel.registerPlugin(tableModelFilteredView);
		headerTableModel.registerPlugin(tableModelThreadedView);
		headerTableModel.registerPlugin(tableModelSorter);

		headerTableActionListener = new HeaderTableActionListener(mainInterface, this);

		menu = new HeaderTableMenu(this);

		initRenderer();

		headerTableDnd = new HeaderTableDnd(headerTable.getTable());

		headerTable.addTreeSelectionListener(this);

		headerTableMouseListener = new HeaderTableMouseListener(this);
		headerTable.getTable().addMouseListener(headerTableMouseListener);

		headerTableFocusListener = new HeaderTableFocusListener();
		headerTable.getTable().addFocusListener(headerTableFocusListener);

		headerItemActionListener = new HeaderItemActionListener(this, headerTableItem);
		filterActionListener = new FilterActionListener(this);

		addMouseListenerToHeaderInTable();

		scrollPane = new JScrollPane(headerTable.getTable());
		scrollPane.setWheelScrollingEnabled(true);
		scrollPane.getViewport().setBackground(Color.white);
		//scrollPane.setPreferredSize( new java.awt.Dimension(300,200) );

		add(scrollPane, BorderLayout.CENTER);

		filterToolbar = new FilterToolbar(this);
		filterToolbar.setAlignmentX(0);
		if (mainInterface.mainFrameWindowItem.isShowFilterToolbar() == true)
			add(filterToolbar, BorderLayout.NORTH);

	}

	public HeaderTableMouseListener getHeaderTableMouseListener()
	{
		return headerTableMouseListener;
	}

	public boolean hasFocus()
	{
		return headerTableFocusListener.hasFocus();
	}

	/**
	 * return FilterToolbar
	 */

	public FilterToolbar getFilterToolbar()
	{
		return filterToolbar;
	}

	/**
	 * return ImageIcon
	 */
	protected ImageIcon getImageIcon(String id, String failCase)
	{
		ImageIcon icon = (ImageIcon) UIManager.getIcon(id);

		if (icon == null)
			icon = ImageLoader.getImageIcon("org/columba/images/" + failCase + ".gif");

		return icon;
	}

	/**
	 * return HeaderTableItem
	 */
	public HeaderTableItem getHeaderTableItem()
	{
		return headerTableItem;
	}

	/**
	 * set the render for each column
	 */
	protected void initRenderer()
	{

		headerTable.setTreeCellRenderer(
			new SubjectTreeCellRenderer(headerTable.getTree()));

		HeaderTableItem v = (HeaderTableItem) mainInterface.headerTableItem.clone();
		v.removeEnabledItem();

		for (int i = 0; i < v.count(); i++)
		{
			String name = v.getName(i);
			int size = v.getSize(i);
			int position = v.getPosition(i);

			if (name.equalsIgnoreCase("size"))
			{
				headerTable.registerRenderer(
					"Size",
					new HeaderTableSizeRenderer(headerTable.getTree()),
					new CommonHeaderRenderer(
						name,
						ResourceLoader.getString("header/mail", "size"),
						getTableModelSorter()),
					size,
					false,
					position);
			}
			else if (name.equalsIgnoreCase("Status"))
			{

				headerTable.registerRenderer(
					"Status",
					new StatusRenderer(),
					new BooleanHeaderRenderer(true, name, getTableModelSorter()),
					23,
					true,
					position);
			}
			else if (name.equalsIgnoreCase("Flagged"))
			{
				headerTable.registerRenderer(
					"Flagged",
					new FlaggedRenderer(),
					new BooleanHeaderRenderer(true, name, getTableModelSorter()),
					23,
					true,
					position);
			}
			else if (name.equalsIgnoreCase("Attachment"))
			{
				headerTable.registerRenderer(
					"Attachment",
					new BooleanRenderer(
						true,
						ImageLoader.getImageIcon("attachmentflag", "attachment")),
					new BooleanHeaderRenderer(true, name, getTableModelSorter()),
					23,
					true,
					position);
			}
			else if (name.equalsIgnoreCase("Date"))
			{
				headerTable.registerRenderer(
					"Date",
					new HeaderTableDateRenderer(headerTable.getTree(), true),
					new DateHeaderRenderer(
						name,
						ResourceLoader.getString("header/mail", "date"),
						getTableModelSorter()),
					size,
					false,
					position);

			}
			else if (name.equalsIgnoreCase("Priority"))
			{
				headerTable.registerRenderer(
					"Priority",
					new PriorityRenderer(true),
					new BooleanHeaderRenderer(true, name, getTableModelSorter()),
					23,
					true,
					position);

			}

			else if (name.equalsIgnoreCase("Subject"))
			{
				/*
				headerTable.registerRenderer( "Subject",
				                              new HeaderTableCommonRenderer( headerTable.getTree() ),
				                              new CommonHeaderRenderer( name,
				                                                        headerLabels.getString("subject"),
				                                                        getTableModelSorter() ),
				                                                        size, false, position );
				*/

				headerTable.registerRenderer(
					"Subject",
					null,
					new CommonHeaderRenderer(
						name,
						ResourceLoader.getString("header/mail", "subject"),
						getTableModelSorter()),
					size,
					false,
					position);
			}
			else
			{
				String str = new String();
				try
				{
					str = ResourceLoader.getString("header/mail", name.toLowerCase());

				}
				catch (MissingResourceException ex)
				{
					System.out.println(ex.getMessage());

				}

				if (str.length() == 0)
					headerTable.registerRenderer(
						name,
						new HeaderTableCommonRenderer(headerTable.getTree()),
						new CommonHeaderRenderer(name, name, getTableModelSorter()),
						size,
						false,
						position);

				else
					headerTable.registerRenderer(
						name,
						new HeaderTableCommonRenderer(headerTable.getTree()),
						new CommonHeaderRenderer(name, str, getTableModelSorter()),
						size,
						false,
						position);

			}
		}

	}

	/**
	 * return a Dialog wich is used by the copy/move message operation
	 * to get the destination folder
	 */
	public org.columba.gui.tree.util.SelectFolderDialog getSelectFolderDialog()
	{
		return mainInterface.treeViewer.getSelectFolderDialog();
	}

	/**
	 * show the filter toolbar
	 */
	public void showToolbar()
	{
		add(filterToolbar, BorderLayout.NORTH);

		validate();
		repaint();
	}

	/**
	 * hide the filter toolbar
	 */
	public void hideToolbar()
	{
		remove(filterToolbar);
		validate();
		repaint();
	}

	/**
	 * return the ActionListener
	 *
	 */
	public HeaderTableActionListener getActionListener()
	{
		return headerTableActionListener;
	}

	/**
	 * save the column state:
	 *  - position
	 *  - size
	 *  - appearance
	 * of every column
	 */
	public void saveColumnConfig()
	{
		HeaderTableItem v = (HeaderTableItem) mainInterface.headerTableItem.clone();
		v.removeEnabledItem();

		for (int i = 0; i < v.count(); i++)
		{
			String c = (String) v.getName(i);
			TableColumn tc = getHeaderTable().getTable().getColumn(c);

			v.setSize(i, tc.getWidth());

			try
			{
				int index = getHeaderTable().getTable().getColumnModel().getColumnIndex(c);
				v.setPosition(i, index);
			}
			catch (IllegalArgumentException ex)
			{
				ex.printStackTrace();
			}

		}

	}

	/**
	 * sets the header, which is going to be viewed
	 *
	 * @param f a <code>Folder</code>
	 * @see Folder
	 */

	public void setHeaderList(HeaderList list)
	{
		headerList = list;

		if (list != null)
		{

			Runnable run = new Runnable()
			{
				public void run()
				{
					getHeaderTableModel().setHeaderList(headerList);

					if (getHeaderTable().getRowCount() != 0)
					{
						getHeaderTable().setSelectedRow(0);
						scrollPane.getVerticalScrollBar().setValue(0);
						getActionListener().changeMessageActions();
					}

				}
			};
			try
			{
				if (!SwingUtilities.isEventDispatchThread())
					SwingUtilities.invokeAndWait(run);
				else
					SwingUtilities.invokeLater(run);

			}
			catch (Exception ex)
			{
			}
		}
		else
		{
			getHeaderTableModel().setHeaderList(null);
		}
	}

	/**
	 * set folder to show
	 */
	public void setFolder(Folder f)
	{
		this.folder = f;
	}

	/**
	 * return currently showed folder
	 */
	public Folder getFolder()
	{
		return folder;
	}

	/**
	 * return an array of all selected message uids
	 */
	public Object[] getUids()
	{
		MessageNode[] nodes = getHeaderTable().getSelectedNodes();

		Object[] uids = new Object[nodes.length];

		for (int i = 0; i < nodes.length; i++)
		{
			uids[i] = nodes[i].getUid();
		}

		return uids;

	}

	/**
	 * return HeaderTable widget which does all the dirty work
	 */
	public HeaderTable getHeaderTable()
	{
		return headerTable;
	}

	/**
	 * return the Model which contains a HeaderList
	 */
	public HeaderTableModel getHeaderTableModel()
	{
		return headerTableModel;

	}
	
	public void backupSelection()
	{
		if ( messageNodes != null )
			getHeaderTable().setSelection( messageNodes );	
	}
	
	public void clearMessageNodeList()
	{
		messageNodes = null;
	}

	/**
	 * return the table model sorter
	 */
	public HeaderTableModelSorter getTableModelSorter()
	{
		return tableModelSorter;
	}

	/**
	 * return the threaded view model
	 */
	public TableModelThreadedView getTableModelThreadedView()
	{
		return tableModelThreadedView;
	}

	/**
	 * return the filtered view model
	 */
	public TableModelFilteredView getTableModelFilteredView()
	{
		return tableModelFilteredView;
	}

	/**
	 * return ActionListener for the headeritem sorting
	 */
	public HeaderItemActionListener getHeaderItemActionListener()
	{
		return headerItemActionListener;
	}

	/**
	 * return ActionListener for FilterToolbar
	 */
	public FilterActionListener getFilterActionListener()
	{
		return filterActionListener;
	}

	/**
	 * show the message in the messageviewer
	 */
	public void showMessage()
	{
		MessageNode node = getHeaderTable().getSelectedNode();
		if (node == null)
			return;
		if (node.equals(oldNode))
			return;

		oldNode = node;
		Object uid = node.getUid();

		ColumbaHeader header = (ColumbaHeader) node.getUserObject();
		String contentType = (String) header.get("Content-Type");

		System.out.println("content-type: " + contentType);

		mainInterface.messageViewer.setSecurityIndicator(SecurityIndicator.NOOP);

		if ((contentType != null))
		{
			if (contentType.toLowerCase().indexOf("encrypted") != -1)
			{
				System.out.println("open encrypted message");

				EncryptionOperation op =
					new EncryptionOperation(Operation.DECRYPT, 1, folder, uid);
				mainInterface.crossbar.operate(op);

				return;
			}
			//FIXME - verifying is not yet implemented correctly
			/*
			    else if ( contentType.toLowerCase().indexOf("signed") != -1 )
			    {
			        System.out.println("open signed message");
			
			
			        EncryptionOperation op = new EncryptionOperation( Operation.VERIFY,
			                                                  1, folder, uid );
			        mainInterface.crossbar.operate( op );
			
			
			        return;
			    }
			*/
			else
			{
				mainInterface.crossbar.operate(
					new GuiOperation(Operation.MESSAGEBODY, 4, folder, uid));
			}
		}
		else
		{
			mainInterface.crossbar.operate(
				new GuiOperation(Operation.MESSAGEBODY, 4, folder, uid));
		}

		//mainInterface.crossbar.operate( new GuiOperation(Operation.MESSAGEBODY, 20, folder, uid) );
	}

	/**
	 * return the PopupMenu for the table
	 */
	public JPopupMenu getPopupMenu()
	{
		return menu.getPopupMenu();
	}

	/************************** actions ********************************/

	/**
	 * create the PopupMenu
	 */

	/**
	 * MouseListener sorts table when clicking on a column header
	 */
	protected void addMouseListenerToHeaderInTable()
	{
		final JTable tableView = getHeaderTable().getTable();

		tableView.setColumnSelectionAllowed(false);

		MouseAdapter listMouseListener = new MouseAdapter()
		{
			public void mouseClicked(MouseEvent e)
			{
				TableColumnModel columnModel = tableView.getColumnModel();
				int viewColumn = columnModel.getColumnIndexAtX(e.getX());
				int column = tableView.convertColumnIndexToModel(viewColumn);

				if (e.getClickCount() == 1 && column != -1)
				{
					getTableModelSorter().setSortingColumn(column);
					getHeaderTableModel().update();
					mainInterface.mainFrame.getMenu().updateSortMenu();
				}
			}
		};

		JTableHeader th = tableView.getTableHeader();
		th.addMouseListener(listMouseListener);
	}

	/**
	 * method is called when the selection changes
	 */
	public void valueChanged(TreeSelectionEvent e)
	{
		
		
		getActionListener().changeMessageActions();
		
		

	}

	// method is called when folder data changed
	// the method updates the model
	public void folderChanged(FolderEvent event)
	{
		//System.out.println("headertableviewer->folderChanged");
		
		
		
		
		final FolderEvent e = event;

		final int rowCount = getHeaderTable().getSelectedRowCount();

		Folder source = (Folder) e.getSource();

		//if ( mainInterface.treeViewer.getSelected().equals(source) == false ) return;

		if (e.getMode() == FolderEvent.UPDATE)
		{
			messageNodes = getHeaderTable().getSelectedNodes();
			
			getHeaderTableModel().update();
		}
		else if (e.getMode() == FolderEvent.ADD)
		{

			Runnable run = new Runnable()
			{
				public void run()
				{
					if (e.getMessage() != null)
					{

						messageNodes = getHeaderTable().getSelectedNodes();

						getHeaderTableModel().addMessage(e.getMessage());

						//getHeaderTable().setSelection(messageNodes);
					}
				}
			};
			try
			{
				if (!SwingUtilities.isEventDispatchThread())
					SwingUtilities.invokeAndWait(run);
				else
					SwingUtilities.invokeLater(run);

			}
			catch (Exception ex)
			{
			}

		}
		else if (e.getMode() == FolderEvent.REMOVE)
		{

			final Object[] uids = e.getUids();
			Runnable run = new Runnable()
			{
				public void run()
				{
					//messageNodes = getHeaderTable().getSelectedNodes();

					getHeaderTableModel().removeMessage(uids);

					//getHeaderTable().setSelection(messageNodes);
				}
			};
			try
			{
				if (!SwingUtilities.isEventDispatchThread())
					SwingUtilities.invokeAndWait(run);
				else
					SwingUtilities.invokeLater(run);

			}
			catch (Exception ex)
			{
			}

		}
		else if (e.getMode() == FolderEvent.MARK)
		{
			final Object[] uids = e.getUids();

			Runnable run = new Runnable()
			{
				public void run()
				{
					
					
					
					messageNodes = getHeaderTable().getSelectedNodes();
					
					getHeaderTableModel().markMessage(uids, e.getSubMode());					
					
					
					
					
				}
			};
			try
			{
				if (!SwingUtilities.isEventDispatchThread())
					SwingUtilities.invokeAndWait(run);
				else
					SwingUtilities.invokeLater(run);

			}
			catch (Exception ex)
			{
			}
			


			
		}
	}

}