// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.gui.header.action;

import java.awt.event.*;
import java.awt.*;

import javax.swing.*;
import javax.swing.tree.*;
import java.util.*;

import org.columba.gui.header.*;
import org.columba.folder.*;
import org.columba.folder.imap.*;
import org.columba.folder.virtual.*;
import org.columba.gui.composer.*;
import org.columba.gui.util.*;
import org.columba.gui.tree.*;
import org.columba.gui.tree.util.*;
import org.columba.message.*;
import org.columba.main.*;
import org.columba.util.*;
import org.columba.config.*;
import org.columba.gui.header.util.*;
import org.columba.gui.header.*;
import org.columba.gui.action.*;
import org.columba.gui.message.*;
import org.columba.gui.config.filter.*;
import org.columba.gui.config.search.*;
import org.columba.gui.util.*;
import org.columba.filter.*;
import org.columba.folder.columba.*;

public class HeaderTableActionListener implements ActionListener
{
	private MainInterface mainInterface;
	private HeaderTableViewer headerTableViewer;

	public BasicAction markAsReadAction;
	public BasicAction markAsFlaggedAction;
	public BasicAction markAsExpungedAction;
	public BasicAction nextAction;
	public BasicAction nextUnreadAction;
	public BasicAction prevAction;
	public BasicAction prevUnreadAction;
	public BasicAction markAllAsReadAction;
	public BasicAction replyAction;
	public BasicAction replyToAction;
	public BasicAction replyToAllAction;
	public BasicAction replyAsAttachmentAction;
	public BasicAction forwardAction;
	public BasicAction forwardInlineAction;
	
	public BasicAction saveAction;
	public BasicAction printAction;
	public BasicAction copyMessageAction;
	public BasicAction moveMessageAction;
	public BasicAction deleteMessageAction;
	public BasicAction addSenderAction;
	public BasicAction addAllSendersAction;
	public BasicAction viewSourceAction;

	public BasicAction filterSubjectAction;
	public BasicAction filterFromAction;
	public BasicAction filterToAction;

	public BasicAction vFolderSubjectAction;
	public BasicAction vFolderFromAction;
	public BasicAction vFolderToAction;

	public BasicAction viewThreadedAction;

	public BasicAction openMessageWithComposerAction;

	

	public HeaderTableActionListener(
		MainInterface mainInterface,
		HeaderTableViewer headerTableViewer)
	{
		this.mainInterface = mainInterface;

		this.headerTableViewer = headerTableViewer;

		

		initActions();
	}

	protected void initActions()
	{

		markAsReadAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_markasread"),
				
				"Mark selected Message as Read",
				"MARK_AS_READ",
				ImageLoader.getImageIcon("markasread_small", "Properties16"),
				ImageLoader.getImageIcon("markasread", "Properties24"),
				'R',
				KeyStroke.getKeyStroke("M"));

		markAsFlaggedAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_markasflagged"),
				
				"Mark selected Message as Flagged",
				"MARK_AS_FLAGGED",
				ImageLoader.getImageIcon("markasflagged_small", "Bookmarks16"),
				ImageLoader.getImageIcon("markasflagged", "Bookmarks24"),
				'F',
				null);

		markAsExpungedAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_markasexpunged"),
				
				"Mark selected Messages as Expunged",
				"MARK_AS_EXPUNGED",
				ImageLoader.getImageIcon("markasexpunged_small", "Delete16"),
				ImageLoader.getImageIcon("markasexpunged", "Delete24"),
				'E',
				null);

		nextAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_view_nextmessage"),
				
				"Goto Next Message",
				"NEXT_MESSAGE",
				null,
				null,
				'M',
				KeyStroke.getKeyStroke("F"));

		nextUnreadAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_view_nextunreadmessage"),
				
				"Goto Next Unread Message",
				"NEXT_UNREAD_MESSAGE",
				null,
				null,
				'U',
				KeyStroke.getKeyStroke("N"));

		prevAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_view_prevmessage"),
				
				"Goto Prev Message",
				"PREV_MESSAGE",
				null,
				null,
				'M',
				KeyStroke.getKeyStroke("B"));

		prevUnreadAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_view_prevunreadmessage"),
				
				"Goto Prev Unread Message",
				"PREV_UNREAD_MESSAGE",
				null,
				null,
				'U',
				KeyStroke.getKeyStroke("P"));

		markAllAsReadAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_markallasread"),
				"Mark all Message as Read",
				"MARK_ALL_AS_READ",
				null,
				null,
				'A',
				KeyStroke.getKeyStroke("A"));

		replyAction =
			new BasicAction(	
				ResourceLoader.getString("action/mail","menu_message_reply"),
				ResourceLoader.getString("action/mail","menu_message_reply"),
				"REPLY",
				ImageLoader.getImageIcon("reply_small", "Redo16"),
				ImageLoader.getImageIcon("reply", "Redo24"),
				'R',
				KeyStroke.getKeyStroke(KeyEvent.VK_R, ActionEvent.CTRL_MASK));

		replyToAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_replyto"),
				ResourceLoader.getString("action/mail","menu_message_replyto"),
				
				"REPLY_MAILINGLIST",
				null,
				null,
				'0',
				null);
		replyToAllAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_replytoall"),
				ResourceLoader.getString("action/mail","menu_message_replytoall"),
				
				"REPLY_TO_ALL",
				null,
				null,
				'0',
				null);
		replyAsAttachmentAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_replyasattachment"),
				ResourceLoader.getString("action/mail","menu_message_replyasattachment"),
				
				"REPLY_AS_ATTACHMENT",
				null,
				null,
				'0',
				null);

		moveMessageAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_move"),
				ResourceLoader.getString("action/mail","menu_message_move"),
				
				"MOVE_MESSAGE",
				ImageLoader.getImageIcon("movemessage_small", ""),
				ImageLoader.getImageIcon("movemessage", ""),
				'M',
				null);

		copyMessageAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_copy"),
				ResourceLoader.getString("action/mail","menu_message_copy"),
				
				"COPY_MESSAGE",
				ImageLoader.getImageIcon("copymessage_small", ""),
				ImageLoader.getImageIcon("copymessage", ""),
				'C',
				null);

		deleteMessageAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_delete"),
				ResourceLoader.getString("action/mail","menu_message_delete"),
				
				"DELETE_MESSAGE",
				ImageLoader.getImageIcon("deletemessage_small", "Delete16"),
				ImageLoader.getImageIcon("deletemessage", "Delete24"),
				'0',
				null);

		replyToAllAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_replytoall"),
				ResourceLoader.getString("action/mail","menu_message_replytoall"),
				
				"REPLY_TO_ALL",
				null,
				null,
				'0',
				null);

		forwardAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_forward"),
				ResourceLoader.getString("action/mail","menu_message_forward"),
				
				"FORWARD",
				ImageLoader.getImageIcon("forward_small", "Export16"),
				ImageLoader.getImageIcon("forward", "Export24"),
				'F',
				KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
		forwardInlineAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_forwardinline"),
				ResourceLoader.getString("action/mail","menu_message_forwardinline"),
				
				"FORWARD_INLINE",
				null,
				null,
				'0',
				null);

		

		addSenderAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_addsender"),
				ResourceLoader.getString("action/mail","menu_message_addsender"),
				
				"ADD_SENDER",
				null,
				null,
				'0',
				null);
		addAllSendersAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_addallsenders"),
				ResourceLoader.getString("action/mail","menu_message_addallsenders"),
				
				"ADD_ALLSENDERS",
				null,
				null,
				'0',
				null);
		
		printAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_print"),
				ResourceLoader.getString("action/mail","menu_message_print"),
				"PRINT",
				ImageLoader.getImageIcon("print_small", ""),
				ImageLoader.getImageIcon("print", ""),
				'0',
				KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.CTRL_MASK));
		saveAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_save"),
				ResourceLoader.getString("action/mail","menu_message_save"),
				
				"SAVE",
				ImageLoader.getImageIcon("savemessage_small", "Save16"),
				ImageLoader.getImageIcon("savemessage", "Save24"),
				'0',
				null);
		viewSourceAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_view_source"),
				ResourceLoader.getString("action/mail","menu_view_source"),
				
				"View Message Source",
				"VIEW_SOURCE",
				null,
				null,
				'0',
				null);

		filterSubjectAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_filteronsubject"),
				ResourceLoader.getString("action/mail","menu_message_filteronsubject"),
				
				"FILTER_ON_SUBJECT",
				null,
				null,
				'0',
				null);
		filterFromAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_filteronfrom"),
				ResourceLoader.getString("action/mail","menu_message_filteronfrom"),
				
				
				"FILTER_ON_FROM",
				null,
				null,
				'0',
				null);
		filterToAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_filteronto"),
				ResourceLoader.getString("action/mail","menu_message_filteronto"),
				
				"FILTER_ON_TO",
				null,
				null,
				'0',
				null);

		vFolderSubjectAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_filteronsubject"),
				ResourceLoader.getString("action/mail","menu_message_filteronsubject"),
				
				"VFOLDER_ON_SUBJECT",
				null,
				null,
				'0',
				null);
		vFolderFromAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_vfolderonfrom"),
				ResourceLoader.getString("action/mail","menu_message_vfolderonfrom"),
				
				"VFOLDER_ON_FROM",
				null,
				null,
				'0',
				null);
		vFolderToAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_message_vfolderonto"),
				ResourceLoader.getString("action/mail","menu_message_vfolderonto"),
				
				"VFOLDER_ON_TO",
				null,
				null,
				'0',
				null);

		viewThreadedAction =
			new BasicAction(
				ResourceLoader.getString("action/mail","menu_view_viewthreaded"),
				ResourceLoader.getString("action/mail","menu_view_viewthreaded"),
				
				"VIEW_THREADED",
				null,
				null,
				'0',
				null);
		openMessageWithComposerAction =
			new BasicAction(	
				ResourceLoader.getString("action/mail","menu_message_openwithcomposer"),
				ResourceLoader.getString("action/mail","menu_message_openwithcomposer"),
				
				"OPEN_MESSAGE",
				ImageLoader.getImageIcon("openmessage_small", "Open16"),
				ImageLoader.getImageIcon("openmessage", "Open24"),
				'0',
				null);

		viewThreadedAction.addActionListener(this);
		markAsReadAction.addActionListener(this);
		markAsReadAction.setEnabled(false);
		markAsFlaggedAction.addActionListener(this);
		markAsFlaggedAction.setEnabled(false);
		markAsExpungedAction.addActionListener(this);
		markAsExpungedAction.setEnabled(false);
		nextAction.addActionListener(this);
		nextAction.setEnabled(false);
		nextUnreadAction.addActionListener(this);
		nextUnreadAction.setEnabled(false);
		prevAction.addActionListener(this);
		prevAction.setEnabled(false);
		prevUnreadAction.addActionListener(this);
		prevUnreadAction.setEnabled(false);
		markAllAsReadAction.addActionListener(this);
		markAllAsReadAction.setEnabled(false);
		replyAction.addActionListener(this);
		replyAction.setEnabled(false);
		moveMessageAction.addActionListener(this);
		moveMessageAction.setEnabled(false);
		copyMessageAction.addActionListener(this);
		copyMessageAction.setEnabled(false);
		deleteMessageAction.addActionListener(this);
		deleteMessageAction.setEnabled(false);
		replyToAllAction.addActionListener(this);
		replyToAllAction.setEnabled(false);
		replyToAction.addActionListener(this);
		replyToAction.setEnabled(false);
		replyAsAttachmentAction.addActionListener(this);
		replyAsAttachmentAction.setEnabled(false);
		forwardAction.addActionListener(this);
		forwardAction.setEnabled(false);
		

		addSenderAction.addActionListener(this);
		addSenderAction.setEnabled(false);
		addAllSendersAction.addActionListener(this);
		addAllSendersAction.setEnabled(false);

		printAction.addActionListener(this);
		printAction.setEnabled(false);
		saveAction.addActionListener(this);
		saveAction.setEnabled(false);
		forwardInlineAction.addActionListener(this);
		forwardInlineAction.setEnabled(false);

		viewSourceAction.addActionListener(this);
		viewSourceAction.setEnabled(false);

		filterSubjectAction.addActionListener(this);
		filterSubjectAction.setEnabled(false);
		filterFromAction.addActionListener(this);
		filterFromAction.setEnabled(false);
		filterToAction.addActionListener(this);
		filterToAction.setEnabled(false);

		vFolderSubjectAction.addActionListener(this);
		vFolderSubjectAction.setEnabled(false);
		vFolderFromAction.addActionListener(this);
		vFolderFromAction.setEnabled(false);
		vFolderToAction.addActionListener(this);
		vFolderToAction.setEnabled(false);

		openMessageWithComposerAction.setEnabled(false);
		openMessageWithComposerAction.addActionListener(this);

	}

	public void changeMessageActions()
	{
		//System.out.println("headertableactionlistner->changemessageaction()");

		//int row = headerTableViewer.getHeaderTable().getSelectedRow();

		int rowCount = headerTableViewer.getHeaderTable().getSelectedRowCount();

		//System.out.println("row: "+ row );
		//System.out.println("rowcount: "+ rowCount );
		int max = headerTableViewer.getHeaderTable().getRowCount();
		//System.out.println("max count: "+ max );

		Folder folder = mainInterface.treeViewer.getSelected();
		//headerTableViewer.getFolder();
		FolderItem item = null;
		if (folder != null)
			item = folder.getFolderItem();

		boolean imap = false;

		if ((folder instanceof ImapFolder) || (folder instanceof ImapRootFolder))
			imap = true;

		if ((max == 0) || (item == null) || (rowCount == 0))
		{
			moveMessageAction.setEnabled(false);
			copyMessageAction.setEnabled(false);
			deleteMessageAction.setEnabled(false);

			mainInterface.globalActionCollection.cutAction.setEnabled(false);
			mainInterface.globalActionCollection.copyAction.setEnabled(false);
			mainInterface.globalActionCollection.deleteAction.setEnabled(false);

			markAsReadAction.setEnabled(false);
			markAsFlaggedAction.setEnabled(false);
			markAsExpungedAction.setEnabled(false);

			replyAction.setEnabled(false);
			replyToAllAction.setEnabled(false);
			replyToAction.setEnabled(false);
			replyAsAttachmentAction.setEnabled(false);
			forwardAction.setEnabled(false);
			forwardInlineAction.setEnabled(false);

			nextAction.setEnabled(false);
			prevAction.setEnabled(false);

			addSenderAction.setEnabled(false);

			viewSourceAction.setEnabled(false);

			filterSubjectAction.setEnabled(false);
			filterFromAction.setEnabled(false);
			filterToAction.setEnabled(false);
			vFolderSubjectAction.setEnabled(false);
			vFolderFromAction.setEnabled(false);
			vFolderToAction.setEnabled(false);

			openMessageWithComposerAction.setEnabled(false);
			//saveAction.setEnabled( false );
			//printAction.setEnabled( false );

		}
		else
		{
			replyAction.setEnabled(true);
			replyToAllAction.setEnabled(true);
			replyToAction.setEnabled(true);
			replyAsAttachmentAction.setEnabled(true);
			forwardAction.setEnabled(true);
			forwardInlineAction.setEnabled(true);

			openMessageWithComposerAction.setEnabled(true);

			if (item.isRemoveAllowed())
			{
				moveMessageAction.setEnabled(true);
				deleteMessageAction.setEnabled(true);
				mainInterface.globalActionCollection.cutAction.setEnabled(true);
				mainInterface.globalActionCollection.deleteAction.setEnabled(true);
			}
			else
			{
				moveMessageAction.setEnabled(false);
				deleteMessageAction.setEnabled(false);
				mainInterface.globalActionCollection.cutAction.setEnabled(false);
				mainInterface.globalActionCollection.deleteAction.setEnabled(false);
			}

			copyMessageAction.setEnabled(true);

			markAsReadAction.setEnabled(true);
			markAsFlaggedAction.setEnabled(true);
			markAsExpungedAction.setEnabled(true);

			mainInterface.globalActionCollection.copyAction.setEnabled(true);

			addSenderAction.setEnabled(true);

			viewSourceAction.setEnabled(true);

			filterSubjectAction.setEnabled(true);
			filterFromAction.setEnabled(true);
			filterToAction.setEnabled(true);
			vFolderSubjectAction.setEnabled(true);
			vFolderFromAction.setEnabled(true);
			vFolderToAction.setEnabled(true);

			/*
			if (row >= 1)
				prevAction.setEnabled(true);
			else
				prevAction.setEnabled(false);
			if (row < headerTableViewer.getHeaderTable().getRowCount() - 1)
				nextAction.setEnabled(true);
			else
				prevAction.setEnabled(false);
			*/
		}

		if (item == null)
			return;

		if (item.isAddAllowed())
			mainInterface.globalActionCollection.pasteAction.setEnabled(true);
		else
			mainInterface.globalActionCollection.pasteAction.setEnabled(false);
		/*
		        String editAction = headerTableViewer.getEditAction();
		        if ( editAction != null )
		        {
		
		        }
		        else
		        {
		            mainInterface.globalActionCollection.pasteAction.setEnabled( false );
		        }
		
		*/

		/*
		  Folder tmpFolder = mainInterface.treeViewer.getTempFolder();
		  if ( tmpFolder.hasMessages() )
		  mainInterface.globalActionCollection.pasteAction.setEnabled( true );
		  else
		  mainInterface.globalActionCollection.pasteAction.setEnabled( false );
		*/
	}

	/*
	private void updateTable( Folder folder )
	{
	      // if folder selected, update tableModel
	    if ( folder.equals( mainInterface.treeViewer.getSelected() ) )
	    {
	
	        IndexedTableModel model = mainInterface.headerTableViewer.getHeaderTable().getIndexedTableModel();
	        model.update();
	    }
	}
	*/

	/*
	  private void updateTree( FolderTreeNode node )
	  {
	  mainInterface.treeViewer.updateTree( node );
	  }
	*/

	private Object[] getUids()
	{
		/*
		int rows[] = new int[headerTableViewer.getSelectedRowCount()];
		rows = headerTableViewer.getSelectedRows();
		
		Folder folder = getFolder();
		*/

		Object uids[] = headerTableViewer.getUids();
		//folder.getUids( rows );

		return uids;
	}

	private Folder getFolder()
	{
		Folder folder = mainInterface.treeViewer.getSelected();

		return folder;
	}

	/*
	private FolderTreeNode getFolderTreeNode()
	{
	    FolderTreeNode node = mainInterface.treeViewer.getSelectedNode();
	    return node;
	}
	*/

	public boolean hasFocus()
	{
		return headerTableViewer.hasFocus();
	}

	public void actionPerformed(ActionEvent evt)
	{
		String command = evt.getActionCommand();

		if (command.equals(markAsReadAction.getActionCommand()))
		{
			Folder folder = getFolder();

			Object uids[] = getUids();

			FolderOperation op = new FolderOperation(Operation.MARK_READ, 0, uids, folder);
			mainInterface.crossbar.operate(op);

			/*
			FolderOp op = new FolderOp( FolderOp.MARK, FolderOp.READ, folder,  uids );
			mainInterface.folderOperator.operate( op );
			*/

		}
		else if (command.equals(markAsFlaggedAction.getActionCommand()))
		{
			Folder folder = getFolder();

			Object uids[] = getUids();

			FolderOperation op =
				new FolderOperation(Operation.MARK_FLAGGED, 0, uids, folder);
			mainInterface.crossbar.operate(op);
			/*
			FolderOp op = new FolderOp( FolderOp.MARK, FolderOp.FLAGGED, folder,  uids );
			mainInterface.folderOperator.operate( op );
			*/
		}
		else if (command.equals(markAsExpungedAction.getActionCommand()))
		{
			Folder folder = getFolder();

			Object uids[] = getUids();

			FolderOperation op =
				new FolderOperation(Operation.MARK_EXPUNGED, 0, uids, folder);
			mainInterface.crossbar.operate(op);
			/*
			FolderOp op = new FolderOp( FolderOp.MARK, FolderOp.EXPUNGED, folder,  uids );
			mainInterface.folderOperator.operate( op );
			*/
		}
		/*
		else if ( command.equals("HEADER_CONF") )
		{
		  HeaderTableItem v = mainInterface.config.getHeaderTableItem();
		
		
		  JPopupMenu menu = new JPopupMenu();
		  for ( int i=0; i< v.count(); i++ )
		  {
		      String c = v.getName(i);
		      boolean b = v.getEnabled(i);
		        //System.out.println("name: "+c+" boolean: "+b );
		
		      JCheckBoxMenuItem item = new JCheckBoxMenuItem( c );
		      item.setEnabled( false );
		      if ( b ) item.setSelected( true );
		
		      menu.add( item );
		  }
		
		  menu.show( button, 0, 0);
		}
		*/

		else if (
			command.equals(
				mainInterface.globalActionCollection.copyAction.getActionCommand()))
		{
			if (hasFocus())
			{
				Object uids[] = getUids();
				Folder folder = getFolder();
				ColumbaFolder clipboard = mainInterface.crossbar.getClipBoard();
				clipboard.removeAll();

				FolderOperation op =
					new FolderOperation(Operation.COPY, 0, uids, folder, clipboard);
				mainInterface.crossbar.operate(op);

				/*
				        mainInterface.folderOperator.operate(
				            new FolderOp( FolderOp.COPY, folder, uids ) );
				
				        mainInterface.globalActionCollection.pasteAction.setEnabled( true );
				*/
			}

		}
		else if (
			command.equals(
				mainInterface.globalActionCollection.cutAction.getActionCommand()))
		{
			if (hasFocus())
			{
				Object uids[] = getUids();
				Folder folder = getFolder();
				Folder clipboard = mainInterface.crossbar.getClipBoard();
				clipboard.removeAll();

				FolderOperation op =
					new FolderOperation(Operation.MOVE, 0, uids, folder, clipboard);
				mainInterface.crossbar.operate(op);

				/*
				        mainInterface.folderOperator.operate(
				            new FolderOp( FolderOp.CUT, folder, uids ) );
				        mainInterface.globalActionCollection.pasteAction.setEnabled( true );
				*/
			}

		}
		else if (
			command.equals(
				mainInterface.globalActionCollection.pasteAction.getActionCommand()))
		{
			//if ( hasFocus() )
			//{
			System.out.println("pastae action");

			Folder clipboard = mainInterface.crossbar.getClipBoard();
			Object uids[] = clipboard.getUids();
			Folder folder = getFolder();

			FolderOperation op =
				new FolderOperation(Operation.COPY, 0, uids, clipboard, folder);
			mainInterface.crossbar.operate(op);

			/*
			        mainInterface.folderOperator.operate(
			            new FolderOp( FolderOp.PASTE, getFolder() ) );
			*/
			//    }

		}
		else if (
			command.equals(
				mainInterface.globalActionCollection.deleteAction.getActionCommand()))
		{
			Folder folder = getFolder();
			FolderItem item = folder.getFolderItem();
			int uid = item.getUid();
			Object uids[] = getUids();

			FolderOperation op = new FolderOperation(Operation.DELETE, 0, uids, folder);
			mainInterface.crossbar.operate(op);

		}
		else if (
			command.equals(
				mainInterface.globalActionCollection.selectAllAction.getActionCommand()))
		{
			//FIXME
			/*
			if ( hasFocus() )
			{
			    int rowCount = headerTableViewer.getTable().getRowCount();
			    headerTableViewer.getTable().setRowSelectionInterval( 0, rowCount-1 );
			
			    changeMessageActions();
			}
			*/

		}
		else if (command.equals(replyAction.getActionCommand()))
		{
			ComposerFrame composer;

			int count;
			int[] selection;
			Folder folder = getFolder();
			Object uids[] = getUids();

			count = uids.length;

			for (int i = 0; i < count; i++)
			{
				FolderOperation op1 =
					new FolderOperation(Operation.MARK_ANSWERED, 0, uids, folder);
				mainInterface.crossbar.operate(op1);

				ComposerOperation op2 =
					new ComposerOperation(Operation.COMPOSER_REPLY, 0, folder, uids[i]);
				mainInterface.crossbar.operate(op2);


			}

		}
		else if (command.equals(replyToAction.getActionCommand()))
		{

			ComposerFrame composer;

			int count;
			int[] selection;
			Folder folder = getFolder();
			Object uids[] = getUids();

			count = uids.length;

			for (int i = 0; i < count; i++)
			{
				FolderOperation op1 = new FolderOperation( Operation.MARK_ANSWERED, 0, uids, folder);
				mainInterface.crossbar.operate( op1 );
				
				ComposerOperation op2 =
					new ComposerOperation(Operation.COMPOSER_REPLY_MAILINGLIST, 0, folder, uids[i]);
				mainInterface.crossbar.operate(op2);

				

			}

		}
		else if (command.equals(replyToAllAction.getActionCommand()))
		{

			ComposerFrame composer;

			int count;
			int[] selection;
			Folder folder = getFolder();
			Object uids[] = getUids();

			count = uids.length;

			for (int i = 0; i < count; i++)
			{
				FolderOperation op1 = new FolderOperation( Operation.MARK_ANSWERED, 0, uids, folder);
				mainInterface.crossbar.operate( op1 );
				
				ComposerOperation op2 =
					new ComposerOperation(Operation.COMPOSER_REPLY_ALL, 0, folder, uids[i]);
				mainInterface.crossbar.operate(op2);

				

			}

		}
		else if (command.equals(replyAsAttachmentAction.getActionCommand()))
		{

			ComposerFrame composer;

			int count;
			int[] selection;
			Folder folder = getFolder();
			Object uids[] = getUids();

			count = uids.length;

			for (int i = 0; i < count; i++)
			{
				FolderOperation op1 = new FolderOperation( Operation.MARK_ANSWERED, 0, uids, folder);
				mainInterface.crossbar.operate( op1 );
				
				ComposerOperation op2 =
					new ComposerOperation(Operation.COMPOSER_REPLY_AS_ATTACHMENT, 0, folder, uids[i]);
				mainInterface.crossbar.operate(op2);

				

			}

		}
		else if (command.equals(forwardAction.getActionCommand()))
		{

			ComposerFrame composer;

			int count;
			int[] selection;
			Folder folder = getFolder();
			Object uids[] = getUids();

			count = uids.length;

			for (int i = 0; i < count; i++)
			{

				
				FolderOperation op = new FolderOperation( Operation.MARK_ANSWERED, 0, uids, folder);
				mainInterface.crossbar.operate( op );
				

				ComposerOperation op2 =
					new ComposerOperation(Operation.COMPOSER_FORWARD, 0, folder, uids[i]);
				mainInterface.crossbar.operate(op2);

				
			}

		}
		else if (command.equals(forwardInlineAction.getActionCommand()))
		{

			ComposerFrame composer;

			int count;
			int[] selection;
			Folder folder = getFolder();
			Object uids[] = getUids();

			count = uids.length;

			for (int i = 0; i < count; i++)
			{

				
				FolderOperation op = new FolderOperation( Operation.MARK_ANSWERED, 0, uids, folder);
				mainInterface.crossbar.operate( op );
				

				ComposerOperation op2 =
					new ComposerOperation(Operation.COMPOSER_FORWARD_INLINE, 0, folder, uids[i]);
				mainInterface.crossbar.operate(op2);

				
			}

		}
		else if (command.equals(moveMessageAction.getActionCommand()))
		{
			/*
			final SelectFolderDialog dialog = new SelectFolderDialog( mainInterface );
			*/
			SelectFolderDialog dialog = headerTableViewer.getSelectFolderDialog();
			//dialog.setSize( new Dimension( 300,430 ) );

			//dialog.show();

			if (dialog.success())
			{
				//int uid = dialog.getUid();
				//Folder destFolder = mainInterface.treeViewer.getFolder( uid );
				Folder destFolder = dialog.getSelectedFolder();

				if (destFolder != null)
				{
					Folder folder = getFolder();
					Object uids[] = getUids();

					FolderOperation op =
						new FolderOperation(Operation.MOVE, 0, uids, folder, destFolder);
					mainInterface.crossbar.operate(op);
				}
			}
		}
		else if (command.equals(copyMessageAction.getActionCommand()))
		{
			/*
			final SelectFolderDialog dialog = new SelectFolderDialog( mainInterface );
			*/
			SelectFolderDialog dialog = headerTableViewer.getSelectFolderDialog();
			//dialog.setSize( new Dimension( 300,430 ) );

			//dialog.show();

			if (dialog.success())
			{
				//int uid = dialog.getUid();
				//Folder destFolder = mainInterface.treeViewer.getFolder( uid );
				Folder destFolder = dialog.getSelectedFolder();

				if (destFolder != null)
				{
					Folder folder = getFolder();

					Object uids[] = getUids();

					FolderOperation op =
						new FolderOperation(Operation.COPY, 0, uids, folder, destFolder);
					mainInterface.crossbar.operate(op);

				}
			}
		}
		else if (command.equals(deleteMessageAction.getActionCommand()))
		{

			Folder folder = getFolder();
			FolderItem item = folder.getFolderItem();
			int uid = item.getUid();
			Object uids[] = getUids();

			FolderOperation op = new FolderOperation(Operation.DELETE, 0, uids, folder);
			mainInterface.crossbar.operate(op);

		}
		else if (command.equals(nextAction.getActionCommand()))
		{

			/*
			int selection = mainInterface.headerTableViewer.getSelectedRow();
			
			selection+=1;
			
			headerTableViewer.setSelection( selection );
			
			headerTableViewer.showMessage();
			
			changeMessageActions();
			
			*/
		}
		else if (command.equals(prevAction.getActionCommand()))
		{
			/*
			int selection = mainInterface.headerTableViewer.getSelectedRow();
			
			selection-=1;
			
			headerTableViewer.setSelection( selection );
			headerTableViewer.showMessage();
			changeMessageActions();
			*/
		}
		else if (command.equals(addSenderAction.getActionCommand()))
		{

			try
			{
				AddressbookXmlConfig config = mainInterface.config.getAddressbookConfig();
				/*
				int rows[] = new int[headerTableViewer.getSelectedRowCount()];
				rows = headerTableViewer.getSelectedRows();
				*/
				Object[] uids = getUids();
				Folder folder = getFolder();

				Message message;
				String sender;
				for (int i = 0; i < uids.length; i++)
				{
					message = folder.uidGet(uids[i]);
					sender = (String) message.getHeader().get("From");

					if (sender != null)
						config.addContact(sender);
				}
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}

		}
		else if (command.equals(addAllSendersAction.getActionCommand()))
		{
			System.out.println("not yet implemented");
		}
		else if (command.equals(viewSourceAction.getActionCommand()))
		{

			Folder folder = getFolder();
			Object[] uids = getUids();

			Object uid = uids[0];

			mainInterface.crossbar.operate(
				new GuiOperation(Operation.MESSAGESOURCE, 20, folder, uid));

		}
		else if (command.equals(filterSubjectAction.getActionCommand()))
		{

			try
			{

				Folder folder = getFolder();
				Object[] uids = getUids();

				for (int i = 0; i < uids.length; i++)
				{

					Message message = folder.uidGet(uids[i]);
					String pattern = (String) message.getHeader().get("Subject");
					AdapterNode parentNode = folder.getNode();
					createFilter("Subject", pattern);
				}
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}

		}
		else if (command.equals(filterFromAction.getActionCommand()))
		{
			try
			{
				Folder folder = getFolder();
				Object[] uids = getUids();

				for (int i = 0; i < uids.length; i++)
				{

					Message message = folder.uidGet(uids[i]);
					String pattern = (String) message.getHeader().get("From");
					AdapterNode parentNode = folder.getNode();
					createFilter("From", pattern);
				}
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}

		}
		else if (command.equals(filterToAction.getActionCommand()))
		{
			Folder folder = getFolder();
			Object[] uids = getUids();

			try
			{
				for (int i = 0; i < uids.length; i++)
				{

					Message message = folder.uidGet(uids[i]);
					String pattern = (String) message.getHeader().get("To");
					AdapterNode parentNode = folder.getNode();
					createFilter("To", pattern);
				}
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}

		}
		else if (command.equals(vFolderSubjectAction.getActionCommand()))
		{
			Folder folder = getFolder();
			Object uids[] = getUids();

			try
			{
				for (int i = 0; i < uids.length; i++)
				{
					Message message = folder.uidGet(uids[i]);

					createVFolder(message, "Subject");
				}
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
		}
		else if (command.equals(vFolderFromAction.getActionCommand()))
		{
			Folder folder = getFolder();
			Object uids[] = getUids();

			try
			{
				for (int i = 0; i < uids.length; i++)
				{
					Message message = folder.uidGet(uids[i]);

					createVFolder(message, "From");
				}
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
		}
		else if (command.equals(vFolderToAction.getActionCommand()))
		{
			Folder folder = getFolder();
			Object uids[] = getUids();

			try
			{
				for (int i = 0; i < uids.length; i++)
				{
					Message message = folder.uidGet(uids[i]);

					createVFolder(message, "To");
				}
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}

		}
		else if (command.equals(viewThreadedAction.getActionCommand()))
		{
			headerTableViewer.getTableModelThreadedView().toggleView();

			//headerTableViewer.getHeaderTableModel().setFolder( headerTableViewer.getFolder() );
		}
		else if (command.equals(openMessageWithComposerAction.getActionCommand()))
		{
			Object[] uids = getUids();
			Folder folder = getFolder();

			for (int i = 0; i < uids.length; i++)
			{
				
				ComposerOperation op2 =
					new ComposerOperation(Operation.COMPOSER_OPEN, 0, folder, uids[i]);
				mainInterface.crossbar.operate(op2);
			}
		}
	}

	protected void createFilter(String headerItem, String pattern)
	{
		Folder folder = getFolder();
		FilterList list = folder.getFilterList();

		Filter filter = list.addEmtpyFilter();
		FilterRule rule = filter.getFilterRule();

		FilterCriteria criteria = rule.get(0);

		criteria.setType(headerItem);
		criteria.setHeaderItem(headerItem);
		criteria.setPattern(pattern);
		criteria.setCriteria("contains");

		filter.setName(headerItem + ": " + pattern);

		FilterDialog dialog = new FilterDialog(mainInterface, filter);

	}

	protected void createVFolder(Message message, String headerItem)
	{

		String pattern = (String) message.getHeader().get(headerItem);

		VirtualFolder vFolder =
			(VirtualFolder) mainInterface.treeViewer.getFolderTree().addVirtualFolder(
				pattern);

		Folder selectedFolder = getFolder();
		int uid = selectedFolder.getUid();
		Integer uidInt = new Integer(uid);

		Search search = vFolder.getSearchFilter();

		search.setUid(uidInt.intValue());
		Filter filter = search.getFilter();
		FilterRule rule = filter.getFilterRule();
		FilterCriteria criteria = rule.get(0);

		criteria.setType(headerItem);
		criteria.setHeaderItem(headerItem);
		criteria.setCriteria("contains");
		criteria.setPattern(pattern);

		SearchFrame searchDialog = new SearchFrame(mainInterface, vFolder);

	}

}