// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.header.util;

import org.columba.message.*;
import org.columba.config.*;
import org.columba.folder.*;
import org.columba.gui.header.*;
import javax.swing.JTable;

import javax.swing.table.*;
import javax.swing.event.*;

import java.util.*;

import java.awt.event.*;

import java.text.Collator;


public class IndexedTableModel extends AbstractTableModel
{
    private Vector indexList;
    protected CTableModel tableModel;
    protected EventListenerList listenerList = new EventListenerList();
    private boolean mutex;


    public IndexedTableModel( CTableModel tableModel )
        {
            this.tableModel = tableModel;

            indexList = new Vector();

            mutex = false;
        }

    private synchronized void getMutex()
    {
        while( mutex ) {
            try {
                wait();
            }
            catch( InterruptedException e )
            {
            }
        }

        mutex = true;
    }

    private synchronized void releaseMutex()
    {
        mutex = false;
        notify();
    }


    public int getIndex( int i )
    {
        getMutex();

        if ( indexList.size() == 0 ) return -1;

        if ( ( i >= indexList.size() ) || ( i < 0 ) ) return -1;

        Integer in = (Integer) indexList.get( i );

        releaseMutex();

        if ( in == null ) return -1;

        return in.intValue();
    }

    public int getTableIndex( int i )
    {
        getMutex();

        for ( int n=0; n<indexList.size();n++)
        {
            Integer in = (Integer) indexList.get(n);
            int value = in.intValue();

            if ( i==value )
            {
                releaseMutex();
                return n;
            }

        }

        releaseMutex();

        return -1;
    }


    public Message getMessage( int row )
    {
        Message message = null;
        try
        {
            message = tableModel.getFolder().get( getIndex(row) );
        }
        catch ( Exception ex )
        {
        }

        return message;
    }


    public void update()
    {
        fireTableChanged( new TableModelEvent( this ) );
    }


/*
    public Vector getIndexList()
    {
        getMutex();

        return indexList;
    }

*/

    public void setIndexList( Vector v )
    {
        getMutex();

        indexList = v;

        releaseMutex();
    }


    public void sort( Comparator c )
    {
        getMutex();
        Collections.sort( indexList, c );
        releaseMutex();
    }



/*    public MessageCollection getMessageCollection()
    {
          //return tableModel.getMessageCollection();
        return null;
    }
*/
    public Folder getFolder()
    {
        return tableModel.getFolder();
    }


    public void setFolder( Folder f )
    {
        tableModel.setFolder( f );
        if ( f == null ) return;

        getMutex();

        indexList.clear();

        for ( int i=0; i<f.count(); i++ )
        {
            indexList.add( new Integer(i) );
        }

        releaseMutex();

        fireTableChanged( new TableModelEvent( this ) );
    }



      /*
    public void setMessageCollection( MessageCollection mc )
    {
        if ( mc != null )
        {
            indexList.clear();


            for ( int i=0; i<mc.count(); i++ )
            {
                indexList.add( new Integer(i) );
            }


            tableModel.setMessageCollection( mc );
            fireTableChanged( new TableModelEvent( this ) );
        }
    }
      */

    public void recalculate()
    {
        Folder folder = getFolder();
        if ( folder == null ) return;

        getMutex();

        indexList.clear();

        for ( int i=0; i<folder.count(); i++ )
        {
            indexList.add( new Integer(i) );
        }

        releaseMutex();
    }


    public int getRowCount()
    {
        int result;

        getMutex();

        result = indexList.size();

        releaseMutex();

        return result;
    }


    public Object getValueAt(int row, int col)
        {
              //System.out.println("indexed  row: "+ row + " column: "+col);

            String column = getColumnName( col );

              //System.out.println("row: "+ row + "column: " +column );

            getMutex();

            if ( indexList == null ) return null;
            if ( indexList.size() == 0 ) return null;


            Integer i = (Integer) indexList.get(row);

            releaseMutex();

            if ( i == null ) return null;

            int indexedNumber  = i.intValue();

            return tableModel.getValueAt( indexedNumber, col );
        }

    public int getColumnCount()
    {
        return tableModel.getColumnCount();
    }

    public String getColumnName(int col)
    {
        return tableModel.getColumnName( col );
    }

    public int getColumnNumber(String str)
    {
        return tableModel.getColumnNumber( str );
    }


    public Class getColumnClass(int c)
    {
        return tableModel.getColumnClass( c );
    }

    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
        return tableModel.isCellEditable( rowIndex, columnIndex );
    }


    public void setValueAt(Object aValue, int rowIndex, int columnIndex)
    {
        tableModel.setValueAt( aValue, rowIndex, columnIndex );
    }



    public void addTableModelListener(TableModelListener l)
    {
	listenerList.add(TableModelListener.class, l);
    }

    public void removeTableModelListener(TableModelListener l)
    {
	listenerList.remove(TableModelListener.class, l);
    }



    //
//  Fire methods
//

    /**
     * Notifies all listeners that all cell values in the table's
     * rows may have changed. The number of rows may also have changed
     * and the <code>JTable</code> should redraw the
     * table from scratch. The structure of the table (as in the order of the
     * columns) is assumed to be the same.
     *
     * @see TableModelEvent
     * @see EventListenerList
     */
    public void fireTableDataChanged() {
        tableModel.fireTableChanged(new TableModelEvent(this));
    }

    /**
     * Notifies all listeners that the table's structure has changed.
     * The number of columns in the table, and the names and types of
     * the new columns may be different from the previous state.
     * If the <code>JTable</code> receives this event and its
     * <code>autoCreateColumnsFromModel</code>
     * flag is set it discards any table columns that it had and reallocates
     * default columns in the order they appear in the model. This is the
     * same as calling <code>setModel(TableModel)</code> on the
     * <code>JTable</code>.
     *
     * @see TableModelEvent
     * @see EventListenerList
     */
    public void fireTableStructureChanged() {
        tableModel.fireTableChanged(new TableModelEvent(tableModel, TableModelEvent.HEADER_ROW));
    }

    /**
     * Notifies all listeners that rows in the range
     * <code>[firstRow, lastRow]</code>, inclusive, have been inserted.
     *
     * @param  firstRow  the first row
     * @param  lastRow   the last row
     *
     * @see TableModelEvent
     * @see EventListenerList
     *
     */
    public void fireTableRowsInserted(int firstRow, int lastRow) {
        tableModel.fireTableChanged(new TableModelEvent(tableModel, firstRow, lastRow,
                             TableModelEvent.ALL_COLUMNS, TableModelEvent.INSERT));
    }

    /**
     * Notifies all listeners that rows in the range
     * <code>[firstRow, lastRow]</code>, inclusive, have been updated.
     *
     * @param firstRow  the first row
     * @param lastRow   the last row
     *
     * @see TableModelEvent
     * @see EventListenerList
     */
    public void fireTableRowsUpdated(int firstRow, int lastRow) {
        tableModel.fireTableChanged(new TableModelEvent(tableModel, getIndex(firstRow), getIndex(lastRow),
                             TableModelEvent.ALL_COLUMNS, TableModelEvent.UPDATE));
    }

    /**
     * Notifies all listeners that rows in the range
     * <code>[firstRow, lastRow]</code>, inclusive, have been deleted.
     *
     * @param firstRow  the first row
     * @param lastRow   the last row
     *
     * @see TableModelEvent
     * @see EventListenerList
     */
    public void fireTableRowsDeleted(int firstRow, int lastRow) {
        tableModel.fireTableChanged(new TableModelEvent(tableModel, getIndex(firstRow), getIndex(lastRow),
                             TableModelEvent.ALL_COLUMNS, TableModelEvent.DELETE));
    }

    /**
     * Notifies all listeners that the value of the cell at
     * <code>[row, column]</code> has been updated.
     *
     * @param row  row of cell which has been updated
     * @param column  column of cell which has been updated
     * @see TableModelEvent
     * @see EventListenerList
     */
    public void fireTableCellUpdated(int row, int column) {
        tableModel.fireTableChanged(new TableModelEvent(tableModel, row, row, column));
    }

    /**
     * Forwards the given notification event to all
     * <code>TableModelListeners</code> that registered
     * themselves as listeners for this table model.
     *
     * @param e  the event to be forwarded
     *
     * @see #addTableModelListener
     * @see TableModelEvent
     * @see EventListenerList
     */
    public void fireTableChanged(TableModelEvent e) {
	// Guaranteed to return a non-null array
	Object[] listeners = listenerList.getListenerList();
	// Process the listeners last to first, notifying
	// those that are interested in this event
	for (int i = listeners.length-2; i>=0; i-=2) {
	    if (listeners[i]==TableModelListener.class) {
		((TableModelListener)listeners[i+1]).tableChanged(e);
	    }
	}
    }

    /**
     * Returns an array of all the listeners of the given type that
     * were added to this model.
     *
     * @returns all of the objects receiving <code>listenerType</code>
     *		notifications from this model
     *
     * @since 1.3
     */
    public EventListener[] getListeners(Class listenerType) {
	return listenerList.getListeners(listenerType);
    }


}







































