// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.gui.mainframe;

import org.columba.gui.attachment.*;
import org.columba.gui.header.*;
import org.columba.gui.tree.*;
import org.columba.gui.tree.util.*;
import org.columba.gui.util.*;
import org.columba.gui.action.*;
import org.columba.gui.composer.*;
import org.columba.main.*;
import org.columba.message.*;
import org.columba.gui.mainframe.util.*;
import org.columba.gui.mainframe.menu.*;
import org.columba.parser.*;
import org.columba.gui.statusbar.*;
import org.columba.config.*;
import org.columba.gui.util.ImageLoader;
import org.columba.folder.*;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;
import javax.swing.undo.*;

import java.util.*;

import java.io.*;

public class MainFrame extends JFrame implements ActionListener
{
	private Toolbar toolbar;

	private org.columba.gui.mainframe.menu.Menu menu;
	private StatusBar statusBar;
	public JSplitPane mainSplitPane;
	public SplitPane rightSplitPane;

	private JPanel pane;

	private MainInterface mainInterface;

	public BasicAction exitAction;

	public ResourceBundle guiLabels;

	public MainFrame(MainInterface iface)
	{
		mainInterface = iface;
		mainInterface.mainFrame = this;

		new DialogStore(this);

		initActions();

		//pack();
		int count = mainInterface.config.getAccountList().count();

		if (count == 0)
		{
			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			setSize(screenSize);
		}
		else
		{
			java.awt.Point point = mainInterface.mainFrameWindowItem.getPoint();
			java.awt.Dimension dim = mainInterface.mainFrameWindowItem.getDimension();

			Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
			setSize(dim);
		}

		init();

		setVisible(true);

		addWindowListener(new WindowAdapter()
		{
			public void windowClosing(WindowEvent e)
			{
				ExitWorker worker = new ExitWorker(mainInterface);
				worker.register(mainInterface.taskManager);
				worker.start();

			}
		});

	}

	public void saveWindowPosition()
	{

		Runnable run = new Runnable()
		{
			public void run()
			{
				java.awt.Dimension d = getSize();
				//mainInterface.config.saveWindowPosition( 0, 0, d.width, d.height);
				WindowItem item = mainInterface.mainFrameWindowItem;
				item.setXPosition(0);
				item.setYPosition(0);
				item.setWidth(d.width);
				item.setHeight(d.height);

				item.setMainSplitPane(mainSplitPane.getDividerLocation());
				item.setRightSplitPane(rightSplitPane.getDividerLocation());

			}
		};
		try
		{

			if (!SwingUtilities.isEventDispatchThread())
				SwingUtilities.invokeAndWait(run);
			else
				SwingUtilities.invokeLater(run);

		}
		catch (Exception ex)
		{
		}

	}

	private void init()
	{

		this.getContentPane().setLayout(new BorderLayout());
		JPanel panel = (JPanel) this.getContentPane();

		setTitle("Columba v" + org.columba.main.MainInterface.version);

		statusBar = mainInterface.statusBar;
		this.getContentPane().add(statusBar, BorderLayout.SOUTH);

		mainSplitPane = new JSplitPane();

		mainSplitPane.setBorder(BorderFactory.createEmptyBorder(1, 0, 0, 0));

		this.getContentPane().add(mainSplitPane, BorderLayout.CENTER);

		mainSplitPane.setOrientation(JSplitPane.HORIZONTAL_SPLIT);
		mainSplitPane.add(mainInterface.treeViewer.scrollPane, JSplitPane.LEFT);

		rightSplitPane =
			new SplitPane(
				mainInterface.headerTableViewer,
				mainInterface.messageViewer,
				mainInterface.attachmentViewer.scrollPane);

		mainSplitPane.add(rightSplitPane, JSplitPane.RIGHT);

		// menu has to be created at last, because it uses the action-classes of the other main-viewer classes
		menu = new org.columba.gui.mainframe.menu.Menu(mainInterface);
		setJMenuBar(menu.getMenuBar());

		pane = new JPanel();
		pane.setLayout(new BoxLayout(pane, BoxLayout.Y_AXIS));

		// same as menu
		toolbar = new Toolbar(mainInterface);
		if (mainInterface.mainFrameWindowItem.isShowToolbar() == true)
		{
			pane.add(toolbar);
		}

		mainInterface.folderInfoPanel = new FolderInfoPanel(mainInterface);

		pane.add(mainInterface.folderInfoPanel);

		getContentPane().add(pane, BorderLayout.NORTH);

		int count = mainInterface.config.getAccountList().count();

		if (count == 0)
		{
			pack();
			rightSplitPane.setDividerLocation(150);
		}
		else
		{
			mainSplitPane.setDividerLocation(
				mainInterface.mainFrameWindowItem.getMainSplitPane());

			rightSplitPane.setDividerLocation(
				mainInterface.mainFrameWindowItem.getRightSplitPane());
		}

	}

	public void hideToolbar(boolean b)
	{
		remove(toolbar);
		validate();
		repaint();

		if (b)
		{
			getContentPane().remove(pane);
			getContentPane().add(mainInterface.folderInfoPanel, BorderLayout.NORTH);
			validate();
			repaint();
		}
		else
		{
			getContentPane().remove(toolbar);
			validate();
			repaint();
		}

	}

	public void showToolbar(boolean b)
	{
		getContentPane().add(toolbar, BorderLayout.NORTH);

		validate();
		repaint();

		if (b)
		{
			pane.removeAll();
			pane.add(toolbar);
			pane.add(mainInterface.folderInfoPanel);
			getContentPane().add(pane, BorderLayout.NORTH);
			validate();
			repaint();
		}
		else
		{
			getContentPane().remove(pane);
			getContentPane().add(toolbar, BorderLayout.NORTH);
			validate();
			repaint();
		}

	}

	public void hideFolderInfo(boolean b)
	{

		if (b)
		{
			getContentPane().remove(pane);
			getContentPane().add(toolbar, BorderLayout.NORTH);
			validate();
			repaint();
		}
		else
		{
			getContentPane().remove(mainInterface.folderInfoPanel);
			validate();
			repaint();
		}

	}

	public void showFolderInfo(boolean b)
	{

		if (b)
		{
			pane.removeAll();
			pane.add(toolbar);
			pane.add(mainInterface.folderInfoPanel);
			getContentPane().add(pane, BorderLayout.NORTH);
			validate();
			repaint();
		}
		else
		{
			getContentPane().remove(pane);
			getContentPane().add(mainInterface.folderInfoPanel, BorderLayout.NORTH);
			validate();
			repaint();
		}

	}

	public StatusBar getStatusBar()
	{
		return statusBar;
	}

	public org.columba.gui.mainframe.menu.Menu getMenu()
	{
		return menu;
	}

	public void initActions()
	{

		exitAction =
			new BasicAction(
				ResourceLoader.getString("action/global", "menu_file_exit"),
				ResourceLoader.getString("action/global", "menu_file_exit"),
				ResourceLoader.getString("action/global", "menu_file_exit"),
				"EXIT",
				ImageLoader.getImageIcon("exit_small", "Home16"),
				ImageLoader.getImageIcon("exit", "Home24"),
				'X',
				null);
		exitAction.addActionListener(this);

		mainInterface.globalActionCollection.newMessageAction.addActionListener(this);

	}

	public void actionPerformed(ActionEvent e)
	{
		String command = e.getActionCommand();

		System.out.println("command: " + command);

		if (command.equals(exitAction.getActionCommand()))
		{
			ExitWorker worker = new ExitWorker(mainInterface);
			worker.register(mainInterface.taskManager);
			worker.start();

		}
		else if (
			command.equals(
				mainInterface.globalActionCollection.newMessageAction.getActionCommand()))
		{
			new ComposerFrame(mainInterface);
		}
	}
}