// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.gui.message;

import org.columba.gui.message.util.*;
import org.columba.main.MainInterface;
import org.columba.gui.composer.util.*;
import org.columba.parser.*;
import org.columba.gui.util.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import java.io.*;

public class HyperlinkTextViewer
	extends JTextPane
	implements AbstractDocumentViewer, MouseMotionListener, MouseListener
{
	boolean active;

	private MessageDocument document;

	private MainInterface mainInterface;

	private JMenuItem menuItem;
	private JPopupMenu popup;

	private String address;
	private String link;

	public HyperlinkTextViewer(MainInterface mainInterface)
	{
		super();

		this.mainInterface = mainInterface;

		document = new MessageDocument();
		Font font =
			new Font(
				mainInterface.themeItem.getTextFontName(),
				Font.PLAIN,
				mainInterface.themeItem.getTextFontSize());
		document.setFont(font);

		setStyledDocument(document);

		setMargin(new Insets(5, 5, 5, 5));
		setEditable(false);

		addMouseListener(new PopupListener());
		addMouseListener(this);
		addMouseMotionListener(this);

		active = false;
	}

	public String getAddress()
	{
		return address;
	}

	public String getLink()
	{
		return link;
	}

	public void setDoc(String s)
	{
		//System.out.println("hyperlinktextviewer->setText");

		final String str = s;
		Runnable run = new Runnable()
		{
			public void run()
			{
				document.reset();

				if (str == null)
					return;

				if (str.length() > 0)
				{
					document.append(str, 0);

					document.parse();

					setCaretPosition(0);
				}

			}
		};
		try
		{
			if (!SwingUtilities.isEventDispatchThread())
				SwingUtilities.invokeAndWait(run);
			else
				SwingUtilities.invokeLater(run);

		}
		catch (Exception ex)
		{
		}

	}

	public String getDoc()
	{
		return document.getText();
	}

	public void setActive(boolean b)
	{
		active = true;
	}

	public boolean isActive()
	{
		return active;
	}

	public void clearDoc()
	{
		document.reset();
	}

	public boolean isLink(int pos)
	{
		Element element = document.getCharacterElement(pos);
		AttributeSet set = element.getAttributes();

		Color color = StyleConstants.getForeground(set);

		if (color != null)
		{
			if (color == Color.blue)
			{
				return true;
			}
		}

		return false;
	}

	protected String getUrl(String str, int pos)
	{
		int index1 = pos;
		char c = str.charAt(index1);
		StringBuffer left = new StringBuffer();
		while (isLink(index1) == true)
		{
			if (index1 < 0)
				break;
			c = str.charAt(index1);
			left.insert(0, c);
			index1--;
		}
		System.out.println("left: " + left);

		int index2 = pos + 1;
		c = str.charAt(index2);
		StringBuffer right = new StringBuffer();
		while (isLink(index2) == true)
		{
			if (index2 >= str.length())
				break;
			c = str.charAt(index2);
			right.append(c);
			index2++;
		}
		System.out.println("right: " + right);

		String result = left.toString() + right.toString();
		System.out.println("str: " + result);

		return result;
	}

	public void mouseMoved(MouseEvent e)
	{

		java.awt.Point point = e.getPoint();

		int pos = viewToModel(point);

		Element element = document.getCharacterElement(pos);
		AttributeSet set = element.getAttributes();

		Color color = StyleConstants.getForeground(set);

		if (color != null)
		{
			if (color == Color.blue)
			{
				if (getCursor().getType() != Cursor.HAND_CURSOR)
					setCursor(new Cursor(Cursor.HAND_CURSOR));
			}
			else
			{
				if (getCursor().getType() != Cursor.DEFAULT_CURSOR)
					setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			}
		}
		else
		{
			if (getCursor().getType() != Cursor.DEFAULT_CURSOR)
				setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}

	}

	public void mouseDragged(MouseEvent e)
	{
	}

	public void mouseClicked(MouseEvent e)
	{

		java.awt.Point point = e.getPoint();

		int pos = viewToModel(point);

		if (isLink(pos))
		{
			String str = document.getText();

			String result = getUrl(str, pos);

			if (result.indexOf("@") != -1)
			{
				// e-mail address

				address = result;

				/*
				popup = new JPopupMenu();
				menuItem = new JMenuItem("Add Contact to Addressbook");
				menuItem.addActionListener( mainInterface.messageViewer );
				menuItem.setActionCommand("CONTACT");
				popup.add(menuItem);
				menuItem = new JMenuItem("Compose Message for "+ result );
				menuItem.setActionCommand("COMPOSE");
				menuItem.addActionListener( mainInterface.messageViewer );
				popup.add(menuItem);
				*/

				UrlController controller = new UrlController(mainInterface);
				controller.setAddress(address);
				controller.createContactMenu(address).show(e.getComponent(), point.x, point.y);

			}
			else
			{
				// http link
				System.out.print("http link");

				link = result;

				System.out.println(" no popup trigger");

				UrlController controller = new UrlController(mainInterface);
				controller.open(link);

				/*
				boolean save = false;
				String viewer;
				ChooseViewerDialog viewerDialog;
				
				viewer = MimeRouter.getViewer( "text", "html" );
				if( viewer == null )
				{
				
				    viewerDialog = new ChooseViewerDialog(mainInterface.mainFrame,
				                                          "text",
				                                          "html",
				                                          save);
				
				    viewer = viewerDialog.getViewer();
				    save = viewerDialog.saveViewer();
				
				    if(save) MimeRouter.setViewer( "text", "html", viewer );
				}
				
				try
				{
				    Runtime.getRuntime().exec( viewer+" "+result );
				}
				catch( IOException ex )
				{
				    ex.printStackTrace();
				    JOptionPane.showMessageDialog( mainInterface.mainFrame,
				                                   "Error: "+ex.getMessage() );
				}
				*/

			}
		}

	}

	public void mouseEntered(MouseEvent e)
	{
	}

	public void mouseExited(MouseEvent e)
	{
	}

	public void mousePressed(MouseEvent e)
	{
	}

	public void mouseReleased(MouseEvent e)
	{
	}

	class PopupListener extends MouseAdapter
	{
		public void mousePressed(MouseEvent e)
		{
			maybeShowPopup(e);
		}

		public void mouseReleased(MouseEvent e)
		{
			maybeShowPopup(e);
		}

		private void maybeShowPopup(MouseEvent e)
		{
			if (e.isPopupTrigger())
			{
				java.awt.Point point = e.getPoint();

				int pos = viewToModel(point);

				if (isLink(pos))
				{
					String str = document.getText();

					String result = getUrl(str, pos);

					if (result.indexOf("@") != -1)
					{
						// e-mail address

						address = result;
						/*
						popup = new JPopupMenu();
						menuItem = new JMenuItem("Add Contact to Addressbook");
						menuItem.addActionListener( mainInterface.messageViewer );
						menuItem.setActionCommand("CONTACT");
						popup.add(menuItem);
						menuItem = new JMenuItem("Compose Message for "+ result );
						menuItem.setActionCommand("COMPOSE");
						menuItem.addActionListener( mainInterface.messageViewer );
						popup.add(menuItem);
						*/

						UrlController controller = new UrlController(mainInterface);
						controller.setAddress(address);
						controller.createContactMenu(address).show(e.getComponent(), point.x, point.y);
					}
					else if (result.indexOf("http://") != -1)
					{
						link = result;

						/*
						popup = new JPopupMenu();
						menuItem = new JMenuItem("Open");
						menuItem.addActionListener( mainInterface.messageViewer );
						menuItem.setActionCommand("OPEN");
						popup.add(menuItem);
						menuItem = new JMenuItem("Open with..." );
						menuItem.setActionCommand("OPEN_WITH");
						menuItem.addActionListener( mainInterface.messageViewer );
						popup.add(menuItem);
						popup.addSeparator();
						menuItem = new JMenuItem("Open with internal browser" );
						menuItem.setActionCommand("OPEN_WITHINTERNALBROWSER");
						menuItem.addActionListener( mainInterface.messageViewer );
						popup.add(menuItem);
						*/

						UrlController controller = new UrlController(mainInterface);
						controller.setLink(link);
						controller.createLinkMenu().show(e.getComponent(), e.getX(), e.getY());
					}
				}
			}
		}
	}

}