// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.message;

import org.columba.main.*;

import javax.swing.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.text.*;
//import java.awt.*;
import java.awt.Dimension;
import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.Insets;
import java.util.StringTokenizer;
import javax.swing.text.*;
import java.io.*;
import java.awt.*;
import java.util.*;
import javax.swing.text.html.*;
import java.net.URL;


import org.columba.message.*;
import org.columba.config.*;
import org.columba.gui.attachment.*;
import org.columba.folder.*;
import org.columba.gui.util.*;
import org.columba.coder.*;
import org.columba.util.SwingWorker;
import org.columba.gui.action.*;
import org.columba.gui.composer.util.*;
import org.columba.gui.statusbar.*;
import org.columba.gui.message.util.*;
import org.columba.gui.mainframe.util.*;
import org.columba.gui.themes.*;
import org.columba.gui.addressbook.util.*;
import org.columba.gui.composer.*;
import org.columba.gui.header.util.*;
import org.columba.parser.*;
import org.columba.gui.message.menu.*;
import org.columba.gui.message.action.*;




/**
 * this class shows the messagebody
 */

public class MessageViewer extends JPanel
{
    private MessageHeader messageHeader;

    private MainInterface mainInterface;

    private Folder folder;
    private Object uid;

    private DocumentOperator documentOperator;

    private MessageMenu menu;

    private MessageFocusListener focusListener;
    private MessageActionListener actionListener;
    private MessagePopupListener popupListener;

    public MessageViewer( MainInterface mainInterface  )
        {
            this.mainInterface = mainInterface;

            setLayout( new BorderLayout() );

            focusListener = new MessageFocusListener();
            popupListener = new MessagePopupListener( this );

            documentOperator = new DocumentOperator( mainInterface, this );
            documentOperator.setPopupListener( popupListener );

            add( documentOperator, BorderLayout.CENTER );

            String[] keys = new String[4];
            keys[0] = new String("Subject");
	    keys[1] = new String("From");
            keys[2] = new String("Date");
            keys[3] = new String("To");


            Font mainFont = new Font( mainInterface.themeItem.getMainFontName(),
                                      Font.PLAIN,
                                      mainInterface.themeItem.getMainFontSize() );

            messageHeader = new MessageHeader( keys,
                                               2, mainInterface );

            add( messageHeader , BorderLayout.NORTH );


            menu = new MessageMenu( mainInterface, this );


            actionListener = new MessageActionListener( mainInterface, this );



        }

    public DocumentOperator getDocumentOperator()
    {
        return documentOperator;
    }

    public JPopupMenu getPopupMenu()
    {
        return menu.getPopupMenu();
    }


    public void update( boolean b )
    {
        if ( b == true )
        {
            if ( documentOperator != null )
               remove( documentOperator );

            add(documentOperator, BorderLayout.CENTER);
        }
    }

     public void createTextPane( boolean b, boolean htmlViewer )
    {


        if ( b == true )
        {
            if ( htmlViewer == true )
            {
                // use the html viewer

                documentOperator.enableViewer( DocumentOperator.HTML );

                if ( documentOperator != null )
                   remove( documentOperator );

                add(documentOperator, BorderLayout.CENTER);


            }
            else
            {
                // use the advanced viewer

                documentOperator.enableViewer( DocumentOperator.ADVANCED );

                if ( documentOperator != null )
                   remove( documentOperator );

                add(documentOperator, BorderLayout.CENTER);


            }
        }
        else
        {
            // use plain text viewer ( fast )

            documentOperator.enableViewer( DocumentOperator.SIMPLE );

            if ( documentOperator != null )
                   remove( documentOperator );

            add(documentOperator, BorderLayout.CENTER);

        }

    }



    public void setViewerFont( Font font )
    {
        //textPane.setFont( font );
    }

    public void resetRenderer()
    {
        messageHeader.resetRenderer();
    }


    public Object getUid()
    {
        return uid;
    }

    public Folder getFolder()
    {
        return folder;
    }

    public synchronized void setFolder( Folder f )
    {
        this.folder = folder;
    }

    public synchronized void setUid( Object o )
    {
        this.uid = o;
    }

    public synchronized void showMessage( Message message ) throws Exception
    {
        boolean htmlViewer = false;

	/*

        ColumbaHeader header = message.getHeader();
        String contentType = (String) header.get("Content-Type");

        System.out.println("content-type: "+contentType );

        if ( ( contentType != null ) )
        {
            if ( contentType.toLowerCase().indexOf("text") != -1 )
            {
                if ( contentType.toLowerCase().indexOf("html") != -1 )
                {
                    htmlViewer = true;
                }
            }

        }


	*/

	


	MimePart bodyPart = message.getBodyPart();
	Decoder decoder = CoderRouter.getDecoder(bodyPart.getHeader().contentTransferEncoding);
	StringWriter decodedBody = new StringWriter();

	decoder.setupDecoder( new StringReader( bodyPart.getBody() ) , decodedBody );
	decoder.run();
	
	htmlViewer = bodyPart.getHeader().contentSubtype.equals( "html" );
	
	System.out.println( "Content-Typ of BodyPart:"+bodyPart.getHeader().contentType );
	
        insertText( decodedBody.toString(), htmlViewer );
        //scrollToBegin();
        messageHeader.setValues( message );


	
    }

    public void setSecurityIndicator( int value )
    {
	messageHeader.setSecurityValue( value );
    }


    public synchronized void showMessageSource( String rawText )
    {
        insertText( rawText, false );
        scrollToBegin();
    }


    public MessageActionListener getActionListener()
    {
        return actionListener;
    }

    public MessageFocusListener getFocusListener()
    {
        return focusListener;
    }


    public String getAddress()
    {
        HyperlinkTextViewer viewer = (HyperlinkTextViewer) documentOperator.getViewer( DocumentOperator.ADVANCED );

        if ( viewer != null ) return viewer.getAddress();
        else return new String();
    }

    public String getLink()
    {
        HyperlinkTextViewer viewer = (HyperlinkTextViewer) documentOperator.getViewer( DocumentOperator.ADVANCED );

        if ( viewer != null ) return viewer.getLink();
        else return new String();
    }


    protected void insertText( String s, boolean htmlView )
    {
        boolean advanced = mainInterface.mainFrameWindowItem.getAdvancedViewer();

        if ( htmlView == true )
        {

            boolean b = getDocumentOperator().enableViewer( DocumentOperator.HTML );
            update(b);
            getDocumentOperator().setDoc( s );
        }
        else if ( advanced == true )
        {

            boolean b = getDocumentOperator().enableViewer( DocumentOperator.ADVANCED );
            update(b);
            getDocumentOperator().setDoc( s );
        }
        else
        {

            boolean b = getDocumentOperator().enableViewer( DocumentOperator.SIMPLE );
            update(b);
            getDocumentOperator().setDoc( s );

        }

    }


    protected void scrollToBegin()
        {
            Runnable run = new Runnable()
             {
                 public void run()
                 {
                     documentOperator.setCaretPosition(0);
                 }
             };
             try
             {
                if ( !SwingUtilities.isEventDispatchThread() )
                   SwingUtilities.invokeAndWait(run);
                else
                   SwingUtilities.invokeLater(run);

             }
             catch( Exception ex )
             {
             }

        }


      /********************* context menu *******************************************/





}


