// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.mimetype;

import org.columba.gui.attachment.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class ChooseViewerDialog extends JDialog implements ActionListener
{
    private GridBagLayout gridBag;
    private GridBagConstraints constraints;

    private JTextField viewerName;
    private JButton searchButton;
    private JButton okButton;
    private String viewer;
    private JCheckBox saveCButton;


    public ChooseViewerDialog( Frame owner, String contentType, String contentSubtype, boolean save )
    {
        super(owner);

        setTitle("Select Viewer for "+contentType+" / "+contentSubtype);
        setModal( true );
        setSize( 320, 160 );

        gridBag = new GridBagLayout();
        getContentPane().setLayout( gridBag );

        createDialog(save);

        setVisible(true);
    }

    private void createDialog(boolean save)
    {
        constraints = new GridBagConstraints();
        constraints.ipadx = 5;
        constraints.ipady = 5;

        JLabel label;

        searchButton = new JButton( "..." );
        label = new JLabel( "Viewer : " );
        constraints.weightx = 0.0;
        constraints.fill = GridBagConstraints.NONE;
        constraints.anchor = GridBagConstraints.EAST;
        constraints.gridwidth = 1;

        gridBag.setConstraints( label, constraints );
        getContentPane().add( label );

        viewerName = new JTextField();
        viewerName.setActionCommand("OK");
        viewerName.addActionListener( this );


        constraints.weightx = 1.0;
        constraints.fill = GridBagConstraints.BOTH;
        constraints.anchor = GridBagConstraints.WEST;
        constraints.gridwidth = 1;

        gridBag.setConstraints( viewerName, constraints );
        getContentPane().add( viewerName );



        searchButton.addActionListener(this);
        searchButton.setActionCommand("SEARCH");

        constraints.weightx = 0.0;
        constraints.fill = GridBagConstraints.NONE;
        constraints.anchor = GridBagConstraints.CENTER;
        constraints.gridwidth = GridBagConstraints.REMAINDER;

        gridBag.setConstraints( searchButton, constraints );
        getContentPane().add( searchButton );

        saveCButton = new JCheckBox( "Always open with this Viewer", save );
//        saveCButton.addActionListener(this);
//        saveCButton.setActionCommand("SAVE");


        constraints.weightx = 1.0;
        constraints.fill = GridBagConstraints.NONE;
        constraints.anchor = GridBagConstraints.CENTER;
        constraints.gridwidth = GridBagConstraints.REMAINDER;

        gridBag.setConstraints( saveCButton, constraints );
        getContentPane().add( saveCButton );

        okButton = new JButton( "OK" );
        okButton.addActionListener(this);
        okButton.setActionCommand("OK");


        constraints.weightx = 1.0;
        constraints.fill = GridBagConstraints.NONE;
        constraints.anchor = GridBagConstraints.CENTER;
        constraints.gridwidth = GridBagConstraints.REMAINDER;

        gridBag.setConstraints( okButton, constraints );
        getContentPane().add( okButton );


    }

    public String getViewer()
    {
        return viewer;

    }

    public boolean saveViewer()
    {
        return saveCButton.isSelected();
    }


    public void actionPerformed( ActionEvent e )
    {
        String command = e.getActionCommand();

        if( command.equals("SEARCH") ) {
            JFileChooser fileChooser = new JFileChooser();

            fileChooser.setDialogTitle("Choose Program");
            int returnVal = fileChooser.showDialog( this, "OK");
            if( returnVal == JFileChooser.APPROVE_OPTION ) {
                viewerName.setText( fileChooser.getSelectedFile().toString() );
            }

            return;
        }
        if( command.equals("OK") ) {
            viewer = viewerName.getText();

            setVisible(false);
        }

    }

}





