// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.pop3;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import org.columba.message.*;
import org.columba.gui.util.*;
import org.columba.pop3.*;
import org.columba.gui.pop3.util.*;
import org.columba.main.*;
import org.columba.gui.statusbar.*;
import org.columba.gui.pop3.action.*;




public class ManageFrame extends JFrame
{

    private MainInterface mainInterface;
    public ManageHeaderTableViewer headerTableViewer;
    public ManageActionListener manageActionListener;

    public ManageInfoPanel manageInfoPanel;
    public ManageFilterToolbar filterToolbar;
        //private PopServer popServer;

    public JLabel statusBar;
    private JMenuBar menuBar;
    private JToolBar toolBar;

    private JWindow window;

    private PopServer popServer;



    public ManageFrame( MainInterface mainInterface, PopServer popServer )
        {
            this.popServer = popServer;
            this.mainInterface = mainInterface;

            headerTableViewer = new ManageHeaderTableViewer( mainInterface );

            manageActionListener = new ManageActionListener( this, headerTableViewer );

            manageInfoPanel = new ManageInfoPanel( mainInterface );
            manageInfoPanel.setPopServer( getPopServer() );

            filterToolbar = new ManageFilterToolbar( mainInterface, headerTableViewer );


            setupWindow();



            addWindowListener(
                new WindowAdapter()
                    {
                        public void windowClosing(WindowEvent e)
                            {
                                /*
                                  if ( popServer != null )
                                  {
                                  popServer.setHide(true);
                                  }
                                  else
                                  {
                                  System.out.println("popserver is null");
                                  }
                                */

                            }
                    }
                );
        }



    public void setPopServer( PopServer popServer )
    {
        this.popServer = popServer;
        headerTableViewer.setPopServer( popServer );
    }

    public PopServer getPopServer()
    {
        return popServer;
    }


    public void show()
    {
        super.show();
    }

    public ManageHeaderTableViewer getHeaderTableViewer()
    {
        return headerTableViewer;
    }


    /*
    public void updateWindow(boolean status)
    {

        if ( status == true )
        {
            setJMenuBar( menuBar );
            getContentPane().add( toolBar, BorderLayout.NORTH );
            getContentPane().add( headerTableViewer.scrollPane, BorderLayout.CENTER );
        }
        else
        {
            setJMenuBar( null );
            remove( toolBar );
            remove( headerTableViewer.scrollPane );
        }

    }
    */


    private void setupWindow()
        {
            Container contentPane;

            contentPane = this.getContentPane();

            menuBar = createMenubar();
            setJMenuBar( menuBar );

                //this.setJMenuBar( createMenubar() );

            contentPane.setLayout( new BorderLayout() );

            toolBar = createToolbar();


            JPanel topPanel = new JPanel();
            topPanel.setLayout( new BorderLayout() );

            topPanel.add( toolBar, BorderLayout.NORTH );
            topPanel.add( manageInfoPanel, BorderLayout.CENTER );


            contentPane.add( topPanel, BorderLayout.NORTH );

            JPanel centerPanel = new JPanel();
            centerPanel.setLayout( new BorderLayout() );

            centerPanel.add( filterToolbar, BorderLayout.NORTH );
            centerPanel.add( headerTableViewer.scrollPane, BorderLayout.CENTER );

            contentPane.add( centerPanel );


            JLabel statusBar = new JLabel("Manage POP3 Server");
            statusBar.setBorder( UIManager.getBorder("statusbar") );

            contentPane.add( statusBar, BorderLayout.SOUTH );




        }

        private ManageActionListener getActionListener()
        {
            return manageActionListener;
        }

        private JToolBar createToolbar()
        {
            JToolBar toolBar;
            CButton button;

            toolBar = new JToolBar();
	    toolBar.putClientProperty("JToolBar.isRollover",Boolean.TRUE);
            toolBar.setFloatable(false);

            toolBar.addSeparator();

            button = new CButton( getActionListener().synchronizeAction );
            toolBar.add( button );


            toolBar.addSeparator();

            button = new CButton( getActionListener().fetchAction );
            toolBar.add( button );

            button = new CButton( getActionListener().deleteAction );
            toolBar.add( button );

            toolBar.addSeparator();
            toolBar.addSeparator();
            toolBar.addSeparator();

            button = new CButton( getActionListener().closeAction );
            toolBar.add( button );

            /*
            toolButton = new JButton("Synchronize",ImageLoader.getImageIcon("","Host24") );
            toolButton.setMargin( new Insets(0,0,0,2) );
            toolButton.addActionListener( this );
            toolButton.setActionCommand("SYNCHRONIZE");
            //toolButton.setEnabled(false);
            toolBar.add( toolButton );

            toolBar.addSeparator();

            toolButton = new JButton("Fetch",ImageLoader.getImageIcon("","WebComponent24") );
            toolButton.setMargin( new Insets(0,0,0,2) );
            toolButton.addActionListener( this );
            toolButton.setActionCommand("FETCH");
            //toolButton.setEnabled(false);
            toolBar.add( toolButton );

            toolBar.addSeparator();

            toolButton = new JButton("Delete",ImageLoader.getImageIcon("","Delete24") );
            toolButton.setMargin( new Insets(0,0,0,2) );
            toolButton.addActionListener( this );
            toolButton.setActionCommand("DELETE");
            //toolButton.setEnabled(false);
            toolBar.add( toolButton );

            toolBar.addSeparator();

            toolButton = new JButton("Close",ImageLoader.getImageIcon("","Home24") );
            toolButton.setMargin( new Insets(0,0,0,2) );
            toolButton.addActionListener( this );
            toolButton.setActionCommand("CLOSE");
            toolBar.add( toolButton );

            */
            return toolBar;
        }



    private JMenuBar createMenubar()
        {
            JMenu fileMenu = new JMenu("Action");
            JMenuBar menuBar = new JMenuBar();
            menuBar.add( fileMenu );

            fileMenu.add( getActionListener().synchronizeAction );

            fileMenu.addSeparator();

            fileMenu.add( getActionListener().fetchAction );
            fileMenu.add( getActionListener().deleteAction );

            fileMenu.addSeparator();

            fileMenu.add( getActionListener().closeAction );

            JMenu viewMenu = new JMenu("View");
            menuBar.add( viewMenu );

            JCheckBoxMenuItem cbMenuItem = new JCheckBoxMenuItem(
                              getActionListener().threadedViewAction );

            viewMenu.add( cbMenuItem );


            return menuBar;
        }


        /*
    public void updateHeader()
    {
        headerTableViewer.setPopServer( headerTableViewer.getPopServer() );
    }
    */



}







