// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.pop3;

import org.columba.message.*;
import org.columba.folder.*;
import org.columba.parser.*;
import org.columba.pop3.*;
import org.columba.util.*;
import org.columba.main.*;


import org.columba.gui.header.*;
import org.columba.gui.pop3.util.*;
import org.columba.gui.header.util.*;
import org.columba.gui.message.*;
import org.columba.gui.util.*;
import org.columba.config.*;


import java.io.*;
import javax.swing.text.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.util.*;

import java.awt.dnd.*;
import java.awt.datatransfer.*;

/**
 * This class shows the messageheader
 *
 *
 * @version 1.0
 * @author leFrog
 */

public class ManageHeaderTableViewer implements TreeModelListener, TreeSelectionListener
{

    private JPopupMenu popup;
    private HeaderTable headerTable;
    private HeaderTableModel headerTableModel;
    private MainInterface mainInterface;

    public JScrollPane scrollPane;
    private boolean ready = true;
    private PopServer server;

    protected TableModelFilteredView manageTableModelFilteredView;
    protected TableModelSorter tableModelSorter;
    protected TableModelThreadedView tableModelThreadedView;

    public ManageHeaderTableViewer(MainInterface mainInterface)
        {
            super();
	    this.mainInterface = mainInterface;

            HeaderTableItem item = (HeaderTableItem) mainInterface.config.getPopManageOptionsConfig().getHeaderTableItem();
            //item.removeEnabledItem();

            headerTableModel = new HeaderTableModel( item );
            headerTableModel.addTreeModelListener( this );

            manageTableModelFilteredView = new TableModelFilteredView( headerTableModel );
            tableModelSorter = new TableModelSorter( headerTableModel );
            //tableModelSorter.setSortingColumn("In Order Received");
            //tableModelSorter.setWindowItem( mainInterface.windowItem );
            tableModelThreadedView = new TableModelThreadedView( headerTableModel );

            headerTable = new HeaderTable( headerTableModel);
            headerTableModel.registerPlugin( manageTableModelFilteredView );
            headerTableModel.registerPlugin( tableModelThreadedView );
            headerTableModel.registerPlugin( tableModelSorter );



            initRenderer();

            headerTable.addTreeSelectionListener( this );


            //headerTable.getTable().addMouseListener( new HeaderTableMouseListener() );


            //addMouseListenerToHeaderInTable();


            popup  = new JPopupMenu();
            MouseListener popupListener = new PopupListener();
            headerTable.getTable().addMouseListener(popupListener);

            scrollPane = new JScrollPane( headerTable.getTable() );
            scrollPane.getViewport().setBackground( Color.white );
            scrollPane.setPreferredSize(new Dimension(600, 200));



        }

    public void toggleThreadedView()
    {
        boolean result = getTableModelThreadedView().isEnabled();
        if ( result ) getTableModelThreadedView().setEnabled( false );
           else getTableModelThreadedView().setEnabled( true );
    }


    public void initRenderer()
    {
       

        headerTable.setTreeCellRenderer( new SubjectTreeCellRenderer( headerTable.getTree() ) );

        HeaderTableItem v = (HeaderTableItem) mainInterface.config.getPopManageOptionsConfig().getHeaderTableItem();
        v.removeEnabledItem();


        for ( int i=0; i<v.count(); i++ )
        {
            String name = v.getName(i);
            int size = v.getSize(i);

            if ( name.equalsIgnoreCase("size") )
            {
                headerTable.registerRenderer( "Size",
                                              new HeaderTableSizeRenderer( headerTable.getTree() ) ,
                                              new CommonHeaderRenderer( "size",
                                                                        ResourceLoader.getString("header/mail","size"),
                                                                        getTableModelSorter() ),
                                              size, false );
            }
            /*
            else if ( name.equalsIgnoreCase("Flagged") )
            {
                headerTable.registerRenderer( "Flagged",
                                              new FlaggedRenderer(),
                                              new BooleanHeaderRenderer( true, name, getTableModelSorter() ),
                                              23, true );
            }

            else if ( name.equalsIgnoreCase("Status") )
            {

                headerTable.registerRenderer( "Status",
                                              new StatusRenderer(),
                                              new BooleanHeaderRenderer( true, name, getTableModelSorter() ),
                                              23, true );
            }
            */
	    else if ( name.equalsIgnoreCase("Fetch") )
            {
                headerTable.registerRenderer( "Fetch",
                                              null,
                                              new CommonHeaderRenderer( "Already Fetched",
                                                                         "Already Fetched",
                                                                         getTableModelSorter() ),
                                              size, true );
            }
            /*
            else if ( name.equalsIgnoreCase("Delete") )
            {
                headerTable.registerRenderer( "Delete",
                                              new ImageRenderer( ImageLoader.getImageIcon("","Delete16") ),
                                              new BooleanHeaderRenderer( true, name, getTableModelSorter() ),
                                              23, true );
            }
            */
            else if ( name.equalsIgnoreCase("Attachment") )
            {
                headerTable.registerRenderer( "Attachment",
                                              new BooleanRenderer( true, getImageIcon("table.attachment", "Save16" ) ),
                                              new BooleanHeaderRenderer(true, name, getTableModelSorter() ),
                                              23, true );
            }
            else if ( name.equalsIgnoreCase("Date") )
            {
                headerTable.registerRenderer( "Date",
                                              new HeaderTableDateRenderer( headerTable.getTree(), true ),
                                              new CommonHeaderRenderer( name,
                                                                        ResourceLoader.getString("header/mail","date"),
                                                                        getTableModelSorter() ),
                                                                        size, false );


            }
            else if ( name.equalsIgnoreCase("Priority") )
            {
                headerTable.registerRenderer( "Priority",
                                              new PriorityRenderer( true  ),
                                              new BooleanHeaderRenderer(true, name, getTableModelSorter() ),
                                              23, true );

            }
	    else if ( name.equalsIgnoreCase("Subject") )
            {
                headerTable.registerRenderer( "Subject",
                                              null,
                                              new CommonHeaderRenderer( name,
                                                                        ResourceLoader.getString("header/mail","subject"),
                                                                        getTableModelSorter() ),
                                                                        size, false );

            }
            else
            {
                String str = new String();
                try
                {
                	str = ResourceLoader.getString("header/mail", name.toLowerCase());
                   
                }
                catch (MissingResourceException ex)
                {
                    System.out.println( ex.getMessage() );

                }

                if ( str.length() == 0 )
                   headerTable.registerRenderer( name,
                                                 new HeaderTableCommonRenderer( headerTable.getTree() ) ,
                                                 new CommonHeaderRenderer( name,
                                                                           name,
                                                                           getTableModelSorter() ),
                                                                           size, false );

                else
                    headerTable.registerRenderer( name,
                                                 new HeaderTableCommonRenderer( headerTable.getTree() ) ,
                                                 new CommonHeaderRenderer( name,
                                                                           str,
                                                                           getTableModelSorter() ),
                                                                           size, false );

            }
        }



    }

    protected ImageIcon getImageIcon( String id, String failCase )
    {
        ImageIcon icon = (ImageIcon) UIManager.getIcon( id );

        if ( icon == null )
            icon = ImageLoader.getImageIcon("org/columba/images/"+failCase+".gif");

        return icon;
    }



        /**
         * sets the selected folder, which is going to be viewed
         *
         * @param f a <code>Folder</code>
         * @see Folder
         */
    public void setPopServer( PopServer server )
    {
        this.server = server;
            // register the new folder in the tablemodel

        //getHeaderTableModel().setMessageCollection( server.getMessageCollection() );

        getHeaderTableModel().setHeaderList( server.getHeaderList() );
    }

    public PopServer getPopServer()
    {
        return server;
    }





        /**
         * returns the table
         * @return the JTable class capsulated by HeaderTable
         */
    public JTable getTable()
        {
                // return JTable from class HeaderTable
            return getHeaderTable().getTable();
        }



    public HeaderTable getHeaderTable()
        {
            return headerTable;
        }

    public HeaderTableModel getHeaderTableModel()
        {
            return headerTableModel;

        }

    public TableModelSorter getTableModelSorter()
    {
        return tableModelSorter;
    }

    public TableModelThreadedView getTableModelThreadedView()
    {
        return tableModelThreadedView;
    }

    public TableModelFilteredView getTableModelFilteredView()
    {
        return manageTableModelFilteredView;
    }



    protected void initContextMenu()
        {


            popup.removeAll();

            JMenuItem menuItem;

            menuItem = new JMenuItem("Mark as Read");
                //menuItem.addActionListener(new MainFrameActionListener(frame));
            menuItem.setEnabled(true);

            popup.add(menuItem);

            popup.addSeparator();


        }

    class PopupListener extends MouseAdapter
    {
        public void mousePressed(MouseEvent e)
            {
                maybeShowPopup(e);
            }

        public void mouseReleased(MouseEvent e)
            {
                maybeShowPopup(e);
            }

        private void maybeShowPopup(MouseEvent e)
            {
                if (e.isPopupTrigger())
                {
                    initContextMenu();
                    popup.show(e.getComponent(),  e.getX(), e.getY());
                }
            }
    }


    /*
    public Object[] getUids()
    {
        MessageNode[] nodes = getHeaderTable().getSelectedNodes();

        Object[] uids = new Object[ nodes.length ];

        for ( int i=0; i<nodes.length; i++ )
        {
            uids[i] = nodes[i].getUid();
            //System.out.println("uid: "+uids[i] );
        }

        return uids;

    }
    */

    public MessageNode[] getSelectedNodes()
    {
        return getHeaderTable().getSelectedNodes();
    }

    private class HeaderTableMouseListener extends MouseAdapter
    {

        public HeaderTableMouseListener()
            {
                super();

            }

        public void mousePressed( MouseEvent event )
            {
                if ( event.isPopupTrigger() )
                {
                    System.out.println("popup-menu pressed");
                }
            }


        public void mouseReleased( MouseEvent event )

            {
                if ( event.isPopupTrigger() )
                {
                    System.out.println("pop-menu release");
                }
            }

        public void mouseClicked( MouseEvent event )
            {




                if ( event.getClickCount() == 1 )
                {

                    if ( (event.isControlDown()) || (event.isShiftDown()) || (event.isMetaDown()) )
                    {
                            //System.out.println("selection one-click");

                    } else
                    {
                            //showMessage();
                    }

                }

            }

        private void processDoubleClick()
            {
                System.out.println("double-click");
            }
    }



    public void treeStructureChanged(TreeModelEvent e)
    {

        //System.out.println("headerTableviewer->treeStructureChanged");


        /*
        getHeaderTable().treeStructureChanged( e );
        */



    }


    public void treeNodesRemoved(TreeModelEvent e)
    {}

    public void treeNodesInserted(TreeModelEvent e)
    {}

    // message is called when we only sort or  filter the messages
    public void treeNodesChanged(TreeModelEvent e)
    {
        //System.out.println("headerTable->treeNodesChanged");
    }

    public void updateTable()
    {
        //getHeaderTableModel().setMessageCollection( server.getMessageCollection() );
        getHeaderTableModel().setHeaderList( server.getHeaderList() );
    }

    public void addMouseListenerToHeaderInTable()
        {
            final JTable tableView = getTable();

            tableView.setColumnSelectionAllowed(false);

            MouseAdapter listMouseListener = new MouseAdapter()
                {
                    public void mouseClicked(MouseEvent e)
                    {
                        System.out.println("mouseclicked");

                        TableColumnModel columnModel = tableView.getColumnModel();
                        int viewColumn = columnModel.getColumnIndexAtX(e.getX());
                        int column = tableView.convertColumnIndexToModel(viewColumn);

                        if (e.getClickCount() == 1 && column != -1)
                        {
                            System.out.println("mouseclicked 1");

                            getTableModelSorter().setSortingColumn( column );

                            updateTable();


                        }
                    }
                };

            JTableHeader th = tableView.getTableHeader();
            th.addMouseListener(listMouseListener);
        }


        public void valueChanged(TreeSelectionEvent e)
        {
            //System.out.println("manageheadertableviewer->valueChanged for tree");

            //Object[] uids = getUids();

            /*
            MessageNode node = (MessageNode) getHeaderTable().getSelectedNode();
            if (node == null) return;

            setSelected( node );
            */


            //getActionListener().changeMessageActions();


        }

}




