// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.statusbar;


import javax.swing.*;
import java.util.*;
import java.text.*;
import java.awt.*;
import java.awt.event.*;


import org.columba.main.*;

public class TaskManager implements ActionListener
{
    private javax.swing.Timer timer;
    private Vector workerList;
    private StatusBar statusBar;

    private int value=0;

    private MainInterface mainInterface;

    private JButton cancelButton;

    protected Collator collator;

    protected boolean mutex;

    public TaskManager( MainInterface mainInterface, StatusBar statusBar )
    {
        this.mainInterface = mainInterface;
        this.statusBar = statusBar;


        statusBar.initActions( this );

          


        workerList = new Vector();

	collator = Collator.getInstance();

    }


    protected void insertElement( WorkerItem item )
    {
	getMutex();
	workerList.insertElementAt(item,0 );	
        releaseMutex();

	sortElements();
    }

    protected void sortElements()
    {
            
	getMutex();

	Collections.sort( workerList, new WorkerComparator() );

	WorkerItem item = (WorkerItem) workerList.get(0);
	

	releaseMutex();
    }

    // old method - don't use it anymore
    public void register( Thread t )
    {
	
    }

    public void register( Thread t, int priority )
    {
	

	getMutex();
        WorkerItem worker = new WorkerItem( t, workerList.size(), priority );
	releaseMutex();

        insertElement( worker );

        enableCancelButton( true );

        getMutex();
        int count = workerList.size();
        releaseMutex();

        setTaskCount( count );

        if ( count > 1 )
        {
            enableLeftButton( true );
            enableRightButton( true );
        }
        else
        {
            //worker.setAllowed( true );
        }

	worker.setAllowed( true );

        mainInterface.imageSequenceTimer.start();


          //System.out.println("thread registered");
    }


    public void unregister( Thread thread )
    {
        Thread t;
        WorkerItem worker;
	
	getMutex();
	int size2 = workerList.size();
	releaseMutex();

	for ( int i=0; i<size2 ;  i++ )
        {
            getMutex();
            worker = (WorkerItem) workerList.get(i);
            releaseMutex();
            t = worker.getThread();


            if ( t.equals( thread ) )
            {

                getMutex();
                workerList.remove(i);
                int count = workerList.size();
                releaseMutex();


                setTaskCount( count );

                if ( count > 0 )
                {
		    
                    WorkerItem item;
                    getMutex();

		    Collections.sort( workerList, new WorkerComparator() );

                    if ( workerList.size() > 0 )
                       item = (WorkerItem) workerList.get(0);
                    else
                       item = null;

                    releaseMutex();

                    if ( item != null )
                    {
                        setText( item.getText() );                 
                        setProgressBarMaximum( item.getProgressBarMaximum() );
                        setProgressBarValue( item.getProgressBarValue() );
                        enableCancelButton( true );
                    }
                }
                else
                {
		    setProgressBarMaximum(100);
                    setProgressBarValue(0);
                    setText( "" );
                    enableCancelButton( false );

                    mainInterface.imageSequenceTimer.stop();
                }

                getMutex();
                int size = workerList.size();
                releaseMutex();
                if ( size < 2 )
                {
                    enableLeftButton( false );
                    enableRightButton( false );

                }


		return;
            }
        }



    }



      /******************************** Show other Thread Label *********************************/

    public void left()
    {
        getMutex();
        int size = workerList.size();
        releaseMutex();

        if ( size > 1 )
        {
            getMutex();
            WorkerItem worker = (WorkerItem) workerList.remove( 0 );
            workerList.add( worker );

            WorkerItem item = (WorkerItem) workerList.get(0);
            releaseMutex();

            setText( item.getText() );
            setProgressBarMaximum( item.getProgressBarMaximum() );
            setProgressBarValue( item.getProgressBarValue() );

            //System.out.println("progressbar total size: "+ item.getProgressBarMaximum() );
            //System.out.println("progressbar total value: "+ item.getProgressBarValue() );
        }
    }

    public void right()
    {
        getMutex();
        int size = workerList.size();
        releaseMutex();

        if ( size > 1 )
        {
            getMutex();
            WorkerItem worker = (WorkerItem) workerList.remove( workerList.size()-1 );

            workerList.insertElementAt( worker, 0 );

            WorkerItem item = (WorkerItem) workerList.get(0);
            releaseMutex();

            setText( item.getText() );
            setProgressBarMaximum( item.getProgressBarMaximum() );
            setProgressBarValue( item.getProgressBarValue() );

            //System.out.println("progressbar total size: "+ item.getProgressBarMaximum() );
            //System.out.println("progressbar total value: "+ item.getProgressBarValue() );
        }


    }


      /***************************************** Timer ******************************************/



    public void resetTimer()
    {

    }


    public synchronized void start()
    {
          //timer.start();
    }

    public synchronized void stop()
    {
          //timer.stop();
    }






      /******************************************** Progressbar **********************************/

      /*
    public JProgressBar getProgressBar()
    {
        return statusBar.getProgressBar();
    }
    */


    public WorkerItem getWorkerItem( Thread thread )
    {
        WorkerItem worker;
        Thread t;

        getMutex();
        int size = workerList.size();
        //releaseMutex();

        for ( int i=0; i< size;  i++ )
        {
            //getMutex();
            worker = (WorkerItem) workerList.get(i);


            t = worker.getThread();


            if ( t.equals( thread ) )
            {
                releaseMutex();
                return worker;
            }
        }

        releaseMutex();

        return null;
    }



    public void setProgressBarValue( Thread t, int v )
    {
        getMutex();
        WorkerItem worker = (WorkerItem) workerList.get(0);
        releaseMutex();

	Thread act = worker.getThread();

        getWorkerItem(t).setProgressBarValue( v );

	if ( t.equals( act ) )
	    {


                if ( worker.isAllowed() == true )
		   setProgressBarValue(v);
	    }
	else
	    {

	    }
    }


    public void incProgressBarValue( Thread t, int i )
    {
        getMutex();
        WorkerItem worker = (WorkerItem) workerList.get(0);
        releaseMutex();

	Thread act = worker.getThread();

        getWorkerItem(t).setProgressBarValue( getWorkerItem(t).getProgressBarValue()+i );

	if ( t.equals( act ) )
	    {

                int newValue = worker.getProgressBarValue();

                if ( worker.isAllowed() == true )
		   setProgressBarValue( newValue );
	    }

    }


    public void setProgressBarMaximum( Thread t, int v )
    {
        getMutex();
        WorkerItem worker = (WorkerItem) workerList.get(0);
        releaseMutex();

	Thread act = worker.getThread();

        getWorkerItem(t).setProgressBarMaximum( v );

	if ( t.equals( act ) )
	    {
                if ( worker.isAllowed() == true )
		   setProgressBarMaximum(v);
	    }

    }

    public void setProgressBarMaximum( int m )
    {
        getStatusBar().setMaximum( m );
    }

    public void setProgressBarValue( int v )
    {
        getStatusBar().setValue( v );
    }

    public int getProgressBarValue()
    {
        return getStatusBar().getValue();
    }


      /*************************************** Cancel Operation ******************************/



    public boolean getCancel( Thread t )
    {
    	/*
        getMutex();
        WorkerItem worker = (WorkerItem) workerList.get(0);
        releaseMutex();

	Thread act = worker.getThread();
		*/
		
	WorkerItem item = getWorkerItem(t);

	if ( item != null ) return item.getCancel();
	else
          return false;

	
    }

    public void setCancel ( Thread t, boolean b )
    {
		/*
        getMutex();
        WorkerItem worker = (WorkerItem) workerList.get(0);
        releaseMutex();

		Thread act = worker.getThread();
		*/

        getWorkerItem(t).setCancel( b );

	

    }

      /**************************************** show active thread info **************************/

    public void setText( String s )
    {
        getStatusBar().setText( s );
    }


    public void setText( Thread t, String s )
    {
	getMutex();
        WorkerItem worker = (WorkerItem) workerList.get(0);
        releaseMutex();

	Thread act = worker.getThread();

        getWorkerItem(t).setText( s );

	if ( t.equals( act ) )
	    {
                
                if ( worker.isAllowed() == true )
		   setText( s );
	    }



    }


    public void setTaskCount( int i )
    {
        getStatusBar().setTaskCount( i );
    }


    public StatusBar getStatusBar()
    {
        return statusBar;
    }

    public void setCancelButton( JButton b )
    {
        cancelButton = b;
    }

    public JButton getCancelButton()
    {
        return cancelButton;
    }


    public void enableLeftButton( boolean b )
    {
        getStatusBar().getLeftButton().setEnabled( b );
    }

    public void enableRightButton( boolean b )
    {
        getStatusBar().getRightButton().setEnabled( b );
    }


    public void enableCancelButton( boolean b )
    {
        getCancelButton().setEnabled( b );
    }

    public void enableBusyButton( boolean b )
    {
        getStatusBar().getBusyButton().setEnabled( b );
    }


      /*********************************** action **************************************/



    public void actionPerformed(ActionEvent e)
    {
        Object source = e.getSource();
        String command = e.getActionCommand();
		WorkerItem worker = null;

        if ( command.equals("CANCEL_ACTION") )
        {
            getMutex();
            if( workerList.size() != 0 )  worker = (WorkerItem) workerList.get(0);
            releaseMutex();

            if( worker != null ) worker.setCancel( true );

        }
        else if ( command.equals("LEFT") )
        {
            left();
        }
        else if ( command.equals("RIGHT") )
        {
            right();
        }
        else
        {

        }

    }


    /************************* thread safe work *******************************/




    private synchronized void getMutex()
    {
        while( mutex ) {
            try {
                wait();
            }
            catch( InterruptedException e )
            {
            }
        }


        mutex = true;
    }



    private synchronized void releaseMutex()
    {

        mutex = false;
        notify();
    }


    class WorkerComparator implements Comparator
    {


        public WorkerComparator()
            {                
            }

        public int compare(Object o1, Object o2)
            {             
                WorkerItem item1 = (WorkerItem) o1;
		WorkerItem item2 = (WorkerItem) o2;

		if ( ( item1==null ) || ( item2==null ) ) return 0;
		int priority1 = item1.getPriority();
		int priority2 = item2.getPriority();

                if ( priority1 < priority2 ) return 1;
		else if ( priority1 > priority2 ) return -1;
		else return 0;

            }


        public boolean equals(Object obj)
        {

            if (obj instanceof WorkerComparator)
            {

                WorkerComparator compObj = (WorkerComparator) obj;

                return true;

            }

            return false;

        }

    }


}
