package org.columba.gui.themes.thincolumba;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import javax.swing.plaf.metal.*;

import javax.swing.text.View;

public class ThinColumbaButtonUI extends MetalButtonUI implements java.io.Serializable
{
    private final static ThinColumbaButtonUI columbaButtonUI = new ThinColumbaButtonUI();

    
    public ThinColumbaButtonUI() {}

    public static ComponentUI createUI( JComponent c )
    {
        return columbaButtonUI;
    }

    private static Rectangle viewRect = new Rectangle();
    private static Rectangle textRect = new Rectangle();
    private static Rectangle iconRect = new Rectangle();

    public void paint(Graphics g, JComponent c) 
    {
        AbstractButton b = (AbstractButton) c;
        ButtonModel model = b.getModel();

        FontMetrics fm = g.getFontMetrics();

        Insets i = c.getInsets();

        viewRect.x = i.left;
        viewRect.y = i.top;
        viewRect.width = b.getWidth() - (i.right + viewRect.x);
        viewRect.height = b.getHeight() - (i.bottom + viewRect.y);

        textRect.x = textRect.y = textRect.width = textRect.height = 0;
        iconRect.x = iconRect.y = iconRect.width = iconRect.height = 0;

        Font f = c.getFont();
        g.setFont(f);

        // layout the text and icon
        String text = SwingUtilities.layoutCompoundLabel(
            c, fm, b.getText(), b.getIcon(), 
            b.getVerticalAlignment(), b.getHorizontalAlignment(),
            b.getVerticalTextPosition(), b.getHorizontalTextPosition(),
            viewRect, iconRect, textRect, 
            b.getText() == null ? 0 : defaultTextIconGap
        );

        clearTextShiftOffset();

        // perform UI specific press action, e.g. Windows L&F shifts text
        if (model.isArmed() && model.isPressed()) {
            paintButtonPressed(g,b); 
        }

	/*
	if ( b.isRolloverEnabled() && model.isRollover() ) 
	{
	    UIManager.getBorder("Button.border").paintBorder( c,g,  0,0, c.getWidth(),c.getHeight());
	}
	else if ( b.isRolloverEnabled() )
	{
	    c.setBorder(null);
	}
	*/

        // Paint the Icon
        if(b.getIcon() != null) { 
            paintIcon(g,c,iconRect);
        }

        if (text != null && !text.equals("")){
	    View v = (View) c.getClientProperty(BasicHTML.propertyKey);
	    if (v != null) {
		v.paint(g, textRect);
	    } else {
		paintText(g, c,textRect, text);
	    }
        }

        if (b.isFocusPainted() && b.hasFocus()) {
            // paint UI specific focus
            paintFocus(g,b,viewRect,textRect,iconRect);
        }
    }

    public void installUI(JComponent c)
    {
        super.installUI(c);
    }

    public void uninstallUI(JComponent c)
    {
        super.uninstallUI(c);
    }
    
}
