// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.gui.tree;

import java.awt.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.io.*;
import javax.swing.plaf.basic.*;
import java.util.Enumeration;
import java.util.*;

import org.columba.config.*;
import org.columba.message.*;
import org.columba.folder.*;
import org.columba.folder.imap.*;
import org.columba.folder.columba.*;
import org.columba.folder.outbox.*;
import org.columba.folder.virtual.*;
import org.columba.gui.tree.util.*;
import org.columba.gui.util.*;
import org.columba.pop3.*;
import org.columba.main.MainInterface;
import org.columba.filter.*;

/**
 * this class does all the dirty work for the TreeViewer
 */
public class FolderTree implements TreeSelectionListener, TreeNodeChangeListener
{

    //protected AdapterNode rootNode;

    protected Folder rootFolder;
    //public FolderModel treeModel;

    protected FolderXmlConfig folderXmlConfig;

    protected JTree tree;

    private String selectedLeaf = new String();

    //private FolderTreeNode selectedNode;
    private Folder selectedFolder;

    private MainInterface mainInterface;

    private JTextField textField;

    private DefaultTreeModel treeModel;

 

    public FolderTree( MainInterface mainInterface, FolderXmlConfig folderConfig )
    { 
            this.mainInterface = mainInterface;
            this.folderXmlConfig = folderConfig;
            

            AdapterNode rootNode = folderXmlConfig.getRootNode();

            rootFolder = new Folder( rootNode, null );


            createDirectories( mainInterface, rootNode, rootFolder );



            treeModel = new DefaultTreeModel( rootFolder );
            tree = new JTree( treeModel );

            tree.putClientProperty("JTree.lineStyle", "Angled");

            tree.setShowsRootHandles( true );
            tree.setRootVisible(false);

            //rootFolder.addTreeNodeListener( this );
            addTreeNodeChangeListener( rootFolder );


            FolderTreeCellRenderer renderer = new FolderTreeCellRenderer( true );
            tree.setCellRenderer(renderer);
            /*
            tree.setEditable(true);
            textField = new JTextField();
            tree.setCellEditor( new FolderTreeCellEditor( textField ) );
            */

            tree.getSelectionModel().setSelectionMode (TreeSelectionModel.SINGLE_TREE_SELECTION);
            tree.addTreeSelectionListener( this  );

            expand();


        }

    public Folder getRootFolder()
    {
        return rootFolder;
    }


    public void expand()
    {
        int rowCount = getTree().getRowCount();

	getTree().expandRow(0);
	rowCount = getTree().getRowCount();

	getTree().expandRow(1);

	getTree().repaint();


    }

    public void addTreeNodeChangeListener( Folder parent )
    {
        int count = parent.getChildCount();

        for ( int i=0; i<count; i++ )
        {
            Folder child = (Folder) parent.getChildAt( i );
            //child.addTreeNodeListener( this );
            addTreeNodeChangeListener( child );
        }
    }

    public void createDirectories( MainInterface mainInterface, AdapterNode parentTreeNode, Folder parentFolder )
    {
        int count = parentTreeNode.getChildCount();
        AdapterNode child;


        if ( count > 0 )
        {
            for ( int i=0; i<count; i++ )
            {

                child = parentTreeNode.getChild(i);
                String name = child.getName();
                AdapterNode nameNode = child.getChild("name");

                //System.out.println( "node: "+child );
                //System.out.println( "nodename: "+nameNode.getValue());
                if (  (name.equals("tree")) || (name.equals("folder")) )
                {
                  Folder folder = add( mainInterface, child, parentFolder );
                  createDirectories( mainInterface, child, folder );
                }
            }
        }
    }

    public void addUserFolder( String name )
    {
        addUserFolder( getSelected(), name );

    }

    public void addUserFolder( Folder parentFolder, String name )
    {
        AdapterNode adapterNode = mainInterface.config.getFolderConfig().addFolderNode(
                            parentFolder.getNode() , name, "user", "true","columba", "true", "true","true",-1 );

        FolderItem item = folderXmlConfig.getFolderItem( adapterNode );

        ColumbaFolder newFolder = new ColumbaFolder( adapterNode, item,  mainInterface);
        //newFolder.addTreeNodeListener( this );


        parentFolder.add( newFolder );

        TreeNodeEvent updateEvent2 = new TreeNodeEvent( parentFolder, TreeNodeEvent.STRUCTURE_CHANGED );
        treeNodeChanged(updateEvent2);
    }


    public Folder addVirtualFolder( String name )
    {
        Folder parentFolder = getSelected();

        AdapterNode adapterNode = mainInterface.config.getFolderConfig().addFolderNode(
                            parentFolder.getNode() , name, "user", "true","virtual", "false", "false","true",-1 );

        FolderItem item = folderXmlConfig.getFolderItem( adapterNode );

        VirtualFolder newFolder = new VirtualFolder( adapterNode, item );
        Search search = new Search( adapterNode, newFolder, mainInterface );
       // newFolder.addTreeNodeListener( this );
        parentFolder.add( newFolder );
        VirtualOperator operator = new VirtualOperator(mainInterface);
        operator.start();
        newFolder.setOperator( operator );

        TreeNodeEvent updateEvent2 = new TreeNodeEvent( parentFolder, TreeNodeEvent.STRUCTURE_CHANGED );
        treeNodeChanged(updateEvent2);

        return newFolder;

    }

    /*
    public Folder addVirtualFolder( String name, String headerItem, String pattern )
    {
        Folder parentFolder = getSelected();
        int uid = parentFolder.getUid();

        AdapterNode adapterNode = mainInterface.config.getFolderConfig().addVirtualFolderNode(
                            parentFolder.getNode() , uid, name, headerItem, pattern);

        FolderItem item = folderXmlConfig.getFolderItem( adapterNode );

        VirtualFolder newFolder = new VirtualFolder( adapterNode, item,  mainInterface);
        newFolder.addTreeNodeListener( this );
        parentFolder.add( newFolder );

        return newFolder;
    }
    */

    public Folder addImapFolder( Folder parentFolder, String name, ImapItem imapItem, Folder rootFolder, int uid  )
    {
        AdapterNode adapterNode = mainInterface.config.getFolderConfig().addFolderNode(
                            parentFolder.getNode() , name, "user", "true","imap", "true", "true","true", uid );

        FolderItem item = folderXmlConfig.getFolderItem( adapterNode );

        ImapFolder newFolder = new ImapFolder( adapterNode, item, imapItem, (ImapRootFolder) rootFolder, mainInterface);
        //newFolder.addTreeNodeListener( this );
        parentFolder.add( newFolder );

        TreeNodeEvent updateEvent2 = new TreeNodeEvent( parentFolder, TreeNodeEvent.STRUCTURE_CHANGED );
        treeNodeChanged(updateEvent2);

        return newFolder;
    }

    public Folder addImapRootFolder( String name, ImapItem imapItem, int uid )
    {
        Folder parentFolder = getRootFolder();
        AdapterNode adapterNode = mainInterface.config.getFolderConfig().addFolderNode(
                            parentFolder.getNode() , name, "system", "false","imaproot", "true", "false","false", uid );

        FolderItem item = folderXmlConfig.getFolderItem( adapterNode );

        ImapRootFolder newFolder = new ImapRootFolder( adapterNode, item, imapItem, uid, mainInterface);
        //newFolder.addTreeNodeListener( this );
        parentFolder.add( newFolder );
        ImapOperator operator = new ImapOperator(mainInterface);
        operator.start();
        newFolder.setOperator( operator );

        TreeNodeEvent updateEvent2 = new TreeNodeEvent( parentFolder, TreeNodeEvent.STRUCTURE_CHANGED );
        treeNodeChanged(updateEvent2);

        return newFolder;
    }


    public Folder add( MainInterface mainInterface, AdapterNode childNode, Folder parentFolder)
    {


        FolderItem item = mainInterface.config.getFolderConfig().getFolderItem( childNode );
        if ( item == null ) return null;


        // i18n stuff

        String name = null;

        int uid = item.getUid();

        try
        {
            if ( uid == 100 )
                name =  ResourceLoader.getString("tree/mail","localfolders") ;
            else if ( uid == 101 )
            	name =  ResourceLoader.getString("tree/mail","inbox") ;
               
            else if ( uid == 102 )
            	name =  ResourceLoader.getString("tree/mail","drafts") ;
                
            else if ( uid == 103 )
            	name =  ResourceLoader.getString("tree/mail","outbox") ;
               
            else if ( uid == 104 )
            	name =  ResourceLoader.getString("tree/mail","sent") ;
                
            else if ( uid == 105 )
            	name =  ResourceLoader.getString("tree/mail","trash") ;
                
            else if ( uid == 106 )
            	name =  ResourceLoader.getString("tree/mail","search") ;
             else if ( uid == 107 )
            	name =  ResourceLoader.getString("tree/mail","templates") ;
                
            else
                name = item.getName();

            item.setName( name );

        }
        catch ( MissingResourceException ex )
        {
            name = item.getName();
        }



        // now instanciate the folder classes

        if ( item.getType().equals("columba") )
        {
            ColumbaFolder f = new ColumbaFolder( childNode, item,  mainInterface );

	    if( uid == 100 ) {
		ColumbaOperator operator = new ColumbaOperator(mainInterface);
		operator.start();
		f.setOperator( operator );
	    }

            FilterList list = new FilterList( f, mainInterface );
            parentFolder.add( f );

            return f;
        }
        else if ( item.getType().equals("virtual") )
        {

            VirtualFolder f = new VirtualFolder( childNode, item );
            Search search = new Search( childNode, f, mainInterface );
            parentFolder.add( f );

            VirtualOperator operator = new VirtualOperator(mainInterface);
            operator.start();
            f.setOperator( operator );

            return f;
        }
        else if ( item.getType().equals("outbox") )
        {
            OutboxFolder f = new OutboxFolder( childNode, item, mainInterface );

	    OutboxOperator operator = new OutboxOperator(mainInterface, f);
	    operator.start();

            parentFolder.add( f );      // Do never exchange with line below!!
	    f.setOperator( operator );

            return f;
        }
        else if ( item.getType().equals("imap") )
        {
            AccountItem accountItem = mainInterface.config.getAccountList().uidGet( item.getAccountUid() );


            ImapItem item2 = accountItem.getImapItem();


            ImapRootFolder imapRootFolder = null;


            ImapFolder f = new ImapFolder( childNode, item,  item2 , imapRootFolder, mainInterface );
            FilterList list = new FilterList( f, mainInterface );
            parentFolder.add( f );


            return f;

        }
        else if ( item.getType().equals("imaproot") )
        {

            AccountItem accountItem = mainInterface.config.getAccountList().uidGet( item.getAccountUid() );

            ImapItem item2 = accountItem.getImapItem();


            ImapRootFolder f = new ImapRootFolder( childNode, item,  item2 , item.getAccountUid(), mainInterface );
	    ImapOperator operator = new ImapOperator(mainInterface);
	    operator.start();
	    f.setOperator( operator );

            parentFolder.add( f );


            return f;
        }

        return null;
    }



    public Folder getSelected()
    {
        return selectedFolder;
    }

    public void setSelected( Folder f )
    {
        //TreePath treePath = f.getSelectionTreePath();
        //getTree().setSelectionPath( treePath );

        selectedFolder=f;
    }




    protected DefaultTreeModel getModel()
    {
        return treeModel;
    }



    public JTree getTree()
        {
            return tree;
        }


      /************************ tree structure methods ********************************/



        // this method is called when the user selects another folder
    public void valueChanged(TreeSelectionEvent e)
        {
            //System.out.println("foldertree->valueChanged");
            selectedFolder = (Folder) tree.getLastSelectedPathComponent();
            if (selectedFolder == null) return;
        }




    public Folder findFolder( Folder parentFolder, String str )
    {

	int count = parentFolder.getChildCount();
        Folder child;
        Folder folder;

        for (Enumeration e = parentFolder.children() ; e.hasMoreElements() ;)
        {
            child = (Folder) e.nextElement();
            //System.out.println( "child: "+child.getName());
	    if ( child.getName().equals( str ) ) return child;
	}

	return null;
    }



    public void treeNodeChanged( TreeNodeEvent e )
    {

        int mode = e.getMode();


        Folder folder = (Folder) e.getSource();
        if ( folder == null ) return;

        if ( mode == TreeNodeEvent.UPDATE )
           getModel().nodeChanged( folder );
        else if ( mode == TreeNodeEvent.STRUCTURE_CHANGED )
           getModel().nodeStructureChanged( folder );

    }

    class FolderTreeCellEditor extends DefaultCellEditor
    {


      public FolderTreeCellEditor( JTextField textField )
      {
          super( textField );
          setClickCountToStart( 2 );

      }

      public Component getTreeCellEditorComponent( JTree tree, Object value,
             boolean isSelected, boolean expanded, boolean leaf, int row )
      {
        JTextField textField = (JTextField) editorComponent;

        Folder folder = (Folder) value;

        textField.setText( folder.getName() );

        return textField;
      }
    }



    public Folder getFolder( int uid )
    {
        Folder root = getRootFolder();

        for (Enumeration e = root.getPreorderEnumeration() ; e.hasMoreElements() ;)
        {
            Folder node = (Folder) e.nextElement();

            FolderItem item = node.getFolderItem();
            if ( item == null ) continue;

            int id = item.getUid();

            if ( uid == id )
            {
                return node;
            }

        }
        return null;

    }

    public Folder getImapFolder( int accountUid )
    {

        Folder root = (Folder) getRootFolder();

        for (Enumeration e = root.getPreorderEnumeration() ; e.hasMoreElements() ;)
        {
            Folder node = (Folder) e.nextElement();

            FolderItem item = node.getFolderItem();
            if ( item == null ) continue;

            if ( item.getType().equals("imaproot") )
            {
                int account = item.getAccountUid();

                if ( account == accountUid )
                {
                    int uid = item.getUid();

                    return getFolder( uid );
                }

            }




        }
        return null;

    }



    public Folder getFolder( TreeNodeList list )
    {

        Folder folder = null;

        Folder parentFolder = getRootFolder() ;

        if ( list == null ) return parentFolder;

         if ( list.count() == 0 )
         {
             System.out.println("list count == null ");

             return parentFolder;
         }

	Folder child = parentFolder;
	for ( int i=0; i < list.count(); i++)
	    {
		String str = list.get(i);
                System.out.println( "str: "+ str );
                child = findFolder( child, str );
	    }

        return child;

    }





      /**
       *
       * return Folder where to save the messages from the specified popserver
       *
       *
       **/
    public Folder getFolder( PopServer server )
    {
        String uidString = server.getAccountItem().getSpecialFoldersItem().getInbox();
        int uid = new Integer(uidString).intValue();

        //int uid = server.getPopItem().getUid();

        Folder node = getFolder( uid );

        return node;
    }

}








