// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.gui.tree;

import org.columba.message.*;
import org.columba.filter.*;
import org.columba.folder.*;
import org.columba.folder.imap.*;
import org.columba.folder.outbox.*;
import org.columba.folder.columba.*;
import org.columba.folder.virtual.*;
import org.columba.config.*;
import org.columba.protocol.Imap4;
import org.columba.pop3.PopServer;
import org.columba.gui.util.*;
import org.columba.gui.action.*;
import org.columba.gui.config.search.*;
import org.columba.util.*;
import org.columba.main.*;
import org.columba.gui.tree.util.*;
import org.columba.gui.header.util.*;
import org.columba.gui.statusbar.*;
import org.columba.gui.tree.action.*;
import org.columba.gui.themes.*;
import org.columba.gui.tree.menu.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import java.awt.dnd.*;
import java.awt.datatransfer.*;

import java.io.*;

import javax.swing.JTree;

/**
 * this class shows the the folder hierarchy
 */

public class TreeViewer implements TreeNodeChangeListener
{
	private FolderTree folderTree;
	private MainInterface mainInterface;
	private boolean b = false;
	private TreePath treePath;
	private JPopupMenu popup;

	private FolderInfoPanel messageFolderInfoPanel;

	private FolderTreeActionListener folderTreeActionListener;

	public JScrollPane scrollPane;

	private Folder oldSelected;

	private FolderTreeMenu menu;

	private FolderTreeMouseListener mouseListener;

	public TreeViewer(MainInterface mainInterface)
	{
		this.mainInterface = mainInterface;

		folderTree =
			new FolderTree(mainInterface, mainInterface.config.getFolderConfig());

		mouseListener = new FolderTreeMouseListener(this);

		getFolderTree().getTree().addMouseListener(mouseListener);

		getFolderTree().getTree().setBorder(
			BorderFactory.createEmptyBorder(2, 0, 2, 0));

		FolderTreeDnd dnd = new FolderTreeDnd(mainInterface, this);

		scrollPane = new JScrollPane(getFolderTree().getTree());

		folderTreeActionListener = new FolderTreeActionListener(mainInterface, this);

		menu = new FolderTreeMenu(this);

	}

	public FolderTree getFolderTree()
	{
		return folderTree;
	}

	public SelectFolderDialog getSelectFolderDialog()
	{
		SelectFolderDialog dialog =
			new SelectFolderDialog(mainInterface.mainFrame, this);
		return dialog;
	}

	public EditFolderDialog getEditFolderDialog(String name)
	{
		EditFolderDialog dialog = new EditFolderDialog(mainInterface.mainFrame, name);

		return dialog;
	}

	public FolderTreeActionListener getActionListener()
	{
		return folderTreeActionListener;
	}

	public JPopupMenu getPopupMenu()
	{
		return menu.getPopupMenu();
	}

	public DefaultTreeModel getModel()
	{
		return getFolderTree().getModel();
	}

	// FIXME
	public void setSelected(Folder folder)
	{
		getFolderTree().setSelected(folder);

		mainInterface.headerTableViewer.setFolder(folder);

		GuiOperation op = new GuiOperation(Operation.HEADERLIST, 4, folder);
		mainInterface.crossbar.operate(op);

	}

	public Folder getSelected()
	{
		Folder folder =
			(Folder) getFolderTree().getTree().getLastSelectedPathComponent();
		return folder;
	}

	public void selectFolder()
	{
		Folder folder = getSelected();
		if (folder == null)
			return;

		if (folder.equals(oldSelected))
			return;
		else
			oldSelected = folder;

		FolderItem item = folder.getFolderItem();

		getActionListener().changeActions();

		MessageFolderInfo info = folder.getMessageFolderInfo();
		mainInterface.folderInfoPanel.set(item, info);

		if (item.isMessageFolder())
		{
			mainInterface.headerTableViewer.setFolder(folder);
			GuiOperation op = new GuiOperation(Operation.HEADERLIST, 20, folder);

			mainInterface.crossbar.operate(op);
		}
		else
		{
			mainInterface.headerTableViewer.setHeaderList(null);
		}
	}

	public void expandImapRootFolder()
	{
		Folder folder = getSelected();
		if (folder == null)
			return;

		FolderItem item = folder.getFolderItem();

		if (item.getType().equals("imaproot"))
		{
			ImapRootFolder root = (ImapRootFolder) folder;
			FolderOperation op = new FolderOperation(Operation.FOLDER_LIST, 20, root);
			mainInterface.crossbar.operate(op);
		}
	}

	public void treeNodeChanged(TreeNodeEvent e)
	{
		final TreeNodeEvent event = e;

		Runnable run = new Runnable()
		{
			public void run()
			{
				getFolderTree().treeNodeChanged(event);

				if (mainInterface.folderInfoPanel != null)
					mainInterface.folderInfoPanel.update();
			}
		};
		try
		{
			if (!SwingUtilities.isEventDispatchThread())
				SwingUtilities.invokeAndWait(run);
			else
				SwingUtilities.invokeLater(run);

		}
		catch (Exception ex)
		{
		}

	}

}