package org.columba.gui.tree.action;


import org.columba.gui.tree.*;
import org.columba.main.*;
import org.columba.gui.action.*;
import org.columba.gui.tree.util.*;
import org.columba.folder.*;
import org.columba.folder.imap.*;
import org.columba.folder.columba.*;
import org.columba.folder.outbox.*;
import org.columba.gui.config.search.*;
import org.columba.config.*;
import org.columba.gui.header.util.*;
import org.columba.gui.config.subscribe.*;
import org.columba.gui.util.*;
import org.columba.filter.*;

import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;


public class FolderTreeActionListener implements ActionListener
{
    public BasicAction addAction;
    public BasicAction addVirtualAction;
    public BasicAction removeAction;
    public BasicAction renameAction;
    public BasicAction expungeAction;
    public BasicAction emptyAction;
    public BasicAction compactAction;
    public BasicAction applyFilterAction;
    public BasicAction filterPreferencesAction;
    public BasicAction moveupAction;
    public BasicAction movedownAction;
    public BasicAction subscribeAction;
    public BasicAction unsubscribeAction;

    private MainInterface mainInterface;
    private TreeViewer treeViewer;

    public FolderTreeActionListener( MainInterface mainInterface, TreeViewer treeViewer )
        {
            this.mainInterface = mainInterface;
            this.treeViewer = treeViewer;



            initActions();

        }




    public void actionPerformed(ActionEvent e)
    {
	String command = e.getActionCommand();


        if ( command.equals( addAction.getActionCommand() ) )
	    {

                EditFolderDialog dialog = treeViewer.getEditFolderDialog( "New Folder" );
		dialog.showDialog();

		String name;

		if ( dialog.success() == true )
		    {
			// ok pressed
			name = dialog.getName();
		    }
		else
		    {
			// cancel pressed
			return;
		    }

                Folder folder = treeViewer.getSelected();
		//System.out.println( name );

                FolderOperation op = new FolderOperation( Operation.FOLDER_ADD, 0, folder, name, null );
                mainInterface.crossbar.operate( op );



	    }
        else if ( command.equals( addVirtualAction.getActionCommand() ) )
	    {

                EditFolderDialog dialog = treeViewer.getEditFolderDialog( "New Folder" );

		dialog.showDialog();

		String name;

		if ( dialog.success() == true )
		    {
			// ok pressed
			name = dialog.getName();
		    }
		else
		    {
			// cancel pressed
			return;
		    }
                Folder vFolder = treeViewer.getFolderTree().addVirtualFolder( name );
                changeActions();

                AdapterNode node = vFolder.getNode();

                SearchFrame searchDialog = new SearchFrame(
                    mainInterface, vFolder);




	    }
	else if ( command.equals( renameAction.getActionCommand() ) )
	    {


                Folder folder = treeViewer.getSelected();

                EditFolderDialog dialog = treeViewer.getEditFolderDialog( folder.getName() );

		dialog.showDialog();

		String name;

		if ( dialog.success() == true )
		    {
			// ok pressed
			name = dialog.getName();


		    }
		else
		    {
			// cancel pressed
			return;
		    }

		if ( folder.getFolderItem().getType().equals("virtual") )
		    {
			Folder parent = (Folder) folder.getParent();
			folder.setName(name);
			TreeNodeEvent updateEvent = new TreeNodeEvent( 
					parent, 
					TreeNodeEvent.STRUCTURE_CHANGED );
			mainInterface.crossbar.fireTreeNodeChanged( updateEvent );

			changeActions();
			return;
		    }

                     FolderOperation op = new FolderOperation( Operation.FOLDER_RENAME, 0, folder, name, null );
                     mainInterface.crossbar.operate( op );



            }
        else if ( command.equals( applyFilterAction.getActionCommand() ) )
	    {

                Folder folder = treeViewer.getSelected();
                Object[] uids = folder.getUids();

                FolderOperation op = new FolderOperation( Operation.FOLDER_APPLYFILTER, 10, uids, folder);
                mainInterface.crossbar.operate( op );
                /*
                try
                {
                    folder.applyFilter( uids );
                }
                catch ( Exception ex )
                {
                    ex.printStackTrace();
                }
                */


                changeActions();
            }
	else if ( command.equals( removeAction.getActionCommand() ) )
	    {
                Folder folder = treeViewer.getSelected();

                FolderItem item = folder.getFolderItem();
                String access = item.getAccessRights();

                if ( access.equals("user") )
                {
		    if ( item.getType().equals("virtual") )
		    {
			Folder parent = (Folder) folder.getParent();
			folder.removeFromParent();
			TreeNodeEvent updateEvent = new TreeNodeEvent( 
					parent, 
					TreeNodeEvent.STRUCTURE_CHANGED );
			mainInterface.crossbar.fireTreeNodeChanged( updateEvent );

			changeActions();
			return;
		    }

                    if ( folder.isLeaf() )
                       {

                            FolderOperation op = new FolderOperation( Operation.FOLDER_REMOVE, 0, folder );
                            mainInterface.crossbar.operate( op );


                       }
                    else
                      {
                          // warn user
			JOptionPane.showMessageDialog( mainInterface.mainFrame,
						       "Your can only remove leaf folders !");
			return;
                      }


                }

                changeActions();

	    }
        else if ( command.equals("FILTER_PREFERENCES") )
        {

            Folder folder = treeViewer.getSelected();
	    if ( folder  == null ) return;


            FolderItem item = folder.getFolderItem();
            if ( item == null ) return;

            if ( (item.getType().equals("columba")) || (item.getType().equals("imap")) )
            {
                FilterList filterList = folder.getFilterList();

                org.columba.gui.config.filter.ConfigFrame dialog =
                            new org.columba.gui.config.filter.ConfigFrame(
                                filterList, mainInterface );

            }
            else if ( item.getType().equals("virtual") )
            {
                //AdapterNode searchNode = folder.getNode();

                SearchFrame dialog = new SearchFrame( mainInterface,
                                                      folder );


            }

        }
        else if ( command.equals( expungeAction.getActionCommand() ) )
	    {
                JOptionPane.showMessageDialog( mainInterface.mainFrame,
                "Expunge Folder Action\nnot yet implemented");

                /*
                Folder folder = treeViewer.getSelected();

                try
                {
                    folder.expunge();
                    mainInterface.headerTableViewer.setFolder( folder );
                }
                catch ( Exception ex )
	        {
                    JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                 "Error while trying to expunge folder:  "+ex.getMessage() );
	        }


                changeActions();
                */

        }
        else if ( command.equals( emptyAction.getActionCommand() ) )
	    {
                System.out.println("not yet implemented");
            }

        else if ( command.equals( moveupAction.getActionCommand() ) )
        {
            System.out.println("move up");
            Folder folder = treeViewer.getSelected();

            folder.moveUp();

            TreeNodeEvent updateEvent2 = new TreeNodeEvent( folder.getParent(), TreeNodeEvent.STRUCTURE_CHANGED );
            mainInterface.crossbar.fireTreeNodeChanged(updateEvent2);

            //changeActions();

        }
        else if ( command.equals( movedownAction.getActionCommand() ) )
	{
            System.out.println("move down");

            Folder folder = treeViewer.getSelected();

            folder.moveDown();

            TreeNodeEvent updateEvent2 = new TreeNodeEvent( folder.getParent(), TreeNodeEvent.STRUCTURE_CHANGED );
            mainInterface.crossbar.fireTreeNodeChanged(updateEvent2);
            //changeActions();

        }
        else if ( command.equals( subscribeAction.getActionCommand() ) )
	{
            System.out.println("subscsribe");

            Folder folder = treeViewer.getSelected();
            FolderItem item = folder.getFolderItem();
            ImapRootFolder imapRootFolder=null;
            if ( ( item.getType().equals("imap") ) || (item.getType().equals("imaproot")) )
            {
                SubscribeDialog dialog = new SubscribeDialog( mainInterface );
                //dialog.pack();
                //dialog.setSize( dialog.getPreferredSize() );

                if ( item.getType().equals("imap") )
                {

                    imapRootFolder = ( (ImapFolder) folder).getImapRootFolder();
                    dialog.setFolder( imapRootFolder );
                }
                else
                    dialog.setFolder( folder );

                dialog.showDialog();
            }


        }

    }


    public void initActions()
    {
       

	addAction = new BasicAction(
						ResourceLoader.getString("action/mail","menu_folder_newfolder"),
					  	ResourceLoader.getString("action/mail","menu_folder_newfolder"),
					  	ResourceLoader.getString("action/mail","menu_folder_newfolder"),
					   
					   "CREATE_SUBFOLDER",
                                           ImageLoader.getImageIcon("newfolder_small", "folder1"),
                                           ImageLoader.getImageIcon("newfolder", "folder1"),
					   'N',
					   KeyStroke.getKeyStroke( KeyEvent.VK_N, ActionEvent.ALT_MASK )
					   );
	addAction.addActionListener( this );
	addAction.setEnabled( false );

        addVirtualAction = new BasicAction(
        		ResourceLoader.getString("action/mail","menu_folder_newvirtualfolder"),
        		ResourceLoader.getString("action/mail","menu_folder_newvirtualfolder"),
        		ResourceLoader.getString("action/mail","menu_folder_newvirtualfolder"),
					  	
					   "CREATE_VIRTUAL_SUBFOLDER",
					   ImageLoader.getImageIcon("newvirtualfolder_small", "Zoom16"),
                                           ImageLoader.getImageIcon("newvirtualfolder", "Zoom24"),
					   '0',
					   KeyStroke.getKeyStroke( KeyEvent.VK_V, ActionEvent.ALT_MASK )
					   );
	addVirtualAction.addActionListener( this );
	addVirtualAction.setEnabled( false );

	renameAction = new BasicAction(
						ResourceLoader.getString("action/mail","menu_folder_renamefolder"),
						ResourceLoader.getString("action/mail","menu_folder_renamefolder"),
						ResourceLoader.getString("action/mail","menu_folder_renamefolder"),
					   
					   "RENAME_FOLDER",
					   null,
					   null,
					   'R',
					   null
					   );
	renameAction.addActionListener( this );
	renameAction.setEnabled( false );

	removeAction = new BasicAction(
						ResourceLoader.getString("action/mail","menu_folder_removefolder"),
						ResourceLoader.getString("action/mail","menu_folder_removefolder"),
						ResourceLoader.getString("action/mail","menu_folder_removefolder"),
					  
					   "REMOVE_FOLDER",
					   ImageLoader.getImageIcon("deletefolder_small", "Delete16"),
                                           ImageLoader.getImageIcon("deletefolder", "Delete24"),
					   'D',
					   KeyStroke.getKeyStroke( KeyEvent.VK_D, ActionEvent.ALT_MASK )
					   );
	removeAction.addActionListener( this );
	removeAction.setEnabled( false );

	expungeAction = new BasicAction(
		ResourceLoader.getString("action/mail","menu_folder_expungefolder"),
		ResourceLoader.getString("action/mail","menu_folder_expungefolder"),
		ResourceLoader.getString("action/mail","menu_folder_expungefolder"),
					  
					   "EXPUNGE_FOLDER",
                                           null,null,
					   'P',
					   null
					   );
	expungeAction.addActionListener( this );
	expungeAction.setEnabled( false );

	emptyAction = new BasicAction(
		ResourceLoader.getString("action/mail","menu_folder_emtpyfolder"),
		ResourceLoader.getString("action/mail","menu_folder_emtpyfolder"),
		ResourceLoader.getString("action/mail","menu_folder_emtpyfolder"),
					   
					   "EMPTY_FOLDER",
					   null,
					   null,
					   'E',
					   null
					   );
	emptyAction.addActionListener( this );
	emptyAction.setEnabled( false );

	compactAction = new BasicAction(
					ResourceLoader.getString("action/mail","menu_folder_compactfolder"),
					ResourceLoader.getString("action/mail","menu_folder_compactfolder"),
					ResourceLoader.getString("action/mail","menu_folder_compactfolder"),
					   
					   "COMPACT_FOLDER",
					   null,
					   null,
					   'F',
					   null
					   );
	compactAction.addActionListener( this );
	compactAction.setEnabled( false );

        applyFilterAction = new BasicAction(
        		ResourceLoader.getString("action/mail","menu_folder_applyfilter"),
        		ResourceLoader.getString("action/mail","menu_folder_applyfilter"),
        		ResourceLoader.getString("action/mail","menu_folder_applyfilter"),
					   
					   "APPLYFILTER",
					   null,
					   null,
					   'F',
					   null
					   );
	applyFilterAction.addActionListener( this );
	applyFilterAction.setEnabled( false );

        filterPreferencesAction = new BasicAction(
        		ResourceLoader.getString("action/mail","menu_folder_filterconfig"),
        		ResourceLoader.getString("action/mail","menu_folder_filterconfig"),
        		ResourceLoader.getString("action/mail","menu_folder_filterconfig"),
					  
					   "FILTER_PREFERENCES",
					   null,
					   null,
					   'U',
					   null
					   );
	filterPreferencesAction.addActionListener( this );
        filterPreferencesAction.setEnabled( false );

        moveupAction = new BasicAction(
        	ResourceLoader.getString("action/mail","menu_folder_moveup"),
        	ResourceLoader.getString("action/mail","menu_folder_moveup"),
        	ResourceLoader.getString("action/mail","menu_folder_moveup"),
					   
					   "MOVEUP",
					   null,
					   null,
					   '0',
					   null
					   );
	moveupAction.addActionListener( this );
        moveupAction.setEnabled( false );

        movedownAction = new BasicAction(
        	ResourceLoader.getString("action/mail","menu_folder_movedown"),
        	ResourceLoader.getString("action/mail","menu_folder_movedown"),
        	ResourceLoader.getString("action/mail","menu_folder_movedown"),
					  
					   "movedown",
					   null,
					   null,
					   '0',
					   null
					   );
	movedownAction.addActionListener( this );
        movedownAction.setEnabled( false );

        subscribeAction = new BasicAction(
        	ResourceLoader.getString("action/mail","menu_folder_subscribe"),
        	ResourceLoader.getString("action/mail","menu_folder_subscribe"),
        	ResourceLoader.getString("action/mail","menu_folder_subscribe"),
					  
					   "Subscribe...",
					   null,
					   null,
					   '0',
					   null
					   );

	subscribeAction.addActionListener( this );
        subscribeAction.setEnabled( false );

    }

    public void changeActions()
    {
        Folder folder = (Folder) treeViewer.getSelected();
        if ( folder == null ) return;
        FolderItem item = folder.getFolderItem();

        addAction.setEnabled(true);
        addVirtualAction.setEnabled(true);

        if ( item != null )
        {
            if ( item.isSubfolderAllowed() )
                addAction.setEnabled( true );
            else
                addAction.setEnabled( false );



            if ( ( item.getAccessRights().equals("user") ) &&
                 ( folder.isLeaf() ) )
                removeAction.setEnabled(true);
            else
                removeAction.setEnabled(false);

            if ( ( item.getAccessRights().equals("user") ) )
            {
               renameAction.setEnabled( true );

               Folder parentFolder = (Folder) folder.getParent();
               int index = parentFolder.getIndex( folder );
               int count = parentFolder.getChildCount();

               if ( index >= 1 )
                    moveupAction.setEnabled( true );
               else
                    moveupAction.setEnabled( false );


              if ( index < count-1 )
                   movedownAction.setEnabled( true );
              else
                  movedownAction.setEnabled( false );

            }
            else
            {
                renameAction.setEnabled( false );
                movedownAction.setEnabled( false );
                moveupAction.setEnabled( false );
            }

            if ( item.isMessageFolder() )
            {

                if ( folder.hasMessages() && folder.hasFilters() )
                    applyFilterAction.setEnabled(true);
                else
                    applyFilterAction.setEnabled(false);

                filterPreferencesAction.setEnabled( true );

                addVirtualAction.setEnabled( true );
            }
            else
            {
                applyFilterAction.setEnabled(false);
                filterPreferencesAction.setEnabled( false );
                addVirtualAction.setEnabled( false );
            }


            if ( (item.getType().equals("imap")) || (item.getType().equals("imaproot")) )
            {
                if ( folder.hasMessages() )
                {
                    expungeAction.setEnabled( true );
                }
                else
                  expungeAction.setEnabled( false );

                subscribeAction.setEnabled( true );
                
                filterPreferencesAction.setEnabled( true );
            }
            else
            {
                expungeAction.setEnabled( false );
                subscribeAction.setEnabled( false );
            }

            if ( item.getType().equals("virtual") )
            {
                applyFilterAction.setEnabled(false);
//                removeAction.setEnabled(true);
                addVirtualAction.setEnabled(false);
                filterPreferencesAction.setEnabled( true );
            }
        }
        else
        {
            System.out.println("item is null");
        }

    }

}

