package org.columba.gui.tree.action;


import org.columba.gui.tree.*;
import org.columba.config.*;
import org.columba.main.*;
import org.columba.util.*;
import org.columba.folder.*;
import org.columba.gui.header.util.*;
import org.columba.gui.util.*;
import org.columba.gui.header.treetable.JTreeTable;
import java.awt.dnd.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.awt.*;
import java.io.*;

import javax.swing.tree.*;
import javax.swing.*;


public class FolderTreeDnd implements DropTargetListener, DragGestureListener, DragSourceListener
{
    private MainInterface mainInterface;
    private TreeViewer treeViewer;

    private Folder sourceFolder;

    private DragSource dragSource;

    public FolderTreeDnd( MainInterface mainInterface, TreeViewer treeViewer )
        {
            this.mainInterface = mainInterface;
            this.treeViewer = treeViewer;


            new DropTarget( treeViewer.getFolderTree().getTree(), // component
                           DnDConstants.ACTION_COPY_OR_MOVE, // actions
                           this); // DropTargetListener



            dragSource = DragSource.getDefaultDragSource();

            dragSource.createDefaultDragGestureRecognizer(
                treeViewer.getFolderTree().getTree(), // component where drag originates
                DnDConstants.ACTION_MOVE, // actions
                this); // drag gesture listener
        }




      /****************************************** drop ***********************************/



    public void drop(DropTargetDropEvent event)
    {


        final DropTargetDropEvent e = event;

        try
        {
            DataFlavor stringFlavor = DataFlavor.stringFlavor;
            Transferable tr = e.getTransferable();
            if(e.isDataFlavorSupported(stringFlavor))
            {

                String filename =
                    (String)tr.getTransferData(stringFlavor);
                e.acceptDrop(DnDConstants.ACTION_COPY_OR_MOVE);

                System.out.println("string: "+ filename );

                Point p = e.getLocation();

                final TreePath path = treeViewer.getFolderTree().getTree().getPathForLocation(p.x, p.y);

		// this produces a invalidDndOperationException!
		/*
                if ( path == null )
                    e.rejectDrop();
		*/
		// we have to use this dropComplete here:
		if ( path == null ) 
		    {
			e.dropComplete(false);
			return;
		    }

                else
                {

                    if ( filename.equals("folder") )
                    {
                        // folder dnd operation


                        Folder selectedFolder = treeViewer.getSelected();
                        //System.out.println( "folder name: "+ selectedFolder.getName() );


                        if ( selectedFolder.equals( sourceFolder ) )
                        {
                            //System.out.println( "source equals destination folder -> ABORTING" );
                            e.rejectDrop();
                            return;
                        }

                        if ( sourceFolder.isParent( selectedFolder ) )
                        {
                            //System.out.println( "source is parent folder -> ABORTING" );
                            e.rejectDrop();
                            return;
                        }

                        selectedFolder.append( sourceFolder );
			TreeNodeEvent updateEvent = new TreeNodeEvent( selectedFolder.getParent(), TreeNodeEvent.STRUCTURE_CHANGED );
			mainInterface.crossbar.fireTreeNodeChanged( updateEvent );
                        e.dropComplete(true);
                    }
                    else
                    {
                        // message dnd operation


                        Folder selectedFolder = null;
                        Folder srcFolder = null;

                        treeViewer.getFolderTree().getTree().setSelectionPath( path );

                        selectedFolder = treeViewer.getSelected();

                        if ( selectedFolder != null )
                        {

                            sourceFolder = mainInterface.headerTableViewer.getFolder();

                            if ( selectedFolder == null )
                            {
                                e.rejectDrop();
                                return;
                            }


                            if ( ( selectedFolder != sourceFolder ) &&
                                 ( selectedFolder.getFolderItem().isAddAllowed() ) )
                            {


                                Object[] uids = mainInterface.headerTableViewer.getUids();
				
				
                                if ( e.getDropAction() == 1 )
                                {
                                    System.out.println("copy action");

				    treeViewer.setSelected( sourceFolder );
				    
				    FolderOperation op = new FolderOperation(Operation.COPY,
									     0,
									     uids,
									     sourceFolder,
									     selectedFolder
									     );
				    mainInterface.crossbar.operate(op);

       

                                    e.dropComplete(true);
       
                                }
                                else if ( e.getDropAction() == 2 )
                                {
                                    System.out.println("move action");

				    treeViewer.setSelected( sourceFolder );

				    FolderOperation op = new FolderOperation(Operation.MOVE,
									     0,
									     uids,
									     sourceFolder,
									     selectedFolder
									     );
				    mainInterface.crossbar.operate(op);

                                    e.dropComplete(true);

                                } else
                                {
                                    System.out.println("drop action not supported");
                                    e.rejectDrop();
                                }


                            }
                            else
                            {
                                 JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                                                "Invalid Folder!");
                                e.rejectDrop();
                            }




                        }
			else
			    {
				e.rejectDrop();
			    }


                    }		    

                }

            }
            else
            {
                e.rejectDrop();
            }
        }
        catch(IOException ioe) {
            ioe.printStackTrace();
            e.rejectDrop();
        }
        catch(UnsupportedFlavorException ufe) {
            ufe.printStackTrace();
            e.rejectDrop();
        }


    }

    public void dragEnter(DropTargetDragEvent e)
    {

    }

    public void dragExit(DropTargetEvent e)
    {
      System.out.println( "dragExit lllllll" );
    }

    public void dragOver(DropTargetDragEvent e)
    {

        Point p = e.getLocation();

        TreePath path = treeViewer.getFolderTree().getTree().getPathForLocation(p.x, p.y);

        if ( path == null )
        {
            System.out.println("no treepath");

              //e.rejectDrag();
        }
        else
        {
            treeViewer.getFolderTree().getTree().setSelectionPath( path );


              /*
            FolderTreeNode node = treeViewer.getSelectedNode();
            if ( node == null ) return;

            FolderItem item = node.getFolderItem();

            System.out.println("itemname: "+ item.getName() );

            if ( !( item.isAddAllowed() == true ) ) e.rejectDrag();


                  //Folder selectedFolder = node.getFolder();

                  //if (! node.getFolderItem().isMessageFolder()) e.rejectDrag();
              */
        }

        if ( e.getDropAction() == 1 )
        {
                //copy
                System.out.println( "copy" );

        }
        else if ( e.getDropAction() == 2 )
        {
                //movee

                System.out.println( "move" );
        } else
        {

        }
    }

    public void dropActionChanged(DropTargetDragEvent e) { }






      /********************************** drag ********************************************/




    public void dragGestureRecognized(DragGestureEvent e)
    {
        InputEvent event =  e.getTriggerEvent();
        int mod = event.getModifiers();

        if ( (mod & InputEvent.BUTTON2_MASK) == InputEvent.BUTTON2_MASK )
        {
            System.out.println("drag n drop action initiated");

            sourceFolder = treeViewer.getSelected();
            FolderItem item = sourceFolder.getFolderItem();
            String access = item.getAccessRights();
            String type = item.getType();

            if ( ( access.equals( "user" ) ) && ( !type.equals( "imap" )) && ( !type.equals( "imaproot" )))
            {
              e.startDrag(  DragSource.DefaultMoveDrop,
                                      //ImageLoader.getImageIcon("","folder1").getImage(),
                                      //new Point(10,10), // cursor
                         new StringSelection("folder"), // transferable
                         this); // drag source listener
            }

              /*

            if ( ( mod & InputEvent.CTRL_MASK) == InputEvent.CTRL_MASK )
            {

            }
              */

        }

    }


    public void dragDropEnd(DragSourceDropEvent e)
    {
              //System.out.println("dragdropend");
    }

    public void dragExit(DragSourceEvent e)
    {
            System.out.println("dragexit");
    }


    public void dragEnter(DragSourceDragEvent e)
    {
          //System.out.println("start enter");
    }

    public void dragOver(DragSourceDragEvent e)
    {
          //System.out.println("dragover");
    }
    public void dropActionChanged(DragSourceDragEvent e)
    {
          //System.out.println("dragactioncahnged");
    }


}



