/* -*-mode: java; compile-command:"javac FolderTreeViewer.java -classpath ../"; -*- */
package org.columba.gui.tree.util;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

import java.awt.dnd.*;
import java.awt.datatransfer.*;

import java.io.*;
import javax.swing.JTree;


import org.columba.main.*;
import org.columba.config.*;
import org.columba.folder.*;
import org.columba.gui.util.*;




public class FolderInfoPanel extends JPanel
{
    private JLabel label;
    private int exists;
    private int unseen;
    private int recent;

    private ImageIcon image1;
    private ImageIcon image2;
    private ImageIcon image3;
    private ImageIcon image4;
    private ImageIcon image5;
    private ImageIcon image6;

    private MessageFolderInfo info;
    private FolderItem item;


    private MainInterface mainInterface;

    public FolderInfoPanel( MainInterface mainInterface )
    {
        this.mainInterface = mainInterface;

        image1 = ImageLoader.getImageIcon("folder", "Open24");
        image2 = ImageLoader.getImageIcon("localhost", "Host24" );
        image3 = ImageLoader.getImageIcon("remotehost", "WebComponent24");
        image4 = ImageLoader.getImageIcon("virtualfolder", "Zoom24");

        initComponents();
    }

    public void initComponents()
    {
        removeAll();

        GridBagLayout gridbagLayout = new GridBagLayout();
        setLayout( gridbagLayout );
        GridBagConstraints gridbagConstraints = new GridBagConstraints();;

        setLayout( gridbagLayout );

        setBackground( UIManager.getColor("controlShadow") );



        gridbagConstraints.gridwidth = GridBagConstraints.RELATIVE;
        gridbagConstraints.gridx = 0;
        gridbagConstraints.weightx = 0.5;
        Box box = Box.createHorizontalBox();
        gridbagLayout.setConstraints( box, gridbagConstraints );
        add( box );

	
	
        Font font = UIManager.getFont("Label.font");
	//Font font = mainInterface.columbaTheme.getControlTextFont();
	

        font = font.deriveFont( font.BOLD );
        int size = font.getSize();
        font = font.deriveFont( font.BOLD, size+2 );
	

        label = new JLabel("Total: Unseen:");
        label.setForeground( Color.white );
        label.setBorder( BorderFactory.createEmptyBorder( 2,2,2,2 ) );
        label.setFont( font );
        label.setIconTextGap( 10 );
        label.setIcon(image1);
        label.setText("Folder");

        gridbagConstraints.gridwidth = GridBagConstraints.REMAINDER;
        gridbagConstraints.gridx = 1;
        gridbagConstraints.insets = new Insets( 0, 0, 0, 20 );
        gridbagConstraints.anchor = GridBagConstraints.EAST;
        gridbagLayout.setConstraints( label, gridbagConstraints );

        add( label );

        //setBorder( UIManager.getBorder("FolderInfoPanel.border") );
    }


    public void updateUI()
    {
        super.updateUI();
	setBackground( UIManager.getColor("controlShadow") );
	//setBorder( UIManager.getBorder("ToolBar.border") );
 
    }

    public void resetRenderer()
    {
        initComponents();
    }


    public void update()
    {
        if ( item == null ) return;

        int uid = item.getUid();

        int total = info.getExists();
        int unread = info.getUnseen();
        int recent = info.getRecent();

        String name;


        name = item.getName();

        String type = item.getType();

        boolean noInfo = false;

        if ( type.equals("virtual") )
            {
                label.setIcon( image4 );
            }
            else if ( ( type.equals("imaproot") ) && ( item.isMessageFolder() == false ) )
            {
                label.setIcon( image3 );
                noInfo=true;
            }
            else if ( item.getAccessRights().equals("system") )
            {
                if ( name.equals( ResourceLoader.getString("tree/mail","localfolders") ) )
                {
                    noInfo=true;
                    label.setIcon( image2 );
                }

                else
                    label.setIcon( image1 );

            }
            else
            {
                label.setIcon( image1 );

            }


        if ( noInfo == true )
            label.setText( name );
        else
        {
            if ( recent>0 )
            {
                label.setText( name+" "+total+" / "+unread+" ("+recent+")" );
            }
            else
            {
                label.setText( name+" "+total+" / "+unread );
            }

        }

    }



    public void set( FolderItem item, MessageFolderInfo info )
    {
        this.item = item;
        this.info = info;

        update();

    }




}











