// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.gui.util;

import java.util.zip.*;
import java.io.*;
import java.awt.*;
import java.net.*;
import javax.swing.*;

import java.util.*;

import org.columba.config.*;
import org.columba.main.*;


public class ImageLoader
{
    static boolean ICON_SET = false;
    
    private static Locale currentLocale;
    private static ResourceBundle bundle;
    private static Properties properties;

    private static MainInterface mainInterface;
    private static String iconset;

    public ImageLoader( MainInterface mainInterface )
    {
	this.currentLocale = Locale.getDefault();
	this.mainInterface = mainInterface;

	iconset = mainInterface.themeItem.getIconset();

	if ( iconset.toLowerCase().equals("default") ) ICON_SET = false;
	else ICON_SET = true;

	if ( ICON_SET == true )
        {
	    File zipFile = new File( mainInterface.config.configDirectory
				     +"/iconsets/"+iconset+".jar" );
	    //System.out.println("zipfile:"+zipFile );

	    String zipFileEntry = new String(iconset+"/icons.properties");
	    
	    properties = loadProperties( zipFile, zipFileEntry );
	}


    }
	
    // load an image from package org.columba.images 
    // works even if the application is bundled as jar-file
    protected static synchronized Image loadImage( URL url )
    {
        if( url == null) return null;
        
        java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
        String protocol = url.getProtocol();

        if( protocol.equals("file") ) {
            return tk.createImage( url.getPath() );
        }
        if( protocol.equals("jar") ) 
	{	    
            String path = url.getPath();
            String jarpath = path.substring(
                path.indexOf("file:")+5,
                path.indexOf("!"));

            String jarEntry = path.substring(
                path.indexOf("!")+2 );

            Image image = null;
            
	    byte bytes[] = loadBytes( new File(path), jarpath );

	    image = tk.createImage(bytes);

            return image;
        }

        return null;
    }

    protected static synchronized Image load( URL url )
    {
        if( url == null) return null;
        
        java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
        String protocol = url.getProtocol();

        if( protocol.equals("file") ) {
            return tk.createImage( url.getPath() );
        }
        if( protocol.equals("jar") ) 
	{	    
            String path = url.getPath();
            String jarpath = path.substring(
                path.indexOf("file:")+5,
                path.indexOf("!"));

            String jarEntry = path.substring(
                path.indexOf("!")+2 );

            Image image = null;
            

            try
            {
                ZipFile zipfile = new ZipFile( jarpath );
                ZipEntry zipentry = zipfile.getEntry( jarEntry );
   
                if( zipentry != null )
                {
                    long size = zipentry.getSize();
                    if( size > 0 )
                    {
                        byte[] bytes = new byte[ (int) size ];
                        InputStream in = new BufferedInputStream(
                            zipfile.getInputStream( zipentry ) );
                        in.read( bytes );
                        in.close();   
			image = tk.createImage( bytes );
                    }
                }
            }
            catch( ZipException e )
            {
                System.out.println(e);
                return null;
            }
            catch( IOException e )
            {
                System.out.println(e);
                return null;
            }


            return image;
        }
	
	return null;
    }

    


    // load image out of jar/zip file
    public static synchronized Image loadImage( File zipFile, String entry )
    {
	java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
	byte[] bytes = loadBytes( zipFile, entry );

        Image image = tk.createImage(bytes);

	return image;
    }

    // load image out of jar/zip file
    public static synchronized Properties loadProperties( File zipFile, String entry )
    {
	java.awt.Toolkit tk = java.awt.Toolkit.getDefaultToolkit();
	byte[] bytes = loadBytes( zipFile, entry );

	ByteArrayInputStream input = new ByteArrayInputStream( bytes );

	Properties properties = new Properties();
	try
	    {
		properties.load( input );
	    }
	catch ( Exception ex )
	    {
		ex.printStackTrace();
	    }

	return properties;
    }


    // load byte-array out of jar/zip file
    protected static synchronized byte[] loadBytes( File zipFile, String entry )
    {	
	byte[] bytes = null;

            try
            {
                ZipFile zipfile = new ZipFile( zipFile );
                ZipEntry zipentry = zipfile.getEntry( entry );
   
                if( zipentry != null )
                {
                    long size = zipentry.getSize();
                    if( size > 0 )
                    {
                        bytes = new byte[ (int) size ];
                        InputStream in = new BufferedInputStream(
                            zipfile.getInputStream( zipentry ) );
                        in.read( bytes );
                        in.close();                 
                    }
                }
            }
            catch( ZipException e )
            {
                System.out.println(e);
                return null;
            }
            catch( IOException e )
            {
                System.out.println(e);
                return null;
            }

	return bytes;      
    }




    public synchronized static ImageIcon getImageIcon( String id, String failCase )
    {
	if ( ICON_SET == true )
	    {
		String str = (String) properties.getProperty(id);
		if ( str == null ) return getDefaultImageIcon( id, failCase );

		//System.out.println("str="+str);

		ImageIcon icon = null;
		try
		    {			
			File zipFile = new File( mainInterface.config.configDirectory
						 +"/iconsets/"+iconset+".jar" );
			//System.out.println("zipfile:"+zipFile );
			
			String zipFileEntry = iconset+"/"+str;

			icon = new ImageIcon( loadImage( zipFile, zipFileEntry ) );
		    }
		catch ( Exception ex )
		    {
			//ex.printStackTrace();
			return getDefaultImageIcon( id, failCase );
		    }

		if ( icon == null ) return getDefaultImageIcon( id, failCase );

		return icon;
	    }
	else
	    {
		return getDefaultImageIcon( id, failCase );
	    }
    }


    public synchronized static ImageIcon getDefaultImageIcon( String id, String failCase )
    {
        ImageIcon icon = (ImageIcon) UIManager.getIcon( id );

        if ( icon == null )
        {
            java.net.URL url;

	    
            url = ClassLoader.getSystemResource("org/columba/images/"+failCase+".gif");
	    if ( url == null ) 
	    {
		url = ClassLoader.getSystemResource("org/columba/images/"+failCase+".jpeg");
		if ( url == null )
	        {
		    url = ClassLoader.getSystemResource("org/columba/images/"+failCase+".png");
		    if ( url == null )
		    {
			if ( id.indexOf("small") != -1 )
			{
			    url = ClassLoader.getSystemResource("org/columba/images/brokenimage_small.png");
			}
			else
			{
			    url = ClassLoader.getSystemResource("org/columba/images/brokenimage.png");
		    
			}
		    }
		}
	    }

	    if ( url == null ) return null;

	    Image image = load(url);

	    if ( image == null ) return null;

            icon = new ImageIcon( image );
            
        }
        
        return icon;
    }

    // this is deprecated -- don't use this
    public synchronized static ImageIcon getImageIcon( String path )
    {
        ImageIcon icon;

        java.net.URL url;
        
        url = ClassLoader.getSystemResource( path );
  
	if ( url == null ) return null;

       
        icon = new ImageIcon( load( url ) );
        
        return icon;
    }

    
    
}




