// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.main;

import org.columba.config.*;
import org.columba.message.*;
import org.columba.gui.attachment.*;
import org.columba.gui.header.*;
import org.columba.gui.tree.*;
import org.columba.gui.message.*;
import org.columba.gui.statusbar.*;
import org.columba.gui.mainframe.*;
import org.columba.gui.util.*;
import org.columba.gui.action.*;
import org.columba.gui.config.accountwizard.*;
import org.columba.folder.*;
import org.columba.folder.gui.*;
import org.columba.folder.encryption.*;
import org.columba.pop3.*;
import org.columba.parser.*;
import org.columba.folder.*;
import org.columba.pgp.*;

import org.columba.coder.*;
import org.columba.gui.themes.*;

import javax.swing.*;
import javax.swing.plaf.metal.*;
import javax.swing.event.*;
import java.awt.event.*;
import java.awt.Font;

import java.awt.datatransfer.*;
import java.util.*;

import java.lang.reflect.Array;

import org.columba.util.*;
import org.columba.config.*;
import org.columba.gui.themes.thincolumba.*;
import org.columba.gui.themes.contrastcolumba.*;
import org.columba.folder.composer.*;

public class Main {
	public static void main(String[] arg) {
		final String[] args = arg;
		final MainInterface mainInterface = new MainInterface();

		System.setProperty("file.encoding", "8859_1");

		final StartUpFrame frame = new StartUpFrame();

		final StartUpWorker worker = new StartUpWorker() {

			public Object construct() {

				if (Array.getLength(args) > 1) {
					if (args[0].equals("--path"))
						mainInterface.config = new Config(mainInterface, args[1]);
				} else {
					mainInterface.config = new Config(mainInterface);
				}

				new TempFileStore(mainInterface);

				mainInterface.headerTableItem =
					mainInterface.config.getMainFrameOptionsConfig().getHeaderTableItem();

				mainInterface.mainFrameWindowItem =
					mainInterface.config.getMainFrameOptionsConfig().getWindowItem();

				mainInterface.themeItem =
					mainInterface.config.getOptionsConfig().getThemeItem();

				ThemeSwitcher.setTheme(mainInterface);

				frame.advance();

				new ImageLoader(mainInterface);

				new ResourceLoader();

				mainInterface.currentLocale = Locale.getDefault();

				mainInterface.popServerCollection = new PopServerCollection(mainInterface);

				//frame.setText(" Initialising Parser ... ");
				new MimeRouter(mainInterface);

				//frame.setText(" Initialising Coders... ");
				new CoderRouter();
				new QuotedPrintableDecoder();
				new QuotedPrintableEncoder();
				new Base64Decoder();
				new Base64Encoder();

				mainInterface.pgpController = new PGPController(mainInterface);

				mainInterface.globalActionCollection =
					new GlobalActionCollection(mainInterface);

				mainInterface.statusBar = new StatusBar();

				mainInterface.popServerCollection.enableMailCheckIcon();

				mainInterface.imageSequenceTimer = new ImageSequenceTimer(mainInterface);

				mainInterface.taskManager =
					new TaskManager(mainInterface, mainInterface.statusBar);

				frame.advance();

				mainInterface.headerTableViewer = new HeaderTableViewer(mainInterface);

				frame.advance();

				mainInterface.attachmentViewer = new AttachmentViewer(mainInterface);

				mainInterface.messageViewer = new MessageViewer(mainInterface);

				mainInterface.treeViewer = new TreeViewer(mainInterface);

				mainInterface.globalActionCollection.addActionListeners();

				// NEW METHOD !!

				mainInterface.crossbar = new Crossbar(mainInterface);
				mainInterface.crossbar.guiOperator = new GuiOperator(mainInterface);
				mainInterface.crossbar.guiOperator.start();
				mainInterface.crossbar.encryptionOperator =
					new EncryptionOperator(mainInterface);
				mainInterface.crossbar.encryptionOperator.start();
				mainInterface.crossbar.composerOperator = new ComposerOperator(mainInterface);
				mainInterface.crossbar.composerOperator.start();

				new MainFrame(mainInterface);

				AccountWizard wizard = new AccountWizard(mainInterface);

				return null;
			}

			public void finished() {
				frame.setVisible(false);
			}

		};

		worker.start();

	}
}