// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.parser;

import org.columba.message.*;
import org.columba.parser.*;
import org.columba.coder.*;
import org.columba.folder.*;
import org.columba.gui.config.subscribe.SubscribeTreeNode;

import java.util.*;

public class Imap4Parser
{

	public Imap4Parser()
	{
	}

	// prepare header for parsing with Rfc822Parser-class
	public ColumbaHeader parseHeader(String s)
	{
		Rfc822Parser parser = new Rfc822Parser();

		return parser.parseHeader(s);
	}

	protected String removeClubs(String s)
	{
		String str = s;

		if (s.startsWith("\""))
		{
			str = str.substring(1, str.length() - 1);
		}
		return str;
	}

	public void parseBodyStructure(String s, Message message)
	{
		//System.out.println("------------------------parseBodyStructure");

		int i = 1;
		int count = 0;
		int elementCount = 0;
		Vector parts = new Vector();

		StringBuffer mimePart = new StringBuffer();

		message.getMimePartCollection().clear();

		// System.out.println("bodystructure: \n"+s);

		i = s.indexOf("((") + 1;

		while (i < s.length() - 2)
		{
			if (s.charAt(i) == '(')
			{
				// increase mimepart count
				count++;
				//System.out.println("----------------begin mimepart: "+count);

				mimePart = new StringBuffer();
				elementCount = 0;

				//System.out.println("char: "+s.charAt(i));

				i++;

				// while not end of mimepart reached
				while ((s.charAt(i) != ')') || (elementCount != 0))
				{
					mimePart.append(s.charAt(i));
					//System.out.println("char: "+s.charAt(i));

					if (s.charAt(i) == '(')
					{
						// found beginning of inner element list
						elementCount++;
					}
					else if (s.charAt(i) == ')')
					{
						// found end of inner element list
						elementCount--;
					}
					i++;
				}

				parts.add(mimePart);

				//System.out.println("end mimepart: "+count);
				//System.out.println("string: "+ mimePart);
				//System.out.println("-----------------------------");

				//i++;
			}
			else
			{
				// we nearly reached the end
				// there only the multipart mixed thing left to parse

				elementCount = 0;
				StringBuffer multipartMixed = new StringBuffer();

				//System.out.println("char: "+s.charAt(i));

				while ((s.charAt(i) != ')') || (elementCount != 0))
				{
					multipartMixed.append(s.charAt(i));
					//System.out.println("char: "+s.charAt(i));

					if (s.charAt(i) == '(')
					{
						// found beginning of inner element list
						elementCount++;
					}
					else if (s.charAt(i) == ')')
					{
						// found end of inner element list
						elementCount--;
					}
					i++;
				}

				//System.out.println("end multipartmixed:");
				//System.out.println("string: "+ multipartMixed);
				//System.out.println("--------------------------------------------------");

				i++;
			}

			i++;
		}

		if (parts.size() > 0)
		{
			//System.out.println("found multipart message");

			for (int j = 0; j < parts.size(); j++)
			{
				//System.out.println("part: "+j);

				MimePart mp = new MimePart();
				MimeHeader header = mp.getHeader();

				StringBuffer partString = (StringBuffer) parts.get(j);

				StringBuffer buf;
				int pos = 0;
				int column = 0;

				while (pos < partString.length() - 2)
				{

					if (partString.charAt(pos) == '"')
					{
						column++;
						pos++;

						if (column == 1)
						{
							// content-type

							buf = new StringBuffer();
							while (partString.charAt(pos) != '"')
							{
								buf.append(partString.charAt(pos));
								//System.out.println("char: "+partString.charAt(pos));
								pos++;
							}

							header.contentType = buf.toString();
						}
						else if (column == 2)
						{
							// content-suptype

							buf = new StringBuffer();
							while (partString.charAt(pos) != '"')
							{
								buf.append(partString.charAt(pos));
								//System.out.println("char: "+partString.charAt(pos));
								pos++;
							}

							header.contentSubtype = buf.toString();
						}

					}
					pos++;
				}

				if (!(header.contentType.equalsIgnoreCase("message")))
				{

					pos = partString.toString().indexOf("FILENAME");
					if (pos != -1)
					{
						// found filename, for example: "FILENAME" "test.txt"

						pos = partString.toString().indexOf(" ", pos + 1);
						buf = new StringBuffer();
						if (pos != -1)
						{
							// skip "
							pos += 2;
							while (partString.toString().charAt(pos) != '"')
							{
								buf.append(partString.charAt(pos));
								//System.out.println("char: "+partString.charAt(pos));
								pos++;
							}

							//System.out.println("filename found: "+ buf );

							mp.setFile(new java.io.File(buf.toString()));
						}
					}
				}

				if (partString.toString().indexOf("7bit") != -1)
				{
					mp.getHeader().contentTransferEncoding = new String("7bit");
				}
				else if (partString.toString().indexOf("7BIT") != -1)
				{
					mp.getHeader().contentTransferEncoding = new String("7bit");
				}
				else if (partString.toString().indexOf("quoted-printable") != -1)
				{
					mp.getHeader().contentTransferEncoding = new String("quoted-printable");
				}
				else if (partString.toString().indexOf("QUOTED-PRINTABLE") != -1)
				{
					mp.getHeader().contentTransferEncoding = new String("quoted-printable");
				}

				message.getMimePartCollection().add(mp);
			}
		}
	}

	public void parseBodyStructure2(String s, Message message)
	{
		StringBuffer mimePart = new StringBuffer();
		StringBuffer innerPart = new StringBuffer();
		ParsePosition pos = new ParsePosition();
		StringTokenizer tok;
		String str;
		int itemIndex = 0;
		int count = 0;
		Vector v = new Vector();
		boolean attach;

		message.getMimePartCollection().clear();

		pos.setIndex(s.indexOf("(("));

		// System.out.println("bodystructure: "+s );

		int i = pos.getIndex() + 1;

		while (i < s.length())
		{
			//System.out.println("char: "+ s.charAt(i) );

			// found first mimepart
			if (s.charAt(i) == '(')
			{
				// increase mimepart number
				count++;
				//System.out.println("begin mimepart: "+ count);
				// increase character number
				i++;

				itemIndex = 0;

				mimePart = new StringBuffer();

				MimePart attachment = new MimePart();
				MimeHeader header = attachment.getHeader();
				attach = true;

				while (s.charAt(i) != ')')
				{
					if (s.charAt(i) == '(')
					{
						// inner part is starting here
						i++; // skip (

						// System.out.println("begin inner part: ");
						innerPart = new StringBuffer();
						while (s.charAt(i) != ')')
						{
							// if INLINE skip inner bracet
							if (s.charAt(i) == '(')
								i++;
							else
							{
								innerPart.append(s.charAt(i));
								i++;
							}

						}
						//  System.out.println("end inner part: ");
						// System.out.println("inner part: "+ innerPart.toString());

						StringTokenizer innerToken = new StringTokenizer(innerPart.toString(), " ");
						String lastToken = new String();
						boolean inline = false;

						while (innerToken.hasMoreElements())
						{
							str = (String) innerToken.nextElement();
							str = removeClubs(str);
							// System.out.println(" token: "+ str );

							if (lastToken.length() == 0)
							{
								if ((str.equalsIgnoreCase("inline")) || (str.equalsIgnoreCase("attachment")))
									inline = true;
							}

							if (lastToken.equalsIgnoreCase("charset"))
							{
								// System.out.println("charset found: "+str );
								header.putContentParameter("charset", str);
							}
							else if (lastToken.equalsIgnoreCase("name"))
							{
								//System.out.println("name found: "+str );

								header.putContentParameter("filename", str);
							}
							else if (lastToken.equalsIgnoreCase("filename"))
							{
								// System.out.println("filename found: "+str );
								header.putDispositionParameter("filename", str);
							}

							lastToken = str;

						}

					}
					else
					{
						mimePart.append(s.charAt(i));
					}

					i++;
				}

				// System.out.println("end mimepart: "+ count);

				//System.out.println("mimepart: "+ mimePart.toString() );

				StringTokenizer mimePartToken = new StringTokenizer(mimePart.toString(), " ");
				int j = 0;
				while (mimePartToken.hasMoreElements())
				{
					str = (String) mimePartToken.nextElement();

					// System.out.println("token: "+ str );

					if (removeClubs(str).equalsIgnoreCase("boundary"))
					{
						attach = false;
						break;
					}

					if (j == 0)
					{
						//  System.out.println("contentType found: "+str );
						header.contentType = removeClubs(str);
					}
					else if (j == 1)
					{
						//  System.out.println("contentsubtype found: "+str );
						header.contentSubtype = removeClubs(str);
					}
					else if (j == 4)
					{
						// System.out.println("transferencoding found: "+str );
						header.contentTransferEncoding = removeClubs(str);
					}
					else if (j == 5)
					{
						//  System.out.println("size found: "+str );
						header.size = removeClubs(str);
					}

					j++;

				}

				if (attach == true)
					message.addMimePart(attachment);
				//System.out.println("inline: "+ buffer2.toString() );

				/*
				  // parse the mimepart
				tok = new StringTokenizer( buffer.toString(), " " );
				int j=0;
				MimePart attachment = new MimePart();
				boolean attach = true;
				while ( tok.hasMoreElements() )
				{
				  str = (String) tok.nextElement();
				
				 // System.out.println("tok: "+str);
				  if ( removeClubs( str ).equals("BOUNDARY") )
				  {
				      attach=false;
				      break;
				  }
				
				
				  if ( j==0 )
				  {
				      attachment.getHeader().contentType = removeClubs( str );
				  }
				  else if ( j==1 )
				  {
				      attachment.getHeader().contentSubtype = removeClubs( str );
				  }
				  else if ( j==4 )
				  {
				      attachment.getHeader().contentTransferEncoding = removeClubs( str );
				  }
				  else if ( j==5 )
				  {
				      attachment.getHeader().size = removeClubs( str );
				  }
				
				  if ( !str.equals("") )
				      j++;
				}
				
				if ( attach == true ) message.addMimePart( attachment );
				*/

			}
			i++;
		}

		count--;

		//System.out.println("count: "+count);
	}

	public String parseBodyText(String s)
	{
		return s;
		/*int index = s.indexOf( "}" );
		String result;
		
		
		if ( s.length() == 0 ) return new String("");
		
		result = s.substring( index+1 , s.length()-2 );
		return result;*/
	}

	public static String createMessageSet(Object[] uids)
	{
		String messageSet = new String();
		StringBuffer messageSetList = new StringBuffer();
		Message message = null;
		//Integer lastPos;
		int lastPos = 0;
		int newPos = 0;
		char lastChar = 'p';

		for (int i = 0; i < uids.length; i++)
		{
			Integer uid = new Integer(Integer.parseInt((String) uids[i]));

			if (i == 0)
			{

				lastPos = (uid).intValue();
				messageSetList.append(lastPos);
				//System.out.println("append_0: "+lastPos);

			}
			else if (i == 1)
			{
				newPos = (uid).intValue();
				if (lastPos + 1 == newPos)
				{
					//messageSetList.append("-");
					messageSetList.append(":");
					lastChar = '-';
					lastPos = (uid).intValue();

					System.out.println("i==1");

					//System.out.println("append_1a: "+ message.getUID());
				}
				else
				{
					lastPos = (uid).intValue();
					messageSetList.append("," + lastPos);
					lastChar = 'p';
					//System.out.println("append_1b: "+  message.getUID());
				}
			}
			else if (i == uids.length - 1)
			{
				if (lastChar == '-')
				{
					lastPos = (uid).intValue();
					messageSetList.append(lastPos);
					//System.out.println("append_last_a: "+ message.getUID());
				}
				else
				{
					lastPos = (uid).intValue();
					messageSetList.append("," + lastPos);
					//System.out.println("append_last_b: "+ message.getUID());
				}

			}
			else
			{
				newPos = (uid).intValue();

				if (lastChar == '-')
				{
					if (lastPos + 1 == newPos)
					{
						lastPos = (uid).intValue();
						//System.out.println("append_between_aa: "+ message.getUID());
					}
					else
					{
						messageSetList.append(lastPos);
						lastPos = (uid).intValue();
						messageSetList.append("," + lastPos);
						lastChar = 'p';
						//System.out.println("append_between_ab: "+  message.getUID());
					}
				}
				else
				{
					if (lastPos + 1 == newPos)
					{
						messageSetList.append(":");

						lastChar = '-';
						lastPos = (uid).intValue();

						//System.out.println("append :");
						//System.out.println("append_between_ba: "+  message.getUID());
					}
					else
					{
						lastPos = (uid).intValue();
						messageSetList.append("," + lastPos);

						//System.out.println("append_between_bb: "+  message.getUID());
					}

				}

			}

		}

		// System.out.println("messageSet: "+ messageSetList );
		return messageSetList.toString();
	}

	/*
	public Flags parseAttributs( String s )
	{
	    String str,str2;
	    StringBuffer result;
	
	    StringTokenizer tok = new StringTokenizer( s, " " );
	    StringTokenizer tok2;
	
	    Flags flags = new Flags();
	
	
	
	    while ( tok.hasMoreElements() )
	    {
	        str = (String) tok.nextElement();
	
	        if ( str.indexOf("FLAGS") != -1 )
	        {
	            result = new StringBuffer();
	            while (  tok.hasMoreElements() )
	            {
	                    str = (String) tok.nextElement();
	                    result.append( str );
	            }
	
	            flags = parseFlags( result.toString()  );
	
	        }
	
	    }
	    return flags;
	}
	*/
	public Integer parseSize(String s)
	{
		//System.out.println("size: "+ s );
		String str, str2;
		StringBuffer result;

		StringTokenizer tok = new StringTokenizer(s, " ");
		StringTokenizer tok2;

		Integer i = new Integer(0);

		while (tok.hasMoreElements())
		{
			str = (String) tok.nextElement();

			if (str.indexOf("SIZE") != -1)
			{
				str = (String) tok.nextElement();
				i = new Integer(str);

				//System.out.println("found size: "+i);
			}
		}
		return i;
	}

	public Vector parseUids(String s)
	{
		String str, str2;
		StringBuffer result;

		StringTokenizer tok = new StringTokenizer(s, "\n");
		StringTokenizer tok2;

		Vector v = new Vector();

		//  System.out.println("parsing for UIDs");

		while (tok.hasMoreElements())
		{
			str = (String) tok.nextElement();
			tok2 = new StringTokenizer(str, " ");

			while (tok2.hasMoreElements())
			{
				str2 = (String) tok2.nextElement();

				// if token equals UID parse number
				if (str2.indexOf("UID") != -1)
				{
					str2 = (String) tok2.nextElement();
					//System.out.println("str: "+str2);
					str2 = str2.substring(0, str2.length() - 1);
					//System.out.println("str: "+str2);
					//Integer i = new Integer( str2 );

					v.add(str2);
					//System.out.println("found UID: "+i);
				}

			}
		}

		//for ( int i=0; i< v.size(); i++)
		// {
		//  System.out.println("uid: "+ v.get(i));
		// }

		return v;
	}

	public static Vector parseLsub(String s)
	{
		StringTokenizer tok = new StringTokenizer(s, "\n");

		String str;

		Vector v = new Vector();
		boolean firstTime = false;

		while (tok.hasMoreElements())
		{
			str = (String) tok.nextElement();

			String path = null;

			path = extractPath(str);
			System.out.println("path:" + path);

			v.add(path);

		}

		return v;
	}
	
	protected static boolean validDelimiter( int pos, String path )
	{
		System.out.println("validate string:"+path);
		System.out.println("pos:"+pos+" - char:"+path.charAt(pos) );
		
		char left = path.charAt(pos-1);
		char right = path.charAt(pos+1);
		
		if ( ( left == '"' ) && ( right == '"')  ) return true;
					
		return false;	
	}

	public static String parseDelimiter(String list)
	{
		StringTokenizer tok = new StringTokenizer(list, "\n");

		String s;

		String delimiter = new String();

		while (tok.hasMoreElements())
		{
			s = (String) tok.nextElement();
			int index;
			String result;
			int delimiterIndex;
			
			if (s.endsWith("\""))
			{
				// string ends with "
				// this means the path is enclosed in ""
				// for example: "My Files"

				index = s.lastIndexOf("\"", s.length() - 2);
				result = s.substring(index + 1, s.length() - 1);
				System.out.println("result:" + result);

				delimiterIndex = index - 3;
				delimiter = s.substring(delimiterIndex, delimiterIndex + 1);
			}
			else
			{
				// path is not enclosed in ""
				// this means it does not contain whitespaces
				// for example: testorder but not "test order"

				index = s.lastIndexOf(" ");
				result = s.substring(index + 1, s.length());
				delimiterIndex = index - 2;
				delimiter = s.substring(delimiterIndex, delimiterIndex + 1);
			}

			if ( validDelimiter( delimiterIndex, s ) == true )
				return delimiter;
		}

		return null;
	}

	protected static String extractPath(String s)
	{
		String result = new String();

		if (s.endsWith("\""))
		{
			// string ends with "
			// this means the path is enclosed in ""
			// for example: "My Files"

			int index = s.lastIndexOf("\"", s.length() - 2);
			result = s.substring(index + 1, s.length() - 1);

		}
		else
		{
			// path is not enclosed in ""
			// this means it does not contain whitespaces
			// for example: testorder but not "test order"

			result = s.substring(s.lastIndexOf(" ") + 1, s.length());
		}

		return result;
	}

	public static Vector parseList(String s)
	{
		//System.out.println("parsing:\n" + s);

		StringTokenizer tok = new StringTokenizer(s, "\n");
		StringTokenizer tok2;

		String str;
		String str2 = new String();

		Vector v = new Vector();

		boolean firstTime = true;

		while (tok.hasMoreElements())
		{
			str = (String) tok.nextElement();

			//str2 = str.substring( str.lastIndexOf(" ")+1, str.length() );
			str2 = extractPath(str);

			// System.out.println("str2: "+ str2 );

			String str3 = str2;
			int index = str2.lastIndexOf("/");
			if (index != -1)
			{
				if (index != str.length() - 1)
					str3 = str2.substring(index + 1, str2.length());
			}

			// System.out.println("str3: "+ str3 );

			SubscribeTreeNode treeNode = new SubscribeTreeNode(str3);
			treeNode.setDirectoryPath(str2);

			str = str.toLowerCase();

			if (str.indexOf("noselect") != -1)
			{
				// this is a directory
				treeNode.setDirectory(true);
			}
			else
			{
				treeNode.setDirectory(false);
			}

			if (str.indexOf("noinferiors") != -1)
			{
				// this is a directory
				treeNode.setSubFolder(true);
			}
			else
			{
				treeNode.setSubFolder(false);
			}

			v.add(treeNode);

		}
		return v;
	}

	/*
	public static Vector parseSubList( String s )
	{
	    StringTokenizer tok = new StringTokenizer( s, "\n" );
	    StringTokenizer tok2;
	
	    String str;
	    String str2 = new String();
	
	    Vector v = new Vector();
	
	    boolean firstTime = true;
	
	    while ( tok.hasMoreElements() )
	    {
	        str = (String) tok.nextElement();
	        System.out.println( "token: "+ str );
	
	        if ( firstTime )
	        {
	            firstTime = false;
	            continue;
	        }
	
	        //str2 = str.substring( str.lastIndexOf("/")+1, str.length() );
	        str2 = extractPath( str );
	
	
	
	        SubscribeTreeNode treeNode = new SubscribeTreeNode( str2 );
	        treeNode.setDirectoryPath( str2 );
	
	        str = str.toLowerCase();
	
	        if ( str.indexOf("noselect") != -1 )
	        {
	            // this is a directory
	            treeNode.setDirectory( true );
	        }
	        else
	        {
	            treeNode.setDirectory( false );
	        }
	
	        if ( str.indexOf("noinferiors") != -1 )
	        {
	            // this is a directory
	            treeNode.setSubFolder( true );
	        }
	        else
	        {
	            treeNode.setSubFolder( false );
	        }
	
	
	
	        v.add( treeNode );
	
	
	    }
	    return v;
	}
	*/

	public static MessageFolderInfo parseMessageFolderInfo(String s)
	{
		MessageFolderInfo info = new MessageFolderInfo();

		StringTokenizer tok = new StringTokenizer(s, "\n");

		String str, str2;
		Integer i;

		while (tok.hasMoreElements())
		{
			str = (String) tok.nextElement();

			if (str.indexOf("EXISTS") != -1)
			{

				str2 = str.substring(str.indexOf("*") + 2, str.indexOf("EXISTS") - 1);

				i = new Integer(str2);

				info.setExists(i.intValue());

			}
			else if (str.indexOf("RECENT") != -1)
			{

				str2 = str.substring(str.indexOf("*") + 2, str.indexOf("RECENT") - 1);

				i = new Integer(str2);

				info.setRecent(i.intValue());

			}
			else if (str.indexOf("UIDVALIDITY") != -1)
			{

				str2 = str.substring(str.indexOf("UIDVALIDITY") + 11 + 1, str.indexOf("]"));

				i = new Integer(str2);

				info.setUidValidity(i.intValue());

			}
			else if (str.indexOf("UIDNEXT") != -1)
			{

				str2 = str.substring(str.indexOf("UIDNEXT") + 7 + 1, str.indexOf("]"));

				i = new Integer(str2);

				info.setUidNext(i.intValue());

			}
			else if (str.indexOf("FLAGS") != -1)
			{

				str2 = str.substring(str.indexOf("(") + 1, str.indexOf(")"));

				//info.setFlags( parseFlags(str2) );
			}
			else if (str.indexOf("PERMANENTFLAGS") != -1)
			{

				str2 = str.substring(str.indexOf("(") + 1, str.indexOf(")"));

				//info.setPermanentFlags(parseFlags(str2) );
			}
			else if (str.indexOf("UNSEEN") != -1)
			{

				str2 = str.substring(str.indexOf("UNSEEN") + 6 + 1, str.indexOf("]"));

				i = new Integer(str2);

				info.setUnseen(i.intValue());

			}
			else if (str.indexOf("READ-WRITE") != -1)
			{

				info.setReadWrite(true);

			}

		}

		return info;
	}

	public static void parseFlags(String str, Message message)
	{
		Flags flags = message.getFlags();

		//System.out.println("flags: "+ str );

		if (str.indexOf("Seen") != -1)
		{
			//System.out.println("seen is true ");
			flags.setSeen(true);
		}
		if (str.indexOf("Answered") != -1)
		{
			//System.out.println("answered is true ");
			flags.setAnswered(true);
		}
		if (str.indexOf("Flagged") != -1)
		{
			//System.out.println("flagged is true ");
			flags.setFlagged(true);
		}
		if (str.indexOf("Deleted") != -1)
		{
			//System.out.println("deleted is true ");
			flags.setDeleted(true);
		}
		if (str.indexOf("Draft") != -1)
		{
			flags.setDraft(true);
		}

	}

	public MimePartCollection parseBodyStructure3(String input)
	{
		MimePartCollection result = new MimePartCollection();
		int start = input.indexOf("BODYSTRUCTURE");

		if (start == -1)
		{
			System.out.println("String from Server / not expected: " + input);
			return null;
		}

		String bodystructure = input.substring(start);

		int open = bodystructure.indexOf("(");
		int close = getClosingParenthesis(bodystructure, open);

		// Is MultiPart ?
		if (bodystructure.charAt(open + 1) != '(')
		{
			MimePart onlyOne =
				new MimePart(parseMimeStructure(bodystructure.substring(open + 1, close)));
			result.add(onlyOne);
			return result;
		}

		MimeHeader header;

		while (bodystructure.charAt(open + 1) == '(')
		{
			open++;
			close = getClosingParenthesis(bodystructure, open);

			header = parseMimeStructure(bodystructure.substring(open + 1, close));
			result.add(new MimePart(header));
			open = close;
		}

		// WE DO NOT (YET) DISTINGUISH BETWEEN DIFFERENT MULTIPART TYPES

		return result;
	}

	private MimeHeader parseMimeStructure(String structure)
	{
		MimeHeader header = new MimeHeader();
		Interval pos = new Interval();
		String value;
		int open;

		// Content-Type    	
		value = getNextItem(structure, pos);
		if (value != null)
			header.contentType = value.toLowerCase();

		// ContentSubtype
		value = getNextItem(structure, pos);
		if (value != null)
			header.contentSubtype = value.toLowerCase();

		// are there some Content Parameters ?
		open = structure.indexOf("(", pos.b);
		if (((open - pos.b) < 5) && (open != -1))
		{
			int close = getClosingParenthesis(structure, open);
			pos.b = close;
			Interval subPos = new Interval();
			String subPart = structure.substring(open + 1, close);
			value = getNextItem(subPart, subPos);
			String secondValue;
			while (value != null)
			{
				secondValue = getNextItem(subPart, subPos);

				header.putContentParameter(value.toLowerCase(), secondValue);

				value = getNextItem(subPart, subPos);
			}
		}

		// ID
		value = getNextItem(structure, pos);
		if (value != null)
			header.contentID = value.toLowerCase();

		// Description
		value = getNextItem(structure, pos);
		if (value != null)
			header.contentDescription = value.toLowerCase();

		// Encoding
		value = getNextItem(structure, pos);
		if (value != null)
			header.contentTransferEncoding = value.toLowerCase();

		// Size
		value = getNextItem(structure, pos);
		if (value != null)
			header.size = value.toLowerCase();

		// Is this a Message/RFC822 Part ?
		if ((header.contentType.equals("message"))
			& (header.contentSubtype.equals("rfc822")))
		{

		}

		// Is this a Text - Part ?
		if (header.contentType.equals("text"))
		{
			// Number of lines
			value = getNextItem(structure, pos);
			// NO USE
		}

		// Are there Extensions ?
		// MD5
		value = getNextItem(structure, pos);
		if (pos.a >= structure.length())
			return header;
		// NO USE

		// are there some Disposition Parameters ?
		open = pos.b + 2;
		if (structure.charAt(open) == '(')
		{
			int close = getClosingParenthesis(structure, open);
			pos.b = close;
			String subPart = structure.substring(open + 1, close);
			Interval subPos = new Interval();
			String subValue;

			// Disposition
			value = getNextItem(subPart, subPos);
			if (value != null)
				header.contentDisposition = value.toLowerCase();

			if (subPos.b + 2 < subPart.length())
			{

				// Disposition Parameters
				open = subPos.b + 2;
				if (subPart.charAt(subPos.b + 2) == '(')
				{
					int subClose = getClosingParenthesis(subPart, open);
					subPos.b = subClose;
					Interval subsubPos = new Interval();
					String subsubPart = subPart.substring(open + 1, subClose);
					value = getNextItem(subsubPart, subsubPos);
					String secondValue;
					while (value != null)
					{
						secondValue = getNextItem(subsubPart, subsubPos);

						header.putDispositionParameter(value.toLowerCase(), secondValue);

						value = getNextItem(subsubPart, subsubPos);
					}
				}
			}
		}
		else
		{
			value = getNextItem(structure, pos);
			if (value != null)
				header.contentDisposition = value.toLowerCase();
		}

		// WE DO NOT GATHER FURTHER INFORMATION

		return header;
	}

	private String getNextItem(String s, Interval i)
	{
		i.a = i.b + 2;
		if (i.a >= s.length())
			return null;

		// Quoted

		if (s.charAt(i.a) == '\"')
		{
			i.b = s.indexOf("\"", i.a + 1);

			return s.substring(i.a + 1, i.b);
		}

		// Parenthesized

		if (s.charAt(i.a) == '(')
		{
			return null;
		}

		// NIL or Number

		i.b = s.indexOf(" ", i.a + 1);
		String item = s.substring(i.a, i.b);
		i.b--;

		if (item.equals("NIL"))
			return null;
		else
			return item;
	}

	private int getClosingParenthesis(String s, int openPos)
	{
		int nextOpenPos = s.indexOf("(", openPos + 1);
		int nextClosePos = s.indexOf(")", openPos + 1);

		while ((nextOpenPos < nextClosePos) & (nextOpenPos != -1))
		{
			nextClosePos = s.indexOf(")", nextClosePos + 1);
			nextOpenPos = s.indexOf("(", nextOpenPos + 1);
		}
		return nextClosePos;
	}

	public static Vector parseSearch(String s)
	{
		Vector result = new Vector();

		StringTokenizer tok = new StringTokenizer(s, "\n");
		while (tok.hasMoreTokens())
		{
			String nextLine = tok.nextToken();

			if (!nextLine.startsWith("* SEARCH"))
				continue;
			if (nextLine.length() <= 9)
				continue;

			String numberString = nextLine.substring(9, nextLine.length());
			System.out.println("<" + numberString + ">");
			numberString = numberString.trim();
			StringTokenizer tok2 = new StringTokenizer(numberString, " ");
			while (tok2.hasMoreTokens())
			{
				String nextLine2 = tok2.nextToken();
				//System.out.println("added <"+nextLine2+">");
				result.add(nextLine2);
			}
		}

		return result;
	}

}

class Interval
{

	public int a, b;

	public Interval(int a, int b)
	{
		this.a = a;
		this.b = b;
	}

	public Interval()
	{
		a = -1;
		b = -2;
	}

	public void reset()
	{
		a = -1;
		b = -2;
	}

}