// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


/** Document the purpose of this class.
 *
 * @version 1.0
 * @author Timo Stich
 */

package org.columba.parser;

import java.util.*;
import org.columba.message.*;
import java.net.*;
import java.io.*;
import org.columba.config.*;
import org.columba.main.MainInterface;

public class MimeRouter
{
    static private MimeTypeParser standardParser;
    static private OptionsXmlConfig config;
    static private int mutex;
    static private MainInterface mainInterface;

    /*
    static public final int PARSE_MESSAGE = 0;
    static public final int SHOW_MESSAGE = 1;

    static private int action = 0;
    static private PGPItem pgpItem;
    */


    public MimeRouter(MainInterface mainInterface )
    {
        this.mainInterface = mainInterface;

        this.config = mainInterface.config.getOptionsConfig();

        standardParser = new MimeStandardParser();

        mutex = 0;
    }

    /*
    public static void setPGPItem( PGPItem item )
    {
        pgpItem = item;
    }

    public static PGPItem getPGPItem()
    {
        return pgpItem;
    }

    public static void setAction( int i )
    {
        action = i;
    }

    public static int  getAction()
    {
        return action;
    }
    */


    static public MimeTypeParser getTypeParser(MimeHeader input, int action )
    {
        String parserName;
        MimeTypeParser parser;

        if( input.contentType == null ) return standardParser;

        parserName = getValue( input.contentType, input.contentSubtype,
                                    "parser" );

        if( parserName != null ) {
            parser =  loadParser( parserName, action );
            if( parser != null) return parser;
        }

        return standardParser;
    }

    static public synchronized String getViewer( MimeHeader input )
    {
        String output;

        while( mutex==1 );
        mutex = 1;
        output = getValue( input.contentType, input.contentSubtype, "viewer" );
        mutex = 0;

        return output;
    }

    static public synchronized String getViewer( String contentType, String contentSubtype )
    {
        String output;

        while( mutex==1 );
        mutex = 1;
        output = getValue( contentType, contentSubtype, "viewer" );
        mutex = 0;

        return output;
    }

    static public synchronized void setViewer( MimeHeader input, String viewer )
    {
        while( mutex==1 );
        mutex = 1;
        setViewer( input.contentType, input.contentSubtype, viewer );
        mutex = 0;
    }


    static private String getValue(String contentType, String subType, String key)
    {

        AdapterNode section = config.getMimeTypeNode();

	AdapterNode actMime = null;
	AdapterNode type = null;

	for ( int i=0; i< section.getChildCount(); i++)
	    {
		actMime = section.getChild(i) ;

		type = actMime.getChild("type");

		if ( type.getValue().equals( contentType ) )
		    break;
		actMime = null;
	    }
        if ( actMime == null )
	    {
		return null;
	    }


        String output = null;

	AdapterNode outputNode = actMime.getChild(key);
	if ( outputNode != null ) output = outputNode.getValue();


        AdapterNode subMime;
        AdapterNode subParserNode;


        for( int i=0; i< actMime.getChildCount(); i++ ) {
            subMime = actMime.getChild(i);

            //System.out.println("i="+i+ " -> " + subMime.getName() );

            if( subMime.getName().equals("subtype") )
            {
                if( subMime.getChild("type").getValue().equals(subType) )
                {
                    subParserNode = subMime.getChild(key);
                    if( subParserNode!= null) output = subParserNode.getValue();
                }
            }
        }

	return output;
    }



    static public void setViewer(String contentType, String subType, String value)
    {
        AdapterNode section = config.getMimeTypeNode();

	AdapterNode actMime = null;
	AdapterNode type = null;

	for ( int i=0; i< section.getChildCount(); i++)
	    {
		actMime = section.getChild(i) ;

		type = actMime.getChild("type");

		if ( type.getValue().equals( contentType ) )
		    break;
		actMime = null;
	    }
        if ( actMime == null )
	    {
                actMime = section.addElement( config.createElementNode("mimetype") );
                actMime.addElement( config.createTextElementNode("type",contentType) );
                actMime.addElement( config.createTextElementNode("viewer", value ) );

                return;
	    }

        AdapterNode subMime;
        AdapterNode viewerNode;

        for( int i=0; i<actMime.getChildCount(); i++ ) {
            subMime = actMime.getChild(i);

            if( subMime.getName().equals("subtype") )
            {
                if( subMime.getChild("type").getValue().equals(subType) )
                {
                    viewerNode = subMime.getChild("viewer");
                    if( viewerNode != null ) {
                        viewerNode.setValue( value );
                        return;
                    }

                    subMime.addElement( config.createTextElementNode("viewer", value ) );
                    return;
                }
            }
        }
        subMime = actMime.addElement( config.createElementNode("subtype") );
        subMime.addElement( config.createTextElementNode("type", subType ) );
        subMime.addElement( config.createTextElementNode("viewer", value ) );
    }


    static private MimeTypeParser loadParser( String name, int action )
    {
        URL[] urlList = new URL[1];
        URLClassLoader loader;
        Class actClass;

        //System.out.println("loading parser: "+ name );

        try {
            urlList[0] = new URL( "file://" + Config.userDir );
            loader = new URLClassLoader(urlList);

            actClass = loader.loadClass("org.columba.parser."+name);

            /*
            // I inserted this code to differenciate between
            // a parsing action inclusive the encryption and verification
            // of a message( for example: message viewing )
            // and parsing action which only parses the message bodystructure
            switch ( action )
            {
                case MimeTypeParser.PARSE_MESSAGE:
                {
                    // only parse bodystructure
                    // do not decrypt or verify message

                    if ( ( actClass.getName().equals("MimeMultipartEncryptedParser") ) ||
                         ( actClass.getName().equals("MimeMultipartSignedParser") ) )
                    {
                        actClass = loader.loadClass("org.columba.parser.MimeMultipartParser");
                    }

                    break;
                }
                case MimeTypeParser.SHOW_MESSAGE:
                {
                    // message should be decrypted and verified

                    break;
                }
            }
            */

            if( actClass.getSuperclass().getName().equals("org.columba.parser.MimeTypeParser") )
            {

                MimeTypeParser parser = (MimeTypeParser) actClass.newInstance();
                return parser;
            }

        }
        catch( Exception e ) {
            System.out.println( e );
        }

        return null;
    }


}



