// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.pgp;

import java.io.*;
import java.util.*;
import javax.swing.*;

import org.columba.config.*;
import org.columba.gui.util.*;
import org.columba.main.MainInterface;
import org.columba.util.*;

public class DefaultUtil
{
    public StreamThread outputStream = null;
    public StreamThread errorStream = null;

    public File outputFile;

    protected String cmdString;

    protected Process executeCommand( String cmd ) throws Exception
    {
        return null;
    }

    protected String parse( String str )
    {
        return str;
    }

    // error gets parsed
    public String getErrorString()
    {
        String str = parse( errorStream.getBuffer() );
        return str;
    }

    public String getOutputString()
    {
        String str = outputStream.getBuffer();
        return str;
    }

    public File getOutputFile()
    {
        return outputFile;
    }

    protected String getCommandString()
    {
        return cmdString;
    }

    protected void setCommandString( String s )
    {
        cmdString = s;
    }

    protected void sendPassphrase( Process p, String passphrase ) throws Exception
    {
        PrintWriter out = new PrintWriter( p.getOutputStream() );
        out.println( passphrase );
        out.flush();
        out.close();
    }

    public int decrypt( String path, String pgpMessage, String passphrase ) throws Exception
    {
        int exitVal = -1;

        Process p = executeCommand( getCommandString() );

        errorStream = new StreamThread(p.getErrorStream(), "ERROR");
        outputStream = new StreamThread(p.getInputStream(), "OUTPUT");

        sendPassphrase( p, passphrase );

        errorStream.start();
        outputStream.start();

        exitVal = p.waitFor();

        System.out.println("exitvalue: "+ exitVal );

        // wait for stream threads to die
        outputStream.join();
        errorStream.join();

        return exitVal;
    }

    public int verify( String path, String pgpMessage, String signatureString, String passphrase ) throws Exception
    {
        int exitVal = -1;

        Process p = executeCommand( getCommandString() );

        errorStream = new StreamThread(p.getErrorStream(), "ERROR");
        outputStream = new StreamThread(p.getInputStream(), "OUTPUT");

        sendPassphrase( p, passphrase );

        errorStream.start();
        outputStream.start();

        exitVal = p.waitFor();

        System.out.println("exitvalue: "+ exitVal );

        // wait for stream threads to die
        outputStream.join();
        errorStream.join();

        return exitVal;
    }

    public int encrypt( String path, String pgpMessage, String passphrase, boolean signValue,  Vector recipient, String id ) throws Exception
    {
        int exitVal = -1;

        Process p = executeCommand( getCommandString() );

        errorStream = new StreamThread(p.getErrorStream(), "ERROR");
        outputStream = new StreamThread(p.getInputStream(), "OUTPUT");

        if ( signValue == true )
        {
            sendPassphrase( p, passphrase );
        }

        errorStream.start();
        outputStream.start();

        exitVal = p.waitFor();

        System.out.println("exitvalue: "+ exitVal );

        // wait for stream threads to die
        outputStream.join();
        errorStream.join();

        return exitVal;
    }
}
