// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

package org.columba.pgp;

import java.io.*;
import java.util.*;
import javax.swing.*;

import org.columba.config.*;
import org.columba.gui.util.*;
import org.columba.main.MainInterface;

public class PGPController
{
    public final static int GPG = 0;
    public final static int PGP2 = 1;
    public final static int PGP5 = 2;
    public final static int PGP6 = 3;

    public final static int DECRYPT_ACTION = 0;
    public final static int ENCRYPT_ACTION = 1;
    public final static int SIGN_ACTION = 2;
    public final static int VERIFY_ACTION = 3;
    public final static int ENCRYPTSIGN_ACTION = 4;



    private MainInterface mainInterface;

    //private String passphrase;
    private int type;
    private String path;
    private String id;
    //private boolean save;
    private int exitVal;


    private DefaultUtil util;

    public PGPController( MainInterface mainInterface )
    {
        this.mainInterface = mainInterface;
        //passphrase = new String("");
    }

    protected void getInformation()
    {

    }

    /*
    public void forgetPassphrase()
    {
        passphrase = new String("");
    }
    */

    public int getReturnValue()
    {
        return exitVal;
    }

    protected DefaultUtil load( int type )
    {
        if ( util != null ) return util;

        switch( type )
        {
            case GPG:
            {
                util = new GnuPGUtil();
                break;
            }
            case PGP2:
            {
                util = new PGP2Util();
                break;
            }
            case PGP5:
            {
                util = new PGP5Util();
                break;
            }
            case PGP6:
            {
                util = new PGP6Util();
                break;
            }
        }

        return util;
    }

    public String decrypt( String pgpMessage, PGPItem item )
    {

        exitVal=-1;

        type = item.getType();
        path = item.getPath();
        id = item.getId();


	String passphrase = item.getPassphrase();
	boolean save = false;

	

            if  ( passphrase.length()==0 ) 
            {
                PGPPassphraseDialog dialog = new PGPPassphraseDialog(id,false);
                //dialog.showDialog( id, false );

                if ( dialog.success() )
                {
                    passphrase = new String( dialog.getPassword(), 0, dialog.getPassword().length );

                    save = dialog.getSave();

                }
                else
                {
                    return new String("");
                }
            }

	    
        try
        {


            util = load( type );

            exitVal = util.decrypt( path, pgpMessage, passphrase );



            if ( exitVal == 2 )
            {
                // unknown error

                JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                               util.getErrorString() );
                if ( save == false ) item.clearPassphrase();

                return null;
            }
            else if ( exitVal == 1 )
            {
                // this means: decrypted message successfully, but
                // was not able to verify the signature

                    JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                               "Couldn't verify signature: "+util.getErrorString() );


		    if ( save == false ) item.clearPassphrase();
                return null;
            }

        }
        catch ( Exception ex )
        {
            ex.printStackTrace();

            JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                           util.getErrorString() );

            if ( save == false ) item.clearPassphrase();

            return null;
        }



        if ( save == false ) item.clearPassphrase();



        return util.getOutputString();
    }


    public String verify( String pgpMessage, String signatureString, PGPItem item )
    {
        //System.out.println("message file:\n"+ pgpMessage );
        //System.out.println("signature file:\n"+ signatureString );

        exitVal=-1;


        type = item.getType();
        path = item.getPath();
        id = item.getId();


        File outputFile;

	String passphrase = item.getPassphrase();
	boolean save = false;

        if ( ( passphrase.length()==0 ) )
            {
                PGPPassphraseDialog dialog = new PGPPassphraseDialog(id,false);
                //dialog.showDialog( id, false );

                if ( dialog.success() )
                {
                    passphrase = new String( dialog.getPassword(), 0, dialog.getPassword().length );

                    save = dialog.getSave();

                }
                else
                {
                    return new String("");
                }
            }


        try
        {

            util = load( type );

            exitVal = util.verify( path, pgpMessage, signatureString, passphrase);

            System.out.println("exitvalue: "+ exitVal );

        }
        catch ( Exception ex )
        {
            ex.printStackTrace();


            JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                           util.getErrorString() );
        }


        if ( exitVal > 0 )
        {
            // error
            String errorString = util.errorStream.getBuffer();
	    

            JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                           util.getErrorString() );
        }

	if ( save == false ) item.clearPassphrase();

        return util.getOutputString();
    }


    public String encrypt( String pgpMessage, boolean signValue, Vector recipients, PGPItem item )
    {
	System.out.println("pgpcontroller->encrypt");
        exitVal=-1;

        type = item.getType();
        path = item.getPath();
        id = item.getId();

	/*
        while ( exitVal != 0 )
        {
	*/

	String passphrase = item.getPassphrase();
	boolean save = false;

        if ( passphrase.length()==0 ) 
            {
                PGPPassphraseDialog dialog = new PGPPassphraseDialog( id, false );
                //dialog.showDialog( id, false );

                if ( dialog.success() )
                {
                    passphrase = new String( dialog.getPassword(), 0, dialog.getPassword().length );

                    save = dialog.getSave();
                }
                else
                {
                    return new String("");
                }
            }


        try
        {
            util = load( type );

            exitVal = util.encrypt( path, pgpMessage, passphrase, signValue, recipients, id );




            if ( exitVal == 2 )
            {
                JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                               util.getErrorString() );
                if ( save == false ) item.clearPassphrase();

                return null;
            }
            else if ( exitVal == 1 )
            {

                if ( save == false ) item.clearPassphrase();
                return null;
            }

        }
        catch ( Exception ex )
        {
            ex.printStackTrace();

            JOptionPane.showMessageDialog( mainInterface.mainFrame,
                                           util.getErrorString() );

            if ( save == false ) item.clearPassphrase();

            return null;
        }

	//        }

        if ( save == false ) item.clearPassphrase();


        // now read encrypted message from File
        StringBuffer strbuf=null;
        try
        {
            BufferedReader in = new BufferedReader( new FileReader( util.outputFile ) );
            String str;
            strbuf = new StringBuffer();

            while ((str = in.readLine()) != null)
            {
                strbuf.append( str+"\n" );
            }

	    System.out.println("----------------------->encrpted message:\n"+strbuf);
            in.close();

            // delete tmp file
            util.outputFile.delete();
        }
        catch ( IOException ex )
        {
            ex.printStackTrace();
        }

        return strbuf.toString();
    }


}
