package org.columba.pop3;

import org.columba.main.MainInterface;
import org.columba.message.*;
import org.columba.parser.*;
import org.columba.folder.*;
import org.columba.gui.util.*;
import org.columba.folder.temp.*;

import java.io.*;
import java.net.UnknownHostException;
import java.net.*;
import java.util.Vector;
import java.util.Date;
import java.util.Arrays;

/**
 * Title:
 * Description:
 * Copyright:    Copyright (c) 2001
 * Company:
 * @author
 * @version 1.0
 */

public class FetchMessageWorker extends SynchronizeMessageWorker
{

	protected Vector fetchResult;
	protected Folder folder;
	protected Message m;

	protected TempFolder tempFolder;

	public void init(Operator root, Operation op)
	{
		super.init(root, op);

		this.folder = pop3Op.source;

		tempFolder = new TempFolder(root.mainInterface);
	}

	public void fetch() throws Exception
	{
		if (getCancel() == false)
		{
			if ( totalMessages == 0 )
			{
				// no messages on server
				return;
			}
			
			fetchResult = synchronize();

			//pop3Op.channel.setTransVolume( (fetchResult.size()) );

			if (fetchResult.size() == 0)
				return;

			if (pop3Op.mailcheck == true)
			{
				playsound();

				boolean autodownload = popServer.getPopItem().isAutoDownload();
				if (autodownload == false)
					return;
			}

			boolean b = false;
			for (int i = 0; i < fetchResult.size(); i++)
			{
				try
				{

					Integer intValue = (Integer) fetchResult.get(i);
					b = fetchMessage(intValue.intValue());

					if (getCancel() == true)
					{
						b = false;
						break;
					}
					
					// fail-case
					if ( m==null ) continue; 
					
					

					Object uid = tempFolder.add(m, m.getRawString());
					Object[] uids = new Object[1];
					uids[0] = uid;

					Channel channel = new Channel(1, 1);
					FolderOperation sendOp =
						new FolderOperation(Operation.SEND, 0, uids, tempFolder, channel);
					sendOp.destructive = true;
					tempFolder.getOperator().addOp(sendOp);

					//Folder destFolder = folder;

					FolderOperation inboxOp =
						new FolderOperation(Operation.RECEIVE_POP3, 0, pop3Op.source, channel);
					pop3Op.source.getOperator().addOp(inboxOp);

					
					//System.out.println( "Send Message "+i );
				}
				catch (Exception e)
				{
					ExceptionDialog dialog = new ExceptionDialog(mainInterface);
					dialog.showDialog(e);

					setCancel(true);
					b = false;
					e.printStackTrace();
				}

				if (b == false)
					break;
			}

			// if fetch operation was cancelled
			// do not cleanly logout
			// just close the socket
			if (b == false)
				login = false;
		}
	}


	protected void delete() throws IOException
	{
		Object[] indexList = ManageRemoveWorker.createIndexList(uids.toArray(), uids);

		Arrays.sort(indexList);

		setProgressBarMaximum(indexList.length);

		for (int i = indexList.length - 1; i >= 0; i--)
		{
			setProgressBarValue((indexList.length-1)-i);
			Integer index = (Integer) indexList[i];

			//System.out.println("index:"+index);

			pop3Connection.deleteMessage(index.intValue());
			popServer.getMessageCollection().remove( index.intValue()-1 );            

		}
	}

	public Object construct()
	{

		try
		{
			//System.out.println("--------------> trying to open port");
			
			openPort();
			
			//System.out.println("--------------> trying to login");

			login();
			
			//System.out.println("--------------> trying to check");

			check();

			//System.out.println("--------------> trying to fetch");
			
			fetch();

			// if fetch operation was cancelled
			// do not cleanly logout
			// just close the socket
			if (login == false)
			{
				closePort();
			}
			else
			{
				
				if (pop3Op.popServer.getAccountItem().getPopItem().isLeaveMessage() == false)
				{
					if ( totalMessages > 0 )
						delete();
				}

				
				
				logout();
			}

		}
		catch (Exception ex)
		{
			if (ex instanceof UnknownHostException)
			{
				Exception e =
					new Exception("Unknown host: " + ex.getMessage() + "\nAre you  online?");
				NotifyDialog dialog = new NotifyDialog(mainInterface);
				dialog.showDialog(e);
			}
			else
			{
				ExceptionDialog dialog = new ExceptionDialog(mainInterface);
				dialog.showDialog(ex);
			}

			ex.printStackTrace();
		}

		popServer.enableActions(true);

		unregister();
		return null;
	}

	protected boolean fetchMessage(int number) throws Exception
	{
		String rawString = new String();
		ColumbaHeader header = new ColumbaHeader();
		Rfc822Parser parser = new Rfc822Parser();
		boolean attach;

		setText(popServer.getFolderName() + " : Fetching messages... ");

		rawString = pop3Connection.getMessage(number, this);

		if (getCancel() == true)
		{
			return false;
		}

		int index = rawString.indexOf("\n\n");
		if (index == -1)
		{
			System.out.println("non-standard-compliant message:\n" + rawString);

			// nevertheless we try to go on
			index = rawString.length() - 1;
			if (index == -1)
			{
				// we found empty string
				System.out.println("found empty message string:\n This should never happen..");
			}
		}

		String headerString = rawString.substring(0, index);

		header = parser.parseHeader(rawString);

		m = new Message(header);
		ColumbaHeader h = m.getHeader();
		m.setRawString(rawString);

		/*
		Flags flags = new Flags();
		flags.setRecent( true );
		m.setFlags( flags );
		*/
		h.set("columba.flags.recent", new Boolean(true));

		int octetString;
		if (messageSizes == null)
			octetString = pop3Connection.getSize();
		else
			octetString = ((Integer) messageSizes.get(number - 1)).intValue();

		int size = -1;

		size = Math.round(octetString / 1024);

		if (size == 0)
			size = 1;
		//m.setSize( size );
		h.set("columba.size", new Integer(size));

		//m.setHost( item.getHost() );
		h.set("columba.host", item.getHost());

		Date date = DateParser.parseString((String) h.get("date"));
		//System.out.println("date1: "+ h.get("date") );
		//m.setDate( date );
		h.set("columba.date", date);
		Date date2 = (Date) h.get("columba.date");

		//System.out.println("date2: "+ h.get("columba.date") );

		String shortFrom = (String) header.get("From");
		if (shortFrom != null)
		{
			if (shortFrom.indexOf("<") != -1)
			{
				shortFrom = shortFrom.substring(0, shortFrom.indexOf("<"));
				if (shortFrom.length() > 0)
				{
					if (shortFrom.startsWith("\""))
						shortFrom = shortFrom.substring(1, shortFrom.length() - 1);
					if (shortFrom.endsWith("\""))
						shortFrom = shortFrom.substring(0, shortFrom.length() - 1);
				}

			}
			else
				shortFrom = shortFrom;

			//m.setShortFrom( shortFrom );
			h.set("columba.from", shortFrom);
		}
		else
		{
			//m.setShortFrom("");
			h.set("columba.from", new String(""));
		}

		String priority = (String) header.get("X-Priority");
		if (priority != null)
		{

			//m.setPriority( prio );
			h.set("columba.priority", Integer.getInteger(priority));
		}
		else
		{
			//m.setPriority( 3 );
			h.set("columba.priority", new Integer(3));
		}

		String attachment = (String) header.get("Content-Type");
		if (attachment != null)
		{
			attachment = attachment.toLowerCase();

			if (attachment.indexOf("multipart") != -1)
			{
				//m.setAttachment(true);
				h.set("columba.attachment", new Boolean(true));
			}
			else
			{
				h.set("columba.attachment", new Boolean(false));
				//m.setAttachment(false);
			}
		}
		else
		{
			h.set("columba.attachment", new Boolean(false));
			//m.setAttachment(false);
		}

		//m.setFetchState( true );
		h.set("columba.fetchstate", new Boolean(true));

		//m.setPop3Uid( (String) uidList.get( number-1 ) );
		h.set("columba.pop3uid", (String) uids.get(number - 1));

		//System.out.println("add message if the header is not fetched already!");
		//if ( !existsLocally( (String) m.getPop3Uid(), 0 ) )
		if (!existsLocally((String) h.get("columba.pop3uid"), 0))
		{
			//System.out.println("fetched header");

			Message pop3Message = (Message) m.clone();
			pop3Message.freeMemory();
			ColumbaHeader head = pop3Message.getHeader();
			head.set("columba.flags.recent", new Boolean(false));
			//pop3Message.getFlags().setRecent( false );

			popServer.getMessageCollection().add(pop3Message);
		}
		else
		{
			Message serverMessage = popServer.getMessageCollection().get(number - 1);
			//serverMessage.setFetchState( true );
			ColumbaHeader head = serverMessage.getHeader();
			head.set("columba.fetchstate", new Boolean(true));
		}

		return true;

	}

	protected void playsound()
	{
	    boolean playsound = popServer.getPopItem().isPlaysound();
	    if ( playsound == false ) return;

		String soundfile = popServer.getPopItem().getSoundfile();

		System.out.println("soundfile:" + soundfile);

		if (soundfile.equalsIgnoreCase("default"))
		{
			org.columba.util.PlaySound.play("newmail.wav");
		}
		else
		{
			URL url = null;
			try
			{
				url = new URL("file://" + soundfile);

			}
			catch (MalformedURLException e)
			{
				System.err.println(e.getMessage());
			}

			org.columba.util.PlaySound.play(url);
		}
	}
}
