// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.pop3;


import org.columba.message.*;
import org.columba.config.*;
import org.columba.gui.pop3.*;
import org.columba.gui.util.*;
import org.columba.protocol.*;
import org.columba.util.*;
import org.columba.main.*;
import org.columba.gui.statusbar.*;
import org.columba.parser.*;
import org.columba.folder.*;

import java.io.*;
import java.util.Vector;
import java.util.Date;
import java.util.Hashtable;
import java.awt.event.*;
import javax.swing.*;

import org.columba.gui.action.*;

public class PopServer implements ActionListener
{
    private final static int ONE_SECOND = 1000;

    private AccountItem accountItem;
    private MessageCollection messageCollection;
    private MainInterface mainInterface;
    private File file;
    private boolean hide;
    private ManageFrame frame = null;
    public Pop3 pop3Connection;
    private boolean status = false;
    private int uid;

    protected Operator operator;

    public BasicAction checkAction;
    private BasicAction manageAction;

    private Timer timer;

    private boolean alreadyLoaded;


    public PopServer(MainInterface mainInterface, AccountItem accountItem)
    {
        this.mainInterface = mainInterface;
        this.accountItem = accountItem;

        messageCollection = new MessageCollection();
        file = new File( mainInterface.config.pop3Directory, accountItem.getName() );

        if ( file.exists() == true )
           alreadyLoaded = false;
        else
            alreadyLoaded = true;

        PopItem  item = accountItem.getPopItem();

        uid = accountItem.getUid();


        pop3Connection = new Pop3( item.getUser(), item.getPassword(), item.getHost() );
        hide = true;

	checkAction = new BasicAction( accountItem.getName()+" ("+accountItem.getIdentityItem().getAddress()+")",
				  "",
				  "CHECK",
				  null,
				  null,
				  0,
				  null );
	checkAction.addActionListener( this );

	manageAction = new BasicAction( accountItem.getName()+" ("+accountItem.getIdentityItem().getAddress()+")",
				  "",
				  "MANAGE",
				  null,
				  null,
				  0,
				  null );
	manageAction.addActionListener( this );



        restartTimer();

    }



    public Operator getOperator()
    {
        return operator;
    }

    public void setOperator( Operator op )
    {
        this.operator = op;
    }

    public void restartTimer()
    {
        PopItem item = accountItem.getPopItem();

        if ( item.isMailCheck() == true )
        {
            int interval = item.getMailCheckInterval();

            timer = new Timer( ONE_SECOND*interval*60, this );
            timer.restart();

            //System.out.println("---------------->timer restarted");
        }
        else
        {
            //System.out.println("----------------->timer stopped");

            if ( timer != null )
            {
                timer.stop();
                timer=null;
            }
        }
    }


    public AccountItem getAccountItem()
    {
        return accountItem;
    }

    public void updateAction()
    {
        checkAction.setName( accountItem.getName()+" ("+accountItem.getIdentityItem().getAddress()+")" );
        manageAction.setName( accountItem.getName()+" ("+accountItem.getIdentityItem().getAddress()+")" );
        uid = accountItem.getUid();
    }

    public int getUid()
    {
        return uid;
    }


    public BasicAction getCheckAction()
    {
	return checkAction;
    }

    public BasicAction getManageAction()
    {
	return manageAction;
    }

    public void setStatus( boolean b )
    {
        status = b;
    }

    public void enableActions( boolean b )
    {
        getCheckAction().setEnabled( b );
        getManageAction().setEnabled( b );
    }



    public boolean getStatus()
    {
        return status;
    }

    public ManageFrame getFrame()
    {
        return frame;
    }


    public void setFrame( ManageFrame f )
    {
        frame = f;
    }


    public void showWindow( boolean b )
    {
        if ( frame == null )
        {
            frame = new ManageFrame( mainInterface, this );
            frame.setPopServer( this );
            //frame.setTitle( accountItem.getName() );
            //frame.updateWindow( b );
            frame.pack();

            frame.setVisible(true);


            synchronize();


            status = b;
        }
        else
        {
            frame.setVisible( true );


            synchronize();


        }


    }

    protected Folder getFolder()
    {
	SpecialFoldersItem foldersItem = accountItem.getSpecialFoldersItem();
	String inboxStr = foldersItem.getInbox();
	int inboxInt = Integer.parseInt( inboxStr );

	Folder f = mainInterface.treeViewer.getFolderTree().getFolder( inboxInt );

	return f;
    }

    public boolean fetchAll( boolean mailcheck )
    {
        enableActions( false );

        //System.out.println("--------------> loaded: "+ alreadyLoaded );

        if ( ( file.exists() ) && ( alreadyLoaded == false ) )
        {
            load();
            alreadyLoaded = true;
        }

        Folder source = getFolder();

        Pop3Operation op = new Pop3Operation( Operation.POP3_FETCHALL, 10,
                                              source, this, mailcheck );
        mainInterface.crossbar.operate( op );



        return true;
    }

    public void setHide( boolean b )
    {
        hide = b;
    }

    public boolean getHide()
    {
        return hide;
    }


    public void fetch( Vector uidList )
    {

        enableActions( false );

        if ( ( file.exists() ) && ( alreadyLoaded == false ) )
        {
            load();
            alreadyLoaded = true;
        }



	Folder source = getFolder();

        Pop3Operation op = new Pop3Operation( Operation.POP3_MANAGE_FETCH, 10,
                                              uidList.toArray(),
                                              source,
                                              this
                                               );
        mainInterface.crossbar.operate( op );

        /*
        fetcher = new FetchMessage(  mainInterface, this,
                                     mainInterface.taskManager,
                                     new Boolean( true ), uidList, false);
        fetcher.start();
        */
    }

    public void synchronize()
    {

        enableActions( false );

        if ( ( file.exists() ) && ( alreadyLoaded == false ) )
        {
            load();
            alreadyLoaded = true;
        }

        Pop3Operation op = new Pop3Operation( Operation.POP3_MANAGE_SYNCHRONIZE, 10,
                                              this );
        mainInterface.crossbar.operate( op );

        /*
        fetcherHeader = new FetchMessageHeader( mainInterface , this );
        fetcherHeader.start();
        */
    }

    public String getFolderName()
    {
        return accountItem.getName();
    }


    public void delete( Vector uidList )
    {

        enableActions( false );

        if ( ( file.exists() ) && ( alreadyLoaded == false ) )
        {
            load();
            alreadyLoaded = true;
        }

        Pop3Operation op = new Pop3Operation( Operation.POP3_MANAGE_REMOVE, 10,
                                              uidList.toArray(),
                                              this );
        mainInterface.crossbar.operate( op );

        /*
        fetcher = new FetchMessage(  mainInterface, this,
                                     mainInterface.taskManager,
                                     new Boolean( true ), uidList, true);
        fetcher.start();
        */
    }

    public void save()
    {
            if ( alreadyLoaded == false ) return;

            //System.out.println("--------------> saving headerfile: "+ file.toString() );

            HeaderTableItem v = mainInterface.config.getPopManageOptionsConfig().getHeaderTableItem();
            //HeaderTableItem v = mainInterface.headerTableItem;


            try
            {

                FileOutputStream istream = new FileOutputStream( file.getPath() );
                ObjectOutputStream p = new ObjectOutputStream(istream);

                p.writeInt( messageCollection.count()  );

                //System.out.println("count: "+ messageCollection.count() );


                ColumbaHeader h;
                for (int i=0; i<messageCollection.count(); i++)
                {
		    Message message = messageCollection.get(i);
                    p.writeInt( i + 1 );
                    h = messageCollection.get(i).getHeader();

                    //p.writeObject( message.getUID() );

                    p.writeObject( h.get("columba.pop3uid") );

                    Flags flags = message.getFlags();

		    p.writeBoolean( flags.getSeen() );
		    p.writeBoolean( flags.getAnswered() );
		    p.writeBoolean( flags.getFlagged() );
		    p.writeBoolean( flags.getDeleted() );
		    p.writeBoolean( flags.getDraft() );
		    p.writeBoolean( flags.getRecent() );

                    p.writeObject( h.get("columba.date") );

                    p.writeObject( h.get("columba.size") );

                    p.writeObject( h.get("columba.fetchstate") );

                    p.writeObject( h.get("columba.from") );

                    p.writeObject( h.get("columba.attachment") );

                    p.writeObject( h.get("columba.priority") );

                    p.writeObject( h.get("columba.host") );

                    /*
		    p.writeObject( message.getDate() );

		    p.writeInt( message.getSize() );

                    p.writeObject( new Boolean( message.getFetchState() ) );

                    p.writeObject( message.getShortFrom() );

                    p.writeObject( new Boolean( message.getAttachment() ) );

                    p.writeInt( message.getPriority() );

                    p.writeObject( message.getHost() );
                    */

                    //p.writeObject( messageCollection.get(i).getHeader().get("Status") );

                    String column;
                    HeaderItem item;
                    Object o;
                    for ( int j=0; j< v.count(); j++)
                    {
                        column = (String) v.getName( j );
                        o = messageCollection.get(i).getHeader().get( column );
                        p.writeObject( o );
                    }
                }

                p.flush();
                p.close();
            }
            catch (IOException ex)
            {
                System.out.println("error while writing to file: "+ex.getMessage() );
                ex.printStackTrace();
            }
    }

    public void load()
    {
        //System.out.println("------------------> loading headerfile: "+ file.toString() );

            messageCollection.reset();

            try
            {
                FileInputStream istream = new FileInputStream( file.getPath() );
                ObjectInputStream p = new ObjectInputStream(istream);


                int capacity = p.readInt();

                //System.out.println("pop capacity: "+ capacity );

                int number;

                for ( int i=1; i<=capacity; i++)
                {
                    Message message = new Message();
                    ColumbaHeader h = message.getHeader();

                        // read current number of message
                    number = p.readInt();


                    //message.setPop3Uid( p.readObject() );

                    h.set("columba.pop3uid", p.readObject() );


		    Flags flags = message.getFlags();

		    flags.setSeen( p.readBoolean() );
		    flags.setAnswered( p.readBoolean() );
		    flags.setFlagged( p.readBoolean() );
		    flags.setDeleted( p.readBoolean() );
		    flags.setDraft( p.readBoolean() );
		    flags.setRecent( p.readBoolean() );
		    //message.setFlags( flags );

                    Date date = (Date) p.readObject();
                    h.set("columba.date", date );

                    h.set("columba.size", (Integer) p.readObject() );

                    Boolean fetchstate = (Boolean) p.readObject();
                    h.set("columba.fetchstate", fetchstate );

                    String from = (String) p.readObject();
                    h.set("columba.from", from );

                    Boolean b = (Boolean) p.readObject();
                    h.set("columba.attachment", b );

                    //int priority = p.readInt();
                    h.set("columba.priority", (Integer) p.readObject() );

                    String host = (String) p.readObject();
                    h.set("columba.host", host );

                    /*
		    Date date = (Date) p.readObject();
		    message.setDate( date );

		    int size = p.readInt();
		    message.setSize( size );

                    Boolean fetchstate = (Boolean) p.readObject();
                    message.setFetchState( fetchstate.booleanValue() );

                    message.setShortFrom( (String) p.readObject() );

		    Boolean b = (Boolean) p.readObject();
		    message.setAttachment( b.booleanValue() );

                    int priority = p.readInt();
                    message.setPriority( priority );


                    String host = (String) p.readObject();
                    message.setHost( host );
                    */


                    HeaderTableItem v = mainInterface.config.getPopManageOptionsConfig().getHeaderTableItem();
                    //HeaderTableItem v = mainInterface.headerTableItem;
                    String column;
                    Object o;
                    for ( int j=0; j< v.count(); j++)
                    {
                        column = (String) v.getName( j );
                        o = p.readObject();

                        if ( o == null ) {}
                        else if ( o instanceof String )
                        {
                            String value = (String) o;
                            h.set( column, value );
                        }
                        else if ( o instanceof Integer )
                        {
                            Integer value = (Integer) o;
                            h.set( column, value );
                        }
                        else if ( o instanceof Boolean )
                        {
                            Boolean value = (Boolean) o;
                            h.set( column, value );
                        }
                        else if ( o instanceof Date )
                        {
                            Date value = (Date) o;
                            h.set( column, value );
                        }

                    }
                    messageCollection.add( message );

                }

                //System.out.println("pop count: "+ messageCollection.count() );

                    // close stream
                p.close();
            }
            catch (Exception ex)
            {
                System.out.println("loadMessageHeaders: error while trying to load Headers:  "+ex.getMessage() );
		ex.printStackTrace();

            }

    }


    /*
    public void parseHeaders( Vector v ) throws Exception
       {
            //System.out.println("---------------------> parsing headers");

            try
            {
            for ( int i=0; i<v.size(); i++ )
            {
                Message message = (Message) v.get(i);
                Rfc822Header h = message.getHeader();

                if  ( h.get("date") != null )
                {
                    //System.out.println("---------------------> parsing date");
                    //System.out.println("date: " +h.get("date") );
                    Date date = DateParser.parseString( (String) h.get( "date" ) );
                    h.set("columba.date", date);
                    //System.out.println("date: " +h.get("columba.date") );
                }

                if ( h.get("columba.priority") != null )
                {
                    String str = (String) h.get("columba.priority");
                    h.set("columba.priority", new Integer(str) );
                }

                if ( h.get("columba.size") != null )
                {
                    String str = (String) h.get("columba.size");
                    h.set("columba.size", new Integer(str) );
                }

                if ( h.get("columba.attachment") != null )
                {
                    String str = (String) h.get("columba.attachment");
                    h.set("columba.attachment", new Boolean(str) );
                }

                if ( h.get("columba.fetchstate") != null )
                {
                    String str = (String) h.get("columba.fetchstate");
                    h.set("columba.fetchstate", new Boolean(str) );
                }

                if ( h.get("columba.flags.seen") != null )
                {
                    String str = (String) h.get("columba.flags.seen");
                    h.set("columba.flags.seen", new Boolean( str ) );
                }

                if ( h.get("columba.flags.recent") != null )
                {
                    String str = (String) h.get("columba.flags.recent");
                    h.set("columba.flags.recent", new Boolean( str ) );
                }

                if ( h.get("columba.flags.flagged") != null )
                {
                    String str = (String) h.get("columba.flags.flagged");
                    h.set("columba.flags.flagged", new Boolean( str ) );
                }

                if ( h.get("columba.flags.expunged") != null )
                {
                    String str = (String) h.get("columba.flags.expunged");
                    h.set("columba.flags.expunged", new Boolean( str ) );
                }

                if ( h.get("columba.flags.draft") != null )
                {
                    String str = (String) h.get("columba.flags.draft");
                    h.set("columba.flags.draft", new Boolean( str ) );
                }

                if ( h.get("columba.flags.answered") != null )
                {
                    String str = (String) h.get("columba.flags.answered");
                    h.set("columba.flags.answered", new Boolean( str ) );
                }
            }
            }
            catch ( Exception ex )
            {
                throw new Exception("parseheader: "+ ex.getMessage() );
            }

       }
       */

       /*
    public void loadxml()
    {
        messageCollection.reset();

        try
            {
                HeaderCacheReader reader = new HeaderCacheReader();

                Vector v = reader.open( file.getPath() );

                reader.close();


                parseHeaders( v );

                for ( int i=0; i<v.size(); i++ )
                {
                    Message m = (Message) v.get(i);
                    messageCollection.add( m );
                }






            }
            catch ( Exception ex )
            {
                System.out.println( "-----------------------> "+ex.getMessage() );
                ex.printStackTrace();
            }

    }
    */

    /*
    public void savexml()
    {
        try
            {
                HeaderCacheWriter writer = new HeaderCacheWriter();

                writer.open( file.getPath() );

                writer.startDocument();

                for ( int i=0; i<messageCollection.count(); i++ )
                {
                    Message message = messageCollection.get(i);
                    Rfc822Header header = message.getHeader();
                    Hashtable hash = header.getHashtable();

                    writer.writeHash( hash );
                }

                writer.endDocument();

                writer.close();
            }
            catch ( Exception ex )
            {
                System.out.println( ex.getMessage() );
            }
    }
    */

    public PopItem getPopItem()
    {
        return accountItem.getPopItem();
    }


    public HeaderList getHeaderList()
    {
        HeaderList headerList = new HeaderList();

        for ( int i=0; i<messageCollection.count(); i++ )
        {
            Message message = messageCollection.get(i);
            ColumbaHeader header = message.getHeader();
            Object uid = header.get("columba.pop3uid");
            headerList.add( header, uid );
        }

        return headerList;

    }

    public MessageCollection getMessageCollection()
    {
        return messageCollection;
    }



    public void actionPerformed(ActionEvent e)
    {
        Object src = e.getSource();

        if ( src.equals( timer ) )
        {
            if ( ( checkAction.isEnabled() == true ) && ( manageAction.isEnabled() == true ) )
               fetchAll(true);
            else
               timer.restart();

            return;
        }

	String action  = e.getActionCommand();
	if ( action == null ) return;
	//System.out.println("action: " +e.getActionCommand() );

	if ( action.equals( checkAction.getActionCommand() ) )
	{
	    //System.out.println("check");
	    boolean result = fetchAll(false);
	    //System.out.println("result collection: "+result );
	}
	else if ( action.equals( manageAction.getActionCommand() ) )
	{
	    //System.out.println("manage");

	    showWindow( true );
	}
    }



}
