// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.protocol;

import java.io.*;
import java.net.*;
import java.security.*;
import java.util.*;
import javax.swing.*;

import org.columba.gui.statusbar.*;
import org.columba.util.*;


public class Pop3
{
    private Socket socket;
    private BufferedReader in;
    private PrintWriter out;

    private boolean statuschecked;
    public  boolean connected;
    private String security;
    public String answer;
    private int logMethod;
    private int totalMessages;
    private int totalSize;
    private Vector messageSize;
    private String selectedMessageSizeString;
    private int lineCount;
    private int selectedMessageSize;
    private String user;
    private String password;
    private String server;

    public static final int DEFAULT_PORT = 110;
    
    public static final int USER = 1;
    public static final int APOP = 2;


    public Pop3(String user, String password, String server)
    {
        this.user = user;
        this.password = password;
        this.server = server;
        
		logMethod = USER;
    }

    public Pop3()
    {
		logMethod = USER;
    }


	public void setLoginMethod(	String str )
	{		
		if ( str.equalsIgnoreCase("USER") ) logMethod = USER;
		else if ( str.equalsIgnoreCase("APOP") ) logMethod = APOP;
		else logMethod = USER;
	}
	

	public void sendString(String s) throws IOException {

		//System.out.println("CLIENT:" + s);

		out.print(s + "\r\n");
		out.flush();
	}

    public String getServerResponse() throws IOException
    {
	return answer;
    }


    public boolean getAnswer() throws IOException
    {
		
		
		answer = in.readLine();
		
		//System.out.println("SERVER:" + answer);


		return (answer.startsWith("+OK"));
    }


	public void getNextLine() throws IOException {
		answer = in.readLine();

		//System.out.println("SERVER:" + answer);
	}

    public boolean openPort(String host) throws IOException
    {
	return openPort(host, DEFAULT_PORT);
    }

    public boolean openPort() throws IOException
    {
	return openPort(server, DEFAULT_PORT);
    }

    public boolean openPort(String host, int port) throws IOException
    {
	socket = new Socket(host, port);
	in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
	out = new PrintWriter(new OutputStreamWriter(socket.getOutputStream()));

	connected = true;
	security = null;

	if(getAnswer()){
	    int i = answer.indexOf("<");
	    if(i != -1){
		security = answer.substring(i,answer.indexOf(">")+1);
		
	    } 

	    return true;
	}
	else return false;
    }

    public int getSize()
    {
        return selectedMessageSize;
    }

    public String getSizeString()
    {
        return selectedMessageSizeString;
    }


    public boolean login(String u,String  pass) throws IOException
    {
	

	switch(logMethod){

	case USER: return userPass(u,pass);

	case APOP: return apop(u,pass);
	}

	return false;
    }

	public boolean login() throws IOException {
		switch (logMethod) {
			case USER :
				{

					return userPass(user, password);
				}

			case APOP :
				{

					return apop(user, password);
				}
		}

		return false;
	}

    private boolean userPass(String usr, String pass) throws IOException
    {
	sendString("USER "+usr);
	if( getAnswer()){
	    sendString("PASS "+pass);
	    return getAnswer();
	}
	return false;
    }

    private boolean apop(String user, String pass) throws IOException
    {
	if( security!=null){
	    try{
		MessageDigest md = MessageDigest.getInstance("MD5");
		md.update(security.getBytes());
		if(pass==null) pass="";
		byte[] digest = md.digest(pass.getBytes());
		sendString("APOP "+user+" "+digestToString(digest));
		return getAnswer();
	    }catch (NoSuchAlgorithmException e){}
	}
	return false;
    }


    public boolean logoff() throws IOException
    {
	if(connected){
	    sendString("QUIT");
	    connected = false;
                //in.close();
                //out.close();
                //socket.close();
	    return getAnswer();
	}
	return false;
    }

    public void close() throws IOException
    {
        in.close();
        out.close();
        socket.close();
    }

	public int getTotalMessages() throws IOException
	{
		/*
		if (!statuschecked)
		{
			if (!checkStat())
			{
				return -1;
			}
		}
		*/
		
		if (!checkStat())
			{
				return -1;
			}
			
		return totalMessages;
	}

    public int getTotalSize() throws IOException
    {
	if( !statuschecked ) {
	    if( !checkStat() ) {
		return -1;
	    }
	}
	return totalSize;
    }

    public int getLineCount()
        {
            return lineCount;
        }


	private boolean checkStat() throws IOException
	{
		String dummy;

		if (connected)
		{
			sendString("STAT");
			if (getAnswer())
			{
				try
				{
					dummy = answer.substring(answer.indexOf(' ') + 1);
					totalMessages = Integer.parseInt(dummy.substring(0, dummy.indexOf(' ')));

					dummy = dummy.substring(dummy.indexOf(' ') + 1);
					totalSize = Integer.parseInt(dummy);

					statuschecked = true;
					return true;
				}
				catch (NumberFormatException e)
				{
				}
			}
		}
		return false;
	}

    public Vector getMessageSizes() throws IOException
    {
	if( updateMessageSizes() != -1 ) {
	    return messageSize;
	}
	return null;
    }



    private  int updateMessageSizes() throws IOException
    {
	int size = -1;
	String dummy;

	if( connected ) {
	    sendString("LIST");
	    if ( getAnswer() ) {
		
		messageSize = new Vector();
		
		getNextLine();
		int i=0;
		while ( !answer.equals(".") )
		{
		    try 
		    {
			size = Integer.parseInt(answer.substring(answer.indexOf(' ')+1));
		    } 
                    catch( NumberFormatException e ) {}
                        
		    messageSize.addElement(new Integer( size ));
		    
		    i++;
		

		    getNextLine();
		}
		

		return i;

		/*
		for (i = 1; i<=totalMessages; i++) {
		    System.out.println("i="+i);
		    getNextLine();
		    if (answer.equals(".")) return i;
		    try {
			size = Integer.parseInt(answer.substring(answer.indexOf(' ')+1));
		    } catch( NumberFormatException e ) {}
                        messageSize.addElement(new Integer( size ));

		}
		System.out.println("for loop finished");
		getNextLine();
		if(answer.equals(".")) return i;
		*/
	    }
	    else
		System.out.println("getAnswer == false");
	}
        return -1;
    }

    /*
    public String getMessage(int messageNumber) throws IOException
    {
	StringBuffer messageBuffer = new StringBuffer();
        lineCount=0;

        //System.out.println(lineCount);


	if( connected ) {
	    sendString("RETR "+messageNumber);
	    if ( getAnswer() ) {
		getNextLine();
		while( !answer.equals(".") ) {
		    messageBuffer.append(answer+"\n");
		    getNextLine();
                    lineCount++;
		}
	    }
	}

	return messageBuffer.toString();
    }
    */


    public String getUIDL(SwingWorker worker) throws IOException
    {
	StringBuffer buffer = new StringBuffer();
        Integer parser = new Integer(0);
        int progress = 0;


	if( connected ) {
	    sendString("UIDL");
	    if ( getAnswer() )
            {
                if ( worker != null )
		   worker.setProgressBarMaximum( totalMessages );
                getNextLine();
		while( !answer.equals(".") )
                {
                    //System.out.println("SERVER: "+ answer );
                    buffer.append( answer+"\n");
                    progress ++;
                    if ( worker != null )
		       worker.setProgressBarValue( progress );
		    getNextLine();
		}
	    }
	}

	return buffer.toString();
    }

    public String getMessage(int messageNumber, SwingWorker worker) throws IOException
    {
	StringBuffer messageBuffer = new StringBuffer();
        Integer parser = new Integer(0);
        int progress = 0;
        String sizeline= new String();
        int test;
        boolean progressBar = true;


	if( connected )
        {
	    sendString("RETR "+messageNumber);
	    if ( getAnswer() )
            {
                /*
                //selectedMessageSizeString = answer.substring(3, answer.indexOf(' ',4) );

                try
                {

                    selectedMessageSize = parser.parseInt( answer.substring(3, answer.indexOf(' ',4) ).trim());
                    //worker.setProgressBarMaximum( selectedMessageSize );
                }
                catch (NumberFormatException ex )
                {
                    System.out.println( ex.getMessage() );
                    //progressBar = false;
                }
                */


		getNextLine();
		while( !answer.equals(".") )
                {
                    if ( worker.getCancel() == true ) break;

		    messageBuffer.append(answer+"\n");

                    progress = answer.length()+2;

                    worker.incProgressBarValue( progress );

		    getNextLine();
		}
                /*
                if ( progressBar == true )
                    worker.setProgressBarValue( selectedMessageSize );
                    */

	    }
	}

	return messageBuffer.toString();
    }

    public String getMessageHeader(int messageNumber ) throws IOException
    {
	StringBuffer messageBuffer = new StringBuffer();
        Integer parser = new Integer(0);
        //int progress = 0;

	if( connected ) {
	    sendString("TOP "+messageNumber+" "+0);
	    if ( getAnswer() ) {
		getNextLine();
		while( !answer.equals(".") ) {
		    messageBuffer.append(answer+"\n");
		    getNextLine();
		}
	    }
	}
	return messageBuffer.toString();
    }

    public boolean deleteMessage( int messageNumber ) throws IOException
    {
	if( connected ) {
	    sendString("DELE "+messageNumber);
	    if( getAnswer() ) return true;
	}
	return false;
    }

    public boolean noop() throws IOException
    {
	if( connected ) {
	    sendString("NOOP");
	    if( getAnswer() ) return true;
	}
	return false;
    }

    public boolean reset() throws IOException
    {
	if( connected ) {
	    sendString("RSET");
	    if( getAnswer() ) return true;
	}
	return false;
    }

    private  String digestToString(byte[] digest) {
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < 16; ++i) {
      if ((digest[i] & 0xFF) < 0x10) {
        sb.append("0");
      }
      sb.append(Integer.toHexString(digest[i] & 0xFF));
    }
    return sb.toString();
  }
}







