// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.protocol;

import java.io.*;
import java.net.*;
import java.util.*;


public class Smtp
{
    private String host;
    private String localHost;
    private int port;

    private Socket socket;
    private DataOutputStream out;
    private BufferedReader in;


    public Smtp( String hostName, int portNr )
        {
            host = hostName;
            port = portNr;

            
        }

    public Smtp( String hostName )
        {
            host = hostName;
            port = 25;

        }

    private int checkAnswer(String answer)
        {
          //  System.out.println("SERVER: "+ answer );
            if( answer.startsWith("2") ) return 0;

            return -1;
        }

    public int openPort()
        {
              String answer;

            try {
                socket = new Socket( host,port );
                out = new DataOutputStream( socket.getOutputStream() );
                in = new BufferedReader( new InputStreamReader(
                    socket.getInputStream() ) );

				if( checkAnswer( in.readLine()) !=0 ) return -1;  // Server there
                // we send a FQDN EHLO/HELO that works
                // sending the actual hostname could be considered spyware
                // and it doesn't work with machines without a FQDN name,
                // or using NAT or a TCP tunnel to another host
                out.writeBytes("HELO localhost.localdomain\r\n");  // Hello Mr.Server
                out.flush();

                answer = in.readLine();
            }
            catch( IOException e )
            {

                return -1;
            }

            return 1;//checkAnswer( answer ); // Did he answer?
        }

    public int setupMessage( String from, Vector to )
        {
            String answer = null;
            int anzAdresses;

            anzAdresses = to.size();

            try
            {
                out.writeBytes("MAIL FROM: "+from+"\r\n");
                out.flush();

                if( checkAnswer( in.readLine() ) != 0 ) return -1;

                for( int i=0;  i < anzAdresses; i++) {
                    out.writeBytes("RCPT TO: "+to.get( i )+"\r\n");
                    out.flush();

                    answer = in.readLine();
                }

            }
            catch( IOException e )
            {
                return -1;
            }

            return checkAnswer( answer )+5;
        }

    public int setupMessage( String from, String to )
        {
            String answer = null;

            try
            {
                out.writeBytes("MAIL FROM: "+from+"\r\n");
                out.flush();

                if( checkAnswer( in.readLine() ) != 0 ) return -1;

                out.writeBytes("RCPT TO: "+to+"\r\n");
                out.flush();

                answer = in.readLine();
            }
            catch( IOException e )
            {
                return -1;
            }

            return checkAnswer( answer )+5;
        }


    public int sendMessage( String message )
        {
            BufferedReader messageReader = new BufferedReader( new StringReader( message ) );
            String line;

            try {
                out.writeBytes("DATA\r\n");
                out.flush();

                if( !in.readLine().startsWith("3") ) return -1;

                line = messageReader.readLine();

                while( line != null ) {

                    if( line.equals(".")) line = new String(" .");

                    out.writeBytes(line+"\r\n");
                    line = messageReader.readLine();
                }
                out.writeBytes(".\r\n");
                out.flush();

                return checkAnswer( in.readLine() );
            }
            catch( IOException e )
            {
                return -1;
            }
        }

    public int closePort()
        {
            try {
                out.writeBytes("QUIT\r\n");
                out.flush();

                return checkAnswer( in.readLine() );
            }
            catch( IOException e )
            {
                return -1;
            }
        }

}







