// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.


package org.columba.util;

import java.util.*;
import java.io.*;


import org.columba.main.*;


public class TempFileStore
{
    private MainInterface mainInterface;
    private static File tempDir;

    public TempFileStore( MainInterface mainInterface )
    {
	this.mainInterface = mainInterface;

	File configDir = mainInterface.config.configDirectory;

	tempDir = new File( configDir, "tmp" );
	if ( tempDir.exists() == false ) tempDir.mkdir();

	cleanupTempFolder();
    }

    // this is called on startup, to get rid of old tempfiles
    protected void cleanupTempFolder()
    {
	File[] list = tempDir.listFiles();
	
        for ( int i=0; i<list.length; i++ )
        {
            File file = list[i];
	    try
	    {
		file.delete();
	    }
	    catch ( Exception ex )
	    {
		ex.printStackTrace();
	    }
	}
    }

    protected static String replaceWhiteSpaces( String s )
    {
	StringBuffer buf = new StringBuffer( s );

	int i=0;
	char ch;
	while ( i<buf.length() )
	{
	    
	    ch = buf.charAt(i);
	    if ( ch==' ' )
	    {
		buf.replace( i,i+1, "_");
		i+=1;
	    }

	    i++;
	}

	return buf.toString();
    }

    public static File createTempFile()
    {
	File file = new File( tempDir, "columba"+System.currentTimeMillis()+".tmp" );

	return file;
    }


    public static File createTempFile( String s )
    {
	String str = replaceWhiteSpaces(s);
	File file = new File( tempDir, str );

	return file;
    }


    public static File createTempFileWithSuffix( String suffix )
    {
	File file = new File( tempDir, "columba"+System.currentTimeMillis()+"."+suffix);

	return file;
    }
}    
